#!/bin/ksh

#****************************************************************//
#* COPYRIGHT_COMMENT_TAG_BEGIN
#* Licensed Materials - Property of IBM (C)
#* Copyright IBM Corp. 2009. All Rights Reserved.
#* US Government Users Restricted Rights - Use, duplication,
#* or disclosure restricted by GSA ADP Schedule Contract with
#* IBM Corp
#* COPYRIGHT_COMMENT_TAG_END
#****************************************************************//

PROGNAME="$(basename $0)"
PROGPATH="$(cd $(dirname $0); pwd)"

# This is all needed for the nls_replace look up
bindir=`pwd`/$(dirname $0)
[ "$(print $0 | grep '\./')" = "" ] && [ "$(print $0 | grep '^/.*')" = "" ] && [ `pwd` = '/' ] && bindir=`pwd`$(dirname $0)
[ "$(print $0 | grep '^/.*')" != "" ] && bindir=$(dirname $0)
export TEXTDOMAINDIR=$bindir/../config/nls/msg
export PATH=$bindir:$PATH


# Usage statement
Use_report() {
    print -u2 "\n$PROGNAME GSKIT_COMMAND *[ARGS]\n"
    exit 2
}

Err_report() {
   case $1 in
      #$NLS-E$ "$PROGNAME warning: $2,continuing ..."
      warn) print -u2 "$PROGNAME `nls_replace 'KCI0520I'`: $2, `nls_replace 'KCI0521I'` ..."
            sleep 2
            return
            ;;
          #$NLS-E$ "$PROGNAME failure: $2."
      fail) print -u2 "$PROGNAME `nls_replace 'KCI0522I'`: $2."
            exit 2
            ;;
      info) print -u2 "$PROGNAME        : $2."
            return
            ;;
   esac
}


# Check if there is at least the gskit_command arg on the CLI
if [ $# -lt 1 ]
then
   Use_report
fi

# Save the incoming argument for execution later - at least in AIX they were
# getting corrupted by the below operations
ARGS=$*
COMMAND=$1
# This shifts off the $1 from $*, and leaves just the arguements they want passed to the command
shift
REST_ARGS=$*

# This test whether the command entered had _64 on it (meaning a 64 bit command)
USER_ENTERED_64_COMMAND=`echo $COMMAND | grep "_64"`

# Set up for and retrieve cArch
CurRegPath=$CANDLEHOME/registry
ArchTbl=$CurRegPath/archdsc.tbl
DynShl=$CANDLEHOME/bin/dynarch.shl

cd $PWD
. $DynShl $ArchTbl

# Now we have cArch, let's find the bin arch by seeing which ver file that is found in

# This will return one, or the last, gs ver file.. the last one will be 64 bit, and that
# is what we want in case there is more than one
verName=`grep -l $cArch $CANDLEHOME/registry/gs*.ver | tail -1`
baseInfo=`basename $verName`
binArch=`echo $baseInfo | sed 's/^gs//g' | sed 's/\.ver//g'`

if [ "$binArch" = "" ] ; then
  #$NLS-E$ "binArch could not be determined"
  Err_report fail "`nls_replace 'KCI1300E'`"
fi

# Set our all important gskit dir
GSKIT_DIR=$CANDLEHOME/$binArch/gs

if [ "$GSKIT_DIR" = "" ] ; then
  #$NLS-E$ "GSKIT_DIR could not be determined"
  Err_report fail "`nls_replace 'KCI1301E'`"
fi

# Determine which lib dir to use - lib or lib64 - We'll assume at first its 64 bit - if we don't find it, we'll reset to lib
GSKIT_LIB="$GSKIT_DIR/lib64"

if [ ! -d "$GSKIT_LIB" ] ; then
  # Lib 64 doesn't exist, so we'll use lib
  GSKIT_LIB="$GSKIT_DIR/lib"
else
  # We only set this one if its 64 bit
  export LD_LIBRARY_PATH_64=$GSKIT_LIB:$LD_LIBRARY_PATH_64
fi

# Now set some library paths needed once we've set the proper lib value
export PATH=$GSKIT_DIR/bin:$PATH
export LD_LIBRARY_PATH=$GSKIT_LIB:$LD_LIBRARY_PATH
export LIBPATH=$GSKIT_LIB:$LIBPATH
export SHLIB_PATH=$GSKIT_LIB:$SHLIB_PATH

print ""

# Check if the GSKit executable is actually there to execute - if its not, check COMMAND_64
# then inform the customer appropriately - the command is $1 from the original args
COMMAND_CHECK=$GSKIT_DIR/bin/$COMMAND

# This will be the default one we assume
COMMAND_TO_EXECUTE=$COMMAND_CHECK

if [ ! -f "$COMMAND_CHECK" ] ; then

  # The file doesn't exist... if they entered 32 bit, we can see about a 64. If they entered 64, we're done.
  if [ -z "$USER_ENTERED_64_COMMAND" ] ; then

    # On some platforms (HPUX at least) has _32 on the gskit commands - so we need to trim that
    # _32 off before we add _64 on the end if we are going to try and check for 64 if 32 fails
    check=`echo $COMMAND | grep '_32'`
    if [ -n "$check" ]; then
      # we found _32 on the command - and we didnt find the command - we now
      # need to trim _32 and add _64 and check that command
      COMMAND2=`echo $COMMAND | sed 's/_32//g'`
      COMMAND_CHECK2=$GSKIT_DIR/bin/$COMMAND2
      COMMAND_CHECK64="${COMMAND_CHECK2}_64"
    else
      # The command didn't have "_32" in there, we'll just append and go
      COMMAND_CHECK64=${COMMAND_CHECK}_64
    fi

    #$NLS-E$ print "You entered a 32 bit GSKit command [$COMMAND_CHECK] and it cannot be found."
    print " `nls_replace 'KCI1302I' $COMMAND_CHECK`"
    #$NLS-E$ print "We will automatically check the 64 bit version [$COMMAND_CHECK64]..."
    print " `nls_replace 'KCI1303I' $COMMAND_CHECK64`"

    if [ ! -f "$COMMAND_CHECK64" ] ; then
      print ""
      #$NLS-E$ print "We checked [$COMMAND_CHECK64] and it could not be found."
      print " `nls_replace 'KCI1304E' $COMMAND_CHECK64`"
      #$NLS-E$ print "Please check the command in the local GSKit directory and try again."
      print " `nls_replace 'KCI1305E'`"
      print ""
      exit 2
    else
      print ""
      #$NLS-E$ print "We were able to find the 64 bit command [$COMMAND_CHECK64] and we will now execute that."
      print " `nls_replace 'KCI1306I' $COMMAND_CHECK64`"
      COMMAND_TO_EXECUTE=$COMMAND_CHECK64
    fi

  else
    # Since it was a 64 bit command, we're done
    #$NLS-E$ print "You entered a 64 bit GSKit command [$COMMAND_CHECK] and it cannot be found."
    print " `nls_replace 'KCI1307E' $COMMAND_CHECK`"
    #$NLS-E$ print "Please check the command in the local GSKit directory and try again."
    print " `nls_replace 'KCI1308E'`"
    print ""
    exit 2
  fi

fi

print ""

# Execute using the proper command (including the full proper path) and the rest of the args.

$COMMAND_TO_EXECUTE $REST_ARGS


