#!/bin/ksh

######################################################################
# * Licensed Materials - Property of IBM
# * 
# * 5724-C04
# *
# * Copyright IBM Corporation 2007-2013 All Rights Reserved               #IV33132
######################################################################

locateCMD () {
  inCMD=$1
  if [ -z "$inCMD" ]
  then
    outCMD=""
  else
    if [ "$osType" = HP-UX ] || [ "$osType" = SunOS ]
    then
      outCMD=`which $inCMD 2> /dev/null | grep -v "^no $inCMD"`
    else
      outCMD=`which $inCMD 2> /dev/null`
    fi
    [ -z "$outCMD" ] && outCMD=`find /bin /usr/bin /local/bin /usr/local/bin /sbin /usr/sbin /local/sbin /usr/local/sbin -name $inCMD 2> /dev/null`
  fi
  print $outCMD
}

report_function()
{
  case $1 in
    warn) if [[ -a "$logFile" ]]; then
            print "$PROGNAME warning:  $2, continuing ..." >> $logFile
          fi
          #$NLS-E$ "$PROGNAME warning:  $2, continuing ..."
          print -u2 "$PROGNAME `nls_replace 'KCI0535I'`:  $2, `nls_replace 'KCI0536I'` ..."
          sleep 2
          return
          ;;
    fail) if [[ -a "$logFile" ]]; then
            print "$PROGNAME failure:  $2."  >> $logFile
            #print "$PROGNAME failure:  $2."
            print -u2 "`nls_replace 'KCI1200E' "UpdateAutoRun.sh" $logFile`"
          else
            print "$PROGNAME failure:  $2."
          fi
          exit 2
          ;;
    info) if [[ -a "$logFile" ]]; then
            print "$PROGNAME info:  $2" >> $logFile
          else
            print "$PROGNAME info:  $2"
          fi
          return
          ;;
    infoCont) if [[ -a "$logFile" ]]; then
            print "$PROGblnk     :  $2" >> $logFile
          else
            print "$PROGblnk     :  $2"
          fi
          return
          ;;
  esac
}

Use_report() {
    print -u2 "\n$PROGNAME [ -h install_dir ] \n"
    exit 2
}

init_rcBase() { #---------------------------------------------------------#IZ58055
  # Initialize rcBaseDir and rcBaseName variables. This is the directory  #IZ58055
  # and file name of the init script. The file name is missing the file   #IZ58055
  # number that is appended to the end: that comes later.                 #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  case "$os" in                                                           #IZ58055
    li|lx|lp|ls)                                                          #IZ58055
      rcBaseDir="/etc/init.d"                                             #IZ58055
      rcBaseName="ITMAgents"                                              #IZ58055
      ;;                                                                  #IZ58055
    hp)                                                                   #IZ58055
      rcBaseDir="/sbin/init.d"                                            #IZ58055
      rcBaseName="ITMAgents"                                              #IZ58055
      ;;                                                                  #IZ58055
    so)                                                                   #IZ58055
      rcBaseDir="/etc/init.d"                                             #IZ58055
      rcBaseName="ITMAgents"                                              #IZ58055
      ;;                                                                  #IZ58055
    ai)                                                                   #IZ58055
      rcBaseDir="/etc"                                                    #IZ58055
      rcBaseName="rc.itm"                                                 #IZ58055
      ;;                                                                  #IZ58055
    *)                                                                    #IZ58055
      report_function fail "$os platform is not supported."               #IZ58055
      ;;                                                                  #IZ58055
  esac                                                                    #IZ58055
}                                                # (init_rcBase())        #IZ58055
                                                                          #IZ58055
init_filenum() { #--------------------------------------------------------#IZ58055
  # Initialize the filenum and filenum_new variables. The filenum         #IZ58055
  # variable represents the file number that is appended to the rcBase    #IZ58055
  # variable to come up with the complete file name in rcFile. The        #IZ58055
  # filenum_new variable indicates whether this is a new file number (1)  #IZ58055
  # which means this is the first time an init script has been created.   #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  filenum=$(cat $CANDLEHOME/registry/AutoStart 2> /dev/null);             #IZ58055
  if [[ $filenum -eq 0 ]]; then                                           #IZ58055
    # begin feature 204833 (RFE 33478)
    # When -s switch for install.sh is used during pristine install we skip updating
    # startup scripts. In this case value 0 is stored in registry/AutoStart file.
    # All subsequent installations/upgrades will also skip startup script update.
    # In order to restore startup script update functionality it's enough to remove
    # registry/AutoStart file.
    if [[ -f "$CANDLEHOME/registry/AutoStart" ]]; then
      #$NLS-E$ "Startup script update has been skipped."
      nls_replace 'KCI1365I';
      exit 0;
    fi
    # end feature 204833 (RFE 33478)
    filenum_new=1                                # first time             #IZ58055
    filenum=1;                                                            #IZ58055
    while [[ -f "$1$filenum" ]]; do                                       #IZ58055
      filenum=$(( $filenum + 1 ))                                         #IZ58055
    done                                                                  #IZ58055
    print $filenum > $CANDLEHOME/registry/AutoStart                       #IZ58055
  else                                                                    #IZ58055
    filenum_new=0                                # not first time         #IZ58055
  fi                                                                      #IZ58055
}                                                # (init_filenum())       #IZ58055
                                                                          #IZ58055
init_script() { #---------------------------------------------------------#IZ58055
  # Create a new init script. This base script is later modified with     #IZ58055
  # agent-specific information.                                           #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  if [[ "$os" = "so" ]]; then                                             #IZ58055
    print "#!/bin/sh"                                             > "$1"  #IZ58055
  else                                                                    #IZ58055
    print "#!/bin/ksh"                                            > "$1"  #IZ58055
  fi                                                                      #IZ58055
                                                                          #IZ58055
  print "# (C) Copyright IBM Corporation 2007, 2012."            >> "$1"  #IV23322
  print "# All Rights Reserved"                                  >> "$1"  #IZ58055
  print "# IBM Tivoli Monitoring"                                >> "$1"  #IZ58055
                                                                          #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  # For Linux, we need to add a block of entries that define our needs    #IZ58055
  # to chkconfig. The format is different for RedHat vs. SUSE.            #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  if [[ "$os" = +(li|ls|lx|lp) ]]; then                                   #IZ58055
    print "#"                                                    >> "$1"  #IZ58055
    print "### BEGIN INIT INFO"                                  >> "$1"  #IZ58055
    if [[ "$linux_dist" = "RedHat" ]]; then      # RedHat...              #IZ58055
      print "# chkconfig: 2345 99 10"                            >> "$1"  #IZ58055
      print "# description: IBM Tivoli Monitoring"               >> "$1"  #IZ58055
    else                                         # SUSE...                #IZ58055
      print "# Provides:          ITMAgents$filenum"             >> "$1"  #IZ58055
      print "# Required-Start:    \$all"                         >> "$1"  #IZ58055
      print "# Required-Stop:"                                   >> "$1"  #IZ58055
      print "# Default-Start:     2 3 4 5"                       >> "$1"  #IZ58055
      print "# Default-Stop:      0 6"                           >> "$1"  #IZ58055
      print "# Short-Description: IBM Tivoli Monitoring"         >> "$1"  #IZ58055
      print "# Description:       IBM Tivoli Monitoring"         >> "$1"  #IZ58055
    fi                                                                    #IZ58055
    print "### END INIT INFO"                                    >> "$1"  #IZ58055
  fi                                                                      #IZ58055
                                                                          #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  # Finally add the code portion of the script. We pull in the content of #IV23322
  # ITMAgents.sh to use as a basis. This gets updated later.              #IV23322
  #-----------------------------------------------------------------------#IZ58055
  print "#"                                                      >> "$1"  #IV23322
  print "CANDLEHOME=\"$CANDLEHOME\""                             >> "$1"  #IV23322
  print "export CANDLEHOME"                                      >> "$1"  #IV23322
  cat $CANDLEHOME/bin/ITMAgents.sh                               >> "$1"  #IV23322
                                                                          #IZ58055
  chmod 755 "$1"                                 # set execute perms      #IZ58055
}                                                # (init_script())        #IZ58055
                                                                          #IZ58055
set_runlevels() { #-------------------------------------------------------#IZ58055
  # Tell the operating system about our init script.                      #IZ58055
  #-----------------------------------------------------------------------#IZ58055
                                                                          #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  # For LINUX systems, we used to create symbolic links manually. Since   #IZ58055
  # SUSE requires chkconfig to manage runlevels and RedHat supports it,   #IZ58055
  # we switched over to that instead. However, we have to clean up the    #IZ58055
  # old links that may have predated our use of chkconfig.                #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  if [[ "$os" = +(li|ls|lx|lp) ]]; then                                   #IZ58055
    runlevelName="$rcBaseName$filenum"                                    #IZ58055
                                                                          #IZ58055
    if [[ $filenum_new -ne 1 ]]; then            # need to rm old links   #IZ58055
      if [[ "$linux_dist" = "RedHat" ]]; then    # RedHat...              #IZ58055
        rm -f /etc/rc.d/rc[0-6].d/K10$runlevelName                        #IZ58055
        rm -f /etc/rc.d/rc[0-6].d/S99$runlevelName                        #IZ58055
      else                                       # SUSE...                #IZ58055
        rm -f /etc/init.d/rc[0-6].d/K10$runlevelName                      #IZ58055
        rm -f /etc/init.d/rc[0-6].d/S99$runlevelName                      #IZ58055
      fi                                                                  #IZ58055
    fi                                                                    #IZ58055
                                                                          #IZ58055
    $chkconfigCMD --del "$runlevelName" > /dev/null 2>&1                  #IZ58055
    $chkconfigCMD --add "$runlevelName" > /dev/null                       #IZ58055
                                                                          #IZ58055
    if [[ "$linux_dist" = "RedHat" ]]; then      # RedHat...              #IZ58055
      $chkconfigCMD --levels 2345 "$runlevelName" on > /dev/null          #IZ58055
    else                                         # SUSE...                #IZ58055
      $chkconfigCMD --set "$runlevelName" 2345 > /dev/null                #IZ58055
    fi                                                                    #IZ58055
  fi                                                                      #IZ58055
                                                                          #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  # For HP systems, we just create symbolic links.                        #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  if [ "$os" = hp ]; then                                                 #IZ58055
    runlevelName="$rcBaseName$filenum"                                    #IZ58055
                                                                          #IZ58055
    ln -fs "$rcFile" "/sbin/rc0.d/K100$runlevelName"                      #IZ58055
    ln -fs "$rcFile" "/sbin/rc1.d/K100$runlevelName"                      #IZ58055
    ln -fs "$rcFile" "/sbin/rc2.d/S500$runlevelName"                      #IZ58055
  fi                                                                      #IZ58055
                                                                          #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  # For Solaris systems, we just create symbolic links. For some reason,  #IZ58055
  # Solaris does not seem to always respect the -f option, so we need to  #IZ58055
  # redirect stderr to prevent messages from showing to the user.         #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  if [ "$os" = so ]; then                                                 #IZ58055
    runlevelName="$rcBaseName$filenum"                                    #IZ58055
                                                                          #IZ58055
    ln -fs "$rcFile" "/etc/rc0.d/K10$runlevelName" 2> /dev/null           #IZ58055
    ln -fs "$rcFile" "/etc/rc1.d/K10$runlevelName" 2> /dev/null           #IZ58055
    ln -fs "$rcFile" "/etc/rc2.d/S99$runlevelName" 2> /dev/null           #IZ58055
  fi                                                                      #IZ58055
                                                                          #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  # For AIX systems, we use mkitab to create the entry we need in         #IZ58055
  # /etc/inittab. We make a backup the first time we are installed.       #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  if [ "$os" = ai ]; then                                                 #IZ58055
    runlevelName="rcitm$filenum"                                          #IZ58055
                                                                          #IZ58055
    if [[ $filenum_new -eq 1 ]]; then                                     #IZ58055
      cp -p /etc/inittab /etc/inittab.itm$filenum                         #IZ58055
    fi                                                                    #IZ58055
                                                                          #IZ58055
    rmitab "$runlevelName" 2> /dev/null          # clean up old entry     #IZ58055
                                                                          #IZ58055
    mkitab_opt=""                                                         #IZ58055
    lsitab rcnfs > /dev/null 2>&1                # check for rcnfs        #IZ58055
    if [[ $? -eq 0 ]]; then                      # if it's there...       #IZ58055
      mkitab_opt="-i rcnfs"                      # rcitm is after rcnfs   #IZ58055
    else                                                                  #IZ58055
      lsitab rctcpip > /dev/null 2>&1            # check for rctcpip      #IZ58055
      if [[ $? -eq 0 ]]; then                    # if it's there...       #IZ58055
        mkitab_opt="-i rctcpip"                  # rcitm is after rctcpip #IZ58055
      fi                                                                  #IZ58055
    fi                                                                    #IZ58055
                                                                          #IZ58055
    mkitab $mkitab_opt "$runlevelName:2:once:$rcFile start > /dev/console 2>&1" #IV23322
  fi                                                                      #IZ58055
}                                                # (set_runlevels())      #IZ58055

#This is to get instance from env file.
getInstance()
{
  pCode=$1
  instance=""
 
  defaultUser=$(getDefaultUser $pCode) 
  defaultAutoStart=$(getDefaultAutoStart $pCode)   
  [[ -n $defaultUser ]] && user="$defaultUser"
# For Oracle and Sybase we will check <hostname>_<pp>_<instance>.cfg file
  if [[ "$pCode" = @(or|oy|mq) ]]; then
    host=$(hostname | cut -d. -f1)
    for rec in $(ls "$CANDLEHOME"/config/"$host"_"$pCode"_*.cfg  "$CANDLEHOME"/config/"$host"\.*_"$pCode"_*.cfg 2>/dev/null |grep -v kcirunas | grep -v kdyrunas)
    do
      rec=${rec##*/}  # Remove the directory path.
      rec=${rec%.*}   # Remove the ".cfg" extension.
      if [[ "${rec}" = "${host}."* ]]; then                               #IZ58877
        instance=${rec#${host}.*_${pCode}_};                              #IZ58877
      else                                                                #IZ58877
        instance=${rec#${host}_${pCode}_};                                #IZ58877
      fi;                                                                 #IZ58877
      [[ -z "$instance" ]] && continue
      isExist=$(grepInStart "agent -o $instance start $pCode");           #IV33132
      [[ -z "$isExist" ]] && writeAgentInstanceToStartStop $user $pCode $instance $defaultAutoStart
    done
    if [[ "$pCode" != "mq" ]]; then
      return
    fi
  fi

# the instance is saved as platform for certain agents
  instanceFile=$(cat $CANDLEHOME/config/.ConfigData/k${pCode}env)
  instanceFile=$(print "$instanceFile" | grep -v ^default | grep -v ^$os | grep -v FIELDSEP)
  instanceFile=$(print "$instanceFile" | cut -d"|" -f1 | sort -u)
  # defect 73338 - use -p for um instances
  if [[ "$pCode" = "um" ]] 
  then
    parm="-p"
  else
    parm="-o"
  fi  
  if [[ -n "$instanceFile" ]]; then
    for instance in $instanceFile
    do
      isExist=$(grepInStart "agent $parm $instance start $pCode");        #IV33132
      [[ -z "$isExist" ]] && writeAgentInstanceToStartStop $user $pCode $instance $defaultAutoStart
    done
  else

# Now let's check for env files.
#Some instances are saved as variables in the env file
    instanceFile=$(cat $CANDLEHOME/config/.ConfigData/k${pCode}env)
    instanceFile=$(print "$instanceFile" | grep -v ^default | grep -w INSTANCE)
    for instanceLine in $instanceFile
    do
      instance=$(print "$instanceLine"| cut -d"|" -f3)
      [[ -z "$instance" ]] && continue
      isExist=$(grepInStart "agent $parm $instance start $pCode");        #IV33132
      [[ -z "$isExist" ]] && writeAgentInstanceToStartStop $user $pCode $instance $defaultAutoStart
     done
  fi

  if [ -f $CANDLEHOME/bin/.agent.$pCode ] ; then
    prdlIst=$pCode
  
    cur_dir=`pwd`
    cd $CANDLEHOME/config
 
    for inst in $(ls ${pCode}_*.environment 2> /dev/null)
    do
      instname=${inst#${pCode}_}
      instname=${instname%.environment}
      isExist=$(grepInStart "agent $parm $instname start $pCode");        #IV33132
      [[ -z "$isExist" ]] && writeAgentInstanceToStartStop $user $pCode $instname $defaultAutoStart
    done
    isExist=$(grepInStart "gent start $pCode");                           #IV33132
    [[ -z "$isExist" ]] && writeAgentToStartStop $user $pCode $defaultAutoStart #IV33132
  
    cd $cur_dir
  fi
  
}

function getCmdWrappers { #-----------------------------------------------#IZ68336
  # Set strings that wrap around the commands to be launched in the       #IZ68336
  # start/stop script:                                                    #IZ68336
  # - For z/OS systems, we need to launch 32-bit TEMS and TEPS in 32-bit  #IZ68336
  #   mode using the s390 command.                                        #IZ68336
  # - For all systems, we need to launch under ksh because some other     #IZ68336
  #   shells (csh and tcsh, specifically) don't support redirection of    #IZ68336
  #   stderr.                                                             #IZ68336
  # - For all systems, we run CANDLEHOME/bin/itmcmd and redirect both     #IZ68336
  #   stdout and stderr to /dev/null.                                     #IZ68336
  # We do this by setting variables "CmdWrapFront" and "CmdWrapBack". The #IZ68336
  # command to be launched is enclosed between these to accomplish the    #IZ68336
  # above items.                                                          #IZ68336
  #-----------------------------------------------------------------------#IZ68336
  pcCode=$1                                                               #IZ68336
                                                                          #IZ68336
  s390cmd=""                                                              #IZ68336
                                                                          #IZ68336
  print "${0}: pcCode: $pcCode" >> $logFile                               #IZ68336
  print "${0}: cArch: $cArch" >> $logFile                                 #IZ68336
                                                                          #IZ68336
  #-----------------------------------------------------------------------#IZ68336
  # Check to see if a) we are setting up a 32-bit TEMS or TEPS and b) if  #IZ68336
  # we are on a a 64-bit zLinux system. If so, then we need to start the  #IZ68336
  # TEMS or TEPS in 32-bit mode using the s390 command.                   #IZ68336
  #-----------------------------------------------------------------------#IZ68336
  if [[ "$cArch" != ls3* ]]; then                                         #IZ68336
    print "${0}: cArch (${cArch}) is not zLinux... no s390" >> $logFile   #IZ68336
  elif [[ "${pcCode}" != +(cq|ms) ]]; then       # only TEMS/TEPS apply   #IZ68336
    print "${0}: ${pcCode} runs 32/64-bit... no s390" >> $logFile         #IZ68336
  elif [[ -n $(find $CANDLEHOME/registry/ -type f -name ${pcCode}ls3*[5-6].ver) ]]; then
    print "${0}: ${pcCode} 64-bit code installed... no s390" >> $logFile  #IZ68336
  else                                                                    #IZ68336
    if [[ -z "${realMode}" ]]; then              # if first time...       #IZ68336 
      realMode=`$CANDLEHOME/bin/checkZMode.sh | grep realMode | cut -d":" -f2` #IZ68336
    fi                                                                    #IZ68336 
    if [[ "$realMode" = "32" ]]; then                                     #IZ68336
      print "${0}: realMode==32, no need for s390 command" >> $logFile    #IZ68336
    else                                                                  #IZ68336 
      print "${0}: realMode==64, use s390 command for 32-bit" >> $logFile #IZ68336
      s390cmd="s390 "                                                     #IZ68336 
    fi                                                                    #IZ68336
  fi                                                                      #IZ68336
                                                                          #IZ68336
  #-----------------------------------------------------------------------#IZ68336
  # For all systems, we need to force the startup command to run under    #IZ68336
  # ksh. This takes care of problems when root's default shell is csh     #IZ68336
  # or tcsh.                                                              #IZ68336
  #-----------------------------------------------------------------------#IZ68336
  CmdWrapSearch="${CANDLEHOME}/bin/itmcmd";                               #IV33132
  CmdWrapFront="/bin/ksh -c '${s390cmd}${CmdWrapSearch}";                 #IV33132
  CmdWrapBack=">/dev/null 2>&1'";                                         #IZ68336
                                                                          #IZ68336
  print "${0}: CmdWrapSearch: \"${CmdWrapSearch}\"" >> $logFile           #IV33132
  print "${0}: CmdWrapFront:  \"${CmdWrapFront}\"" >> $logFile            #IV33132
  print "${0}: CmdWrapBack:   \"${CmdWrapBack}\"" >> $logFile             #IV33132
}                                                # (getCmdWrappers())     #IZ68336

function writeCmdToStartStop { #------------------------------------------#IV33132
   # Write a single autostart command to the start or stop file.          #IV33132
   #                                                                      #IV33132
   # - If kxxrunas specifies not to autostart or if the existing entry is #IV33132
   #   commented out, the new entry is commented out.                     #IV33132
   #                                                                      #IV33132
   # - If kxxrunas specifies a userid of "DO_NOT_SU", the new entry is    #IV33132
   #   generated without the su wrapper.                                  #IV33132
   #                                                                      #IV33132
   # Some commands change over time, so we have a dual search mechanism   #IV33132
   # to check for comments. First, we look for the current                #IV33132
   # implementation. If we don't find it, we look for a set of            #IV33132
   # commands passed to us. These are separated by the ":" character in   #IV33132
   # the variable ${isrch}.                                               #IV33132
   #----------------------------------------------------------------------#IV33132
   typeset icmd="${1}";                          # command to add         #IV33132
   typeset isrch="${2}";                         # old commands to search #IV33132
   typeset iuid="${3}";                          # su userid (kcirunas)   #IV33132
   typeset iautostart="${4}";                    # autostart (kcirunas)   #IV33132
   typeset ifile="${5}";                         # start/stop file        #IV33132
                                                                          #IV33132
   typeset commentMode="   ";                    # assume active entries  #IV33132
   if [[ "${iautostart}" = "no" ]]; then         # kcirunas says no start #IV33132
      commentMode="#  ";                         # comment it             #IV33132
   else                                          # chk commented entries  #IV33132
      ${grepCMD} -q "^[ \t]*#.*${CmdWrapSearch} ${icmd}" "${backFile}" 2>/dev/null; #IV33132.1
                                                                          #IV33132
      if (( ${?} == 0 )); then                   # found commented record #IV33132
         commentMode="#  ";                      # keep it commented      #IV33132
      elif [[ "${isrch}" != "" ]]; then                                   #IV33132
         typeset isrch1;                         # one cmd from ${isrch}  #IV33132
         print "${isrch}" | tr ':' '\n' | while read isrch1; do           #IV33132
            ${grepCMD} -q "^[ \t]*#.*${CmdWrapSearch} ${isrch1}" "${backFile}" 2>/dev/null; #IV33132.1
            if (( ${?} == 0 )); then             # found commented record #IV33132
               commentMode="#  ";                # keep it commented      #IV33132
            fi;                                  # (if($?==0))            #IV33132
         done;                                   # (while(read...))       #IV33132
      fi;                                        # (else($?==0))          #IV33132
   fi;                                           # (else($iauutostar...)) #IV33132
                                                                          #IV33132
   if [[ "${iuid}" = "DO_NOT_SU" ]]; then        # su disabled?           #IV33132
      typeset suWrapFront="";                                             #IV33132
      typeset suWrapBack="";                                              #IV33132
   else                                                                   #IV33132
      typeset suWrapFront="${su_value} - ${iuid} -c \"";                  #IV33132
      typeset suWrapBack="\"";                                            #IV33132
   fi;                                           # (else(${iuid}="DO...)) #IV33132
                                                                          #IV33132
   print -- "${commentMode}${suWrapFront}${CmdWrapFront} ${icmd} ${CmdWrapBack}${suWrapBack}" >> "${ifile}"; #IV33132
                                                                          #IV33132
}                                                # (writeToStartStop())   #IV33132

######################################################################
######################################################################

writeAgentInstanceToStartStop() { #---------------------------------------#IV33132
  # ${1} = su userid from runas file                                      #IV33132
  # ${2} = product code                                                   #IV33132
  # ${3} = instance                                                       #IV33132
  # ${4} = autostart (yes|no) from xxxrunas.cfg... optional               #IV33132
  #-----------------------------------------------------------------------#IV33132
  getCmdWrappers "${2}";                                                  #IV33132

  # defect 73338 - use -p for um instances
  if [[ "${2}" = "um" ]]; then                                            #IV33132
    parm="-p"
  else
    parm="-o"
  fi

  if [[ "${2}" != "cj" ]];  then
    thisCmd="agent ${parm} ${3} start ${2}";                              #IV33132
    writeCmdToStartStop "${thisCmd}" "" "${1}" "${4}" "${startFile}";     #IV33132
  fi

  thisCmd="agent ${parm} ${3} stop ${2}";                                 #IV33132
  writeCmdToStartStop "${thisCmd}" "" "${1}" "${4}" "${stopFile}";        #IV33132
}

writeAgentToStartStop() { #-----------------------------------------------#IV33132
  # ${1} = su userid from runas file                                      #IV33132
  # ${2} = product code                                                   #IV33132
  # ${3} = autostart (yes|no) from runas file... optional                 #IV33132
  #-----------------------------------------------------------------------#IV33132
  getCmdWrappers "${2}";                                                  #IV33132
  
  if [[ "${2}" != "cj" ]]; then                                           #IV33132
      thisCmd="agent start ${2}";                                         #IV33132
      writeCmdToStartStop "${thisCmd}" "" "${1}" "${3}" "${startFile}";   #IV33132
  fi

  thisCmd="agent stop ${2}";                                              #IV33132
  if [ "$2" = "kf" ]; then                                                #IV33132
    thisCmdSrch="agent -f stop ${2}";                                     #IV33132
  else                                                                    #IV33132
    thisCmdSrch="";                                                       #IV33132
  fi;                                                                     #IV33132
  writeCmdToStartStop "${thisCmd}" "${thisCmdSrch}" "${1}" "${3}" "${stopFile}"; #IV33132
}

writeServerToStartStop() { #----------------------------------------------#IV33132
  # ${1} = su userid from runas file                                      #IV33132
  # ${2} = TEMS name                                                      #IV33132 
  # ${3} = autostart (yes|no) from runas file... optional                 #IV33132
  #-----------------------------------------------------------------------#IV33132
  getCmdWrappers "ms";                                                    #IV33132

  thisCmd="server start ${2}";                                            #IV33132
  writeCmdToStartStop "${thisCmd}" "" "${1}" "${3}" "${startFile}.ms";    #IV33132
  
  thisCmd="server stop ${2}";                                             #IV33132
  writeCmdToStartStop "${thisCmd}" "" "${1}" "${3}" "${stopFile}.ms";     #IV33132
}

######################################################################
######################################################################

getValueFromLine()
{
  print $(print $line | cut -d">" -f2 | cut -d"<" -f1)
}

getToken1FromLine()
{
  print $(print $line | cut -d">" -f1 | cut -d"<" -f2)
}

##################################3
# functions gets default user for product configured in kdyrunas/kcirunas
# <productCode>mq</productCode>
# <default>
#   <user>defaultUser</user>
#   <autoStart>yes</autoStart>
# </default>
getDefaultUser(){
  typeset host=$(hostname | cut -d. -f1)
  typeset cfgKdy=$CANDLEHOME/config/${host}_kdyrunas.cfg
  typeset cfgKci=$CANDLEHOME/config/kcirunas.cfg
  typeset defUser1
  [[ -f "$cfgKdy" ]] && defUser1=$(getDefaultUserAndAutoStart $cfgKdy $1 | cut -d":" -f1 )  
  typeset defUser2
  [[ -f "$cfgKci" ]] && defUser2=$(getDefaultUserAndAutoStart $cfgKci $1| cut -d":" -f1 )    
  typeset result=$defUser2
  [[ -n "$defUser1" ]] && result=$defUser1
  #print "default user is: $result"
  print $result
}
##################################3
# functions gets default autostart for product configured in kdyrunas/kcirunas
# See snippet in getDefaultUser
getDefaultAutoStart(){
  typeset host=$(hostname | cut -d. -f1)
  typeset cfgKdy=$CANDLEHOME/config/${host}_kdyrunas.cfg
  typeset cfgKci=$CANDLEHOME/config/kcirunas.cfg
  typeset defAutostart1
  [[ -f "$cfgKdy" ]] && defAutostart1=$(getDefaultUserAndAutoStart $cfgKdy $1 | cut -d":" -f2 )    
  typeset defAutostart2
  [[ -f "$cfgKci" ]] && defAutostart2=$(getDefaultUserAndAutoStart $cfgKci $1| cut -d":" -f2 )  
  typeset result=$defAutostart2
  [[ -n "$defAutostart1" ]] && result=$defAutostart1
  print $result
}

########################################################
########### returns "default user:default Autostart" ###
########### it accept path to kcirunas/kdyrunas file and product code
#<default>
#  <!user>defuseros</user>
#  <autoStart>yes</autoStart>
#</default>
 
######################################################## 
getDefaultUserAndAutoStart(){

  typeset runasFile=$1
  typeset pc=$2
  typeset defUser
  typeset defAutoStart

  #print "parsing $runasFile for default values of $pc"
  typeset defaultFlag=false

  while read line
    do
    #print "$line"
    token1=$(getToken1FromLine)
    if [[ "$token1" = "productCode" ]]; then
      if [[ "$(getValueFromLine)" = "$pc" ]]; then
          prodFlag=true
        else
          prodFlag=false
        fi
    fi   
    if [[ "$prodFlag" = true ]]; then
      case "$token1" in
        /agent)
          prodFlag=false
          ;;
        !productCode)
          prodFlag=false
          ;;
        "default")
          defaultFlag=true	
	  ;;
        autoStart)
          [[ "$defaultFlag" = "true" ]] && defAutoStart=$(getValueFromLine)
	  ;;
        user)
          [[ "$defaultFlag" = "true" ]] && defUser=$(getValueFromLine)
          ;;
        "/default")
	  defaultFalg=false
          #Writing to file default autostart and default user  
	  #print "Def user: $defUser, Def autostart $defAutoStart"
	  
          break;
          ;;
      esac
    fi   #prodFlag
  done < $runasFile
  print "$defUser:$defAutoStart"
}



######################################################################
######################################################################

grepInStart() {
   if [[ -f "${startFile}" ]]; then                                       #IV33132
      print $(grep "$1" "${startFile}" 2> /dev/null);                     #IV33132
   else                                                                   #IV33132
      print "";                                                           #IV33132
   fi;                                                                    #IV33132
}

######################################################################
######################################################################

update_rcFile()
{
print "$editString" > $SEDscript
checkit=0
rcFileTmp=$rcTmp/${rcBaseName}.new.$$
cp -f $rcFile $rcFileTmp
checkit=$?
if [[ $checkit = 0 ]]; then
  sed -f $SEDscript $rcFileTmp > $rcFile
  checkit=$?
  if [[ $checkit != 0 ]]; then
    cp -f $rcFileTmp $rcFile
  fi
fi
rm -f $rcFileTmp
rm -f $SEDscript
}

######################################################################
######################################################################
# Check for <hostname>_kdyrunas.cfg file. The <hostname> is a short name only.
# <agent>
#  <productCode>gb</productCode>
#  <instance>
#   <name>instancename</name>
#   <user>userid</user>
#  </instance>
#  <default>
#   <name>instancename</name>
#   <autoStart>userid</autoStart>
#  </default>

# </agent>


checkForCfg()
{

  pCode=$1
  runas=$2    
  tmp=0
  hostFlag=false
  prodFlag=false
  instanceFlag=false
  host=$(hostname | cut -d. -f1)
  cfgname=$CANDLEHOME/config/${host}_${runas}.cfg
  [[ "$runas" = kcirunas ]] && cfgname=$CANDLEHOME/config/${runas}.cfg

  # if file does not exist, return
  [[ ! -f "$cfgname" ]] && return
  username=""
  instance=""
  autoStart=""  
  defaultAutoStart=$(getDefaultAutoStart $pCode)
  #[[ -z "$defaultAutoStart" ]] && defaultAutoStart="yes"
  defaultUser=$(getDefaultUser $pCode)

  while read line
  do
    token1=$(getToken1FromLine)
    
    if [[ "$token1" = "productCode" ]]; then
        if [[ "$(getValueFromLine)" = "$pCode" ]]; then
          prodFlag=true
        else
          prodFlag=false
        fi
    fi    
    if [[ "$prodFlag" = true ]]; then
      case "$token1" in
        /agent)
          prodFlag=false
          ;;
        !productCode)
          prodFlag=false
          ;;
        autoStart)
	  autoStart=$(getValueFromLine)
	  ;;        
	name)
          instance=$(getValueFromLine)
          ;;
        user)
          username=$(getValueFromLine)
          ;;
        /instance)
	  [[ -z "$username" ]] && [[ -n "$defaultUser" ]] && username="$defaultUser"
          [[ -z "$username" ]] && username=$user
	  [[ -z "$autoStart" ]] && autoStart=$defaultAutoStart	 
          if [[ -z "$instance" ]] || [[ "$instance" = "None" ]]
          then
             isExist=$(grepInStart "agent start $pCode");                 #IV33132
             [[ -z "$isExist" ]] && writeAgentToStartStop $username $pCode $autoStart
          else
             if [[ "$pCode" = "ms" ]]; then                               #IV40954
               isExist=$(grepInStart "server start $instance");           #IV40954
               [[ -z "$isExist" ]] && writeServerToStartStop $username $instance $autoStart; #IV40954
             else                                                         #IV40954
               # defect 73338 - use -p for um instances
               if [[ "$pCode" = "um" ]] 
               then
                 parm="-p"
               else
                 parm="-o"
               fi  
               isExist=$(grepInStart "agent $parm $instance start $pCode" | grep $username); #IV33132
               [[ -z "$isExist" ]] && writeAgentInstanceToStartStop $username $pCode $instance $autoStart
             fi
          fi
          username=""
          instance=""
          autoStart=""
          ;;
      esac
    fi   #prodFlag

  done < $cfgname
  
  return
}


######################################################################
######################################################################
#This is to make sure certain agents have instance included otherwise
#this is an error.
verifyAgents()
{
  # if file does not exist, return
  [[ ! -f "$startFile" ]] && return
  while read readLine
  do
    instOpt=$(print $readLine | cut -d " " -f7)
    instPC=$(print $readLine | cut -d " " -f10)
    [[ "$instOpt" != "-o" && "$instOpt" != "-p" ]] && instPC=$(print $readLine | cut -d " " -f8)

    if [[ "$instPC" = "$pCode" && "$instOpt" != "-o" && "$instOpt" != "-p" ]]; then
      prodLine=$(grep "^$pCode" $CANDLEHOME/registry/proddsc.tbl)  # get product descript
      prodLine=$(print $prodLine| cut -d "|" -f2)
      report_function  warn  "$prodLine ($pCode) required start/stop with instance"
    fi
  done < $startFile
}


######################################################################
######################################################################
#Create two files based on current install.
#The scipt will write to restart script.

create_start_stop_file()
{

  startFile=$rcTmp/start
  stopFile=$rcTmp/stop
  pcFile=$rcTmp/pc
  userFile=$rcTmp/user
  startLine=""
  tmpTemsUser=""
  tmpTemsName=""

  [[ -a "$startFile" ]] && rm $startFile
  [[ -a "$stopFile" ]] && rm $stopFile
  [[ -a "${startFile}.ms" ]] && rm "$startFile}.ms";                      #IV33132
  [[ -a "${stopFile}.ms" ]] && rm "${stopFile}.ms";                       #IV33132
  [[ -a "$pcFile" ]] && rm $pcFile
  [[ -a "$userFile" ]] && rm -f $userFile

  $CANDLEHOME/bin/cinfo -d > $pcFile

  [[ -a "$userTmp" ]] && rm -f $userTmp
  pc=$(grep "^\"..\"" $pcFile | cut -c2,3 | sort -r )
  arch=$(ls $CANDLEHOME |grep ^$os);                                      #IV33132
  for pCode in $pc
  do
    for Arch in $arch
    do
      # here Arch is the arch, pCode is pc
      if [[ -d "$CANDLEHOME/$Arch/$pCode/bin" ]]; then
        user=$(ls -ld "$CANDLEHOME/$Arch/$pCode/bin" | awk '{print $3}')
        
        if [[ "$pCode" != @(iw|ui|ue|gs|w0|ii) ]]; then                      #IV33132
          instance=""

          tmpuser=$user
          inst=""
          checkForCfg $pCode "kdyrunas"
          checkForCfg $pCode "kcirunas"

          [[ "$pCode" != "ms" ]] && getInstance $pCode
          instance=""

          # this is to make sure ud, or, oy, sa, tx and ub agents already have instance setup.
          if [[ "$pCode" = @(sa|tx|qv|gb|ph|ud|or|oy|ub) ]]; then
            verifyAgents
            continue
          fi

      [[ -f $CANDLEHOME/config/${pCode}.ini ]] && result=`grep "INSTALL_AGENT_TYPE=MULTI-INSTANCE" $CANDLEHOME/config/${pCode}.ini`
          if [[ -n $result ]]
          then
            verifyAgents
            continue
          fi

          if [[ "$pCode" = "ms" ]]; then
            cms=`ls $CANDLEHOME/tables`
            for temsName in $cms
             do
               [[ "$temsName" = @(cicatrsq|PLACEHOLDER) ]] && continue
               temsLength=`print $temsName| wc -c`
               temsLength=$((temsLength-4))
               oldTems=`print $temsName | cut -c ${temsLength}-`
               [[ "$oldTems" = "_old" ]] && continue
               [[ ! -d "$CANDLEHOME/tables/$temsName/ATTRLIB" ]] && continue
               writeServerToStartStop $user $temsName "yes";              #IV33132
             done  #end cms
           else  #this is not ms, so we can find agent start pp" or "agent -o instance start pp"
             # defect 73338 - make sure we preserve the default um entry
             isExist=$(grepInStart "agent start $pCode");                 #IV33132
             [[ -z "$isExist" ]] && writeAgentToStartStop $user $pCode "yes"; #IV33132
           fi  #end ms
        fi  #end of ui
      elif [[ "${pCode}" = "kf" ]]; then                                  #IV33132
        #-----------------------------------------------------------------#IV33132
        # kf is special. It doesn't have a bin directory (which is how    #IV33132
        # we got here, but it does need an autostart.                     #IV33132
        #-----------------------------------------------------------------#IV33132
        if [[ -f "$CANDLEHOME/$Arch/$pCode/kkfstart.sh" ]]; then
          user=$(ls -ld "$CANDLEHOME/$Arch/$pCode/kkfstart.sh" | awk '{print $3}')
          isExist=$(grepInStart "start $pCode");                          #IV33132
          [[ -z "$isExist" ]] && writeAgentToStartStop $user $pCode "yes"; #IV33132
        fi
      fi  #end of bin

    done #end arch
  done  #end pc
  
  #-----------------------------------------------------------------------#IV33132
  # We need to start the ms component first and stop it last. To do this, #IV33132
  # we earlier wrote its entries to separate files. No we just need to    #IV33132
  # add them to the real start and stop files. These are later merged     #IV33132
  # into the final script in reverse order. So, what we're doing here     #IV33132
  # looks backwards. To get ms started first, we add it to the end of the #IV33132
  # start file. To get it stopped last, we add it to the beginning of the #IV33132
  # stop file.                                                            #IV33132
  #-----------------------------------------------------------------------#IV33132
  if [ -a "${startFile}.ms" ] ; then                                      #IV33132
     cat "${startFile}.ms" >> "${startFile}";                             #IV33132
     rm "${startFile}.ms";                                                #IV33132
  fi;                                                                     #IV33132
                                                                          #IV33132
  if [ -a "${stopFile}.ms" ] ; then                                       #IV33132
     if [ -a "${stopFile}" ] ; then                                       #IV40953
        cat "${stopFile}" >> "${stopFile}.ms";                            #IV40953
     fi;                                                                  #IV40953
     mv "${stopFile}.ms" "${stopFile}";                                   #IV33132
  fi;                                                                     #IV33132
    
  if [[ "$linux_dist" = "RedHat" ]]; then 
  	touch /var/lock/subsys/$rcBaseName$filenum 2> /dev/null
  	print "touch /var/lock/subsys/$rcBaseName$filenum 2> /dev/null" >> $startFile 
  	print "rm -f /var/lock/subsys/$rcBaseName$filenum 2> /dev/null" >> $stopFile
  fi
}

######################################################################
######################################################################

delete_start_all()
{
   # looping is not required to delete lines using sed instead of ed
   if [[ -n "$(grep 'gent start all' $rcFile 2>/dev/null)" ]]; then
      editString="/gent start all/d"
      update_rcFile
      if [[ $checkit != 0 ]]; then
         report_function fail "Delete of agent start all record failed."
      else
         report_function info "Delete of agent start all record successful."
      fi
   else
     return
   fi
}

delete_stop_all()
{
   # looping is not required to delete lines using sed instead of ed
   if [[ -n "$(grep 'gent stop all' "$rcFile" 2>/dev/null)" ]]; then
      editString="/gent stop all/d"
      update_rcFile
      if [[ $checkit != 0 ]]; then
         report_function fail "Delete of agent stop all record failed."
      else
         report_function info "Delete of agent stop all record successful."
      fi
   else
      return
   fi
}

######################################################################
######################################################################
#Add one line to script

add_start_line()
{
   startPath=$(print $line | cut -d"\"" -f2)
   if [[ -z "$(grep  "${startPath}"  $rcFile)" ]]; then
      print "$line" > $SEDline
      editString="/start_all()/{
N
r $SEDline
}"
      update_rcFile
      rm -f $SEDline
      if [[ $checkit != 0 ]]; then
         report_function fail "Unable to add start agent line in $rcFile."
      else
         report_function info "Add of start agent line successful."
      fi
   else
      report_function info "Start agent line already exists in $rcFile."
   fi
}

######################################################################
######################################################################
#Add one line to script

add_stop_line()
{
   stopPath=$(print "$line" | cut -d"\"" -f2)
   if [[ -z "$(grep  "${stopPath}"  $rcFile)" ]]; then
      print "$line" > $SEDline
      editString="/stop_all()/{
N
r $SEDline
}"
      update_rcFile
      rm -f $SEDline
      if [[ $checkit != 0 ]]; then
         report_function fail "Unable to add stop agent line in $rcFile."
      else
         report_function info "Add of stop agent line successful."
      fi
   else
      report_function info "Stop agent line already exists in $rcFile."
   fi
}
######################################################################
######################################################################





######################################################################
#   MAIN   ###########################################################
######################################################################

typeset logFile
typeset -i filenum                               # script file number     #IZ58055
typeset -i filenum_new                           # 1=first time           #IZ58055

PROGNAME=$(basename $0)
PROGblnk="$(print $PROGNAME | sed 's/./ /g')"

#defect 58729: to support the install.sh from ITM6.1, also need to support the "UpdateAutoRun.sh <ch>"
CANDLEHOME=$1
while getopts :h: opt
do case $opt in
     h) CANDLEHOME=$OPTARG
        CANDLEHOME=$(eval print $CANDLEHOME)  # Process ~ if it was entered.
        ;;
     ?) Use_report
        ;;
   esac
done

[[ -n "$CANDLEHOME" ]] || CANDLEHOME=$(cd $(dirname $0) ; dirname $(pwd) )
export CANDLEHOME

WHITESPACE=$(print -n " \t")                     # grep pattern           #IZ58055

NLS_REPLACE_DIR=$CANDLEHOME/bin
PATH=$PATH:$NLS_REPLACE_DIR
export PATH

[[ ! -d "$CANDLEHOME" ]] && report_function fail "$CANDLEHOME is not a valid installation."

if [[ $(print "$CANDLEHOME" | wc -w) -ne 1 ]]; then
  report_function fail "Blanks are not permitted in CANDLEHOME: $CANDLEHOME"
fi

[[ -z "$(whoami 2>/dev/null)" ]] && alias whoami='id | cut -d"(" -f2 | cut -d")" -f1'

# if no AutoStart and not root exit now.
myID=$(id 2> /dev/null | cut -d"=" -f2 2> /dev/null | cut -d"(" -f1 2> /dev/null)
[[ "$myID" = "0" ]] || report_function fail "You must be \"root\" to update the auto start script."

# secure environment when installing pre 623 images into already secured itm
if [[ "$myID" = "0" && -z "$AUTORUN_SECURE" && -n "$CDNAME" ]];
then
  SECUREFILE=$CANDLEHOME/registry/SecureMode
  if [[ -f "$SECUREFILE" ]];
  then
    $CANDLEHOME/bin/itmsecure.sh autorun
  fi
fi


logFile=$CANDLEHOME/logs/UpdateAutoRun.log
[[ -f $logFile ]] && rm $logFile 2>/dev/null
touch $logFile

rcTmp=$CANDLEHOME/tmp
SEDscript=$rcTmp/SEDscript
SEDline=$rcTmp/SEDline

dynShl=$CANDLEHOME/bin/dynarch.shl
[[ -x "$dynShl" ]] || report_function fail "Cannot execute \"$dynShl\""

regPath=$CANDLEHOME/registry
archTbl=$regPath/archdsc.tbl  # archTbl is needed by dynarch.shl
[[ -r "$archTbl" ]] || report_function fail "Cannot read \"$archTbl\""

# Determine the machine's architecture.

. $dynShl $archTbl  # dynarch.shl will determine a value for cArch.
[[ -n "$cArch" ]] || report_function fail "Could not determine the machine's architecture"
os=$(print "$cArch" |cut -c1-2)
                                                                          #IZ58055
if [[ "$os" = +(li|ls|lx|lp) ]]; then                                     #IZ58055
  chkconfigCMD=$(locateCMD chkconfig)
  [ -z "$chkconfigCMD" ] && report_function fail "Unable to find chkconfig command"
  $chkconfigCMD --terse > /dev/null 2>&1                                  #IZ58055
  if [[ ${?} -eq 1 ]]; then                                               #IZ58055
    linux_dist="RedHat";                                                  #IZ58055
  else                                                                    #IZ58055
    linux_dist="SUSE";                                                    #IZ58055
  fi                                                                      #IZ58055
fi                                                                        #IZ58055

#-------------------------------------------------------------------------#IV33132.1
# We use "grep -q...", which requires the real grep command instead of    #IV33132.1
# the nonstandard one we typically get from Solaris.                      #IV33132.1
#-------------------------------------------------------------------------#IV33132.1
if [[ -f /usr/xpg4/bin/grep ]]; then                                      #IV33132.1
   grepCMD="/usr/xpg4/bin/grep";                                          #IV33132.1
else                                                                      #IV33132.1
   grepCMD="grep";                                                        #IV33132.1
fi;                                                                       #IV33132.1
                                                                          #IV33132.1
# IY96372 start =============================================================================
# Set up what value of su we are going to use - we only want to do this if the platform is HP,
# since that is the only platform right now having a problem. If su is found under the paths
# we want to check, we use them.. otherwise not, which is no worse than right now

#set the default value of su now, and then just use it if its not overwritten

su_value=`which su` 2> /dev/null
[ -f "$su_value" ] || su_value="su"

if [ "$os" = hp ]
then
  # we have an HP platform, we want to check two diff files

  if [[ -f /bin/su ]]
  then
    su_value="/bin/su"
  fi

  if [[ -f /usr/bin/su ]]
  then
    su_value="/usr/bin/su"
  fi

fi

#now at this point, either su_value was found and set on any other platform (or even HP maybe),
#or on HP, the other files were found and set properly - either way, we now have a proper su value
# IY96372 stop ==============================================================================

#-------------------------------------------------------------------------#IZ58055
# Determine the name of the init script.                                  #IV33132
#-------------------------------------------------------------------------#IZ58055
init_rcBase;                                     # get base dir/basename  #IZ58055
init_filenum "$rcBaseDir/$rcBaseName"            # get file number        #IZ58055
rcFile="$rcBaseDir/$rcBaseName$filenum"          # script full dir/fname  #IZ58055
backFile="$rcTmp/$rcBaseName$filenum.bak";                                #IZ58055
                                                                          #IZ58055
#-------------------------------------------------------------------------#IZ58055
# Back up/replace the existing script and then create a new base.         #IZ58055
#-------------------------------------------------------------------------#IZ58055
if [[ $filenum_new = 1 ]] | [[ ! -f "$rcFile" ]]; then                    #IZ58055
  rm -f "$backFile"                              # disregard old backup   #IZ58055
elif [[ ! -r "$rcFile" ]]; then                  # old script readable?   #IZ58055
  report_function fail "Can't read from file $rcFile."                    #IZ58055
elif [[ ! -w "$rcFile" ]]; then                  # old script writable?   #IZ58055
  report_function fail "Can't write to file $rcFile."                     #IZ58055
else                                             # make a backup          #IZ58055
  cp "$rcFile" "$backFile" 2> /dev/null                                   #IZ58055
                                                                          #IZ58055
  report_function info     "File $rcFile was backed up to $backFile"      #IZ58055
  report_function infoCont "before updates.  Please review $backFile and" #IZ58055
  report_function infoCont "make changes to $rcFile if necessary."        #IZ58055
  report_function infoCont "Caution: $backFile will be replaced and any"  #IZ58055
  report_function infoCont "custom changes will be removed when"          #IZ58055
  report_function infoCont "$PROGNAME is run again."                      #IZ58055
fi                                                                        #IZ58055
                                                                          #IZ58055
init_script "$rcFile";                           # start with new script  #IZ58055
                                                                          #IZ58055
#-------------------------------------------------------------------------#IZ58055
# Update the new init script with current start/stop incantations.        #IZ58055
#-------------------------------------------------------------------------#IZ58055
create_start_stop_file

if [[ -s "$startFile" && -s "$stopFile" ]]; then
  delete_start_all
  exec 3< $startFile
  while read -u3 line ; do
    add_start_line
  done
  exec 3<&-

  delete_stop_all                                                         #IV23322
  exec 3< $stopFile                                                       #IV23322
  while read -u3 line ; do                                                #IV23322
    add_stop_line                                                         #IV23322
  done                                                                    #IV23322
  exec 3<&-                                                               #IV23322
else
  print "Start or Stop file is empty" >> $logFile
fi

#-------------------------------------------------------------------------#IZ58055
# Tell the operating system about our shiny new init script.              #IZ58055
#-------------------------------------------------------------------------#IZ58055
set_runlevels;                                   # chkconfig, mkitab, etc #IZ58055
                                                                          #IV23322
#$NLS-E$ "Automatic start at system initialization has been configured."  #IV23322
nls_replace 'KCI0117I';                                                   #IV23322
if [ "${os}" != "ai" ]; then                                              #IV23322
   #$NLS-E$ "Automatic stop at system shutdown has been configured."      #IV23322
   nls_replace 'KCI0118I';                                                #IV23322
else                                                                      #IV23322
   #$NLS-E$ "You must manually configure automatic stop at system         #IV23322
   #$NLS-E$ shutdown. Create or modify /etc/rc.shutdown to invoke the     #IV23322
   #$NLS-E$ command '$1 stop'."                                           #IV23322
   nls_replace 'KCI0119I' "${rcFile}";                                    #IV23322
fi;                                                                       #IV23322
######################################################################
#   END OF MAIN   ####################################################
######################################################################
