#!/bin/ksh
#
# ****************************************************************************
# * Licensed Materials - Property of IBM
# * 
# * OCO Source Materials - IBM Confidential
# * 5724-C04
# *
# * Copyright IBM Corporation 2007-2011 All Rights Reserved
# *
# * The source code for this program is not published or other-
# * wise divested of its trade secrets, irrespective of what has
# * been deposited with the U.S. Copyright Office.
# *****************************************************************************

[ -n "$CANDLEHOME" ] || CANDLEHOME=$(cd $(dirname $0) ; dirname $(pwd) )

verfile=$(ls $CANDLEHOME/registry/iu*.ver)
	
hasAccess=1
	
# checking directory access permissions, needed +r+x 
# $1 - directory full path
# $2 - user
# $3 - group
checkPermissions(){
	directory=$1
	user=$2	
	group=$3

	hasAccess=1

	dirUser=$(ls -ld $directory | awk '{ print $3 }')
	dirGroup=$(ls -ld $directory | awk '{ print $4 }')

	ownerReadPermission=$(ls -ld $directory | awk '{ print $1 }' | cut -c 2 )
	ownerExecutePermission=$(ls -ld $directory | awk '{ print $1 }' | cut -c 4 )
		
	if [[ $user != $dirUser || $ownerReadPermission != "r" || $ownerExecutePermission != "x" ]]
	then
		if [[ -x "/usr/xpg4/bin/id" ]]
		then
			belongsToGroup=$(/usr/xpg4/bin/id -Gn $user | grep -w $dirGroup)
		else
			belongsToGroup=$(id -Gn $user | grep -w $dirGroup)
		fi

		groupReadPermission=$(ls -ld $directory | awk '{ print $1 }' | cut -c 5 )
		groupExecutePermission=$(ls -ld $directory | awk '{ print $1 }' | cut -c 7 )

		if [[ -z $belongsToGroup || $groupReadPermission != "r" || $groupExecutePermission != "x" ]]
		then
			allReadPermission=$(ls -ld $directory | awk '{ print $1 }' | cut -c 8 )
			allExecutePermission=$(ls -ld $directory | awk '{ print $1 }' | cut -c 10 )
				
			if [[ $allReadPermission != "r" || $allExecutePermission != "x" ]]
			then
				hasAccess=0
			fi
		fi
	fi
}

if [[ -n $verfile ]]
then
	verfile=${verfile##*/iu}
	arch=${verfile%%\.*}
	ihs="${CANDLEHOME}/${arch}/iu/ihs/conf"
	ihsUser=$(grep "^User\ " $ihs/httpd.conf | awk '{ print $2 }')
	ihsGroup=$(grep "^Group\ " $ihs/httpd.conf | awk '{ print $2 }')
	documentRoot=$(grep "^DocumentRoot\ " $ihs/httpd.conf | awk '{ print $2 }' | sed -e 's/^"//' | sed -e 's/"$//')

	documentRoot=${documentRoot%/$}

	while [ -n "$documentRoot" ] 
	do
		checkPermissions $documentRoot $ihsUser $ihsGroup

		if [ $hasAccess -eq 0 ]
		then
			exit 1	
		fi

		documentRoot=${documentRoot%/*}
	done
fi
