#!/bin/ksh

# Logic:
#
#       This determines what the machine mode and software mode is for a zLinux machine
#
#       - Reference: "s390" switches into 32 bit mode from 64 bit mode on 64 bit machines - "exit" exists from that mode
#
#       - uname -a and uname -m gives the software mode the machine is in
#       - file /usr/bin/ld on that file tells you the hardware package installed - 32 if 32, 64 if 64
#
#
#       > uname -a         <= 64 bit regular mode
#       Linux TVT1041 2.6.5-7.191-s390x #1 SMP Tue Jun 28 14:58:56 UTC 2005 s390x s390x s390x GNU/Linux
#
#       > uname -m         <= 64 bit regular mode
#       s390x
#
#       > file /usr/bin/ld         <= 64 bit regular mode
#       /usr/bin/ld: ELF 64-bit MSB executable, IBM S/390, version 1 (SYSV), for GNU/Linux 2.4.1, dynamically linked (uses shared libs), stripped
#
# So record two variables, REAL & CURRENT modes
#
# Notes:
#       - as written originally, this is made for zLinux, it had un-tested results on non-zLinux

# We default this to 32, it will be set to 64 as needed
realMode="32"
# We default this to 32, it will be set to 64 as needed
currentMode="32"

# Determine the read mode using architecture info on key file
file /usr/bin/ld | grep 64-bit >/dev/null 2>&1
[[ "$?" = "0" ]] && realMode="64"

uname -m | grep s390x >/dev/null 2>&1
[[ "$?" = "0" ]] && currentMode="64"

# print it out for easy parsing
print "realMode:$realMode"
print "currentMode:$currentMode"

