#!/bin/ksh

# @(#) db2check.sh - determines a machine's DB2 install location

bindir=`pwd`/$(dirname $0)
[ "$(print $0 | grep '\./')" = "" ] && [ "$(print $0 | grep '^/.*')" = "" ] && [ `pwd` = '/' ] && bindir=`pwd`$(dirname $0)
[ "$(print $0 | grep '^/.*')" != "" ] && bindir=$(dirname $0)

export PATH=$bindir:$PATH

if [[ -z "$1" ]]
then
   # In order to maintain backwards compatibility with the HD callpoint from FP5,
   # this script must return some thing other than an error. Returning the hard
   # coded value /usr/opt/db2_08_01  will cause the HD callpoint to leave the hd.ini file
   # unmodified. The KHD_CLASSPATH will be updated by the KHD config code to include
   # the correct path to the jar files based on the users input.

   # At some point in the future, the code may be updated such that the instance name
   # will be a required variable by restoring the two following lines.
   # $NLS-E$ "Usage: db2check.sh instance_name"
   # nls_replace 'KCI1202E'
   # exit 2

   print "/usr/opt/db2_08_01"
else 

   instancehome=$(eval print "~$1" 2>/dev/null)

   if [[ -d "$instancehome/sqllib" ]]
   then

      # Only display if the DB2 dir we got is a real thing and the lib link exists
      if [[ -d "$instancehome/sqllib/lib" ]]
      then
         print $instancehome/sqllib
      else
         # $NLS-E$ "Unable to locate db2 install location."
         nls_replace 'KCI1203E'
         exit 4
      fi

   else
      # $NLS-E$ "$1 is not a valid DB2 instance."
      nls_replace 'KCI1204E' $1
      exit 3
   fi
fi
