#!/bin/ksh
# **********************************************************************
# * Licensed Materials - Property of IBM
# * 5724-C04
# * Copyright IBM Corporation 2007-2015 All Rights Reserved
# **********************************************************************

Use_report()
{
  echo "$PROGNAME -a system architecture -h itm installation directory -p directory with installation packages [ -g gskit version to install ( 32 or 64) ]"
  exit 1
}

Err_report()
{
  case $1 in
          #$NLS-E$ "$PROGNAME warning:  $2, continuing ..."
    warn) print -u2 "$PROGNAME `nls_replace 'KCI0535I'`:  $2, `nls_replace 'KCI0536I'` ..."
          sleep 2
          return
          ;;
          #$NLS-E$ "$PROGNAME failure:  $2."
    fail) print -u2 "$PROGNAME `nls_replace 'KCI0537I'`:  $2."
          return 2
          ;;
    info) [ -n "$VERBOSE" ] && print -u2 "$PROGNAME        :  $2."
          return
          ;;
          #$NLS-E$ "$PROGNAME error: $2."
   error) print -u2 "$PROGNAME `nls_replace 'KCI0737I'`: $2."
          return
          ;;
  esac
}

# cleans layed out gskit files from ITMHOME in case of a failed gskit installation
gskCleanUp() {
  keepBadGSKit
  backDir=$candleGskitRunTimeDirectory/${gskArch}/gs.old
  if [[ -d "$backDir" ]]; 
  then
    Log_report "Reverting gskit $gskArch ver file and component in $ITMHOME"
    RevertGSKit ${gskArch} gs${gskArch}.ver
  else
    Log_report "Removing gskit $gskArch ver file and component from $ITMHOME"
    # remove backup files here.
    rm -f  $ITMHOME/registry/gs$gskArch.ver > /dev/null 2>&1
    rm -rf $ITMHOME/$gskArch/gs > /dev/null 2>&1
  fi
}

errorUnTar() {
  Failure_report "UNTAR ERROR: error while untaring file $1"
}

errorBack() {
  Failure_report "BACKUP ERROR: error while backing up $1/gs"
  # remove backup file here
  RMbackupGSKit $1 $2
}

errorRevert() {
  Failure_report "RESTORE ERROR: error while restoring $1/gs"
}

errorKeepTar() {
  Failure_report "TAR ERROR: error while taring backup file $1"
  # remove backup file here
  RMkeepBadGSKit $1
}

gskErrorCheckInstall() {
  Failure_report_exit "$1 - GSK check error, preInstall test failed" 4
}

gskErrorVerifyInstall() {
  Failure_report_exit "$1 - GSK check error, verifyInstall test failed" 4
}

gskErrorPostInstall() {
  Failure_report_exit "$1 - GSK check error, postInstall test failed" 4
}

Failure_report() {
  print "$(date +%m.%d.%Y\ %T) $PROGNAME: error $1" >> $logFile
  Err_report fail "$1"
}

Failure_report_exit() {
  Failure_report "$1"
  gskCleanUp
  exit $2
}

Info_report() {                                                        # 208983
  print "$(date +%m.%d.%Y\ %T) $PROGNAME: info $1" >> $logFile;        # 208983
Err_report info "$1";                                                  # 208983
}                                                                      # 208983
                                                                       # 208983
Info_report_exit() {                                                   # 208983
Info_report "$1";                                                      # 208983
  gskCleanUp;                                                          # 208983
  exit $2;                                                             # 208983
}                                                                      # 208983
                                                                       # 208983
# -------------------
# Global variable desctiprion
# gskKitHomeDirectory=$ITMHOME               - ITM installation directory
# gskitDirs=$gskKitHomeDirectory/$gskArch      - dest directory for GSK
# --------------------

# --------------
# untar gskit tar archive into ITMHOME/tmp directory
# $1 - tar archive name
# $2 - architecture
# $3 - ver file
# --------------

untarGSKit() {
  typeset TarFile=$candleGSKitProdDir/$1
  Log_report "Untar file $TarFile into $candleGskitRunTimeDirectory"
  [ -f "$TarFile" ] || noExpectedFileError $TarFile
  CMD="gskitUnTarPkg $TarFile $2 >/dev/null"
  runCommandPerfectDirectory $candleGskitRunTimeDirectory $CMD || errorUnTar $TarFile
  # ver file
  getVerFromTar $TarFile $3 $CANDLETMP
  [ -f "$tmpVerFile" ] || packageError $tmpVerFile
  Log_report "Copying $tmpVerFile => $ITMHOME/registry/$3"
  cp $tmpVerFile $ITMHOME/registry/$3 || copyingError $tmpVerFile $ITMHOME/registry/$3
  # set permissions
  if [[ ! -d $candleGskitRunTimeDirectory/$2/gs/icc ]] ; then
    typeset CMD="chmod 777 $candleGskitRunTimeDirectory/$2/gs/bin/* $candleGskitRunTimeDirectory/$2/gs/lib*/* $candleGskitRunTimeDirectory/$2/gs/lib*/*/icc/*/*"
  else
    typeset CMD="chmod 777 $candleGskitRunTimeDirectory/$2/gs/bin/* $candleGskitRunTimeDirectory/$2/gs/lib*/* $candleGskitRunTimeDirectory/$2/gs/icc/*/*"
  fi 
  runCommandPerfect $CMD
}

# --------------
# backup gskit into a new directory
# $1 - architecture
# $2 - ver file
# --------------

backupGSKit() {
  Log_report "Backup current GSKit $1/gs installation into $1/gs.old"
  if [[ -d "$candleGskitRunTimeDirectory/$1/gs" &&
        -f "$ITMHOME/registry/$2" ]]; then
    if [[ -d "$candleGskitRunTimeDirectory/$1/gs.old" ]]; then
      rmCMD="rm -rf $1/gs.old"
      runCommandPerfectDirectory $candleGskitRunTimeDirectory $rmCMD || errorBack $1 $2
    fi
#   Do not log this command. It is just to clean up a stray link if it exists
    find $candleGskitRunTimeDirectory/$1/gs/lib* -type l \( -name lib -o -name lib64 \) -exec unlink {} \; 2>/dev/null
    case $MoveCopy in
      move)
        mvCMD="mv $1/gs $1/gs.old"
        runCommandPerfectDirectory $candleGskitRunTimeDirectory $mvCMD || errorBack $1 $2
      ;;
      copy)
        mkdirCMD="mkdir $1/gs.old"
        runCommandPerfectDirectory $candleGskitRunTimeDirectory $mkdirCMD || errorBack $1 $2
        cpCMD="cp -Rp $1/gs/* $1/gs.old"
        runCommandPerfectDirectory $candleGskitRunTimeDirectory $cpCMD || errorBack $1 $2
      ;;
    esac
    cpCMD="cp -p $2 ${2}.old"
    runCommandPerfectDirectory $ITMHOME/registry $cpCMD || errorBack $1 $2
  else
    Log_report "No local GSKit for $1/gs.  Backup skipped."
  fi
}

# --------------
# Remove backup files because the backup failed
# $1 - architecture
# $2 - ver file
# --------------

RMbackupGSKit() {
  Log_report "Remove GSKit failed backup for $1/gs"
  case $MoveCopy in
    move)
      if [[ -d "$candleGskitRunTimeDirectory/$1/gs.old" ]]; then
        if [[ -d "$candleGskitRunTimeDirectory/$1/gs" ]]; then
          ## Something strange happened, report it.
          Log_report "$MoveCopy - Both $1/gs and $1/gs.old exist"
        else
          ## This is expected.
          mvCMD="mv $1/gs.old $1/gs"
          runCommandPerfectDirectory $candleGskitRunTimeDirectory $mvCMD
        fi
      fi
    ;;
    copy)
      if [[ -d "$candleGskitRunTimeDirectory/$1/gs.old" &&
            -d "$candleGskitRunTimeDirectory/$1/gs" ]]; then
        ## This is expected.
        rmCMD="rm -rf $1/gs.old"
        runCommandPerfectDirectory $candleGskitRunTimeDirectory $rmCMD
      fi
    ;;
  esac
  if [[ -f "$ITMHOME/registry/${2}.old" ]]; then
    rmCMD="rm ${2}.old"
    runCommandPerfectDirectory $ITMHOME/registry $rmCMD
  fi
}

# --------------
# Remove backup files because everything worked
# $1 - architecture
# $2 - ver file
# --------------

CLRbackupGSKit() {
  Log_report "Remove GSKit backup for $1/gs"
  if [[ -d "$candleGskitRunTimeDirectory/$1/gs.old" ]]; then
    rmCMD="rm -rf $1/gs.old"
    runCommandPerfectDirectory $candleGskitRunTimeDirectory $rmCMD
  fi
  if [[ -f "$ITMHOME/registry/${2}.old" ]]; then
    rmCMD="rm ${2}.old"
    runCommandPerfectDirectory $ITMHOME/registry $rmCMD
  fi
}

# --------------
# Restore gskit from backup directory after the backup worked
# $1 - architecture
# $2 - ver file
# --------------

RevertGSKit() {
  Log_report "Restore GSKit from $1/gs.old backup"
  case $MoveCopy in
    move)
      if [[ -d "$candleGskitRunTimeDirectory/$1/gs.old" ]]; then
        if [[ -d "$candleGskitRunTimeDirectory/$1/gs" ]]; then
          ## Something strange happened, report it.
          Log_report "$MoveCopy - Both $1/gs and $1/gs.old exist"
        else
          ## This is expected.
          mvCMD="mv $1/gs.old $1/gs"
          runCommandPerfectDirectory $candleGskitRunTimeDirectory $mvCMD || errorRevert $1 $2
        fi
      fi
    ;;
    copy)
      if [[ -d "$candleGskitRunTimeDirectory/$1/gs.old" &&
            -d "$candleGskitRunTimeDirectory/$1/gs" ]]; then
        ## This is expected.
        # important:
        # use find command wrapper because cp command will fail on locked files
        files=$(ls $candleGskitRunTimeDirectory/$1/gs.old)
        findCMD="find $files \( -type d -exec mkdir ../gs/{} 2> /dev/null \; \) -o \( -type f -exec cp -p {} ../gs/{} \; \)"
        runCommandPerfectDirectory $candleGskitRunTimeDirectory/$1/gs.old $findCMD || errorRevert $1 $2
        rmCMD="rm -rf $1/gs.old"
        runCommandPerfectDirectory $candleGskitRunTimeDirectory $rmCMD
      fi
    ;;
  esac
  if [[ -f "$ITMHOME/registry/${2}.old" ]]; then
    cpCMD="cp ${2}.old $2"
    runCommandPerfectDirectory $ITMHOME/registry $cpCMD || errorRevert $1 $2
    rmCMD="rm -f ${2}.old"
    runCommandPerfectDirectory $ITMHOME/registry $rmCMD
  fi
}

# --------------
# backup and keep bad GSKit into tar into ITMHOME/tmp directory
# --------------

keepBadGSKit() {
  typeset backFile=badGSKit_keep.tar
  Log_report "Backup failed GSKit installation into $CANDLETMP/$backFile"
  gsDirTest=$(ls -d $candleGskitRunTimeDirectory/*/gs 2> /dev/null | head -n 1)
  gsVerTest=$(ls -d $ITMHOME/registry/gs*.ver 2> /dev/null | head -n 1)

  if [[ -d "$gsDirTest" && -f "$gsVerTest" ]]; then
    backCMD="tar cf $CANDLETMP/$backFile registry/gs*.ver */gs"
    runCommandPerfectDirectory $candleGskitRunTimeDirectory $backCMD || errorKeepTar $backFile
  fi
}

# --------------
# Remove bad gskit backup tar from ITMHOME/tmp directory
# $1 - gskit archive name
# OR
# $1 - gskit backup tar name
# --------------

RMkeepBadGSKit() {
  typeset backFile=badGSKit_keep.tar
  Log_report "Remove failed GSKit backup file $CANDLETMP/$backFile"
  if [[ -f "$CANDLETMP/$backFile" ]]; then
    rmCMD="rm -f $CANDLETMP/$backFile"
    runCommandPerfect $rmCMD
  fi
}

# -------------------------
# run gskit check script
# $1 - check script to run
# result:
# 0 - ok
# 1 - check error, fail
# -------------------------

runCheckScript() {
  typeset CheckScriptCmd="${*}";                                        #IZ94248
  typeset CheckScript=$1
  typeset CheckScriptDir=$(dirname "${CheckScript}");                   #IZ96919
  typeset CheckVerProgram;                                              #IZ96919
                                                                        #IZ96919
  if [ ! -f $CheckScript ]; then
    noExpectedFileError $CheckScript
    return 4
  fi
  chmod 777 $CheckScript
  if [ ! -x $CheckScript ]; then
    noExecutableFileError $CheckScript
    return 4
  fi
                                                                        #IZ96919
  #---------------------------------------------------------------------#IZ96919
  # On HP-UX, the 64-bit loader ignores the library path environment    #IZ96919
  # variables when the real user of the process is different than the   #IZ96919
  # effective user of the process. Under that condition, the loader     #IZ96919
  # only searches the RPATH that is internally specified in the         #IZ96919
  # executable. HP-UX's dlopen() man page documents this restriction.   #IZ96919
  #                                                                     #IZ96919
  # Since GSKit can't hardcode an ITM install path in its executables,  #IZ96919
  # this means the GSKit verify step cannot succeed. To get around      #IZ96919
  # this, we change the owner of the GSKit verify program to be the     #IZ96919
  # real user and we add the set-ownerid-on-execution bit so that it    #IZ96919
  # runs as that user. This makes it so the real and effective user of  #IZ96919
  # the resulting process are equal which, in turn, makes it so the     #IZ96919
  # loader can use the library path environment variables to find the   #IZ96919
  # GSKit library files.                                                #IZ96919
  #---------------------------------------------------------------------#IZ96919
  typeset restoreVerProgram="N";            # assume no restore needed  #IZ96919
  if [[ "${thisOS}" = "HP-UX" ]]; then                                  #IZ96919
     if [[ "${thisRealUser}" != "${thisEffectiveUser}" ]]; then         #IZ96919
        if [[ "${CheckScript}" = *_64 ]]; then                          #IZ96919
           CheckVerProgram="${CheckScriptDir}/gsk8ver_64";              #IZ96919
           if [[ ! -x $CheckVerProgram ]] ; then
             CheckVerProgram="${CheckScriptDir}/gsk7ver_64";             
           fi 
           CMD="chown \"${thisRealUser}\" \"${CheckVerProgram}\"";      #IZ96919
           runCommandPerfect "${CMD}";                                  #IZ96919
           CMD="chmod u+s \"${CheckVerProgram}\"";                      #IZ96919
           runCommandPerfect "${CMD}";                                  #IZ96919
           restoreVerProgram="Y";           # need to undo chown/chmod  #IZ96919
        fi;                                 # (if(CheckScript=*_64))    #IZ96919
     fi;                                    # (if(thisReal==thisEff))   #IZ96919
  fi;                                       # (if(thisOS==HP-UX))       #IZ96919
                                                                        #IZ96919
  #---------------------------------------------------------------------#IZ96919
  # Run the verify and report if it fails.                              #IZ96919
  #---------------------------------------------------------------------#IZ96919
  runCommandPerfect ${CheckScriptCmd};                                  #IZ94248
  typeset res=$?
  if [ $res -ne 0 ]; then
    Failure_report "GSKit check failure, script: ${CheckScriptCmd}";    #IZ94248
  fi
                                                                        #IZ96919
  #---------------------------------------------------------------------#IZ96919
  # Now that the verify is complete (regardless of success), we need    #IZ96919
  # to restore the GSKit verify program to its original state.          #IZ96919
  #---------------------------------------------------------------------#IZ96919
  if [[ "${restoreVerProgram}" = "Y" ]]; then                           #IZ96919
     CMD="chown \"${thisEffectiveUser}\" \"${CheckVerProgram}\"";       #IZ96919
     runCommandPerfect "${CMD}";                                        #IZ96919
     CMD="chmod u-s \"${CheckVerProgram}\"";                            #IZ96919
     runCommandPerfect "${CMD}";                                        #IZ96919
  fi;                                       # (if(restoreVerProgram))   #IZ96919
                                                                        #IZ96919
  return $res
}

runVerifyInstallScript() {
   # change also permissions for libs
  typeset CMD="chmod 777 $candleGskitRunTimeDirectory/$1/gs/lib*/*"
  runCommandPerfect $CMD

  typeset InstallDir=${candleGskitRunTimeDirectory}/${1}/gs/bin;        #IZ94248
  InstallDir=$( resolveDir ${InstallDir} );                             #IZ94248
  typeset InstallScript=${InstallDir}/private_verifyinstall             #IZ94248
  if [ ! -f $InstallScript ]; then
    InstallScript=${InstallDir}/private_verifyinstall_32;               #IZ94248
    if [ ! -f $InstallScript ]; then
      Install_64=${InstallDir}/private_verifyinstall_64;                #IZ94248
      if [ ! -f $Install_64 ]; then
        Failure_report "Neither $InstallScript nor $Install_64 script found !"; #IZ94248
      fi
      InstallScript=$Install_64
    fi
  fi
                                                                        #IZ94248
  runCheckScript ${InstallScript} -x;       # run in verbose mode       #IZ94248
  typeset res=$?
  return $res
}
                                                                        #IZ94248
resolveDir() { #--------------------------------------------------------#IZ94248
   # Perform directory canonicalization. In other words, the input      #IZ94248
   # directory string is converted to a fully qualified name (no        #IZ94248
   # wildcards, "..", ".", and so on. For example, "/tmp/.." resolves   #IZ94248
   # to "/". The input directory string must refer to a directory that  #IZ94248
   # exists.                                                            #IZ94248
   #                                                                    #IZ94248
   # AIX and Linux provide the "readlink" command to perform this       #IZ94248
   # function. If readlink does not exist, we use "cd -P". This         #IZ94248
   # approach changes the directory *and* sets PWD to be the            #IZ94248
   # canonicalized directory name. So, we switch to the directory, grab #IZ94248
   # its name, and switch back to the original working directory.       #IZ94248
   #                                                                    #IZ94248.1
   # Solaris has some sort of buffering issue when the cd shell command #IZ94248.1
   # sets $PWD and when the pwd shell command returns the value. In     #IZ94248.1
   # in some cases, one or both of these leave extra garbage on the end #IZ94248.1
   # of the output value. And of course, this creates problems for us   #IZ94248.1
   # later when we try to use it. We get around the problem by using    #IZ94248.1
   # the /usr/bin/pwd binary, which always returns the correct value    #IZ94248.1
   # without corruption.                                                #IZ94248.1
   #--------------------------------------------------------------------#IZ94248
   typeset result;                          # localize output variable  #IZ94248
   result=$(readlink -f ${1} 2> /dev/null); # try readlink              #IZ94248
   if [[ ${?} -ne 0 ]]; then                # if no readlink            #IZ94248
      cd -P ${1} > /dev/null 2>&1;          # use cd to change/resolve  #IZ94248
      if [[ "${os}" = "sol" ]]; then         # $PWD/pwd shell broken... #IZ94248.1
         result=$(/usr/bin/pwd);            # ...use pwd binary         #IZ94248.1
      else                                  # $PWD is ok...             #IZ94248.1
         result="${PWD}";                   # ...use its value          #IZ94248.1
      fi;                                   # (elif($os="sol"))         #IZ94248.1
      cd ${OLDPWD} > /dev/null 2>&1;        # restore working directory #IZ94248
   fi;                                      # (if($?!=0))               #IZ94248
   print "${result}";                       # pass result to caller     #IZ94248
}                                           # (resolveDir())            #IZ94248

runCheckInstall() {
  if [ "$os" != "aix" ]; then
    CheckScript=$candleGskitRunTimeDirectory/$1/gs/private_checkinstall
    runCheckScript $CheckScript
  else
    lslpp -L | grep xlC.rte | grep -qE "[' '][6-9][.]|[' '][0-9][0-9][.]"
    rc=$?
    if [ $rc != "0" ]; then
      Failure_report "GSKit check failure, script: $CheckScript"
    fi
  fi
}

runPostInstall() {
  CheckScript=$candleGskitRunTimeDirectory/$1/gs/private_postinstall
  runCheckScript $CheckScript
}

setGSKDirectory() {
  #createDir $candleGskitRunTimeDirectory
  gskitDirs=$candleGskitRunTimeDirectory/$gskArch/gs
}

# check status of GSKit
# warning: checks only "main" GSKIT 64 (for 64 bit box)
# return:
#  2 - no local GSKit
#  1 - new version
#  0 - old version, upgrade needed

checkVersion() {
  # what GSKit
  typeset tarName=$gskTar
  typeset verFile=$gskVer
  # important:
  # check version using only 64 bit GSKit on 64 bit box or
  # 32 bit gSKit on 32 bit box
  # do not check: 32 bit GSKit on 64 bit box
  checkGSKitVersionStatus $candleGSKitProdDir/$tarName $verFile
  typeset res=$?
  case "$res" in
    0) Log_report "No GSKit upgrade is needed"
       return 0
    ;;
    1) Log_report "Older version already installed, upgrade needed"
       return 1
    ;;
    2) Log_report "No local GSKit found, installation needed"
       return 2
    ;;
    *) return 9999
    ;;
  esac
}

# --------------------------
# create gsKit.config file
# --------------------------

setGskitLocation() {
  variable="GskitInstallDir"
  [[ "$GSBIT" = "64" ]] && variable="GskitInstallDir_64"
  if [[ -f $candleGskitgskInfoFile ]];
  then
    content=$(grep -v "${variable}=" $candleGskitgskInfoFile)
    removeFile $candleGskitgskInfoFile
  fi
  Log_report "Setting variable $variable=$gskitDirs in $candleGskitgskInfoFile"
  echo "$variable=$gskitDirs" > $candleGskitgskInfoFile
  [[ -n "$content" ]] && echo $content >> $candleGskitgskInfoFile
}


# -------------------------
# untar and check GSKit
# $1 - gskTar
# $2 - gskArch
# $3 - gskVer
# result
#  if failure exit installation
# ------------------------

setInstallGSKit() {
  typeset lTar=$1
  typeset lArch=$2
  typeset lVer=$3
  untarGSKit $lTar $lArch $lVer
  runCheckInstall $lArch || gskErrorCheckInstall $lArch
  runPostInstall $lArch || gskErrorPostInstall $lArch
  runVerifyInstallScript $lArch || gskErrorVerifyInstall $lArch
}

# ------------------------------
# AIX "locks" shared libraries, so we get an "in use" failure.
# Other platforms do not "lock" shared libraries, so there is no "in use" failure.
# For AIX, move files if possible to get around locked files.
#   Copy files if a TEMS, TEPS or TEP is running in deploy mode.
# ------------------------------
TemsTepsTepRunning() {
  case $os in
    aix)
      if [[ -n "$(genkld | grep '.*/libgsk.*')" ]]; then
        MoveCopy=move
      else
        MoveCopy=copy
      fi
      ;;
    *)
      MoveCopy=copy
      ;;
  esac
}

# ------------------------------
# check version and run install
# ------------------------------

checkandinstallGSK() {
  checkVersion
  typeset versionres=$?
  if [ $versionres -eq 0 ]; then
    return
  fi

# Set the move - copy flag for backup processing
  TemsTepsTepRunning

  backupGSKit $gskArch $gskVer

  Log_report "Removing old gskit directory $ITMHOME/$gskArch/gs"
  rm -rf $ITMHOME/$gskArch/gs >/dev/null 2>&1
  setInstallGSKit $gskTar $gskArch $gskVer

  CLRbackupGSKit $gskArch $gskVer

  # remove old $ITMHOME/gskit
  if [ -d "$oldCandleGskitDirectory" ];
  then
    removeDirectory $oldCandleGskitDirectory
    print "\n`nls_replace 'KCI1248I'`\n"
  fi

  # important:
  # set gsKit.config file only if installing or upgrading
  # left if nothing to do with GSKit
  # important: set gs.config file before changeAllEnvFiles
  setGskitLocation
}

# ------------------------------
# set OS library variable
# ------------------------------
setLibraryPath() {
  newLibPath=$gskitDirs/lib64:$gskitDirs/lib
  case $os in
    aix)
      oldLibPath=$LIBPATH
      export LIBPATH=$newLibPath
      ;;
    hp)
      oldLibPath=$SHLIB_PATH
      export SHLIB_PATH=$newLibPath
      ;;
    hpi)
      oldLibPath=$SHLIB_PATH
      export SHLIB_PATH=$newLibPath
      ;;
     *)
      oldLibPath=$LD_LIBRARY_PATH
      export LD_LIBRARY_PATH=$newLibPath
      ;;
  esac
}

# ------------------------------
# restore OS library variable
# ------------------------------
unsetLibraryPath() {
  case $os in
    ai)
      export LIBPATH=$oldLibPath
      ;;
    hp)
      export SHLIB_PATH=$oldLibPath
      ;;
    hpi)
      export SHLIB_PATH=$oldLibPath
      ;;      
     *)
      export LD_LIBRARY_PATH=$oldLibPath
      ;;
  esac
}

setCertKey() {

# Build the keyfile directory
  keyDir=$ITMHOME/keyfiles
  createDir $keyDir

# Define keyfile and certificate parameters
  CertificateCommonName="IBM Tivoli Monitoring Self-Signed Certificate"
  CertificateOrgUnit="IBM Tivoli"
  CertificateOrg="IBM"
  CertificateCountry="US"
  CertificateExpirationTime=3650
  DatabaseName="$keyDir/keyfile.kdb"
  DatabasePassword="IBM61TIV"
  PasswordExpiration=0
  KeyLabel="IBM_Tivoli_Monitoring_Certificate"
  DistinguishedName="CN=$CertificateCommonName,O=$CertificateOrg,C=$CertificateCountry"

  useGskitDirs=$gskitDirs

  if [[ "$GSBIT" = "64" ]];
  then
    gsk8cmd=gsk8capicmd_64
    gsk7cmd=gsk7capicmd_64
  else
    gsk8cmd=gsk8capicmd
    gsk7cmd=gsk7capicmd
    if [[ "$os" = "hpi" ]]; 
    then
      gsk8cmd=gsk8capicmd_32
      gsk7cmd=gsk7capicmd_32
    fi
    useGskitDirs=$gskitDirs
  fi

  runProg=$useGskitDirs/bin/$gsk8cmd
  if [ ! -x $runProg ]; then
    gsk8cmd=$gsk7cmd
    runProg=$useGskitDirs/bin/$gsk8cmd
  fi

  if [ ! -x $runProg ]; then
    noExecutableFileError $runProg
  fi

# Set os library variable
  setLibraryPath

# Skip the keyfile and certificate creation if we already have a keyfile, otherwise rewriting the
# encryption key means all previously encrypted data will be undecipherable
# 6.3fp2 - perform additional check if self-signed certificate should be regenerated
  if [ -f "$keyDir/keyfile.kdb" ]
  then
    weekKeySize=$($ITMHOME/bin/ITMLangC $useGskitDirs/bin/$gsk8cmd -cert -details -db $DatabaseName -stashed -label $KeyLabel 2>/dev/null | grep "Key Size" | grep 1024 )
    isIBMSelfSigned=$($ITMHOME/bin/ITMLangC $useGskitDirs/bin/$gsk8cmd -cert -details -db $DatabaseName -stashed -label $KeyLabel 2>/dev/null | grep "Subject" | grep "$CertificateCommonName" )
    if [[ -n "$weekKeySize" && -n "$isIBMSelfSigned" ]];
    then
      Log_report "Regenerating strong self-signed certificate."
      Log_report "Deleting existing IBM self-signed certificate"
      $useGskitDirs/bin/$gsk8cmd -cert -delete -db $DatabaseName -stashed -label $KeyLabel >> $logFile 2>&1
      Log_report "return code from command is $?"
      Log_report "Creating cerficate"
      $useGskitDirs/bin/$gsk8cmd -cert -create -db $DatabaseName -stashed -size 2048 -dn "$DistinguishedName" -label $KeyLabel -default_cert -expire $CertificateExpirationTime -ca false -sigalg SHA256WithRSA -fips >> $logFile 2>&1
      Log_report "return code from command is $?" 
      return 0
    else
      #$NLS-E$ "Keyfile.kdb already exists, skipping keyfile and certificate creation"
      Err_report info "`nls_replace 'KCI0747I' Keyfile.kdb`"
      Log_report "Custom Keyfile.kdb already exists or certificate is already strong enough; skipping keyfile and certificate creation"
      return 0
    fi
  fi

# Create the keyfile and certificate
  #$NLS-E$ "creating security files"
  Err_report info "`nls_replace 'KCI0748I'`"
  Log_report "...creating GSkit security keyfile and certificate"
  Log_report "...GSkit version info"
  $useGskitDirs/bin/$gsk8cmd -version >> $logFile 2>&1
  Log_report "return code from command is $?"

# Create the keyfile
  #$NLS-E$ "create keyfile"
  Err_report info "`nls_replace 'KCI0749I'`"
  Log_report "...create GSkit security keyfile"
  $useGskitDirs/bin/$gsk8cmd -keydb -create -db $DatabaseName -pw $DatabasePassword -type cms -stash >> $logFile 2>&1
  Log_report "return code from command is $?"

# Create the certificate (self-signed)
  #$NLS-E$ "create certificate"
  Err_report info "`nls_replace 'KCI0750I'`"
  Log_report "...create GSkit security certificate"
  $useGskitDirs/bin/$gsk8cmd -cert -create -db $DatabaseName -stashed -size 2048 -dn "$DistinguishedName" -label $KeyLabel -default_cert -expire $CertificateExpirationTime -ca false -sigalg SHA256WithRSA -fips >> $logFile 2>&1
  cc=$?
  Log_report "return code from command is $cc"

# Unset os library variable
  unsetLibraryPath

  #defect 86791
  chmod 777 $keyDir/keyfile.* >> $logFile 2>&1

  rm -f $propFile

  return $cc
}

writedebugGlobalGSkit() {
  print $globalpkgName_64
  print $globalgskitVersion_64
  print $globalgskitDirs_64
  print $globalpkgName
  print $globalgskitVersion
  print $globalgskitDirs
}

setSomeCandleEnvironment() {
  oldCandleGskitDirectory=$ITMHOME/gskit
  candleGskitRunTimeDirectory=$ITMHOME
  candleGskitgskInfoFile=$ITMHOME/config/gsKit.config
  CANDLETMP=$ITMHOME/tmp
  candleGSKitProdDir=$PROD_PATH/GSKit
}

checkGSKtarnames() 
{
  [[ -z "$gskArch" || -z "$gskTar" || -z "$gskVer" ]] && gskSupportError
}

createDir() {
  [ -d $1 ] || mkdir $1
}

# check for upgrade
# $1 - path to new tar file
# $2 - version file name
# return :
#   0 - no upgrade, new version lower or equal to version installed
#   1 - upgrade: new version greater than version installed
#   2 - global: no local gskit installation at all
checkGSKitVersionStatus() {
  typeset ExistingVersion=$ITMHOME/registry/$2
  if [ ! -f $ExistingVersion ]; then
    Log_report "No $ExistingVersion file, upgrade from global to local installation needed"
    return 2
  fi
  Log_report "Untar GSKit version file from $1 file"
  removecreateDir $CANDLETMP/tmp
  getVerFromTar $1 $2 $CANDLETMP
  [ -f "$tmpVerFile" ] || packageError $tmpVerFile
  Log_report "Compare $ExistingVersion with $tmpVerFile"
  unset levelStrg3
  checkLevel3 $ExistingVersion $tmpVerFile
  case $levelStrg3 in
    Same)
      Log_report "The same versions found"
      return 0;;
    Newer)
      Log_report "New version of GSKit found"
      return 1;;
    Older)
      Log_report "Older version of GSKit found in tar provided"
      return 0;;
    *)
      Failure_report "$levelStrg3 - INTERNAL ERROR, no expected result from checkLevel2 proc"
      return 9999;;
  esac
}

removecreateDir() {
  [ -d $1 ] && rm -rf $1
  mkdir $1
}

removeDirectory() {
  [ -d $1 ] && rm -rf $1
}

removeFile() {
  [ -f $1 ] && rm $1
}

Log_report() {
  print "$(date +%m.%d.%Y\ %T) $PROGNAME: $1" >> $logFile
  Err_report info "$1"
}

noExpectedFileError() {
  Failure_report "CONFIGURATION ERROR: no file $1"
}

runCommandPerfectDirectory() {
  typeset DIR=$1
  shift
  typeset cmd=$*
  typeset ADIR=$PWD
  cd $DIR
  runCommandPerfect $cmd
  typeset cc=$?
  cd $ADIR
  return $cc
}

# Extracts ver file from tar file
# $1 - path to tar file
# $2 - ver file name to extract
# $3 - temporary directory for use
# example getVerFromTar {...}/GSKit/gsli6243.tar gsli6243.ver /tmp
# output:
# success: tmpVerFile=constains ver file
# failure: tmpVerFile empty

getVerFromTar()
{
  typeset ltarPath=$1
  typeset lverName=$2
  typeset ltempdir=$3

  unset tmpVerFile
  CMD="unloadTmpFile $lverName $ltarPath"
  runCommandPerfectDirectory $ltempdir $CMD
  if [ $? -ne 0 ] ; then
    packageVerError tmp/$lverName $ltarPath
    return
  fi
  tmpVerFile=$ltempdir/tmp/$lverName
}

runCommandPerfect() {
  typeset cmd=$*
#  Log_report "cmd: $cmd"
  typeset cc
  Log_report "----- Running command: $cmd -----"
  eval $cmd >> $logFile 2>&1
  cc=$?
  Log_report "return code from command is $cc"
  Log_report "----- End of running command -----"
  if [ $cc -gt 0 ]; then
    Failure_report "Return error code: $cc"
  fi
  return $cc
}

gskSupportError() {
  # Allow missing GSkit packages in Deploy Mode. Deploy can determine that they do not have to be sent.         # 208983
  if [[ "$DPMODE" = "true" ]]; then                                                                             # 208983
    Info_report_exit "GSK INFO: configuration, package not found : os = $SYSARCH bitMode = $gsdigit" 0;         # 208983
  else                                                                                                          # 208983
    Failure_report_exit "GSK ERROR: configuration, platform not supported : os = $SYSARCH bitMode= $gsdigit" 2; # 208983
  fi;                                                                                                           # 208983
}

packageVerError() {
  Failure_report "GSK package error: no $1 file in $2 package file"
}

packageError() {
  Failure_report "PACKAGE ERROR: no file $1"
}

copyingError() {
  Failure_report "COPYING ERROR: cannot copy from $1 to $2"
}

noExecutableFileError() {
  Failure_report "CONFIGURATION ERROR: file $1 not exists or is not executable"
}

checkLevel3()
{
# $1 = the name of the current .ver file.
# $2 (if specified) = the name of the new .ver file.
  #set -x
  typeset -i curRel=0 curVer=0 newRel=0 newVer=0 useVRMF=0

  # Get and compare the versions.

  curFile=$1
  # Ensure the file exist.
  #$NLS-E$ "could not read \"$curFile\""
  [ -r "$curFile" ] || Failure_report "`nls_replace 'KCI1008E' \\"$curFile\\"`"
  # f32162 - need to use VRMF if present
  curVer=$(grep "^VRMF *=" $curFile | cut -d"=" -f2)
  [ $curVer -ne 0 ] && useVRMF=1
  [ $curVer -ne 0 ] || curVer=$(grep "^ver *=" $curFile | cut -d"=" -f2)
  #$NLS-E$ "could not determine the version number from \"$curFile\""
  [ $curVer -ne 0 ] || Failure_report "`nls_replace 'KCI1009E' \\"$curFile\\"`"

  # Return Same and leave if the name of the new ver file was not provided
  if [ -z "$2" ]; then
    levelStrg3=Same
    return
  fi

  # Continue comparing if the name of the new ver file was provided
  newFile=$2
  # Ensure the file exist.
  #$NLS-E$ "could not read \"$newFile\""
  [ -r "$newFile" ] || Failure_report "`nls_replace 'KCI1008E' \\"$newFile\\"`"
  newVer=$(grep "^VRMF *=" $newFile | cut -d"=" -f2)
  [ $newVer -ne 0 ] && useVRMF=1
  [ $newVer -ne 0 ] || newVer=$(grep "^ver *=" $newFile | cut -d"=" -f2)
  #$NLS-E$ "could not determine the version number from \"$newFile\""
  [ $newVer -ne 0 ] || Failure_report "`nls_replace 'KCI1009E' \\"$newFile\\"`"

  # f32162 - need to normalize versions to match VRMF format
  curVer=$(print $curVer | $AWK '{ nval=$1; pad=10^(8-length(nval)); printf("0%d", nval*pad)}')
  newVer=$(print $newVer | $AWK '{ nval=$1; pad=10^(8-length(nval)); printf("0%d", nval*pad)}')

  if [ newVer -lt curVer ]; then
    levelStrg3=Older
    return
  fi

  if [ newVer -gt curVer ]; then
    levelStrg3=Newer
    return
  fi

  # If we are not using VRMF, and the versions are identical, compare the releases (pre ITM 6.1 FP5 comparison).

  if [ $useVRMF -ne 1 ]; then
    curRel=$(grep "^rel *=" $curFile | cut -d"=" -f2)
    newRel=$(grep "^rel *=" $newFile | cut -d"=" -f2)
    #$NLS-E$ "could not determine the release number from \"$curFile\""
    [ -n "$curRel" ] || Failure_report "`nls_replace 'KCI1010E' \\"$curFile\\"`"
    #$NLS-E$ "could not determine the release number from \"$newFile\""
    [ -n "$newRel" ] || Failure_report "`nls_replace 'KCI1010E' \\"$newFile\\"`"

    # Same version, older release.

    if [ newRel -lt curRel ]; then
      levelStrg3=Older
      return
    fi

    # Same version, same release.

    if [ newRel -eq curRel ]; then
      levelStrg3=Same
      return
    fi

    # New version
    levelStrg3=Newer
    return
  fi

  # Same VRMF version.

  levelStrg3=Same
  return
}

gskitUnTarPkg()
{
# $1 = full pathname of the file to untar.

  typeset -i i=0 maxTry=2
  typeset ext rc

  [ -r "$1" ] || Failure_report "cannot read package \"$1\""

  ext=${1##*.}  # ext (extension) will be either "ctp", "tar", tdd or "zdd" where d = a digit.
  while [ i -lt maxTry ]
  do
    if [[ "$ext" = t* ]]; then     # Uncompressed tar file.
      tar -$tarOpt $1 $2> /dev/null 2>&1
      rc=$?
      # do chown and chmod for Linux.  defect 83100
      if [[ "$chownRoot" = y ]]; then
        for tvf in $(tar tvf $1 2> /dev/null | $AWK '{print $6}')
        do
          if [ -e $tvf ]; then
            chown root:root $tvf
            chmod 755 $tvf
          fi
        done
      fi
      [ -r $1 ] || rc=-1
    else  # Compressed tar file.
      zcat < $1 | tar -$tarOpt - > /dev/null 2>&1
      rc=$?
      # do chown and chmod for Linux.  defect 83100
      if [[ "$chownRoot" = y ]]; then
        for tvf in $(zcat < $1 | tar tvf - 2> /dev/null | $AWK '{print $6}')
        do
          if [ -e $tvf ]; then
            chown root:root $tvf
            chmod 755 $tvf
          fi
        done
      fi
    fi

    [ "$rc" = "0" ] && return

    i=i+1
  done

  # If we get to here, the untar may have failed.
  Err_report warn "unarchive of \"$1\" may have failed"
}

# Unload a single file from an archive's tmp/ directory.

unloadTmpFile()
{ # $1 = name of file to unload.
  # $2 = full pathname of archive.
  #set -x

  typeset ext tFile

  #$NLS-E$ "cannot read package \"$2\""
  [ -r "$2" ] || Err_report fail "`nls_replace 'KCI1020E' \\"$2\\"`"

  ext=${2##*.}  # Get the extension, either "tar", "ctp" or "zxx".

  if [[ "$ext" = t* ]]; then     # Uncompressed tar file.
    tar -$tarOpt $2 tmp/$1 > /dev/null 2>&1
    # do chown and chmod for Linux.  defect 83100
    if [[ "$chownRoot" = y ]]; then
      for tvf in $(tar tvf $2 tmp/$1 2> /dev/null | $AWK '{print $6}')
      do
        chown root:root $tvf
        chmod 755 $tvf
      done
    fi
        #$NLS-E$ "could not unload file \"$1\""
    [ -r tmp/$1 ] || Err_report fail "`nls_replace 'KCI1021E' \\"$1\\"`"
  else  # Compressed tar file.
    zcat < $2 | tar -$tarOpt - tmp/$1 > /dev/null 2>&1
        #$NLS-E$ "could not unload file \"$1\""
    [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1021E' \\"$1\\"`"
    # do chown and chmod for Linux.  defect 83100
    if [[ "$chownRoot" = y ]]; then
      for tvf in $(zcat < $2 | tar tvf - tmp/$1 2> /dev/null | $AWK '{print $6}')
      do
        chown root:root $tvf
        chmod 755 $tvf
      done
    fi
  fi
}

#perform os dependend tasks
systemTasks()
{
  # run slibclean as root user on aix
  [[ "$os" = aix ]] && [[ "$thisUserId" = "0" ]] && runCommandPerfect "slibclean" 
}

# find proper GSKit package

find_gskit_package() {
  #set -x
  gskPgks=$(ls $candleGSKitProdDir/gs$os*$gsdigit.ctp 2>/dev/null)
  if [[ -z "$gskPgks" ]]; then                                         # 208983
    gskArch="";                                                        # 208983
    gskTar="";                                                         # 208983
    gskVer="";                                                         # 208983
  else                                                                 # 208983
    for p in $gskPgks; do
      parch=$(basename $p .ctp | cut -c3-)
      parchprefix=${parch%%+([0-9])}
      # make sure os and package architecture prefix is the same
      [[ "$parchprefix" = "$os" ]] || continue
      # for solaris, make sure first digit of system architecture is the same
      if [[ "$os" = "sol" ]];
      then
        osprefix=$(echo $SYSARCH | cut -c1-4)
        parchprefix=$(echo $parch | cut -c1-4)
        [[ "$osprefix" = "sol6" && ! "$parchprefix" = "sol6" ]] && continue
        [[ ! "$osprefix" = "sol6" && "$parchprefix" = "sol6" ]] && continue
      fi
      gskArch=$parch
      gskTar=$(basename $p)
      gskVer="gs$parch.ver"
    done
  fi;                                                                  # 208983
}

###################################################################
#                          Main                                   #
###################################################################
bindir=`pwd`/$(dirname $0)
[ "$(print $0 | grep '\./')" = "" ] && [ "$(print $0 | grep '^/.*')" = "" ] && [ `pwd` = '/' ] && bindir=`pwd`$(dirname $0)
[ "$(print $0 | grep '^/.*')" != "" ] && bindir=$(dirname $0)

PROGNAME=$(basename $0)
CMDPARAMS="$0 $*"

export TEXTDOMAINDIR=$bindir/../config/nls/msg
export PATH=$bindir:$PATH:/usr/sbin

thisOS=$(uname -a | cut -d" " -f1)  # thisOS -> AIX
thisUserId=$(id 2> /dev/null | cut -d"=" -f2 2> /dev/null | cut -d"(" -f1 2> /dev/null)

# tar file option for unix and root use oxf
# set flag to do chown and chmod for Linux
tarOpt=xhf                                                              #IV67523
chownRoot=n
if [[ "$thisUserId" = "0" ]]; then
  if [[ "$thisOS" != "Linux" ]]; then
    tarOpt=oxhf                                                         #IV67523
  else
    chownRoot=y
  fi
fi

#-----------------------------------------------------------------------#IZ96919
# Use the id command to get our real and effective user names.          #IZ96919
#-----------------------------------------------------------------------#IZ96919
if [[ "${thisOS}" = "SunOS" ]];                                         #IZ96919
then                                                                    #IZ96919
   id_cmd="/usr/xpg4/bin/id";                                           #IZ96919
else                                                                    #IZ96919
   id_cmd="id";                                                         #IZ96919
fi;                                                                     #IZ96919

thisRealUser=$(${id_cmd} -unr);             # get real user name        #IZ96919
thisEffectiveUser=$(${id_cmd} -un);         # get effective user name   #IZ96919
                                                                                   
# need to use nawk if it's available
AWK=awk
type nawk >/dev/null 2>&1
[[ $? -eq 0 ]] && AWK=nawk

# save current directory
CDIR=$(pwd)

# itm installation directory
ITMHOME=""

# directory with product packages
PROD_PATH=""

# system architecture
SYSARCH=""

# gskit bit version to install (32 or 64), if not set, system architecture bit mode will be used
GSBIT=""

# Deploy Mode boolean, if not set, it is false                         # 208983
DPMODE="false";                                                        # 208983
                                                                       # 208983
while getopts ":a:d:g:h:p:" opt;                                       # 208983
do
  case $opt in
    a) SYSARCH=$OPTARG
       ;;
    d) DPMODE=$OPTARG;                                                 # 208983
       ;;                                                              # 208983
    g) GSBIT=$OPTARG
       ;;
    h) ITMHOME=$OPTARG
       ITMHOME=$(eval print $ITMHOME)
       ;;
    p) PROD_PATH=$OPTARG
       PROD_PATH=$(eval print $PROD_PATH)
       ;;
    ?) Use_report
       ;;
    :) Use_report
       ;;
  esac
done

shift OPTIND-1
[ $# -eq 0 ] || Use_report

# validate itm installation directory
[[ ! -f "$ITMHOME/registry/cienv.ver" ]] && echo "Invalid ITM installation directory" && exit 1

logFile=$ITMHOME/logs/candle_installation.log

Log_report ""
Log_report "Starting gskit installation command: $CMDPARAMS"

[[ -z "$SYSARCH" ]] && Failure_report_exit "System architecture is required" 1

[[ -d "$PROD_PATH" ]] || Failure_report_exit "Invalid packages directory" 1

# get architecture prefix eg. lx aix sol
os=${SYSARCH%%+([0-9])}

# check if valid gskit bit mode is specified
if [[ -n "$GSBIT" ]];
then 
  [[ "$GSBIT" != @(32|64) ]] && Failure_report_exit "Invalid gskit bit mode value" 1
else
  # no gskit bit mode to install specified, set default one base on system arch
  GSBIT=32
  [[ "$SYSARCH" = @(*6) ]] && GSBIT=64
  Log_report "GSKit installation mode not specified, setting default base on system architecture to ${GSBIT}bit"
fi

# to simplify processing, set variable with only first digit
gsdigit=$(echo $GSBIT | cut -c1)

# 32bit gskit on 64bit xLinux has li prefix
[[ $os = "lx" ]] && [[ $GSBIT = 32 ]] && os=li 

setSomeCandleEnvironment

find_gskit_package

checkGSKtarnames

setGSKDirectory

systemTasks

checkandinstallGSK

setCertKey
