#!/bin/ksh
######################################################################
#                                                                    #
#          (C) COPYRIGHT Candle Corporation 1996 and 2002            #
#                      All Rights Reserved                           #
#                                                                    #
#        Licensed Materials - Property of Candle Corporation         #
#         Duplication or disclosure prohibited without the           #
#        expressed written consent of the Candle Corporation.        #
#                                                                    #
#                                                                    #
######################################################################

# WARNING - kddfind_dnd.sh only works when invoked from Distributor (a.k.a. dnd).

# Meaningful comments:
# @(#) $Header:   Q:/DD/MISC/kddfind.sh   1.26.1.9   04 Oct 2001 10:36:26   SMYUN  $
#
# Env Vars from CandleDBconfig:
#
# CANDLECFG = Config dir to write config files.
# CANDLETBL = Tables directory
# HOST      = Name of this host
# Debug     = -d if so requested on CandleDBconfig command line

# This is a set of functions invoked by CandleDBconfig.
# Only these functions have external visibility to CandleDBconfig:
#    Version       Return kddfind.sh version for logging
#    Verify        Read/write each config file to make all current
#    Product_menu  Configure products.
#    Batch         Pass parms for non-interactive execution
#    Read          Get user's answer to prompt, but don't log passwords

# user-modifable aliases to cover environmental differences
# alias grep=grep
# alias egrep=egrep
# alias awk=awk

oratab=/etc/oratab              # change here if needed

case $(uname) in (AIX) AIX=1;; (HP-UX) HPUX=1;; (SunOS) Solaris=1;; esac

Null=/dev/null
NL=${IFS#??}     # New-line char (third and last char of $IFS)

#alias -x rm="rm 2>&-"  # Removed for dnd.
alias  more="more -ed 2>&3 >$(tty)"  # allow more to work despite pipe

######################################################################
# FUNCTIONS
######################################################################

# Select_product() has been changed for use with dnd.

Select_product()  # OK for dnd. 
{
  #print "CandleDBConfig for $HOST $CANDLEARCH" >>$Logfile

  # Determine what products are installed.

  # CIDEV:  Next line assumes the current directory structure and
  #         product naming conventions.

  # Explicitly piping output to $Logfile only hides it from the user.
  # Everything the user sees is also written to $Logfile.

  #Version >>$Logfile  # TODO - add once under software control.
  [[ $Verify = no ]] || Verify     # read & re-write config files

  # Did caller pass an explicit parm for database type?
  # Product_menu $*  # Not used for dnd.
  # Batch "$@"   # Disabled by GB.

  Do_product_dnd $1

  Display_dnd  # Generates a list of servers to display.
}

Read()  # Not used in dnd. 
{
   # Read prompt
   # Also print results to logfile, but hide passwords
   typeset ans
   read ans?"$1"
   print $ans
   [[ $1 = *password* ]] && ans=xxxx      # except passwords
   print $ans>>$Logfile
}

# Minor changes for dnd.  Not sure yet about the source code control system.  TODO - modify
# once source control is known.

Version()  # OK for dnd. 
{
  # Write CTVC Header info to stdout
  #print '$Header:   Q:/DD/MISC/kddfind.sh   1.26.1.9   04 Oct 2001 10:36:26   SMYUN  $'
#$NLS-E$ "Version = ?"
nls_replace 'KCI0006I'
}


# Read/rewrite each config file that is older than the kddfind.sh
# script to bring them up to current format.
# Verify was changed to work with dnd.

Verify()  # OK for dnd.
{
  cd $CANDLECFG
  typeset -lL3 Prd     # Product calling requirement
  # Note that I'm using kddfind.sh for the comparison instead of kddfind_dnd.sh.  This is
  # okay since the two should always be in sync.
  #$NLS-Q$
  typeset kddfind=$(whence $ddpath/kddfind.sh) bad new host

  # 1st, rename any existing config files to Leonardo naming convention

    for file in *.@(ora|syb|ko?)
    do   [[ -r $file ]] || continue
         case ${file#*.} in (ora) pr=or;; (syb) pr=oy;; (kor) pr=ox;; esac
         eval $(grep db_hostname= $file)
         set -x
           mv $file $db_hostname\_$pr\_${file%.*}.cfg
         set +x
    done

  # Handle file only if older than this kddfind script
  for file in *_o[ryx]_*.cfg
  do  # Handle file only if older than this kddfind script
      [[ ! -r $file || $file -nt $kddfind ]] && continue

      if   (. ./$file)    # Verify integrity by executing in a sub-shell
      then [[ $file -nt $kddfind ]] && continue
           #$NLS-E$ Verifing integrity:  $file
		   nls_replace 'KCI0007I' $file
           set $(IFS=_; print $file)
#$NLS-Q$
           Product_setup $2
           Force=yes Update $file
      else bad=$bad\ $file
           #$NLS-E$ "\ Warning, $file is not a valid ksh script and CANNOT be used as a OMA configuration file."
		   nls_replace 'KCI0008I' $file
      fi
  done

  # The following if was changed for dnd.

  if [ $bad ]
  then 
#   print "
#Unless you interrupt NOW, these files will be renamed to *.bad and ignored,
#and CandleDBconfig will re-discover these databases and build new config
#files for them.  Press Enter to continue, or interrupt and repair them."
#       read
    #$NLS-E$ "These files will be renamed to *.bad and ignored. CandleDBConfig will rediscover the databases and build new config files for them."
	nls_replace "KCI0009I" 
    for file in $bad
    do  mv $file ${file%.*}.bad
    done
  fi
}

# Product_menu() is not needed for dnd.  The permissions check on kddos has been moved to
# main, and Do_product() has been replaced with Do_product_dnd() which is now called from
# Select_product().

Product_menu()  # Not used for dnd. 
{  
  clear
  #$NLS-E$ MenuHeader Welcome to CandleDBconfig
  MenuHeader `nls_replace 'KCI0010I'`
  tmpname1=`print $LANG | sed 's/^\(.....\).*/\1/'`
  cat < $TEXTDOMAINDIR/$tmpname1/kddfind_dnd_text1_$tmpname1.txt
$ddpath/kddos auth || cat < $TEXTDOMAINDIR/$tmpname1/kddfind_dnd_text2_$tmpname1.txt
  typeset ans
#$NLS-E$ 'Press enter to continue, or type "exit" to exit  '
read ans?"`nls_replace 'KCI0017I' \\"exit\\"`"
  [[ $ans = exit ]] && exit

  while true
  do
#$NLS-E$ MenuHeader Select Search limits
  MenuHeader `nls_replace 'KCI0018I'`
       cat < $TEXTDOMAINDIR/$tmpname1/kddfind_dnd_text3_$tmpname1.txt
  MenuFooter Function menu
     if   ans=$(Select 1 2 0 "Help Scan")
     then case $ans in
              ( [012] ) Scan=$ans break
          ;;  (*) $ans
          esac
     fi
  done

# Note:  ans = 1 for "Confine automatic search to running databases"
#        ans = 2 for "Also search for non-running databases  (slower)"
#        ans = 0 for "No automatic search.  Manual update only."

   # if >1 product, show a product-selection menu.  Args are product abbreviations.
   # 99R1 change to CandleDBConfig forces $# always 1

  if (( $# > 1 ))
  then
     while true
#$NLS-E$ do MenuHeader Select Product
       do MenuHeader `nls_replace 'KCI0025I'`
	   typeset -i i=0
       for PR
#$NLS-E$ do  ProductDef="Omegamon Monitoring Agent for `Product_name $PR`"
       do  ProductDef="`nls_replace 'KCI0026I'` `Product_name $PR`"
           print "               $((i+=1))) $ProductDef..."
       done
           #$NLS-E$ print "               $((i+=1))) Verify entire installation"
           print "               $((i+=1))) `nls_replace 'KCI0027I'`"
		   #$NLS-E$ print "               $((i+=1))) List manual configuration steps..."
           print "               $((i+=1))) `nls_replace 'KCI0028I'`"
		   #$NLS-E$ print "               $((i+=1))) Help for CandleDBconfig..."
           print "               $((i+=1))) `nls_replace 'KCI0029I'`"
	   #$NLS-E$ MenuFooter Main Menu
	   MenuFooter `nls_replace 'KCI0030I'`
       #$NLS-Q$
	   if ans=`Select $* IVP "Help system" "Help intro"`
          then case $ans in
               (return | IVP ) $ans
            ;; (Help*)  # don't know which product, take defaults
                    Prod=any Prd=any Product=database Get_login=yes \
                    #$NLS-E$ "All database servers found"
					DoneMsg="`nls_replace 'KCI0031I'`" $ans
            ;; (*)       Do_product $ans
            ;; esac
       fi
     done
  else Do_product $1
  fi
}

# Replaced with IVP_dnd.

IVP()  # Not used in dnd. 
{
  # Propogate interrupts to IVP program
    trap true 2         # Ignore Ctl-C here
    clear
    $ddpath/kddivp.sh $*        # Note runs in different process
    trap 2              # Restore normal Ctl-C
}

IVP_dnd()  # OK for dnd. 
{
  typeset tfile=$CANDLETMP/IVP_dnd.txt
  #print "... args = $*"
  rm -f $tfile > /dev/null 2>&1
  #$NLS-E$ "could not remove temp file \"$tfile\""
  [ -f "$tfile" ] && Err_report fail "`nls_replace 'KCI0032E' \\"$tfile\\"`"
  touch $tfile > /dev/null 2>&1
  #$NLS-E$ "could not create temp file \"$tfile\""
  [ -f "$tfile" ] || Err_report fail "`nls_replace 'KCI0033E' \\"$tfile\\"`"

  { $ddpath/kddivp.sh $*; } > $tfile 
}

# Select description, return only after the "Monitor - "
# Changed for use with dnd.

Product_name()  # OK for dnd. 
{
  #sed  "/^$1|/!d; s/^.*|//; s/ Mon.*//" $CANDLEHOME/registry/proddsc.tbl

  [ "$1" = "or" ] && print "Oracle"
  [ "$1" = "ox" ] && print "Informix"
  [ "$1" = "oy" ] && print "Sybase"
}

# code note: $Prd exists mostly for compatibility with kddos.  Plan to eliminate
# No changes required for dnd.

Product_setup()  # OK for dnd. 
{
  PR=$1
  Product=$(Product_name $PR) # full verbal product:  oracle

  case $PR in
     (or)  Prd=ora  Home_name=ORACLE_HOME  Init_name=init.ora    Get_login=yes
  ;; (oy)  Prd=syb  Home_name=SYBASE       Init_name=runserver   Get_login=yes
  ;; (ox)  Prd=inf  Home_name=INFORMIXDIR  Init_name=onconfig    Get_login=no
  ;; ( *)  Prd=     Home_name="DB home"    Init_name=init_file   # for generic help
  esac
}

# General menu coding:
# while true    # forever
#   MenuHeader  whatever     # Words used in kddhelp.sh to tell who called Help
#   cat <<!                  # Copy everything until ! in col 1 to stdout
#     1) This choice
#     2) That choice
# !
#   MenuFooter  whatever
#   if ans=`Select This That`
#   then $ans                # ans=return if user hit enter to exit
#   fi
# done

# Code note: if ans=`Select a b c` is NOT a comparison!!!  It means to assign
# results of Select() to ans, and also test Select's return code.  If the return
# code is 0, then we take action, which will be one of the passed parms, or
# "return" if he just hit enter).  If the return code is
# non-zero, the user typed an invalid response, so we just redisplay the menu.

# For dnd, Do_product() has been replaced with Do_product_dnd().

Do_product()  # Not used for dnd.
{
   # show menu for a product.  arg is product name: ora, syb, etc.
   Product_setup $1             # initialize product variables:

   if (( Scan ))
#$NLS-E$ "Performing initial scan for $Product.  Please wait..."
   then nls_replace 'KCI0034I' $Product
        Scan
#$NLS-E$ "Initial scan bypassed"
   else nls_replace 'KCI0035I'
   fi

   # 1st menu item depends on whether we need login/passwords.
   # these vars also used by Locate.
     if [[ $Get_login = yes ]]
     #$NLS-E$ "All $Product servers found.  Ready to specify login/passwords..."
     then DoneMsg="`nls_replace 'KCI0036I' $Product`"
          DoneAct=Login
     #$NLS-E$ "All $Product servers found.  Configuration complete"
	 else DoneMsg="`nls_replace 'KCI0037I' $Product`"
          DoneAct=return
     fi

   while true
   #$NLS-E$ do  MenuHeader $Product Options
   do  MenuHeader $Product `nls_replace 'KCI0038I'`
     print "1) $DoneMsg"
     #$NLS-E$ Optional disk searches, manually add servers...
     print "2) `nls_replace 'KCI0039I'`"
     #$NLS-E$ Display or Modify defined servers...
     print "3) `nls_replace 'KCI0040I'`"
     #$NLS-E$ Exclude/Include a server from the configuration..
     print "4) `nls_replace 'KCI0041I'`"
	 #$NLS-E$ Verify $Product OMA installation
     print "5) `nls_replace 'KCI0042I' $Product`"
     #$NLS-E$ List external configuration steps for $Product...
     print "6) `nls_replace 'KCI0043I' $Product`"
     #$NLS-E$ Help...
     print "7) `nls_replace 'KCI0044I'`"
     #$NLS-E$ MenuFooter Function menu
     MenuFooter `nls_replace 'KCI0045I'`
	 # LOOK here for function names.
     if ans=`Select $DoneAct Locate Modify Exclude "IVP $PR" \
                    "Help system" Help`
     then $ans
     fi
   done
}

# Do_product_dnd() is the dnd equivalent to Do_product().

Do_product_dnd()
{
  #print "... in Do_product_dnd"
  Product_setup $1  # Initialize product variables.
  #print "... arg = $1"

  let Scan=$scanVal  # scanVal is an arg to main.
  #print "... value of Scan = $Scan"
  #$NLS-E$ "performing the initial scan for $Product.  Please wait" 
  Err_report info "`nls_replace 'KCI0046E' $Product`" 

  Scan
}

# Next code outside of functions so that the trap on exit occurs after all done
My_fuser=$CANDLETMP/fuser.$CANDLEARCH
trap '[[ -a $My_fuser ]] && rm -f $My_fuser' EXIT

Scan()  # OK for dnd.
{

  [[ $PR = or ]] && or_oratab="or in $oratab" || or_oratab=
  #$NLS-E$ Scanning servers currently running $or_oratab
  nls_replace 'KCI0047I' $or_oratab
  cd $CANDLECFG
  Inventory before

  # Find a copy of fuser that I can execute.
  # see fuser triva in comments at end of this file
  typeset fuser
  if   fuser=$(whence fuser)
  then true
  elif [[ -x $My_fuser ]]
  then fuser=$My_fuser
  elif fuser=/usr/sbin/fuser
       [[ -x $fuser ]]                      # (HP can't execute it)
  then true
  else cp -f $fuser $My_fuser               # copy to my dir...
       fuser=$My_fuser                      # remember have done so
       chmod +x $fuser                      # w/ x-bit...
  fi

  # Make sure it works, else use kddos.
    for fuser in $fuser "$ddpath/kddos fuser" false
    do  $fuser . >$Null 2>&1 && break
    done

  # Pass 1: Scan_$PR return formats:  ?=maybe not there
  #   /home  pid sid? reason?
  #   ~owner pid sid?         (HP Sybase only: ~owner:partial/dbhome )

  typeset Home home     # Home is modified only by Scan_check.  home=scratch pad.
  typeset pid sid ok Lost Serv
  typeset -i n          # n=common idx for Lost and Serv
   {  Scan_$PR
      eval [[ -n \$$Home_name ]] && eval print \$$Home_name # this is free
   } | sort -u | while read entry
   do let n+=1
      if   [[ $entry = ~* ]]              # ~owner?
      then Lost[n]=$n\ $entry
      elif home=$home\ ${entry%% *}       # always save /possible/dbhome
           [[ $entry != *\ * ]]           # only one word=bare home.
      then true                           # thats all for this line
      elif [[ $entry = /*\ [1-9]* ]]      # /home pid = a Serv entry.
      then Serv[n]=$n\ $entry
      #$NLS-Q$
	  else say Invalid Scan_$PR result: $entry
           exit 99
      fi
   done

  for break in action            # let "break" exit code block anytime
  do  Scan_check $home && break

  # Pass 2: Bring in home from existing config files, and see if that finds
  # everything.  Note, this relies upon the "quotes" in config values.
  # enh:  bring in sids, too

    files=$(ls $HOST\_$PR\_*.cfg 2>&- ) &&
       Scan_check $(fgrep -s db_home= $files | cut -f2 -d\" ) &&
         break

  # Pass 3: Resolve any pids with gcore, if available.

    if   [[ -a /usr/bin/gcore ]]    # may be -x only to root
    then for lost in "${Lost[@]}"
         do set   $lost; n=$1     # n home pid
            if    all3=$($ddpath/kddos look $Prd $3)
            then  unset Lost[n]
                  Serv[n]="$n $all3 kddos_look"
            fi
         done
         Scan_check && break     # should always be complete now.
    fi

  # Pass 4:  search in $HOME directories of DBowners. (fast)
  # Weakness: .sh_history permissions are 600

  Scan_check $(
    for lost in "${Lost[@]}"
    do  lost=${lost#* }       # delete leading "n "
        print ${lost%%[ :]*}   # just ~owner
    done | sort -u | while read owner
    do eval Script $owner/.@(profile|login|cshrc|history|sh_history)
    done | sort -u) && break

  # Pass 5: disk search:  See comments outside function.

    typeset fs_users

    # create a list of filesystem-users (only once)

    true ${fs_users=$(
      # field=for AIX, df output lists filesys name last, all else first.
      #     c=fuser parm for "while filesys" (must be null for AIX)
      # fuser 2>&1 so we get the usage-letters after each pid.
      [ $AIX ] && field=NF  c= || field=1  c=-c
      \df | awk "/\// {print \$$field}" | xargs $fuser $c 2>&1)}

    for lost in "${Lost[@]}"
    do  set $lost
        n=$1 owner=$2 pid=$3 sid=$4

        if [[ $owner = *:* ]]
        then  partial=${owner#*:} owner=${owner%:*}
        else  partial=
        fi

        # Find filesystem for text(pgm) file.  fuser result is $pid, immediately
        # followed by various (unpredictable letters.  I want t=text (executable) file

        filesys=$(print "$fs_users" | grep " $pid[^ ]*t")   # req. leading blank
        filesys=${filesys%%:*}            # trim to filesystem name (must be only 1)

        # don't search same owner's home twice.
          if [[ $owners != *:$owner:* ]]
          then eval uhome=$owner          # expand ~owner to real path
              # if owner's $HOME on same filesystem as $filesys, look there first.
              same_fs $uhome && owners=:$owner:$owners || uhome=
          fi

        owner=${owner#~}      # because kddos wants it that way

        # Search hierarchy: partial(if any), uhome(if applicable), filesys, filsys(exhaustive)

        for here in $partial $uhome $filesys /$filesys
        #$NLS-E$ Beginning search under $here
		do say  `nls_replace 'KCI0048I' $here`
           fs=1               # confine search to one filesys, unless...
           [[ $here = $partial ]] && { same_fs $partial || fs=; }
           dbhome=$($ddpath/kddos find $Prd $here $owner $fs) || continue # if no hits

           # Oracle exception: ORACLE_HOME must be exactly as oracle expects,
           # another path to the same location is not good enough (IMR 522277)
           # Note, Oracle only supports ORACLE_HOMEs that are pure physical paths,
           #       but does not enforce this.  ORA80A41 is an counter-example.
           # kddos find only produces physical paths

           # techniques:
           # 1. (only works for oracle 7) look in the control$SID.ctl files (not imp)
           # 2. look for line in orainst/root.sh that says:
           #   ORACLE_HOME=${ORACLE_HOME:=/path...}
           # and execute it.s

           [[ $PR = or ]] &&
            dbhome=$(
              for home in $dbhome
              do  if   line=$(fgrep ORACLE_HOME:= $home/orainst/root.sh 2>&-)
                  then unset ORACLE_HOME           # force ${ORACLE_HOME:=...} to fire
                       eval $line
                       # do not accept ORACLE_HOME unless it really points to this home
                       [[ $home -ef $ORACLE_HOME ]] && print $ORACLE_HOME || print $home
                  else print $home
                  fi
              done | sort -u)

           Scan_check $dbhome  && break 3   # (for here, while(n), for break)
           [[ -z ${Lost[n]} ]] && break
        done
    done     # for lost in $Lost
  done       # for break in action

  # Final report: servers that could not be found
    if (( ${#Lost[*]} ))
    #$NLS-E$ then print Following servers could not be found:
	then nls_replace 'KCI0049I'
         for lost in "${Lost[@]}"
         do  print ${lost#* }  # but not index
         done
    fi

  # Determine sids for running servers, and formally add them.
    typeset -i f
  # 1st, sort into dbhome order.

    { [[ $Scan = 2 ]] && print $Home | xargs -n1
      for serv in "${Serv[@]}"; do print ${serv#* }; done   # convert to row/col, no n
    } | sort -r |                          # foreach home, sort $Home lines after $Serv,
    while read home pid sid reason
    do
        if   [[ -n $sid && $sid != . ]]
        then print $home $sid $reason
        elif [[ -n $pid && -n $Solaris ]] && ok=$($ddpath/kddos look $Prd $pid)
        then set -- $ok
             print $home $3 $reason,kddos_look
        elif #  Must search dbhomes for sids, but only when home changes.
             if [[ $home != $home1 ]]
             then  home1=$home                # dont do this home again
                   # convert finds=(home sid reason init :pids) to a self-indexed array.
                   unset finds
                   f=0
                   Find_sid_$PR $home | while read this
                   do let  f+=1
                      finds[f]="$f $this"
                   done
             fi
             # Scan=2 only: a home-only line means this dbhome does not have any
             # running servers.  So, we just send any remaining finds.
             [[ -z $pid ]] &&
                  (IFS=$NL; print "${finds[*]}" ) | cut -s -d\  -f2- && continue

             # parse note: finds[n] contains "fake" fuser output, :pid\o pid\...
             hit=$(IFS=$NL; print "${finds[*]}" | grep ":$pid[^ ]*o")

        then # hit=sid running this pid (2+ if mult initfiles pretend to use same DB)
             print "$hit" | while read f etc
             do    print $etc
                   unset finds[f]
             done
        #$NLS-E$ "Could not determine server name for:\n" 
		else say  "`nls_replace 'KCI0050I'`\n" \
                  "    PID=$pid home=$home, ignored"
        fi
    done | while read dbhome sid reason init etc
    do   [[ $dbhome != $home1 ]] && dbver=$(DBver) home1=$dbhome
         Update server=$sid dbhome=$dbhome initfile=$init dbver=$dbver \
                reason=$reason $Batch_parms
    done

  Inventory after 
}

# Scan_check potential_dbhome...
# Updates Home=accumulated list of dbhomes, which is localized to Scan,
#              but only updated here.
#         Lost=servers with unknown homes,
#         Serv=found running servers.
# returns 0 if all servers found AND Scan=1  !!          (never called if Scan=0)
#         1+ if Scan=2, indicating do not stop search
#
# The logic of Scan_check assumes that items in Lost move from
# Lost[n] to Serv[n] only.  It keeps no record of fuser output from
# previous calls, only the $Home entries from previous calls.
# The $Home built by this process is also used for discovering
# non-running databases.

Scan_check()  # OK for dnd. 
{
  # args are potential new dbhomes, with dups eliminated.
  # Keeps $Home=list of potential dbhomes already checked.  Only updated here.

  typeset -i OK          # OK=0 means scan done (req Scan==1)
  (( OK = ${#Lost[*]} || Scan==2 )) || return 0     # trival case: none lost
  (( $# )) || return $OK                            # No args, no action.

   # homes = fuser output of new dbhomes
   # xtr   = item under home that server uses
   # x     = how used:  c=current dir, t=text(executable)
   typeset x xtr homes home

   case $PR in
       (or) x=c xtr=/dbs
    ;; (oy) x=t xtr=/bin/dataserver
    ;; (ox) x=t xtr=/bin/oninit
   esac

   # 1st, homes=args, purged of any unqualified ( ! -d $home),
   # any already in $Home, and any dups

     homes=$(
       for home
       do  [[ -d $home && $Home != *" $home "* ]] && print $home   # note require blanks
       done | sort -u)
     [[ -z $homes ]] && return $OK         # none new this call

   # convert $homes to an list of processes using the $homes

   if [[ -z $xtr ]]
   then  homes=$($fuser 2>&1 $homes)
   else  homes=$(print "$homes" | xargs -i $fuser 2>&1 {}$xtr)
   fi

  # Remove any found items from $Lost, add to $Serv (use same index)
    typeset -i n
    typeset pid sid

    for lost in "${Lost[@]}"
    do set $lost              # n ~owner pid sid
       n=$1 pid=$3 sid=${4-.} # sid may be unknown
       if [[ $homes = *\ $pid*([! ])$x* ]]
       then  home=$(print "$homes" | grep " $pid[^ ]*$x")
             Serv[n]="$n ${home%$xtr:*} $pid $sid fuser"
             unset Lost[n]
       fi
    done

  # Update $Home with new ones (with blanks around each dir)
    set -- $(print "$home" | sed "s#$xtr:.*##") ''
    Home="$Home $* "

  return $(( ${#Lost[*]} || Scan==2 ))
}

# Initial scan functions:  Scan_$PR
#   no need to localize,  called only by sub-process.
#   returns rows of  home pid sid reason
#     home=/path            if determinable
#         =~owner           if not
#         =~owner:/partial  Sybase on HP only: only partial path known
#     pid
#     sid if known
#     reason = data source, if relevant.  1 word, no ':'

# ORACLE INITIAL SCAN
# I used to filter all non-aix servers through oratab, and only
# report the ones not found there.  But, oratab can be wrong,
# so it's safer to pass them back and let Scan_check match them.

Scan_or()  # OK for dnd.
{

  # Gather all possible information from the ps command.

  ps=$(ps -ef)                    # only do once

  print "$ps" | grep ora_pmon | while read who pid rest
  do sid=${rest#*ora_pmon_}
     if [ $AIX ]                 # can get home from env string
     then typeset this=`ps eww $pid`   # env string
          this=${this#*ORACLE_HOME=}  # chop off leading junk
          where=${this%% *}          # chop off trailing junk
          reason=AIX_ps_ceww
     else where=~$who
          reason=
     fi
     print "$where $pid $sid $reason"
  done

  # 593505
  # orasrv is the boot-startup server, which may have a path.  Send if so
  # code:  sed '\:string:cmd'  means use : as context address delimiter
  # sed:    only trailing /*/bin/orasrv; remove all before ' /'; remove /bin/orasrv

    print "$ps" | sed '\: /.*/bin/orasrv$:!d; s:.* /:/:; s:/bin/orasrv$::'

  # Send everything in the oratab file

    grep -sv "^#" $oratab | cut -s -f2 -d:  # cut -s = suppress lines w/o :
}

# Sybase is an exception to normal rules:
# 1. The ps command usually has the full path and server name.
# 2. The server environmental variables can be meaningless.
#    (All server parms are usually in the runserver file)
# 3. The dataserver pwd is meaningless unless the executable is a relative path.
# 4. HPUX truncates "ps -ef" CMD  to 60 bytes, Solaris to 79.

Scan_oy()  # OK for dnd. 
{

 typeset -i psn   # numbers below are empircal
 case $(uname) in (AIX) psn=999999;; (HP-UX) psn=60;; (SunOS) psn=79;; esac
 typeset -r pgm=/bin/dataserver
 typeset sid cmd pid etc home

 # grouch :Solaris ps -e truncates executable names to 8 chars
 ps -e | grep dataserv | grep -v grep | while read pid etc
 do  ps=$(ps -fp $pid)
     cmd=${ps##*:[0-9][0-9] }                       # cmd is after TIME(mm:ss)

     # Get Sybase home, if possible.

     if   [[ $cmd = /*$pgm* ]]             # complete path in cmd line?
     then home=${cmd%%$pgm*}               # remove trailing /bin/dataserver

     elif [ $Solaris ] &&                  # Solaris....
          $ddpath/kddos look syb $pid      # look under the covers (home, pid sid -> stdout)
     then continue                         # All done with this server!

     elif [ $AIX ] &&                      # AIX ps -f complete, so $cmd is a relative path
          env=$(ps ceww $pid) &&           # complete env string
          pwd=${env#*PWD=}    &&           # $PWD + blank + junk
          text=${pwd%% *}/${cmd%% *} &&    # $PWD/$cmd
          [[ -a $text ]]                   # be sure itsa file. (no guesswork, so no $fuser check)
     then cd ${text%$pgm}                  # cd to home
          home=$PWD                        # resolve any /./ or /../ in $text

     elif [[ $cmd =  /*/bin/* ]] &&        # HP: at least /bin/ here?
          home=${cmd%%/bin/*}   &&         # its a potential home...
          text=$home/bin/dataserver &&     # running this executable...
          [[ -a $text ]]            &&     # provided it exists...
          [[ $($fuser 2>&1 $text) = *\ $pid*([! ])t* ]]  # and we are really using it.
     then true

     elif set -- ${ps#*$NL}                   # owner=$1
          [[ $cmd = /* ]]                  # HP: is it a truncated path?
     then home=~$1:$cmd                    # cmd is an incomplete sybase home.

     else home=~$1                         # useless relative path, let Scan disk search.s
     fi

     # get server name, if possible

     if   [[ ${#cmd} -le $psn ]]           # complete cmd and args?
     then case $cmd in
		( *-s*-e* ) 
          		sid=${cmd#*-s}
               		sid=${sid%% *}
	;;	( *-s*-d* )		   # if the pattern is *-s*-d* on HP1020
			sid=${cmd#*-s}
			sid=${sid%% *}
	;;	( *-d*-s* ) sid=.	   # if the pattern is *-d*-s* for HP1020
	;;	( * ) sid=SYBASE           # Sybase default if no -s.
	;; esac

     elif [[ $cmd = *-s*\  ]]              # truncated args with complete -s
     then sid=${cmd#*-s}
          sid=${sid%% *}

     else sid=.                            # no clue
     fi

     print "$home $pid $sid ps"
 done
}

Scan_ox()  # OK for dnd. 
{
    ps -ef | grep oninit | while read who pid ppid etc
    do [[ $ppid = 1 ]] || continue        # only want top-level server
       if [ $AIX ]                        # can get home from env string
       then env=$(ps eww $pid)            # env string
            home=${env#*INFORMIXDIR=}     # chop off leading junk
             sid=${env#*INFORMIXSERVER=}  # chop off leading junk
            print "${home%% *} $pid ${sid%% *}"
       else print "~$who $pid"
       fi
    done
}

# Find_sid_$PR: locate all servers under a dbhome.
# Also called by Locate fns to get sids for
# non-running servers, but with fuser=print to disable it.
# Return format:
#    dbhome sid reason initfile :processes....
#
# We put the colon in front of the processes so that the caller can grep for
# ":$pid" with no possibility of the pid accidentally matching an earlier
# part of the line.

# Find_sid_or will never be called for running servers, so it doesn't try to
# extract processes.  Serious cheat:  Find_sid_or returns only 1 row per sid
# found, no matter what the reason.  Also note that this mixes guaranteed good
# sid's extracted from initfiles (via db_name= in kddos) and questionable ones
# extracted from filenames.

Find_sid_or()  # OK for dnd. 
{
  # possible: dbs/  config* ora|init!(*_0).ora (look in ifile)
  # confirm:  dbs/@(ctrl[1-3]*.ctl|@(temp|tool|usr|sgadef)*.dbf)
  # this needs work, no -a *.dbf test yet
  home=$1
  cd $1/dbs
  for file in *
  do  case $file in
         (@(init|config)!(*_0).ora )
            sid=$($ddpath/kddos orasid $file) &&    # only if kddos $?=0.
               print $home $sid kddos_orasid \
                    $( [[ $file = init* ]] && print $file )
            continue                        # already written
      ;; (ctrl[1-9]*.ctl) sid=${file#ctrl[1-9]}
                          sid=${sid%.ctl}
      ;; ( @(temp|tool|usr|sgadef)*.dbf )
             sid=${file#@(temp|tool|usr|sgadef)}
             sid=${sid%.dbf}
      ;; (*) continue               # ignore all others
      esac
      print $home $sid $file # reason=$file, initfile unknown
  done | sort -u +0 -2
}


Find_sid_oy()  # OK for dnd. 
{
  home=$1
  cd $1/install
  for initfile in $(grep -ls bin/dataserver !(*errorlog*|*.log|sybinit|@(start|show)server))
  do  if [[ -r $initfile ]]
      then cmd=$(egrep -v "^[${IFS%?}]*#" $initfile)    # exclude comment lines (# after any whitespace)
      else cmd=$($ddpath/kddos sybsid $initfile)
      fi
      # does the database exist? anyone using it?
        [[ $cmd = *-d* ]] || continue
        x=${cmd#*-d}; database=${x%% *}
        [[ -a $database ]] || continue

      if [[ $cmd = *-s* ]]               # determine sid from file
      then x=${cmd#*-s}; sid=${x%% *}
      else sid=SYBASE
      fi

      set -- $($fuser 2>&1 $database) && shift
      (($#)) && reason=fuser || reason=initfile    # why do we think this is valid?
      print $home $sid $reason $PWD/$initfile :$*          # return list of sids using this dataset
  done | sort -u
}

# An informix onconfig file can be named anything, so we check
# all files in the etc directory which are less than 10K.
# Code notes:
# a) AIX find doesnt support -follow, so we don't follow symbolic links in AIX.
# b) "find -print | xargs" is far more efficient than "find -exec"
# c) We return the pid of the database users' parent, because because Scan_inf
#    returns the pid of only the parent process, and the parent itself does not
#    use the database.

Find_sid_ox()  # OK for dnd. 
{
  home=$1
  cd $1/etc
  find !(*.log|*.out) -prune -size -10000c \
        $(test $AIX || print -follow) -print |
  xargs $ddpath/kddos infsid | while read initfile sid database
  do      set -- $($fuser 2>&1 $database);   shift
          if   (( $# ))
          then reason=fuser
               ps=$(ps -fp ${1%o})
               set ${ps#*$NL}           # ppid is $3
          else reason=initfile
          fi
          print $home $sid $reason $PWD/$initfile :$3o   # pretend open
  done | sort -u
}

# arg in same filesystem as $filesys? (not work on AIX or HP9)
same_fs() { [[ $(df -n $1) = $(df -n $filesys) ]]; }  # OK for dnd.

#---------- here ends automatic-scan functions --------------------------

# Locate() has been replaced with individual calls to the functions invoked by Locate.

Locate()  # Not used for dnd. 
{
   # fool "set $($fuser anything) && shift" into creating no args.
     typeset fuser=print

   while true
   #$NLS-E$ MenuHeader $Product Disk Search
   do MenuHeader $Product `nls_replace 'KCI0051I'`
     typeset -l lproduct=$Product       # force to lower case

# Home_name is defined in Product_setup.

     #$NLS-E$ Search of scripts in the PATH, looking for $Home_name
     print "1) `nls_replace 'KCI0052I' $Home_name`"
     #$NLS-E$ Display a list of Unix users that are possible $Product owners....
     print "2) `nls_replace 'KCI0053I' $Product`"
     #$NLS-E$ Specify a search starting point, usually the \$HOME of the
     #        $Product owner.  For example, ~$lproduct.  You can select
     #        this option multiple times with different starting points.
     print "3) `nls_replace 'KCI0054I' \$HOME $Product ~$lproduct`"
     #$NLS-E$ Manually add a server
     print "4) `nls_replace 'KCI0055I'`"
     print "5) $DoneMsg"
     #$NLS-E$ Help
     print "6) `nls_replace 'KCI0056I'`"

	 # dnd note - Option 5 or $DoneMsg is not needed since the "Login" button has it covered.

     #$NLS-E$ MenuFooter Search Options
	 MenuFooter `nls_replace 'KCI0057I'`

# LOOK here for function names. 

     if ans=`Select Path Userlist Search Add_manual $DoneAct Help`
     then $ans
          [[ $ans = $DoneAct ]] && return
     fi
   done
}


Path()  # OK for dnd.
{
  # Search path for Home_name
  Inventory before
  typeset path=${PATH#*:}      # exclude product bin directory
  while [[ -n $path ]]
  do   dir=${path%%:*}                # 1st entry
       # Remove all but 1st entry from path
         [[ $path = *:* ]] && path=${path#*:} || path=
       # Exemptions for performance
         [[ $dir = $CANDLEHOME/* ]]           # Exempt myself
         [[ $dir = . ]] && continue           # Exempt myself
         [[ $dir = ?(/usr)/?(s)bin ]] && continue # Exempt unix bins
         [[ $path = *:$dir:* ]] && continue   # Ignore most dups in path
     say  searching $dir
     # Cheat: AIX find doesn't support -follow, so ignore links
     # Note, no check for execute bit, because not needed to ". script"
     if [ $AIX ]
     then file $dir/*
     else file $(find $dir/* -prune -follow -type f -print 2>&-)
     fi | while read this
     do #  Results of file is "file: description"
        #  reject scripts provided by Oracle, and
        #  for others, description must contain text or script.
        [[ $this = !(*/?(c)oraenv.*|*/dbhome):*@(text|script)* ]] &&
           Script ${this%%:*}
     done
  done | sort -u | while read dbhome
  do [[ -d $dbhome ]] && Add_servers $dbhome
  done
  Inventory after
}


Script()  # OK for dnd. 
{ 
# return any dbhome assignment found in passed files

  fgrep -s $Home_name $* | while read line
  do   case $line in
          (*$Home_name=/*)   set ${line#*$Home_name=}
                             print ${1%;}  # no trailing ;
       ;; (*setenv\ *$Home_name\ */*) set ${line#*$Home_name}
                             print ${1%;}
       ;; esac
  done
}


Search()  # OK for dnd.  Always call with an arg. 
{
    if [[ $# -eq 0 ]]
    then set -- $(Get_dir search starting directory) || return
    fi
    Inventory before
    for  dbhome in $($ddpath/kddos find $Prd $1)
    do   Add_servers $dbhome
    done
    Inventory after
}

Add_servers()  # OK for dnd. 
{ 
# Add all servers under 1 dbhome

    Find_sid_$PR $1 | while read dbhome server reason initfile
    do   Find $server in $before && continue
         Update server=$server dbhome=$dbhome \
                dbver=$(DBver) \
                ${reason:+reason=$reason} \
                ${initfile:+reason=$initfile} \
                $Batch_parms
    done
}

# Modified for use with dnd.

Userlist()  # OK for dnd. 
{
   # Inventory user accounts.

#   typeset tempfile=/tmp/Userlist.$$
#   trap "rm $tempfile 2>&-" EXIT     # Always delete file on function exit

  typeset tempfile=$CANDLETMP/Userlist_dnd.txt

  rm -f $tempfile > /dev/null 2>&1
  #$NLS-E$ "could not remove temp file \"$tempfile\""
  [ -f "$tempfile" ] && Err_report fail "`nls_replace 'KCI0058E' \\"$tempfile\\"`"
  touch $tempfile > /dev/null 2>&1
  #$NLS-E$ "could not create temp file \"$tempfile\""
  [ -f "$tempfile" ] || Err_report fail "`nls_replace 'KCI0059E' \\"$tempfile\\"`"

   { # Generate userlist, store in file
#     MenuHeader -n Potential $Product Owners for Disk Search
     # Fixed-length variables for display
     typeset -R2   num="( "
#     typeset -L12  user=User
     typeset -L20  user=User
#     typeset -L25  desc=Description
     typeset -L35  desc=Description
#     typeset -L30  fsys=File\ System
     typeset -L40  fsys=File\ System
     typeset -L   dashes=$(Userlist_line) # fix the width
     dashes="--------------------------------------------------------------------------------------------------"
     print "$dashes\n$(Userlist_line)\n$dashes"
     num=0
     typeset Userlist=
     # 605776:  Earlier:  Userlist_gen "$Prd|dba" | while read user fsys desc
     # "while read" into fixed-length vars cores Sol 2.6 ksh.  code-around follows:
     typeset a b c
     Userlist_gen "$Prd|dba" | while read a b c
     do  user=$a  fsys=$b  desc=$c
         ((num+=1))
         Userlist_line
         Userlist=$Userlist\ $user
     done
     print "$dashes"
#     print " $((num+=1))) | All of the above"
#     print " $((num+=1))) | Help..."
#     print "$dashes"
   } >$tempfile

  if [ -z "$Userlist" ]
  then
    #$NLS-E$ "no users found with $Prd or dba in their names or descriptions"
	Err_report info "`nls_replace 'KCI0060I' $Prd`"
    rm -f $tempfile > /dev/null 2>&1
    return
  fi

  # Just generate a list of users if no args (user names in this case) are provided.  The
  # contents of this list will then be displayed in the Search II dialog of dnd.

  [ $# -eq 0 ] && return

  # Search given specific user names.  These names were chosen from the list (displayed in 
  # the Search II dialog of dnd) generated in a prior call to Userlist (no args).  The results
  # of the search will then be displayed in the "Servers" window of dnd.
   
  typeset me

  for me in "$@"
  do
    #$NLS-E$ "searching for user \"$me\""
	Err_report info "`nls_replace 'KCI0061I' \\"$me\\"`"
    eval Search ~$me
  done

  # typeset ans
  # while true
  # do  print "$(<$tempfile)"
  #     if ans=`Select $Userlist All Help`
  #     then case $ans in
  #           (return | Help) $ans
  #        ;; (All) for me in $Userlist
  #                 do  eval Search ~$me
  #                 done
  #        ;; (*)   eval Search ~$ans
  #        ;; esac
  #     fi
  # done
}


Userlist_line() { print "$num) | $user | $desc | $fsys"; }  # OK for dnd. 


Userlist_gen()  # Minor mods; OK for dnd.
{
 # Userlist string
 # Return for each user that maches string: user filesys desc.

 # translation of next statement:
 #  concatenate  /etc/passwd (has priority) with ypcat & nixcat results
 #  select lines with string (-i=ignore case)
 #  cut to only user:description
 #  sort by user (-u=unique keys, t:=delimiter,
 #    +0 -1 means key is beginning of line to next-to-last field
 #    (older form of sort key required on Sol 2.4).
## niscat passwd.org_dir 2>&-

 typeset user desc home pass x
 { cat /etc/passwd
   ypcat  passwd.byname  2>&-
 } | egrep -i "$1" | sort -ut: +0 -1 |
   while IFS=: read user x x x desc home x
   do   
#     [[ -d $home ]] && print $user $(print Userlist_loc $home) $desc
     [[ -d $home ]] && print $user $(Userlist_loc $home) $desc
   done
}


Userlist_loc()  # OK for dnd. 
{
   # Return physical location of provided directory
   # On HP9 systems, /bin/df doesn't work, so customers often
   # install bdf (Berkley df) instead.  Also, various versions of df
   # have different return formats (some have headings, others don't)
   #    system:/mount
   #    /user/home/xxx  (system:/mount)
   # The following code handles all cases.  If df doesn't work, run bdf
   # instead.  If () are present, take only what is between them (if the
   # delimiter isn't present, cut passes the line through as is), then
   # get the first blank-delimited word.

   { bdf $1 || df -k $1 || print "\n<unknown>"; } 2>&- | sed '2!d; s/ .*//'
}

# Add_manual was replaced with a dnd dialog and Add_manual_dnd().

Add_manual()  # Not used for dnd.
{
   # Auto-or-manually add
   #$NLS-E$ MenuHeader "Add server(s)"
   MenuHeader "`nls_replace 'KCI0062I'`"
   dbhome=`Get_dir $Home_name dir for server` || return

   if [[ -d $dbhome ]]
   then
      # Oracle requires dbhome as physically listed (unimplemented)

      #$NLS-E$ print "    Allowing self-discovery to fill in details..."
	  nls_replace 'KCI0063I'
      Search $dbhome       # note this is a slight misuse of $dbhome
   fi

   #$NLS-E$ print "continuing with manual add..."
   nls_replace 'KCI0064I' 
   print
   #$NLS-E$ server=$(Read "Enter Server name or enter to return to menu  ")
   server=$(Read "`nls_replace 'KCI0065I'`  ")
   [[ -z $server ]] && return
   #$NLS-E$ dbver=`Read "Enter $Product software release level  "`
   dbver=$(Read "`nls_replace 'KCI0066I' $Product`  ")
   #$NLS-E$ initfile=$(Read "Enter $Init_name filename  (optional)  ")
   initfile=$(Read "`nls_replace 'KCI0067I' $Init_name`  ")
   if  [[ $Get_login = yes ]]
   then Get_login_info $server
        Update dbhome=$dbhome server=$server \
               dbver=$dbver initfile=$initfile \
               login=$login password=$password
   else Update dbhome=$dbhome server=$server \
               dbver=$dbver initfile=$initfile
   fi
   Modify_server $HOST\_$PR\_$server.cfg

}

Add_manual_dnd()  # OK for dnd.
{
  # $1 = dbhome  
  # $2 = server 
  # $3 = dbver
  # $4 = initfile
  # $5 = login
  # $6 = password

  # Note, args 5 and 6 are not required of Informix since for it, Get_login = no.  However,
  # we no longer provide an Informix agent.  

  # TODO - implement the Get_dir() check for dbhome.
 
  if [[ -d $dbhome ]]
  then
    # Oracle requires dbhome as physically listed (unimplemented)

    #$NLS-E$ print "... allowing self-discovery to fill in details ..."
	nls_replace 'KCI0068I'
    Search $dbhome       # note this is a slight misuse of $dbhome
  fi

  #$NLS-E$ Err_report info "adding server \"$2\""
  Err_report info "`nls_replace 'KCI0069I' \\"$2\\"`"

  if [ "$Get_login" = "yes" ]
  then 
    Update dbhome="$1" server="$2" dbver="$3" initfile="$4" login="$5" password="$6"
    visCmd="$visCmd $1 $2 $3 $4 $5 xxxxxx"  # Do not display the password.
  else 
    Update dbhome="$1" server="$2" dbver="$3" initfile="$4"
    visCmd="$visCmd $1 $2 $3 $4"
  fi
}

# Replaced with a dialog and Mod_login_dnd.

Login()  # Not used for dnd. 
{
  #$NLS-E$ Display 'Login/Password for' Mod_login
  Display '`nls_replace 'KCI0070I'`' Mod_login
}

Modify()  # Not used in dnd. 
{
  Display Display/Modify Modify_server
}

# For dnd, Display_dnd() replaces Display().

Display()  # Not used for dnd. 
{
   # Generic display:  Usage: Display heading-verb select-function
   typeset tempfile=$CANDLECFG/Userlist_$HOST.cfg
   trap "\rm $tempfile 2>&-" EXIT     # Always delete file on function exit
   while true
   do # Note, coding it as:  { list } > tempfile; cat tempfile
      # instead of:          { list } | cat
      # because the latter executes list in a separate process, and
      # all of its variable changes are lost.

      { MenuHeader -n $1 $Product Servers
        files=`Inventory -f`
        List $files || return
        typeset -R2 c=$List_count
        if [[ $Get_login = yes ]]
#$NLS-E$        then print "\
#	$((c+=1))) Specify the same login/password for all servers
#    $((c+=1))) Specifiy login/password for each server in turn
#$dashes"
        then print "\
	$((c+=1))) `nls_replace 'KCI0071I'`
    $((c+=1))) `nls_replace 'KCI0072I'`
$dashes"
        fi
        print "\
    $((c+=1))) Help"
        #$NLS-E$ MenuFooter Select a server to display or modify
		MenuFooter `nls_replace 'KCI0073I'`
      } > $tempfile
     cat $tempfile

     if ans=`Select $files log_all log_each`
     then case $ans in
           (log_all) Get_login_info
              for  file in $files
              do   Update $file login=$login password=$password
              done
        ;; (log_each) for file in $files
              do  Get_login_info ${file%.*}
                  Update $file login=$login password=$password
              done
        ;; (return)  return
        ;; (Help)    Help
        ;; (*)        $2 $ans   # Call function whose name passed by Login or Modify
        ;; esac
     fi
   done
}

# Generate a list of servers; this list will be displayed in the main GUI, JDBConfigMain.java.

Display_dnd()
{
  # Display all servers regardless of status.  The status of each server will be considered
  # in the GUI.  Following, are the different status types (Refer to the List() function.):
  #   "Incomplete" 
  #   "Excluded"
  #   "Fully Configured"
  #   "7.1.4 login must=sys"

  typeset name tfile=$CANDLETMP/Display_dnd.txt

  rm -f $tfile > /dev/null 2>&1
  #$NLS-E$ "could not remove temp file \"$tfile\""
  [ -f "$tfile" ] && Err_report fail "`nls_replace 'KCI0074E' \\"$tfile\\"`"
  touch $tfile > /dev/null 2>&1
  #$NLS-E$ "could not create temp file \"$tfile\""
  [ -f "$tfile" ] || Err_report fail "`nls_replace 'KCI0075E' \\"$tfile\\"`"

  #$NLS-E$ "generating a list of servers to display"
  Err_report info "`nls_replace 'KCI0076I'`"
  files=$(Inventory -a)
  if [ -z "$files" ]
  then
    #$NLS-E$ "could not find any servers"
	Err_report info "`nls_replace 'KCI0077I'`"
    return
  fi
  
  # print "files = $files"
  # The contents of files looks like (without the quotes):
  #   "puffer_or_FakeDB.cfg puffer_or_OR901.cfg puffer_or_glen.cfg"
 
  # Echo each filename into the temp file.  DND will need to use these names as args in
  # calls to this script, kddfind_dnd.sh

  print "<>" > $tfile
  for name in $files
  do
    print $name >> $tfile
  done
  print "<>" >> $tfile

  List $files >> $tfile
}

# Replaced with individual calls to the various Mod_* functions.

Modify_server()  # Not used for dnd. 
{  # arg is config file name
  file=$1
  typeset -L11 init__name=$Init_name   # fix length.
  typeset -L11 home__name=$Home_name   # fix length.
  Update="Update $file"  # short-cut var
  cd $CANDLECFG
  while true; do
    . ./$file   # reread file
    MenuHeader Modify $Product server ${file%.*}
        print "1) $home__name:  $db_home"
        #$NLS-E$ If this is right, version is automaticaly updated. 
        print    `nls_replace 'KCI0078I'`

        #$NLS-E$ (optional)
        print "2) $init__name:  $db_initfilename  (`nls_replace 'KCI0079I'`)"
        #$NLS-E$ Version    :
        print "3) `nls_replace 'KCI0080I'`    :  $db_ver"
    typeset -i n
    typeset choices
    if [[ $Get_login != yes ]]
    then n=3           # as listed above
    else n=5           # plus next two
         choices=Mod_login\ Mod_pass
         if [[ -z $db_password ]]
            #$NLS-Q$
			then db_pass_show="<not configured>"
            else db_pass_show="<encrypted>"
         fi
        #$NLS-E$ login id   :
		print "4) `nls_replace 'KCI0081I'`   :  $db_login"
        #$NLS-E$ Password
		print "5) `nls_replace 'KCI0082I'`   :  $db_pass_show"
    fi
        #$NLS-E$ Display complete configuration details
        print "$((n+=1))) `nls_replace 'KCI0083I'`"
        #$NLS-E$ Help
		print "$((n+=1))) `nls_replace 'KCI0084I'`"
    MenuFooter Modify Server options

    if ans=`Select Mod_home  Mod_init Mod_ver \
                   $choices Details Help`
    then $ans
    fi
  done
}

# Collect the config data for a specific server.  This data will then be displayed in a
# dnd dialog. 

Col_server_data_dnd()  # OK for dnd.
{
  typeset name tfile=$CANDLETMP/Col_server_data_dnd.txt

  name=$1  # $1 = config file name (e.g. puffer_or_OR901.cfg)

  rm -f $tfile > /dev/null 2>&1
  #$NLS-E$ "could not remove temp file \"$tfile\""
  [ -f "$tfile" ] && Err_report fail "`nls_replace 'KCI0085E' \\"$tfile\\"`"
  touch $tfile > /dev/null 2>&1
  #$NLS-E$ "could not create temp file \"$tfile\""
  [ -f "$tfile" ] || Err_report fail "`nls_replace 'KCI0086E' \\"$tfile\\"`"

  #$NLS-E$ "collecting server data from \"$CANDLECFG/$name\""
  Err_report info "`nls_replace 'KCI0087' \\"$CANDLECFG/$name\\"`"

  #$NLS-Q$
  Update="Update $name"  # short-cut var.  What is this?
  cd $CANDLECFG
#  print "... before db_home = $db_home"
  . ./$name   # Reread config file.
#  print "... before db_home = $db_home"

  print "dbHome = $db_home" >> $tfile
  print "dbVer = $db_ver" >> $tfile

  if [ -n "$db_initfilename" ]
  then
    print "dbInitFile = $db_initfilename" >> $tfile
  else
    print "dbInitFile = (optional)" >> $tfile
  fi

  print "dbLogin = $db_login" >> $tfile
 
  if [ -n "$db_password" ]
  then
    print "dbPassword = <encrypted>" >> $tfile
  else 
    print "dbPassword = <not configured>" >> $tfile
  fi
}

# Replaced with Mod_home_dnd().

Mod_home()  # Not used for dnd. 
{
   #$NLS-E$ print "  If possible, Version will be updated"
   nls_replace 'KCI0088I'
   #$NLS-E$ dbhome=`Get_dir "home directory of server(s) to be modified"`
   dbhome=$(Get_dir "`nls_replace 'KCI0089I'`")
   Update $file dbhome=$dbhome \
       $(test -d $dbhome && print dbver=$(DBver))
}

Mod_home_dnd()  # OK for dnd. 
{
  # $1 = config file name (e.g. puffer_or_AUTO.cfg)
  # $2 = full path name of server's home directory

  # TODO - implement Get_dir and automatic version update.

  Update $1 dbhome=$2 
}

# Modified for use with dnd.

Mod_ver()  # OK for dnd. 
{
  # $1 = config file name (e.g. puffer_or_AUTO.cfg)
  # $2 = version number

  #Update $file dbver=`Read "Enter $Product software release level  "`
  Update $1 dbver=$2
}

# Modified for use with dnd.

Mod_init()  # OK for dnd. 
{
  # $1 = config file name (e.g. puffer_or_AUTO.cfg)
  # $2 = init file name.  This will equal "NONE" if no init file was specified.

  #initfile=$(Read "Enter $Init_name filename  (fully qualified)  ")
  #Update $file initfile=$initfile
  
  Update $1 initfile=$2
}

# Replaced with a dnd dialog and Mod_login_dnd. 

Mod_login()  # Not used for dnd. 
{
   file=${1-$file}             # arg overrides file
   Get_login_info ${file%.*}
   Update $file login=$login password=$password
}

# Mod_login_dnd() is called when the user presses the "Specify" button in JDBConfigMain.java.  

Mod_login_dnd()  # OK for dnd.
{
  # $1 = config file name (e.g. puffer_or_AUTO.cfg)
  # $2 = login id

  Update $1 login=$2  # The keyword "login" is needed by Update().
}

# Replaced with Mod_pass_dnd.

Mod_pass() { Update $file password=`Get_password $db_login`; }  # Not used for dnd.

Mod_pass_dnd()  # OK for dnd.
{
  # $1 = config file name (e.g. puffer_or_AUTO.cfg)
  # $2 = password

  Update $1 password=$2  # The keyword "password" is needed by Update().
}

Details()  # Not used in dnd. 
{
  cat $file
  #$NLS-E$ "Press Enter to continue..."
  read x?"`nls_replace 'KCI0090I'`"
}

# Replaced with Exclude_dnd().

Exclude()  # Not used in dnd. 
{
   # Toggle exclude status
   typeset server status sure file verb
   while true; do
     MenuHeader $Product Inclusion Status
     files=`Inventory -a`     # All files, including excluded ones
     List $files || return
     MenuFooter Toggle Status

     if file=`Select $files`; then
        [[ $file = return ]]  && return
        server=${file##*_}
        server=${server%.cfg}
        eval status=\$db_$server   # variable-named variable set by List()
        [[ $status = Excluded ]] && verb=re-activate || verb=exclude
        sure=`Read "Are you sure you want to $verb server $server (y/n)   "`
        if [[ $sure = y ]]
        then Update $file toggle_status
        #$NLS-E$ "Configuration file for $server unchanged"
		else nls_replace 'KCI0091I' $server
        fi
     fi
   done
}

Exclude_dnd()  # OK for dnd.
{
  # $1 = config file name (e.g. puffer_or_AUTO.cfg)

  Update $1 toggle_status
}

List_line()  # OK for dnd. 
{
print "\
   $num) $db_sid | $db_ver | $db_installstatus | $db_login |"
}

# The display fields were increased for dnd.  The output from List() is retrieved and
# displayed by dnd.

List()  # OK for dnd. 
{
   # List servers, args=list of inventory files.
   if [[ $# = 0 ]]
   #$NLS-E$ "   No $Product servers configured for $HOST."
   then  nls_replace 'KCI0092I' $Product $HOST
         return 99
   fi

   # Define vars in config file, but fix their width for display
     typeset -R2  num="( "
     #typeset -L18 db_sid="Server Name"
     #$NLS-Q$
	 typeset -L28 db_sid="Server Name"
     #typeset -L7  db_ver="Version"
     #$NLS-Q$
	 typeset -L17  db_ver="Version"
     #typeset -L20 db_installstatus="Installation Status"
     #$NLS-Q$
	 typeset -L25 db_installstatus="Installation Status"
     #typeset -L12 db_login="Login"
     #$NLS-Q$
	 typeset -L22 db_login="Login"
     typeset -L   dashes=$(List_line)        # fix width=a line

   dashes="--------------------------------------------------------------------------------------------------"

   print "$(List_line)\n$dashes"
   cd $CANDLECFG
   num=0
   for file; do
       num=$((num+1))
       . ./$file  || continue        # No listing if not valid script

       # Note, supporting archaic forms of db_installstatus in case
       # user doesn't have write authority to the config dir, and
       # Verify couldn't bring everything up to date.
       # (note, all vars have trailing blanks )

       case $db_installstatus in
         NOTCONFIG* | INCOMPLETE* )
                      #$NLS-Q$
					  db_installstatus="Incomplete"     ;;
         DBNOTEXIS* | EXCLUDED*  )
                      #$NLS-Q$
					  db_installstatus="Excluded"       ;;
         #$NLS-Q$
		 CONFIGSUC*)  db_installstatus="Fully Configured" ;;
        #$NLS-Q$
		7.1.4_NOT*)  db_installstatus="7.1.4 login must=sys" ;;
       esac
       List_line

       # remember install status for caller
       eval db_${db_sid%% *}=${db_installstatus%% *}   # only first word
   done
   List_count=$num       # callers want it
   print "$dashes"
}

# Replaced with a dnd dialog and functions Mod_login_dnd() and Mod_pass_dnd().

Get_login_info()  # Not used for dnd.
{
  # Get login and Password
  typeset server=
  test $# -eq 0 && server="servers" || server=$1
  login=`Get_login      $server`
  password=`Get_password $login`
}

# Replaced with a dialog and Mod_login_dnd.

Get_login()  # Not used for dnd. 
{
  #$NLS-E$ "Enter $Product connect-id for $1:    <default=candle>  "
  login=$(Read "`nls_replace 'KCI0093I' $Product \"$1\"`")
  print ${login:-candle}
}

# Replaced with a dialog and Mod_pass_dnd.

Get_password()  # Not used for dnd. 
{
  typeset sure=n
  until [ "$sure" = y ];do
     #$NLS-E$ "Enter $Product password for $1  "
	 password=$(Read "`nls_replace 'KCI0094I' $Product \"$1\"`")
     if [ "$password" != "" ]; then sure=y
     #$NLS-E$ "   Please confirm that you want a null password for $1 ?  (y/n) "
	 else sure=$(Read "`nls_replace 'KCI0095I' \"$1\"`")
          password=NONE
     fi
  done
  print $password
}

Get_dir()  # Not currently used in dnd.  TODO - implement. 
{
    # get a directory, confirm it, convert ~dir, etc to real dir
    # args are description text.  rc=99 if abort
    typeset dir sure=n text=$*
    until [ $sure = y ]; do
       #$NLS-E$ dir=`Read "     Enter $text or <CR> to return  "`
	   dir=$(Read "    `nls_replace 'KCI0096I' $text`  ")
       test $dir || return 99        # caller abort

       if [[ -d $dir ]]                   # real dir or relative
         then dir=`cd $dir; print $PWD`
              sure=y
              break
       fi

       dir2=`eval ls -d $dir`           # maybe a ~user dir
       if [[ -d $dir2 ]]
         then dir="$dir2"
              sure=y
         #$NLS-E$ "$dir is not a directory" 1>&2
		 else nls_replace 'KCI0097E' $dir 1>&2
              Beep
              #$NLS-E$ sure=`Read "Confirm using directory $dir ? (y/n)  "`
			  sure=$(Read "`nls_replace 'KCI0098I' $dir` [1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`]")
			  case $sure in
				1)
					sure=y ;;
				2)
					sure=n ;;
			  esac		
       fi
    done
    print $dir
}

Select()  # Not used for dnd.
{
   # menu selection.  Return text answer based on number
   # Note, Read() function defined in CandleDBconfig.
   typeset -i int=-1 n=$#
   until [[ $int -ge 0 && $int -le $n ]]
   do   # Grouch:  int=`Read ....` sets int=0 if user hits enter on all
        #          Unix except rarely on aix 3.   This is a code-around.
        #$NLS-E$ "     Please select an option or <CR> to exit: "`
		ans=$(Read "    `nls_replace 'KCI0099I'` ")
        test "$ans" = "" && int=0 || int=$ans   # complain if not int.
   done
   typeset choices
   set -A  choices return "$@" # if user executes result, 0=return
   print ${choices[$int]}
}

# Changed for use with dnd.

Inventory()  # OK for dnd. 
{
  # return an inventory of configured databases for prod
  # Following usages are mutually exclusive:
  # Usage Inventory -f  return list of non-excluded files for this host
  #       Inventory -a  return a list of all config files for this host
  #       Inventory     before/after  (excluded marked with *)
  #     Print a report of active databases for this prod, and
  #     export before or after to contain a list of servers.

  cd $CANDLECFG

  # Files for this host only.
    typeset files=$(ls $HOST\_$PR\_*.cfg 2>&-)

  case $1 in
     (-a) print $files
  ;; (-f) [[ -n $files ]] &&
          grep -l 'db_installstatus=".[^X]' $files
  ;; (*)  typeset server status inventory line
          grep -s db_installstatus $files junk |
          while IFS='_.:=" ' read x x server x x x x status x
          do  case $status in
                 (CONFIGSUCCESS) st=
              ;; (EXCLUDED)      st="(X)"
              ;; (*)             st="(I)"
              ;; esac
              inventory=$inventory${inventory:+ }$server$st
          done
          export $1="$inventory"
	  if [[ -z $inventory && $1 = after ]]
          then 
          #$NLS-E$ "No servers known after search."
	  nls_replace 'KCI0100I'
	  return 1
	  fi
	  if [[ -z $inventory && $1 = before ]]
	  then
          #$NLS-E$ "No servers known before search."
	  nls_replace 'KCI0115I'
	  return 1
	  fi
          if [[ -n $inventory && $1 = after ]]
	  then
          #$NLS-E$ "\
          #Inventory of known $Product servers after search:  X=Excluded I=Incomplete
	  nls_replace 'KCI0101I' $Product
          print $inventory
          fi
          if [[ -n $inventory && $1 = before ]]
	  then
          #$NLS-E$ "\
          #Inventory of known $Product servers before search:  X=Excluded I=Incomplete
	  nls_replace 'KCI0116I' $Product 
          print $inventory
          fi
  ;; esac

  # Write the contents of variable inventory to a file.  The contents of this file will
  # then be returned to dnd.

  # TODO - comment out or remove since these results are not needed.  

  typeset tfile

  #if [ "$1" = "before" ] 
  if [ "$1" = "1091" ]  # Skip for now.
  then
    tfile=$CANDLETMP/Inventory_before_dnd
    rm -f $tfile > /dev/null 2>&1
    #$NLS-E$ [ -f "$tfile" ] && Err_report fail "could not remove temp file \"$tfile\""
	[ -f "$tfile" ] && Err_report fail "`nls_replace 'KCI0102E' \\"$tfile\\"`"
    print $inventory > $tfile
  fi

  #if [ "$1" = "after" ]
  if [ "$1" = "1091" ]  # Skip for now.
  then
    tfile=$CANDLETMP/Inventory_after_dnd
    rm -f $tfile > /dev/null 2>&1
    #$NLS-E$ "could not remove temp file \\"$tfile\\""
	[ -f "$tfile" ] && Err_report fail "`nls_replace 'KCI0103E' \\"$tfile\\"`"
    print $inventory > $tfile
  fi
}

Find()  # OK for dnd. 
{
  # Usage: Find word in blank delimited list of words
  #   If so, return 0, else return 1.  Note, words must match exactly
  word=" $1 "
  shift 2   # swallow "$word in"
  list=" $* "
  [[ $list = *$word* ]]
}

MenuHeader()  # Not used for dnd. 
{
  typeset n
  getopts n n && shift   # -n means NoUnderlines
  Menu="$*"   # Not localized, used by Help
  typeset -L80 underscores=_____________________________________________________________________________________
  typeset date=$(date "+%m/%d/%y %H:%M")
  #$NLS-Q$
  typeset -L$((80-${#date})) head="Candle DB config: $Menu"
  print "\n$head$date"
  [[ $n = n ]] || print "$underscores"
  print
}

MenuFooter()  # Not used for dnd. 
{
  typeset msg="$*"
  typeset -L$((36-(${#msg})/2+1)) \
          underscores=__________________________________________
  print "$underscores <$msg> $underscores\n"
}

# Beep is not used in dnd.

Beep () { print '' >&2;  }

DBver()  # OK for dnd. 
{ # get version from dbhome
  case $PR in
       (or)
       if [[ ! -a $dbhome/orainst/unix.rgs ]] then
           set -- $(fgrep rdbms $dbhome/install/unix.rgs)
           home=${2#\"}      # 2th word, but in quotes
           print ${home%\"}
       else
           set -- $(fgrep ocommon $dbhome/orainst/unix.rgs)
           home=${5#\"}      # 5th word, but in quotes
           print ${home%\"}
       fi
    ;; (oy) # SQL Server/10.0.2/... from dataserver  output
        $dbhome/bin/dataserver -v | awk -F/ 'NR==1 { print $2; exit 0 }'
    ;; (ox)
        $dbhome/bin/onaudit -v | awk '/Version/{print $NF; exit 0}'
    #$NLS-E$ ;; (da) say "DB2 code not ready"
	;; (da) say "DB2 `nls_replace 'KCI0104I'`"
    ;;
  esac
}

# Function to normalize file names by removing leading irrelevant directories
# (like /tmp_mnt/).   Arg is var name.

PathNorm()  # OK for dnd. 
{
  typeset file short
  eval file=\$$1
  [[ $file = /* ]] || return   # accept only fully-qualified names

  while short=${file##/+([!/])}
        [[ $file -ef $short ]]
  do    file=$short
  done

  eval $1=$file
}

# Update should work "as is" with dnd.  However, the Version function has been changed
# to print ? until we determine what source code control system to use.

Update()  # OK for dnd. 
{
  # Update file toggle_status keyword=value...
  # env var Force=yes means even excluded servers
  # the Product function must have been called!!!

  # Variable-localization firewall:  like-named variables
  # in calling functions must not affect Update().
  # Tolerating pipedir, although support for pipedir has
  # been removed from the rest of this.

  test $Debug && print Update $*

  typeset dbhome login password server dbver reason \
          initfile file pipedir dbhost syspass toggle

  typeset   db_sid db_home db_initfilename db_errlog db_login  \
            db_password db_ver db_type db_hostname db_pipedir  \
            db_installstatus db_reason

  for Arg
  do  typeset -l arg=$Arg          # lower case
      case $arg in
           (*=*)   keyword=${arg%%=*}
                   eval $keyword=${Arg#*=}
        ;; (*.*)   file=$Arg
        ;; (toggle_status) toggle=yes
        ;; esac
  done

 # if no explicit hostname given anywhere, it's me
   true ${dbhost:=$(uname -n)}
   true ${file:=$dbhost\_$PR\_$server.cfg}

  # Read the existing config file if one exists.
  # Note this relies upon the file integrity check performed in Verify()

   cd $CANDLECFG
   # if file exists, (1) check executability (if it blows, CC>0)
   #                 (2) list all env vars it makes
   [[ -f $file ]] &&
      if vars=$(typeset +x `typeset +x`  #un-export all exported
                . ./$file ; typeset +x)  #list exported per file
      then

        # Tag unsupported env vars created by the script
        # IMPORTANT: any env var created by Update must be in the following
        # list, or it will be created twice

        typeset +t `typeset +t`   #un-tag all existing tagged vars
        for var in $vars
        do  if Find $var in LOGNAME \
                     COLL_SERVERID COLL_LOGONID COLL_PASSWORD \
                     COLL_VERSION  COLL_DBVERSION             \
                     ORACLE_SID ORACLE_HOME                   \
                     DSQUERY SYBASE                           \
                     INFORMIXSERVER INFORMIXDIR ONCONFIG
            then continue
            else typeset -t $var   # tag this env var
            fi
        done
        . ./$file  # now that we know it's safe

      else bad=${file%.*}.bad
           #$NLS-E$ print $file is not a valid shell script, renaming to $bad
		   nls_replace 'KCI0105I' $file $bad
           mv -f $file $bad
      fi


 # Apply manual/discovered parameters
   test $server      && db_sid=$server
   test $dbhome      && db_home=$dbhome
   test $initfile    && db_initfilename=$initfile
   test $dbhost      && db_hostname=$dbhost
   test $dbver       && db_ver=$dbver
   test $reason      && db_reason=$reason
   if [[ $Get_login = yes ]]
   then
   test $login       && db_login=$login
   typeset pass23="$db_login $db_sid"       # 2nd and 3rd parms for password call
   test $password    && db_password=$($ddpath/kddos key $password $pass23)
   else                 db_login=*OS
                        db_password=NONE
   fi
#  check the current password.  If it is 64 bytes then it needs to be re-encrypted
   [[ ${#db_password} = 64 ]] && db_password=$($ddpath/kddos key $db_password $pass23)

 # Normalize names
#  PathNorm db_home
   PathNorm db_initfilename

 # Always use internal product name
   db_type=k$PR

 # For batch:  apply syspass if appropriate
 if [[ $PR = or && $db_ver = 7.1.4* && -n $syspass ]]
 then db_login=sys
      db_password=$($ddpath/kddos key $syspass $pass23)
 fi

 # For sybase12
 db_home_org=$db_home
 db_home_ocs=$db_home

 if [[ $PR = oy && $db_ver = 12* ]]
 then db_home_org=${db_home%%/ASE*}
      db_home_ocs=${db_home_org}/OCS-12_0
 fi

 # N/A no longer supported for db_pipedir
   [[ $db_pipedir = N/A ]] && db_pipedir=

 # Determine config status.  Note, the only external dependency is that
 # CandleDBagent searches for db_installstatus="CONFIGSUCCESS" and
 # db_host="`uname -n`".

 # Otherwise, this script completely controls these definitions

 # CONFIGSUCCESS All required parms present.
 # INCOMPLETE    A required parm is missing.
 # 7.1.4_NOT_SYS Oracle 7.1.4 server defined for some login other than
 # EXCLUDED      User has requested that no agent run on this server,
 #               and future self-discovery searches should disregard it.
 # DBNOTEXIST    Archaic form of EXCLUDED.  Must honor if read.
 # NOTCONFIGURE  Archaic form of INCOMPLETE.  No code support needed.

 # First, eliminate archaic forms
 [[ $db_installstatus = DBNOTEXIST ]] && db_installstatus=EXCLUDED
 unset COLL_VERSION_TYPE

 if   [[ $toggle = yes && $db_installstatus != EXCLUDED ]]
 then db_installstatus=EXCLUDED
 elif [[ $toggle != yes && $db_installstatus = EXCLUDED && $Force != yes ]]
 then return # No parm update on excluded servers
 elif [[ $PR = or && $db_ver = 7.1.4* && $db_login != sys ]]
 then db_installstatus=7.1.4_NOT_SYS
 elif [[ 1245 = ${db_sid:+1}${db_home:+2}${db_login:+4}${db_ver:+5} ]]
 then db_installstatus=CONFIGSUCCESS    # kddexec.sh relies on this text
 else db_installstatus=INCOMPLETE
 fi

# Change for dnd - allow the user to remove the optional init filename when modifying a 
# server.  The original code did not support this type of change.  Note, ox is no longer
# a supported product.

[ "$db_initfilename" = "NONE" ] && db_initfilename=""
   
# Write the file.  Include code to write any new
# If you add a exported variable to this shell, add it to the
# supported list above.

# Note, '#" is unspecial to cat, So $var and $(cmd) substitution work OK
#       Version depends upon CTVC Header format
#       db_pipedir supported in replication only.  No support for Update pipedir=

  #$NLS-E$ Error creating $file, no change.
  cat >.$file <<! && mv -f .$file $file || nls_replace 'KCI0106E' $file
#  Candle Omegamon Monitoring Agent (OMA) for $Product
#  Configuration file: $file
#  Written by kddfind_dnd.sh version: $(set `Version`; print $3) on $(date "+%d%b%y %T")

#  Note: this is a Korn-shell script that is "sourced" to create the environment
#  for a Candle OMA.  It can be used to create the environment to run the
#  Candle-supplied database grant scripts.

#  Candle does not recommend that you modify this file, but you can change the
#  data values or add new exported variables as long as the file is a valid ksh
#  script that executes with zero return code, and values remain quoted with "".

#  Following environment variables are set for convience in running grants.
$(case $PR   in
   (or)      cat <<@
   export ORACLE_SID="$db_sid"
   export ORACLE_HOME="$db_home"
@
;; (oy)      cat <<@
   export DSQUERY="$db_sid"
   export SYBASE="$db_home_org"
   export SYBASE_OCS="$db_home_ocs"
@
;; (ox)      cat <<@
   export INFORMIXSERVER="$db_sid"
   export INFORMIXDIR="$db_home"
   export ONCONFIG="${db_initfilename##*/}"    # ONCONFIG must be a bare filename
@
;; esac)

#  Following variables are set for the setup script
   db_sid="$db_sid"
   db_home="$db_home"
   db_initfilename="$db_initfilename"
   db_login="$db_login"
   db_password="$db_password"
   db_ver="$db_ver"
   db_type="$db_type"
   db_hostname="$db_hostname"
   db_reason="$db_reason"
   db_pipedir="$db_pipedir"
   db_installstatus="$db_installstatus"

#  User-defined environment variables
$( for var in `typeset +t`  # as tagged above
   do  eval val=\$$var
       print "   export $var=\"$val\""
   done )
!
}

Help () {
  # I'm only called if kddhelp.sh hasn't yet defined the REAL Help function
  if   . $ddpath/kddhelp.sh
  then Help $*
  #$NLS-E$ "Help non-functional"
  else nls_replace 'KCI0107I'
  fi
}


Batch () { # Non-interactive driver.   Non-operational for now.
  test $Debug && say Batch "$*"
  # Hierarchy of args:
  # Product      required.
  # Only one of the following three:
  #   scan       Scan running jobs, /etc/oratab
  #   search=    Disk search from starting points.  wildcards OK,
  #              muliple starting points OK if quoted.
  #   modify=    Servers to modify, must be quoted if more than one.
  #              modify=* means all unexcluded servers for this host
  #              No other wildcarding allowed
  #
  #      keyword=value   dbhome initfile dbhost dbver pipedir login password
  #      syspass=        If Oracle 7.1.4, and syspass specified, then
  #                      login=sys, password=$syspass for that Oracle.
  #                      Allows you to configure all Oracle with a
  #                      single call.
  #
  #      toggle_status   Caution: use only with specific server.
  #
  #   Notes:
  #   If you specify any of dbhome, initfile or dbver with a search
  #   or scan call, those values override anything discovered.

  Batch=1
  Product $1             # Set Prod, Product, Prd.  See Product() for defs
  action=$2 shift 2
  Batch_parms=$@
  case $action in
       (scan)     Scan
    ;; (search=*) for search in ${action#search=}
                  do  Search $search
                  done
    ;; (modify=*) if [[ ${action#modify=} = * ]]
                  then servers=`Inventory -f`
                  else servers=${action#modify=}
                  fi
                  for server in $servers
                  do  Update server=${server%.*} $Batch_parms
                  done
    ;; esac
  List `Inventory -a`
}

say() { print "$*" >&2; }  # OK for dnd, I think.

Err_report()  # OK for dnd.
{
  case $1 in
    warn) print -u2 "$PROGNAME warning:  $2, continuing ..."
          sleep 2
          return
          ;;
    fail) print -u2 "$PROGNAME failure:  $2."
          exit 2
          ;;
    info) print -u2 "$PROGNAME :  $2."
          return
          ;;
    msgs) print -u2 "$PROGNAME :  $2"
          return
          ;;
  esac
}  

######################################################################
#                              Main                                  #
######################################################################

# WARNING - kddfind_dnd.sh only works when invoked from Distributor (a.k.a. dnd).

# NOTE - the verification of options and arguments is done in Distributor.  The options
# L, V, d and l (small "L") are carried over from the original kddfind.sh.

bindir=`pwd`/$(dirname $0)
[ "$(print $0 | grep '\./')" = "" ] && [ "$(print $0 | grep '^/.*')" = "" ] && [ `pwd` = '/' ] && bindir=`pwd`$(dirname $0)
[ "$(print $0 | grep '^/.*')" != "" ] && bindir=$(dirname $0)

export TEXTDOMAINDIR=$bindir/../config/nls/msg
export PATH=$bindir:$PATH

export PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)

typeset -i scanVal=0
typeset Debug Logfile PROD cfgFile ddpath logID pWord
typeset DOpt SArg SOpt aOpt eOpt mArg mOpt oArg oOpt vOpt 

# Option letters were chosen to correspond to the buttons in JDBConfigMain.java.  Option args
# are shown indented from the options and include required sub-options:
#
#   -s -> Search (a.k.a. Scan)
#       1 (one) -> search for running databases.
#       2 -> search for running and non-running databases.
#   -S -> Search II
#       1 (one) -> search PATH
#       2 -> list users
#       2 -> search, given a list of users.  The user names follow the product code.
#          billy joey moe
#       3 -> search, starting at a specific directory.  The directory follows the product code.
#          /users/billy
#   -o -> Login; small "L" and "L" were taken, so the second letter of "Login" was used.
#         config_filename|login_id -p password
#   -m -> Modify
#       1 (one) -> collect server data, and write it to <candletmp>/Col_server_data_dnd.txt
#         The config file name (e.g. puffer_or_0R901.cfg) follows the product code.
#       2 -> modify an existing server.  The args following the product code are:
#         cfgfile dbhome dbver initfile login password
#   -e -> Exclude; the config file name follows the product code.
#   -a -> Add; the args follow the product code:  
#         dbhome server dbver initfile login password
#   -v -> Verify OMA installation.
#   -D -> Done; erase the temp files.

# Variable visCmd contains the command string that is written to the "dd" log file.  The 
# actual command is not written since it may contain a password.

typeset visCmd=""

# Note, the checking of options and args is done in JDBConfigMain.java.

while getopts :DLS:Vadeh:k:lm:o:p:s:v opt 
do 
  case $opt in
    (D) DOpt=yes
        visCmd="$visCmd -D"
        ;;
    (L) Logfile=/dev/null
        visCmd="$visCmd -L"
        ;;
    (S) SOpt=yes
        SArg=$OPTARG
        visCmd="$visCmd -S $SArg"
        ;;
    (V) Verify=no
        visCmd="$visCmd -V"
        ;;
    (a) aOpt=yes
        visCmd="$visCmd -a"
        ;;
    (d) #Debug=debug  # Removed from dnd since it didn't do much.
        #visCmd="$visCmd -d"
        # read x?"-d option writes passwords to the log file! (enter)"
        ;;
    (e) eOpt=yes
        visCmd="$visCmd -e"
        ;;
    (h) CANDLEHOME=$OPTARG  # It's valid if it came from dnd.
        visCmd="$visCmd -h $CANDLEHOME"
        ;;
    (k) ddpath=$OPTARG 
        # ddpath is the full pathname (e.g. <candlehome>/<dd_arch>/bin ) to the dd 
        # (common database code) executables like kddos.
        visCmd="$visCmd -k $ddpath"
        ;;
    (l) New_log=yes
        visCmd="$visCmd -l"
        ;;
    (m) mOpt=yes
        mArg=$OPTARG
        visCmd="$visCmd -m $mArg"
        ;;
    (o) oOpt=yes
        oArg=$OPTARG  # oArg -> puffer_or_AUTO.cfg|candle
        visCmd="$visCmd -o $oArg"
        ;;
    (p) pWord=$OPTARG
        visCmd="$visCmd -p xxxxxx"  # Do not write out the password.
        ;;
    (s) let scanVal=$OPTARG  # scanVal must be an integer.
        visCmd="$visCmd -s $scanVal"
        ;;
    (v) vOpt=yes
        visCmd="$visCmd -v"
        ;;
    #$NLS-E$ "invalid option -$OPTARG"
	(*) Err_report fail "`nls_replace 'KCI0108E' -$OPTARG`"
        exit 1
        ;;
  esac
done

shift OPTIND-1
PROD=$1  # $1 can be either "or" or "oy."  ox, Informix, is no longer.

visCmd="$visCmd $PROD"

# Check kddos and its permissions.

#$NLS-E$ "\"$ddpath/kddos\" not found"
[ -f "$ddpath/kddos" ] || Err_report fail "`nls_replace 'KCI0109E' \\"$ddpath/kddos\\"`"
#$NLS-E$ "\"$ddpath/kddos\" root permission not set.  Please
#execute \"Set Agent Permissions\" from the main Distributor GUI, and re-run"
$ddpath/kddos auth || Err_report fail "`nls_replace 'KCI0110E' \\"$ddpath/kddos\\" \\"Set_Agent_Permissions\\"`"



# The following crud was taken from the CandleDBConfig script.  This script is
# not used to invoke kddfind_dnd.sh.  

# Set common environmental variables.

export CANDLEHOME
export HOST=$(uname -n)

set -a  # Auto-export the following:
  CANDLELOG=$CANDLEHOME/logs
  CANDLETBL=$CANDLEHOME/tables
  CANDLECFG=$CANDLEHOME/config
  CANDLEMISC=$CANDLEHOME/misc
  CANDLETMP=$CANDLEHOME/tmp
set +a

#unset CDPATH  # TODO - uncomment?

# Set additional environmental variables.

Config_File=$CANDLECFG/$PROD.config

if [[ -f $Config_File ]]
then 
  . $Config_File
else 
  #$NLS-E$ "config file \"$Config_File\" not found"
  Err_report fail "`nls_replace 'KCI0112E' \\"$Config_File\\"`"
fi

# End of crud from CandleDBConfig.

[ -z "$Logfile" ] && Logfile=$CANDLELOG/$HOST\_dd_config.log
[ "$New_log" = "yes" ] && rm -f $Logfile > /dev/null 2>&1

# Call Select_product.   Redirections:
# 1 stdout               piped to "tee"
# 2 stderr               redirected to stdout to pipe to tee
# 3 "real stderr"        preserved as file descriptor 3 so that more can get to it.
                        
# Update the log file.  

print "CandleDBConfig for $HOST $CANDLEHOME" >> $Logfile
date >> $Logfile

# Delete the temp files.

if [ "$DOpt" = "yes" ] 
then
  rm -f $CANDLETMP/*_dnd.txt > /dev/null 2>&1
  print $PROGNAME $visCmd >> $Logfile
  print >> $Logfile
  exit 0
fi

# Always call Select_product() so that variables are initialized and the initial scan is 
# performed.

Select_product $1

# Perform optional disk searches.  SOpt corresponds to the "Search II" button.

if [ "$SOpt" = "yes" ]
then
  [ "$SArg" = "1" ] && Path
  
  if [ "$SArg" = "2" ] 
  then
    if [ $# -le 1 ]  # It should always be 1 because of the product code.
    then
      Userlist  # Generate a list of users. 
    else
      shift 1  # Skip the product code.
      Userlist "$@"  # Search given a list of user names.
      visCmd="$visCmd $@"
    fi
  fi
  
  if [ "$SArg" = "3" ] 
  then
    shift 1  # Skip the product code.
    Search "$1"  # Search, starting at the provided directory.
    visCmd="$visCmd $1"
  fi
fi

# Add a database server.  aOpt corresponds to the "Add" button.

if [ "$aOpt" = "yes" ]
then
  shift 1  # Skip the product code.
  Add_manual_dnd "$@"  # visCmd is completed in the Add_manual_dnd function.
fi

# Exclude or include a server.

if [ "$eOpt" = "yes" ]
then
  Exclude_dnd $2
  visCmd="$visCmd $2"
fi

# Modify an existing server.  mOpt corresponds to the "Modify" button.

if [ "$mOpt" = "yes" ]
then
  shift 1  # Skip the product code
  #print "... cfgFile = $1"

  # Just write the data to <candletmp>/Col_server_data_dnd.txt.  
  [ "$mArg" = "1" ] && { Col_server_data_dnd $1; visCmd="$visCmd $1"; }
 
  if [ "$mArg" = "2" ]
  then
    Mod_home_dnd $1 $2
    Mod_ver $1 $3
    Mod_init $1 $4
    [ "$PROD" = "ox" ] || Mod_login_dnd $1 $5  # Skip if Informix (ox).
    [ "$PROD" = "ox" ] || Mod_pass_dnd $1 $6   # Skip if Informix (ox).
    visCmd="$visCmd $1 $2 $3 $4 $5 xxxxxx"  # Do not display the password.
  fi
fi

# Set the login ID and password.  oOpt corresponds to the "Login" button.
  
if [ "$oOpt" = "yes" ]
then
  cfgFile=$(print "$oArg" | cut -d"|" -f1)
  logID=$(print "$oArg" | cut -d"|" -f2)
  #print "... cfgFile = $cfgFile"
  #print "... logID = $logID"
  #print "... pWord = $pWord"
  
  Mod_login_dnd $cfgFile $logID $pWord
  Mod_pass_dnd $cfgFile $pWord

  # visCmd was completed in the getopts loop.
fi

# Verify the OMA installation.  vOpt corresponds to the "Verify" button.  IVP_dnd() writes
# out the file $CANDLETMP/IVP_dnd.txt.  This file contains the results of the verification,
# and these should be displayed in the "Messages" window of dnd.
  
if [ "$vOpt" = "yes" ]
then
  IVP_dnd $PROD  # Same as $1 in this case.
  # visCmd was completed just after the getopts loop.
fi

# Always call Display_dnd to update the server list, <candletmp>/Display_dnd.txt.  This
# list will then be retrieved by dnd.

Display_dnd

# Add the command string to the log file.

print $PROGNAME $visCmd >> $Logfile
print >> $Logfile

exit 0
