#!/bin/ksh
######################################################################
#                                                                    #
#             (C) COPYRIGHT Candle Corporation 1996, 2002            #
#                      All Rights Reserved                           #
#                                                                    #
#        Licensed Materials - Property of Candle Corporation         #
#         Duplication or disclosure prohibited without the           #
#        expressed written consent of the Candle Corporation.        #
#                                                                    #
######################################################################

# WARNING - kddhelp_dnd.sh is to be invoked from Distributor (a.k.a. dnd).

helpText()
{
tmpname1=`print $LANG | sed 's/^\(.....\).*/\1/'`
cat < $TEXTDOMAINDIR/$tmpname1/kddhelp_dnd_text_$tmpname1.txt
}

Err_report()
{
  case $1 in
    #$NLS-E$ "$PROGNAME warning:  $2, continuing ..."
    warn) print -u2 "$PROGNAME `nls_replace 'KCI0500I'`:  $2, `nls_replace 'KCI0501I'` ..."
          sleep 2
          return
          ;;
	#$NLS-E$ "$PROGNAME failure:  $2."	  
    fail) print -u2 "$PROGNAME `nls_replace 'KCI0502I'`:  $2."
          exit 2
          ;;
    info) print -u2 "$PROGNAME :  $2."
          return
          ;;
    msgs) print -u2 "$PROGNAME :  $2"
          return
          ;;
  esac
}  

######################################################################
#                              Main                                  #
######################################################################

# WARNING - kddhelp_dnd.sh is to be invoked from Distributor (a.k.a. dnd).

bindir=`pwd`/$(dirname $0)
[ "$(print $0 | grep '\./')" = "" ] && [ "$(print $0 | grep '^/.*')" = "" ] && [ `pwd` = '/' ] && bindir=`pwd`$(dirname $0)
[ "$(print $0 | grep '^/.*')" != "" ] && bindir=$(dirname $0)

export TEXTDOMAINDIR=$bindir/../config/nls/msg
export PATH=$bindir:$PATH

typeset CANDLEHOME HOST PROGNAME Prd Product fail home_name init_name
typeset Logfile New_log visCmd

export PATH=/usr/bin:$PATH
HOST=$(uname -n)
PROGNAME=$(basename $0)

while getopts :H:LP:h:i:lp: opt 
do 
  case $opt in
    (H) home_name=$OPTARG
        visCmd="$visCmd -H $home_name"
        ;;
    (L) Logfile=/dev/null
        visCmd="$visCmd -L"
        ;;
    (P) Product=$OPTARG
        visCmd="$visCmd -P $Product"
        ;;
    (h) CANDLEHOME=$OPTARG
        visCmd="$visCmd -h $CANDLEHOME"
        ;;
    (i) init_name=$OPTARG
        visCmd="$visCmd -i $init_name"
        ;;
    (l) New_log=yes
        visCmd="$visCmd -l"
        ;;
    (p) Prd=$OPTARG
        visCmd="$visCmd -p $Prd"
        ;;
    (*) fail=true
        ;;
  esac
done

# Check that all args are complete.  If not, create an empty Help_dnd.txt file.  Note that
# CANDLEHOME is a must have.

#$NLS-E$ "\"CANDLEHOME\" was not specified"
[ -n "$CANDLEHOME" ] || Err_report fail "`nls_replace 'KCI0503E' \\"CANDLEHOME\\"`"
#$NLS-E$ "cannot access directory \"$CANDLEHOME\""
[ -d "$CANDLEHOME" ] || Err_report fail "`nls_replace 'KCI0504E' \\"$CANDLEHOME\\"`"

[ -n "$Prd" ] || fail=true
[ -n "$Product" ] || fail=true
[ -n "$home_name" ] || fail=true
[ -n "$init_name" ] || fail=true

if [ -n "$fail" ]
then
  print > $CANDLEHOME/tmp/Help_dnd.txt
  #$NLS-E$ "could not generate the help text"
  Err_report fail "`nls_replace 'KCI0505E'`"
fi

[ -z "$Logfile" ] && Logfile=$CANDLEHOME/logs/$HOST\_dd_config.log
[ "$New_log" = "yes" ] && rm -f $Logfile > /dev/null 2>&1

# Update the log file.  

print "CandleDBConfig for $HOST $CANDLEHOME" >> $Logfile #$NON-NLS 1$
date >> $Logfile

# Generate the help text.  This file will then be retrieved and displayed by dnd.

helpText > $CANDLEHOME/tmp/Help_dnd.txt

# Add the command string to the log file.

print $PROGNAME $visCmd  >> $Logfile
print >> $Logfile

exit 0
