#!/bin/ksh
#
# kfwdb2loopback -- prepare a db2 loopback node for TEPS on AIX only
#
# set -x

KFW_DB2INST_ID=$1
KFW_DB2ADMIN_ID=$2
KFW_DB2_DBNAME=`print "$3" |tr "[:lower:]" "[:upper:]"`
KFW_DB2USER_ID=$4
KFW_DB2LOCALE=$5
# d52389 - Use unique log file in kfwdb2pre.sh for each db2admin id
KFW_DB2PREP_LOG=$(dirname $6)/kfwdb2prep_$2.log
KFW_DB2ADMIN_PW=$(grep adminPW  $7|cut -f2 -d"=")
KFW_DB2USER_PW=$(grep DB2PW  $7|cut -f2 -d"=")
KFW_DATE=`date`

# d52389 - if DB user can't write to log path, need an alt path
[ -w $(dirname ${KFW_DB2PREP_LOG}) ] || KFW_DB2PREP_LOG=/tmp/$(basename ${KFW_DB2PREP_LOG})

[ -n "$CANDLEHOME" ] || CANDLEHOME=$(cd $(dirname $0) ; dirname $(pwd) )

print "KFWDB2LOOPBACK started on " $KFW_DATE |tee -a ${KFW_DB2PREP_LOG}

# Process configuration variables for DB2 loopback node processing.
#
# Note:
# Sourcing the cq config variables would be the most convenient way to set the necessary variables, but is not done 
# because of problem causing the DB2 environment to be lost. Also, the $CANDLEHOME variable does not seem to be set.
# . $CANDLEHOME/config/cq.config
# 
# Instead the cq.config is read and the variable values are obtained from there.

KFW_LOOPBACK_NODE=`cat $CANDLEHOME/config/cq.config | grep KFW_LOOPBACK_NODE | cut -d"=" -f2`
KFW_LOOPBACK_DB=`cat $CANDLEHOME/config/cq.config | grep KFW_LOOPBACK_DB | cut -d"=" -f2`
KFW_LOCALHOST_LOOPBCK_IPADR=`cat $CANDLEHOME/config/cq.config | grep KFW_LOCALHOST_LOOPBCK_IPADR | cut -d"=" -f2`
KFW_DB2SERVER_PORT=`cat $CANDLEHOME/config/cq.config | grep KFW_DB2SERVER_PORT | cut -d"=" -f2`

# These would be good values to trace were there some global trace facility for install scripts
# print "KFW_LOOPBACK_NODE " $KFW_LOOPBACK_NODE |tee -a ${KFW_DB2PREP_LOG}
# print "KFW_LOOPBACK_DB " $KFW_LOOPBACK_DB |tee -a ${KFW_DB2PREP_LOG}
# print "KFW_LOCALHOST_LOOPBCK_IPADR " $KFW_LOCALHOST_LOOPBCK_IPADR |tee -a ${KFW_DB2PREP_LOG}
# print "KFW_DB2SERVER_PORT " $KFW_DB2SERVER_PORT |tee -a ${KFW_DB2PREP_LOG}

rm .pwd
cd -

if test ! ${KFW_DB2PREP_LOG}
then
    print "Missing parameter(s)" |tee -a ${KFW_DB2PREP_LOG}
    print "Usage:"|tee -a ${KFW_DB2PREP_LOG}
    print "kfwdb2loopback <instance name> <admin id> <database name> <user id> <db territory> <log file> <pwd file>" |tee -a ${KFW_DB2PREP_LOG}
    return 2;
fi

# d33090 - if DB user can't write to log path, need an alt path
[ -w $(dirname ${KFW_DB2PREP_LOG}) ] || KFW_DB2PREP_LOG=/tmp/$(basename ${KFW_DB2PREP_LOG})

export DB2INSTANCE=${KFW_DB2INST_ID}
export KFW_DB2_CLP=db2

# print "${KFW_DB2_CLP} ATTACH TO ${KFW_DB2INST_ID} USER ${KFW_DB2ADMIN_ID}" | tee -a  ${KFW_DB2PREP_LOG}

${KFW_DB2_CLP} ATTACH TO ${KFW_DB2INST_ID} USER ${KFW_DB2ADMIN_ID} USING ${KFW_DB2ADMIN_PW} 2>&1 >> ${KFW_DB2PREP_LOG}
if [ ! "$?" = "0" ]
then
    print "Failed to attach to the db2 instance" | tee -a  ${KFW_DB2PREP_LOG}
    print "Please check the followings:"| tee -a  ${KFW_DB2PREP_LOG}
    print "Check the existence of the ${KFW_DB2INST_ID} instance."| tee -a  ${KFW_DB2PREP_LOG}
    print "Check the instance is started and running."| tee -a  ${KFW_DB2PREP_LOG}
    print "Check the Admin User  ${KFW_DB2ADMIN_ID} is defined."| tee -a  ${KFW_DB2PREP_LOG}
    print "Check the Admin User password is correct."| tee -a  ${KFW_DB2PREP_LOG}
    return 3;
fi

# Print to the log existing DB2 node catalog entries
print "KFWDB2LOOPBACK List the Node Catalog entries." | tee -a  ${KFW_DB2PREP_LOG}
print "               If this is the first time configuring the loop back node, then the following message is valid:" | tee -a  ${KFW_DB2PREP_LOG}
print "               SQL1037W  The node directory is empty.  SQLSTATE=01606" | tee -a  ${KFW_DB2PREP_LOG}
print "" | tee -a  ${KFW_DB2PREP_LOG}
print "${KFW_DB2_CLP} LIST NODE DIRECTORY" | tee -a  ${KFW_DB2PREP_LOG}

${KFW_DB2_CLP} LIST NODE DIRECTORY >>${KFW_DB2PREP_LOG}

# When cataloging the loopback node the following messages are valid:
#   DB21056W Directory changes may not be effective until the directory cache is refreshed.
#   reason - The directory cache for KfwServices does not have to be refreshed since it should not be running.  It will pick up the
#            directory changes the next time it connects with the TEPS DB.
#   SQL1018N The node name TEPSlpbk specified in the CATALOG NODE command already exists.

print "KFWDB2LOOPBACK Configuring the loopback connection node ${KFW_LOOPBACK_NODE}" | tee -a  ${KFW_DB2PREP_LOG}
print "               The following message is valid:" | tee -a  ${KFW_DB2PREP_LOG}
print "               SQL1018N  The node name ${KFW_LOOPBACK_NODE} specified in the CATALOG NODE command already exists." | tee -a  ${KFW_DB2PREP_LOG}
print "" | tee -a  ${KFW_DB2PREP_LOG}

MYDB2OUTPUT=`${KFW_DB2_CLP} CATALOG tcpip node ${KFW_LOOPBACK_NODE} remote ${KFW_LOCALHOST_LOOPBCK_IPADR} server ${KFW_DB2SERVER_PORT}`
MYDB2RC=$?
print "KFWDB2LOOPBACK Catalog tcpip node return code " ${MYDB2RC} | tee -a ${KFW_DB2PREP_LOG}
print ${MYDB2OUTPUT} | tee -a ${KFW_DB2PREP_LOG}
if [ "${MYDB2RC}" = "0" -o "${MYDB2RC}" = "4" ] 
then
    print "KFWDB2LOOPBACK Cataloging loopback node successful" |tee -a  ${KFW_DB2PREP_LOG}
    print "" |tee -a  ${KFW_DB2PREP_LOG}
else
    print "KFWDB2LOOPBACK Error creating loopback node ${KFW_LOOPBACK_NODE}"  |tee -a  ${KFW_DB2PREP_LOG}
    return $MYDB2RC;
fi

# Print to the log existing DB2 db catalog entries
print "KFWDB2LOOPBACK List the Database Catalog entries." | tee -a  ${KFW_DB2PREP_LOG}

print "${KFW_DB2_CLP} LIST DB DIRECTORY" | tee -a  ${KFW_DB2PREP_LOG}

${KFW_DB2_CLP} LIST DB DIRECTORY>>${KFW_DB2PREP_LOG}

print "KFWDB2LOOPBACK Configuring the loopback database alias ${KFW_LOOPBACK_NODE}..."| tee -a  ${KFW_DB2PREP_LOG}
print "               The following message is valid:" | tee -a  ${KFW_DB2PREP_LOG}
print "               SQL1005N The database alias "TEPSlpdb" already exists in either the local database directory or system database directory." | tee -a  ${KFW_DB2PREP_LOG}
print "" |tee -a  ${KFW_DB2PREP_LOG}

# print "${KFW_DB2_CLP} CATALOG DB ${KFW_DB2_DBNAME} AS ${KFW_LOOPBACK_DB} AT NODE ${KFW_LOOPBACK_NODE}" |tee -a  ${KFW_DB2PREP_LOG}

MYDB2OUTPUT=`${KFW_DB2_CLP} CATALOG db ${KFW_DB2_DBNAME} AS ${KFW_LOOPBACK_DB} AT NODE ${KFW_LOOPBACK_NODE}`
MYDB2RC=$?
print "KFWDB2LOOPBACK Catalog loopback database return code " ${MYDB2RC} | tee -a ${KFW_DB2PREP_LOG}
print ${MYDB2OUTPUT} | tee -a ${KFW_DB2PREP_LOG}

if [ "${MYDB2RC}" = "0" -o "${MYDB2RC}" = "4" ]
then
    print "KFWDB2LOOPBACK Cataloging loopback database successful" |tee -a  ${KFW_DB2PREP_LOG}
    return 0;
fi

return $MYDB2RC

