#!/bin/ksh
#
# kfwdb2prep -- prepare a db2 instance for TEPS
#
# set -x

KFW_DB2INST_ID=$1
KFW_DB2ADMIN_ID=$2
KFW_DB2_DBNAME=`print "$3" |tr "[:lower:]" "[:upper:]"`
KFW_DB2USER_ID=$4
KFW_DB2LOCALE=$5
# d52389 - Use unique log file in kfwdb2pre.sh for each db2admin id
KFW_DB2PREP_LOG=$(dirname $6)/kfwdb2prep_$2.log
KFW_DB2ADMIN_PW=$(grep adminPW  $7|cut -f2 -d"=")
KFW_DB2USER_PW=$(grep DB2PW  $7|cut -f2 -d"=")
KFW_DATE=`date`

# d33090 - if DB user can't write to log path, need an alt path
[ -w $(dirname ${KFW_DB2PREP_LOG}) ] || KFW_DB2PREP_LOG=/tmp/$(basename ${KFW_DB2PREP_LOG})

print "KFWDB2PREP Starting on " $KFW_DATE |tee -a ${KFW_DB2PREP_LOG}

rm .pwd
cd -

if test ! ${KFW_DB2PREP_LOG}
then
    print "Missing parameter(s)" |tee -a ${KFW_DB2PREP_LOG}
    print "Usage:"|tee -a ${KFW_DB2PREP_LOG}
    print "kfwdb2prep <instance name> <admin id> <database name> <user id> <db territory> <log file> <pwd file>" |tee -a ${KFW_DB2PREP_LOG}
    exit 2;
fi

export DB2INSTANCE=${KFW_DB2INST_ID}
export KFW_DB2_CLP=db2

print "Attaching to DB2 instance ${KFW_DB2INST_ID}..." | tee -a  ${KFW_DB2PREP_LOG}
print "${KFW_DB2_CLP} ATTACH TO ${KFW_DB2INST_ID} USER ${KFW_DB2ADMIN_ID} USING [PASSWORD] 2>&1" | tee -a  ${KFW_DB2PREP_LOG}

tmpFile1=/tmp/$$-1.tmp
print "ATTACH TO ${KFW_DB2INST_ID} USER ${KFW_DB2ADMIN_ID} USING '${KFW_DB2ADMIN_PW}'" > $tmpFile1
${KFW_DB2_CLP} -f $tmpFile1 >> ${KFW_DB2PREP_LOG}

if [ ! "$?" = "0" ]
then
    rm -rf $tmpFile1
    print "Failed to attach to the db2 instance" | tee -a  ${KFW_DB2PREP_LOG}
    print "Please check the followings:"| tee -a  ${KFW_DB2PREP_LOG}
    print "Check the existence of the ${KFW_DB2INST_ID} instance."| tee -a  ${KFW_DB2PREP_LOG}
    print "Check the instance is started and running."| tee -a  ${KFW_DB2PREP_LOG}
    print "Check the Admin User  ${KFW_DB2ADMIN_ID} is defined."| tee -a  ${KFW_DB2PREP_LOG}
    print "Check the Admin User password is correct."| tee -a  ${KFW_DB2PREP_LOG}
    exit 3;
fi
rm -rf $tmpFile1

KFW_DB2_DBCREATED=N
print "Creating the database ${KFW_DB2_DBNAME}..." | tee -a  ${KFW_DB2PREP_LOG}
MYDB2OUTPUT=`${KFW_DB2_CLP} CREATE DATABASE ${KFW_DB2_DBNAME} USING CODESET UTF-8 TERRITORY ${KFW_DB2LOCALE} 2>>${KFW_DB2PREP_LOG}`
MYDB2RC=$?
print ${MYDB2OUTPUT}|tee -a ${KFW_DB2PREP_LOG}
if [ "${MYDB2RC}" = "0" ]
then
    KFW_DB2_DBCREATED=Y
else
	print ${MYDB2OUTPUT}|grep SQL1005N >/dev/null
	if [ "$?" = "0" ]
	then
    		KFW_DB2_DBCREATED=Y
    		print "Existing  ${KFW_DB2_DBNAME} database found."|tee -a  ${KFW_DB2PREP_LOG}
	fi
fi

if [ ! "$KFW_DB2_DBCREATED" = "Y" ]
then
    print "Failed to create database ${KFW_DB2_DBNAME}"| tee -a  ${KFW_DB2PREP_LOG}
    print "Please make sure the Admin User  ${KFW_DB2ADMIN_ID} has required rights.| tee -a  ${KFW_DB2PREP_LOG}"
    exit 4;
fi

print "Connecting to the database as ${KFW_DB2ADMIN_ID}..." | tee -a  ${KFW_DB2PREP_LOG}
tmpFile2=/tmp/$$-2.tmp
print "CONNECT TO ${KFW_DB2_DBNAME} USER  ${KFW_DB2ADMIN_ID} USING '${KFW_DB2ADMIN_PW}'" > $tmpFile2
${KFW_DB2_CLP} -f $tmpFile2 >> ${KFW_DB2PREP_LOG}
if [ ! "$?" = "0" ]
then
    rm -rf $tmpFile2
    print "Failed to connect database ${KFW_DB2_DBNAME}"| tee -a  ${KFW_DB2PREP_LOG}
    exit 5;
fi
rm -rf $tmpFile2

if [ ! "$KFW_DB2INST_ID" = "$KFW_DB2USER_ID" ]
then
    print "Granting rights to ${KFW_DB2USER_ID}..." | tee -a  ${KFW_DB2PREP_LOG}
    print ${KFW_DB2_CLP} ${KFW_DB2USER_ID} | tee -a  ${KFW_DB2PREP_LOG} 
    ${KFW_DB2_CLP} GRANT DBADM,CREATETAB,BINDADD,CONNECT,CREATE_NOT_FENCED_ROUTINE,IMPLICIT_SCHEMA,LOAD,CREATE_EXTERNAL_ROUTINE,QUIESCE_CONNECT ON DATABASE  TO USER ${KFW_DB2USER_ID} 2>&1 >> ${KFW_DB2PREP_LOG}
     if [ ! "$?" = "0" ]
     then
         print "Failed to grant rights to ${KFW_DB2USER_ID}"
         exit 6;
     fi
fi

print "Resetting connection..." | tee -a  ${KFW_DB2PREP_LOG}
${KFW_DB2_CLP} CONNECT RESET 2>&1 >> ${KFW_DB2PREP_LOG}

print "Testing Connection to ${KFW_DB2_DBNAME} with ${KFW_DB2USER_ID} UserID..." | tee -a  ${KFW_DB2PREP_LOG}
tmpFile3=/tmp/$$-3.tmp
print "CONNECT TO ${KFW_DB2_DBNAME} USER  ${KFW_DB2USER_ID} USING '${KFW_DB2USER_PW}'" > $tmpFile3
${KFW_DB2_CLP} -f $tmpFile3 >> ${KFW_DB2PREP_LOG}
if [ ! "$?" = "0" ]
then
    rm -rf $tmpFile3
    print "Failed to connect database ${KFW_DB2_DBNAME} with ${KFW_DB2USER_ID}."
    print "Connection test failed."
    exit 8;
fi
rm -rf $tmpFile3

print "KFWDB2PREP done" | tee -a  ${KFW_DB2PREP_LOG}
exit 0;
