#!/bin/ksh
# ****************************************************************************
# * Licensed Materials - Property of IBM
# *
# * OCO Source Materials - IBM Confidential
# * 5724-C04
# *
# * Copyright IBM Corporation 2009-2013 All Rights Reserved
# *
# * The source code for this program is not published or other-
# * wise divested of its trade secrets, irrespective of what has
# * been deposited with the U.S. Copyright Office.
# *****************************************************************************

#
# MAIN
#

if [ $# -eq 2 ]
then
  localSeeding=yes
  temsName=$1
  sqlFile=$2
else
  temsName=$1
  temsAddress=$2
  temsPort=$3
  temsProtocol=$4
  sqlFile=$5
fi


bindir=`pwd`/$(dirname $0)
[ "$(print $0 | grep '\./')" = "" ] && [ "$(print $0 | grep '^/.*')" = "" ] && [ `pwd` = '/' ] && bindir=`pwd`$(dirname $0)
[ "$(print $0 | grep '^/.*')" != "" ] && bindir=$(dirname $0)

export TEXTDOMAINDIR=$bindir/../config/nls/msg
export PATH=$bindir:$PATH

export PATH=/usr/bin:$PATH

[ -n "$CANDLEHOME" ] || CANDLEHOME=$(cd $(dirname $0) ; dirname $(pwd) )
export CANDLEHOME

CurRegPath=$CANDLEHOME/registry
ArchTbl=$CurRegPath/archdsc.tbl
DynShl=$CANDLEHOME/bin/dynarch.shl
JavaHome=`$CANDLEHOME/bin/CandleGetJavaHome`

. $DynShl $ArchTbl 2>/dev/null

baseArch=${cArch%%[0-9]*}

if [ -n "$localSeeding" ]
then
  #local system hub and rtems seeding
  PRODUCT=ms
  setBinLibArchs $cArch $PRODUCT

  msConfig=""
  hname=`hostname | cut -d'.' -f1`
  if [ -f $CANDLEHOME/config/${hname}_ms_${temsName}.config ]
  then
    msConfig=$CANDLEHOME/config/${hname}_ms_${temsName}.config
  fi
  if [ -z "$msConfig" ]
  then
    configTest=`ls -1 $CANDLEHOME/config/*_ms_$temsName.config 2> /dev/null`
    configCount=`print "$configTest" | wc -l`
    if [ $configCount -eq 1 ] && [ -f "configTest" ]
    then
      msConfig=$configTest
    fi
  fi
  if [ -z "$msConfig" ]
  then
    msConfig=$CANDLEHOME/config/*_ms_$temsName.config
  fi

  set -a
  . $msConfig
  if [[ "$KDC_FAMILIES" = @(*HTTP_SERVER*) ]]
  then
    KDC_FAMILIES=$(print $KDC_FAMILIES  | sed 's/HTTP_SERVER:Y/HTTP_SERVER:N/g')
  else
    KDC_FAMILIES="$KDC_FAMILIES HTTP_SERVER:N"
  fi
  set +a

## Required order of entries for kdstsns response file
## empty line at top to make kdstsns discover itself connection settings to TEMS
## Base name of sql file to be seeded
## "end"

  cd $CANDLEHOME/tables/$temsName/
  responseFile=$CANDLEHOME/tables/$temsName/seed.rsp 
  
  print ""	 > $responseFile
  print "$sqlFile"	>> $responseFile
  print "end"		>> $responseFile

  $CANDLEHOME/$binArch/$PRODUCT/bin/kdstsns < $responseFile
  rc=$?

  rm -f $responseFile 2> /dev/null

else
  #remote system seeding

  export KBB_RAS1='ERROR'

  PRODUCT=ue
  ueInstalled=$(ls $CANDLEHOME/registry/$PRODUCT$baseArch*.ver 2>/dev/null)
  if [ -n "$ueInstalled" ]
  then
    setBinLibArchs $cArch $PRODUCT
    [ -f "$CANDLEHOME/$binArch/$PRODUCT/bin/kdstsns" ] && ALLOWSEEDING=true
  fi

  if [ -z "$ALLOWSEEDING" ]
  then
    PRODUCT=ms
    msInstalled=$(ls $CANDLEHOME/registry/$PRODUCT$baseArch*.ver 2>/dev/null)
    if [ -n "$msInstalled" ]
    then
      setBinLibArchs $cArch $PRODUCT
      ALLOWSEEDING=true
    fi
  fi

  if [ -z "$ALLOWSEEDING" ]
  then
    #check if pre 622 ui is installed
    PRODUCT=ui
    uiInstalled=$(ls $CANDLEHOME/registry/$PRODUCT$baseArch*.ver 2>/dev/null)
    if [ -n "$uiInstalled" ]
    then
      setBinLibArchs $cArch ui
      uiver=$(grep "ver =" $CurRegPath/ui${binArch}.ver 2>/dev/null|head -1|cut -d" " -f3)
      if [ "$uiver" -lt "622" ]
      then
        [ -f "$CANDLEHOME/$binArch/$PRODUCT/bin/kdstsns" ] && ALLOWSEEDING=true
      fi
    fi
  fi

  if [ -z "$ALLOWSEEDING" ]
  then
    print "Seeding tool not found!"
    exit 1
  fi

  export SQLLIB=$CANDLEHOME/tables/cicatrsq/SQLLIB
  export KDC_FAMILIES="$temsProtocol PORT:$temsPort HTTP_SERVER:N"

  glbFile=$CANDLEHOME/tmp/$$_glb_site.txt
  export KDC_GLBSITES=$glbFile

  USE_64BIT="N"
  [[ "$binArch" = @(*6) ]] && USE_64BIT="Y"
  gskitLoc=$CANDLEHOME/config/gsKit.config

  if [ "$USE_64BIT" = "Y" ]
  then
    ICCRTE_DIR=$(grep GskitInstallDir_64= $gskitLoc 2>/dev/null | cut -d"="  -f2)
    ICCRTE_DIR_LIB=$ICCRTE_DIR/lib64
  else
    ICCRTE_DIR=$(grep GskitInstallDir= $gskitLoc 2>/dev/null | cut -d"=" -f2)
    ICCRTE_DIR_LIB=$ICCRTE_DIR/lib
  fi
  export ICCRTE_DIR

  export LD_LIBRARY_PATH=$ICCRTE_DIR_LIB:$CANDLEHOME/$binArch/$PRODUCT/lib:$CANDLEHOME/tmaitm6/$binArch/lib:$LD_LIBRARY_PATH
  export LIBPATH=$ICCRTE_DIR_LIB:$CANDLEHOME/$binArch/$PRODUCT/lib:$CANDLEHOME/tmaitm6/$binArch/lib:$LIBPATH
  export SHLIB_PATH=$ICCRTE_DIR_LIB:$CANDLEHOME/$binArch/$PRODUCT/lib:$CANDLEHOME/tmaitm6/$binArch/lib:$SHLIB_PATH

  export KEYFILE_DIR=$CANDLEHOME/keyfiles
  export KDEBE_KEYRING_FILE=$KEYFILE_DIR/keyfile.kdb
  export KDEBE_KEYRING_STASH=$KEYFILE_DIR/keyfile.sth
  export KDEBE_KEY_LABEL=IBM_Tivoli_Monitoring_Certificate

  print "$temsProtocol$temsAddress" > $glbFile
  $CANDLEHOME/$binArch/$PRODUCT/bin/kdstsns $sqlFile $temsName
  rc=$?

  rm -f $glbFile 2>/dev/null
fi

exit $rc
