#!/bin/ksh

# INFO
#
# This script very simply accepts input from the calling script,
# prints out some variables to check that its executing properly,
# then calls a static class in a static jar for the ITCAM RTT team.
#
# How this is being called
# 	/data/css/installs/FP4_GA-CH/bin/uninstall-process.sh JAVA_HOME CANDLE_HOME PRODUCT_CODE LOCATION BINARCH TYPE
# 	/data/css/installs/FP4_GA-CH/bin/uninstall-process.sh JAVA_HOME /data/css/installs/FP4_GA-CH ul preuninstall li6243 uninstall


Err_report()
{
  #set -x
  case $1 in
    fail) print -u2 "UNINSTALL-PROCESS: exiting:  $2."
          exit
          ;;
    logexit) print "$2" >> $CANDLEHOME/logs/uninstall-process.trc
          exit
          ;;
  esac
}


# STATIC VARIABLES

#The values for each of the RT 6.2 product codes 
UNINSTALL_RTT_JAR_FILE=kt_callpoint.jar

UNINSTALL_T1_JAR_CLASS=com.ibm.tivoli.itcam.ft.FileTransferConfig
UNINSTALL_T3_JAR_CLASS=com.ibm.tivoli.itcam.dashboard.DashboardConfig
UNINSTALL_T4_JAR_CLASS=com.ibm.tivoli.itcam.cat.CATConfig
UNINSTALL_T5_JAR_CLASS=com.ibm.tivoli.itcam.wrm.WRMInstall
UNINSTALL_T6_JAR_CLASS=com.ibm.tivoli.itcam.robot.RobotConfig

#This is the list of valid codes that we will execute this script for
UNINSTALL_PRODUCT_CODE_LIST="t1 t3 t4 t5 t6"

touch $CANDLEHOME/logs/uninstall-process.trc
print "" >> $CANDLEHOME/logs/uninstall-process.trc
print "" >> $CANDLEHOME/logs/uninstall-process.trc
date >> $CANDLEHOME/logs/uninstall-process.trc
print "Executing uninstall-process.sh" >> $CANDLEHOME/logs/uninstall-process.trc

PROGNAME=$(basename $0)

CURRENT_DIR=`pwd`

print "ARG 0: $0" >> $CANDLEHOME/logs/uninstall-process.trc
print "ARG 1: $1" >> $CANDLEHOME/logs/uninstall-process.trc
print "ARG 2: $2" >> $CANDLEHOME/logs/uninstall-process.trc
print "ARG 3: $3" >> $CANDLEHOME/logs/uninstall-process.trc
print "ARG 4: $4" >> $CANDLEHOME/logs/uninstall-process.trc
print "ARG 5: $5" >> $CANDLEHOME/logs/uninstall-process.trc

#ARGS ARE IN THIS ORDER:
# #1 - UNINSTALL_JAVA_HOME
# #2 - CANDLEHOME
# #3 - UNINSTALL_PRODUCT_CODE
# #4 - UNINSTALL_LOCATION - preuninstall
# #5 - UNINSTALL_BIN_ARCH - li6243
# #6 - UNINSTALL_TYPE - uninstall

UNINSTALL_JAVA_HOME=$1
CANDLEHOME=$2
UNINSTALL_PRODUCT_CODE=$3
UNINSTALL_LOCATION=$4
UNINSTALL_BIN_ARCH=$5
UNINSTALL_TYPE=$6
FORCE=$7
RESULT=0

# Check YN/YJ DC configuration status. 
if [[ "$UNINSTALL_PRODUCT_CODE" = "yn" || "$UNINSTALL_PRODUCT_CODE" = "yj" ]] 
then

  UNINSTALL_WR_JAR_FILE=DCConfigurationCheck.jar
  if [ ! -f $CANDLEHOME/$UNINSTALL_BIN_ARCH/$UNINSTALL_PRODUCT_CODE/lib/$UNINSTALL_WR_JAR_FILE ]
  then  
	echo "$CANDLEHOME/$UNINSTALL_BIN_ARCH/$UNINSTALL_PRODUCT_CODE/lib/$UNINSTALL_WR_JAR_FILE" 
    exit 0;
  fi  
  
  UNINSTALL_WR_JAR_CLASS=com.ibm.tivoli.itcam.install.common.plugin.DCConfigurationCheck
  UNINSTALL_JAVA_HOME=`$CANDLEHOME/bin/CandleGetJavaHome`

  UNINSTALL_ARCH=$UNINSTALL_BIN_ARCH
  JREEXE=$UNINSTALL_JAVA_HOME/bin/java

  UNINSTALL_CLASSPATH="$CANDLEHOME/classes/install_common.jar:$CANDLEHOME/classes/jlog.jar:$CANDLEHOME/classes/InstallRAS.jar:$CANDLEHOME/$UNINSTALL_BIN_ARCH/$UNINSTALL_PRODUCT_CODE/lib/$UNINSTALL_WR_JAR_FILE:$CANDLEHOME/config/nls/ras"
  UNINSTALL_OPTION="-DInstallRASConfig="$CANDLEHOME/config/ITMConfigRAS.properties""
  
  print "$JREEXE -classpath $UNINSTALL_CLASSPATH $UNINSTALL_OPTION $UNINSTALL_WR_JAR_CLASS $CANDLEHOME $UNINSTALL_ARCH $UNINSTALL_PRODUCT_CODE $FORCE" >> $CANDLEHOME/logs/uninstall-process.trc   
  $JREEXE -classpath $UNINSTALL_CLASSPATH $UNINSTALL_OPTION $UNINSTALL_WR_JAR_CLASS $CANDLEHOME $UNINSTALL_ARCH $UNINSTALL_PRODUCT_CODE $FORCE 
  typeset RESULT=$?
  exit $RESULT;
fi
#uninstall KMC TEMS app support
if [ "$UNINSTALL_PRODUCT_CODE" = "cf" ] 
then

  #get ms architecture code by parsing ms ver file
  CurRegPath=$CANDLEHOME/registry
  ArchTbl=$CurRegPath/archdsc.tbl 
  DynShl=$CANDLEHOME/bin/dynarch.shl
  . $DynShl $ArchTbl

  baseArch=${cArch%%[0-9]*} 
  msInstalled=$(ls $CANDLEHOME/registry/ms$baseArch*.ver 2>/dev/null)
  [ -z "$msInstalled" ] && exit 0;

  setBinLibArchs $cArch ms

  UNINSTALL_KCF_JAR_FILE=msguninstall.jar
  if [ ! -f $CANDLEHOME/$binArch/ms/lib/$UNINSTALL_KCF_JAR_FILE ]
  then  
    exit 0;
  fi
  UNINSTALL_KCF_JAR_CLASS=com.ibm.tivoli.messaging.install.plugin.MsgUninstall
  UNINSTALL_JAVA_HOME=`$CANDLEHOME/bin/CandleGetJavaHome`

  UNINSTALL_MS_ARCH=$binArch
  JREEXE=$UNINSTALL_JAVA_HOME/bin/java

  UNINSTALL_CLASSPATH="$CANDLEHOME/classes/cic.jar:$CANDLEHOME/classes/install_common.jar:$CANDLEHOME/classes/jlog.jar:$CANDLEHOME/classes/InstallRAS.jar:$CANDLEHOME/$binArch/ms/lib/$UNINSTALL_KCF_JAR_FILE:$CANDLEHOME/config/nls/ras"
  UNINSTALL_OPTION="-DInstallRASConfig="$CANDLEHOME/config/ITMConfigRAS.properties""

  print "$JREEXE -classpath $UNINSTALL_CLASSPATH $UNINSTALL_OPTION $UNINSTALL_KCF_JAR_CLASS $CANDLEHOME $UNINSTALL_MS_ARCH $UNINSTALL_PRODUCT_CODE $UNINSTALL_BIN_ARCH" >> $CANDLEHOME/logs/uninstall-process.trc   
  $JREEXE -classpath $UNINSTALL_CLASSPATH $UNINSTALL_OPTION $UNINSTALL_KCF_JAR_CLASS $CANDLEHOME $UNINSTALL_MS_ARCH $UNINSTALL_PRODUCT_CODE $UNINSTALL_BIN_ARCH 
  typeset RESULT=$?
  exit $RESULT;
fi
# This sets $JAVA_HOME and $CLASSPATH
UNINSTALL_JAVA_HOME=`$CANDLEHOME/bin/CandleGetJavaHome`

print "UNINSTALL-PROCESS: Executing uninstall-process.sh" >> $CANDLEHOME/logs/uninstall-process.trc
print "UNINSTALL-PROCESS: PROGNAME: $PROGNAME" >> $CANDLEHOME/logs/uninstall-process.trc
print "UNINSTALL-PROCESS: CURRENT_DIR: $CURRENT_DIR" >> $CANDLEHOME/logs/uninstall-process.trc
print "UNINSTALL-PROCESS: UNINSTALL_RTT_JAR_FILE: $UNINSTALL_RTT_JAR_FILE" >> $CANDLEHOME/logs/uninstall-process.trc
print "UNINSTALL-PROCESS: UNINSTALL_RTT_JAR_CLASS: $UNINSTALL_RTT_JAR_CLASS" >> $CANDLEHOME/logs/uninstall-process.trc
print "" >> $CANDLEHOME/logs/uninstall-process.trc
print "UNINSTALL-PROCESS: ARGS:" >> $CANDLEHOME/logs/uninstall-process.trc
print "UNINSTALL-PROCESS: 	UNINSTALL_JAVA_HOME: $UNINSTALL_JAVA_HOME" >> $CANDLEHOME/logs/uninstall-process.trc
print "UNINSTALL-PROCESS: 	CANDLEHOME: $CANDLEHOME" >> $CANDLEHOME/logs/uninstall-process.trc
print "UNINSTALL-PROCESS: 	UNINSTALL_PRODUCT_CODE: $UNINSTALL_PRODUCT_CODE" >> $CANDLEHOME/logs/uninstall-process.trc
print "UNINSTALL-PROCESS: 	UNINSTALL_LOCATION: $UNINSTALL_LOCATION" >> $CANDLEHOME/logs/uninstall-process.trc
print "UNINSTALL-PROCESS: 	UNINSTALL_BIN_ARCH: $UNINSTALL_BIN_ARCH" >> $CANDLEHOME/logs/uninstall-process.trc
print "UNINSTALL-PROCESS: 	UNINSTALL_TYPE: $UNINSTALL_TYPE" >> $CANDLEHOME/logs/uninstall-process.trc

# Double check that we have the product code we want

print "UNINSTALL-PROCESS: Product codes we want:" >> $CANDLEHOME/logs/uninstall-process.trc
print "	$UNINSTALL_PRODUCT_CODE_LIST" | grep $UNINSTALL_PRODUCT_CODE >> $CANDLEHOME/logs/uninstall-process.trc

[ $? -eq 0 ] || Err_report logexit "Not a product code we want [$UNINSTALL_PRODUCT_CODE_LIST] - returning"

print "UNINSTALL-PROCESS: We have the product code we want - Continuing" >> $CANDLEHOME/logs/uninstall-process.trc


# Make sure we get the right version of the jar for our code.
case $UNINSTALL_PRODUCT_CODE in
	    "t1")
                UNINSTALL_RTT_JAR_CLASS=$UNINSTALL_T1_JAR_CLASS
		;;
            "t3")
                UNINSTALL_RTT_JAR_CLASS=$UNINSTALL_T3_JAR_CLASS
		;;
	    "t4")
                UNINSTALL_RTT_JAR_CLASS=$UNINSTALL_T4_JAR_CLASS
		;;
	    "t5")
                UNINSTALL_RTT_JAR_CLASS=$UNINSTALL_T5_JAR_CLASS
		;;
	    "t6")
                UNINSTALL_RTT_JAR_CLASS=$UNINSTALL_T6_JAR_CLASS
		;;
	esac

print "UNINSTALL-PROCESS: UNINSTALL_RTT_JAR_FILE: $UNINSTALL_RTT_JAR_FILE" >> $CANDLEHOME/logs/uninstall-process.trc
print "UNINSTALL-PROCESS: UNINSTALL_RTT_JAR_CLASS: $UNINSTALL_RTT_JAR_CLASS" >> $CANDLEHOME/logs/uninstall-process.trc
print "" >> $CANDLEHOME/logs/uninstall-process.trc


# We have everything we need ready at this point. Call the java class and

JREEXE=$UNINSTALL_JAVA_HOME/bin/java
UNINSTALL_CLASSPATH="$CANDLEHOME/classes/install_common.jar:$CANDLEHOME/$UNINSTALL_BIN_ARCH/$UNINSTALL_PRODUCT_CODE/lib/$UNINSTALL_RTT_JAR_FILE"

print "UNINSTALL-PROCESS: Executing java call" >> $CANDLEHOME/logs/uninstall-process.trc

print "UNINSTALL-PROCESS: JREEXE: $JREEXE" >> $CANDLEHOME/logs/uninstall-process.trc
print "UNINSTALL-PROCESS: UNINSTALL_CLASSPATH: $UNINSTALL_CLASSPATH" >> $CANDLEHOME/logs/uninstall-process.trc

print "JAVA CALL: $JREEXE -classpath $UNINSTALL_CLASSPATH $UNINSTALL_RTT_JAR_CLASS $CANDLEHOME $UNINSTALL_PRODUCT_CODE $UNINSTALL_LOCATION $UNINSTALL_BIN_ARCH $UNINSTALL_TYPE" >> $CANDLEHOME/logs/uninstall-process.trc

$JREEXE -classpath $UNINSTALL_CLASSPATH $UNINSTALL_RTT_JAR_CLASS $CANDLEHOME $UNINSTALL_PRODUCT_CODE $UNINSTALL_LOCATION $UNINSTALL_BIN_ARCH $UNINSTALL_TYPE

print "UNINSTALL-PROCESS: " >> $CANDLEHOME/logs/uninstall-process.trc

print "UNINSTALL-PROCESS: Exiting normally" >> $CANDLEHOME/logs/uninstall-process.trc

exit $RESULT
