#!/bin/ksh
# **********************************************************************
# * Licensed Materials - Property of IBM
# * 5724-C04
# * Copyright IBM Corporation 2007-2015 All Rights Reserved
# **********************************************************************

JRE_DEBUG_ARGS=
#JRE_DEBUG_ARGS="-Xdebug -Xnoagent -Djava.compiler=NONE -Xrunjdwp:transport=dt_socket,server=y,address=8001,suspend=y"

function Err_report { #-------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  case $1 in
    #$NLS-E$ "$PROGNAME warning:  $2, continuing ..."
    warn) print -u2 "$PROGNAME `nls_replace 'KCI0535I'`:  $2, `nls_replace 'KCI0536I'` ..."
          sleep 2
          return
          ;;
	#$NLS-E$ "$PROGNAME failure:  $2."
    fail) print -u2 "$PROGNAME `nls_replace 'KCI0537I'`:  $2"           #IV67523
          cleanUp 2
          ;;
    info) [ -n "$VERBOSE" ] && print -u2 "$PROGNAME        :  $2"       #IV67523
          return
          ;;
    msgs) print -u2 "$PROGNAME        :  $2"
          return
          ;;
  esac
}

function Use_report { #-------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  print -u2 "$PROGNAME [ -h ITM home ] -v"
  #$NLS-E$ "  Displays the version and release levels of the current installer."
  print -u2 "`nls_replace 'KCI1000I'`"
  print -u2 "$PROGNAME [ -h ITM home ] [ -d PRODUCT_PATH ] [ -c ] [ -p PARAMETER_FILE ] [ -k GROUP ] [ -s ]"
  #$NLS-E$ "  Specify -c to print diagnostic messages to the console."
  print -u2 "`nls_replace 'KCI1001I' -c`"
  #$NLS-E$ "  Specify -d to indicate the location of the product CD."
  print -u2 "`nls_replace 'KCI1002I' -d`"
  #$NLS-E$ "  Specify -h to indicate the location of the CandleHome."
  print -u2 "`nls_replace 'KCI1003I' -h`"
  #$NLS-E$ "  Specify -j to indicate the JRE installed location. This is available on Tandem system only."
  print -u2 "`nls_replace 'KCI1004I' -j`"
  #$NLS-E$ "  Specify -p for silent mode execution using the specified PARAMETER_FILE."
  print -u2 "`nls_replace 'KCI1005I' -p PARAMETER_FILE`"
  #$NLS-E$ "  Specify -k to secure installation."
  print -u2 "`nls_replace 'KCI1340I' -k`"
  #$NLS-E$ "  Specify -s to skip startup scripts update. The startup scripts are not modified with this option. This is available during a pristine installation only."
  print -u2 "`nls_replace 'KCI1364I' -s`"
  exit 2
}

function locateCMD { #--------------------------------------------------#IV51017
   # Find the directory where the executable ${1} resides so we can     #IZ94626
   # invoke it directly.                                                #IZ94626
   #--------------------------------------------------------------------#IZ94626
   eval $(itmsh_trace_func "${0}" "${@}");                              #IV51017
                                                                        #IV51017
   typeset outCMD="";                                                   #IZ94626
   if [ ! -z "${1}" ]; then                                             #IZ94626
      if [ "${thisOS}" = "HP-UX" ] || [ "${thisOS}" = "SunOS" ]; then   #IZ94626
         outCMD=`which ${1} 2> /dev/null | grep -v "^no ${1}"`;         #IZ94626
      else                                                              #IZ94626
         outCMD=`which ${1} 2> /dev/null`;                              #IZ94626
      fi;                                                               #IZ94626
      if [ -z "${outCMD}" ]; then                                       #IZ94626
         outCMD=`find /bin /usr/bin /local/bin /usr/local/bin /sbin /usr/sbin /local/sbin /usr/local/sbin -name ${1} 2> /dev/null | head -n 1`; #IZ94626
      fi;                                                               #IZ94626
   fi;                                                                  #IZ94626
   print "${outCMD}";                                                   #IZ94626
}                                                                       #IZ94626
                                                                        #IZ94626
function run_LAPTool { #------------------------------------------------#IV51017
  # Run the License Acceptance Process (LAP) tool to get the user's     #IV51017
  # agreement to the license.                                           #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  lapJar=lapapp.jar			# LAP jar name
  lapStatus=$CANDLEHOME/LAP/license/status.dat	# LAP status code file path
  lapCode=0
# always display license
#  [ -r $lapStatus ] && lapCode=$(grep "Status=" $lapStatus | cut -d"=" -f2)

  # Skip LAP if there was a previous acceptance in CandleHome
#  [ $lapCode = 9 ] && return

  # Backup any existing LAP files
  [ -d $CANDLEHOME/LAP ] && mv $CANDLEHOME/LAP $CANDLEHOME/LAP_old

  [ -z "$LAPMode" ] && LAPMode="$1 $2"

  # Prepare the LAP output area and environment
  mkdir $CANDLEHOME/LAP

  # Invoke LAP tool (aix requires special LIBPATH changes)
  lapJrePath=${JREEXE%%/bin*}
  [ "$(uname)" = "AIX" ] && export LIBPATH=$lapJrePath/lib:$lapJrePath/bin:$lapJrePath/bin/classic:$LIBPATH

  if [ -n "$silentMode" ]
  then
  #$NLS-E$ "\n\nUse of silent install mode implies license agreement acceptance. You can view the license file details in the $CANDLEHOME/LAP/license directory"
   print "\n"
   nls_replace 'KCI1006I' $CANDLEHOME/LAP/license
   sleep 5
  fi

  $JREEXE -classpath $PROD_PATH/unix/LAP/LAP_HOME/$lapJar:$lapJrePath/lib/rt.jar:$lapJrePath/lib/i18n.jar $JRE_DEBUG_ARGS com.ibm.lex.lapapp.LAP $silentMode -l $PROD_PATH/unix/LAP/LA_HOME -s $CANDLEHOME/LAP $LAPMode

  # Status code=9 is the only valid acceptance code
  lapCode=$?
  [ -r $lapStatus ] && lapCode=$(grep "Status=" $lapStatus | cut -d"=" -f2)

  if [ $lapCode != 9 ]
  then
    rm -rf $CANDLEHOME/LAP
    [ -d $CANDLEHOME/LAP_old ] && mv $CANDLEHOME/LAP_old $CANDLEHOME/LAP
    if [ $lapCode = 230 ]
    then
	  #$NLS-E$ "terminating ... problem with starting Java Virtual Machine"
      Err_report fail "`nls_replace 'KCI1235E'`"
    else
	  #$NLS-E$ "terminating ... license declined"
      Err_report fail "`nls_replace 'KCI1007E'`"
    fi
  fi

  # Copy 3rd party license (if one exists)
  [ -f $PROD_PATH/unix/terms.agr ] && cp -p $PROD_PATH/unix/terms.agr $CANDLEHOME
  [ -f $PROD_PATH/unix/TERMS.AGR ] && cp -p $PROD_PATH/unix/TERMS.AGR $CANDLEHOME

  # Copy additional license files
  [ -f $PROD_PATH/license/Greek.txt ] && [ ! -f $CANDLEHOME/LAP/license/Greek.txt ] && cp -f $PROD_PATH/license/Greek.txt $CANDLEHOME/LAP/license/ > /dev/null 2>&1
  [ -f $PROD_PATH/license/Lithuanian.txt ] && [ ! -f $CANDLEHOME/LAP/license/Lithuanian.txt ] && cp -f $PROD_PATH/license/Lithuanian.txt $CANDLEHOME/LAP/license/ > /dev/null 2>&1
  [ -f $PROD_PATH/license/Russian.txt ] && [ ! -f $CANDLEHOME/LAP/license/Russian.txt ] && cp -f $PROD_PATH/license/Russian.txt $CANDLEHOME/LAP/license/ > /dev/null 2>&1
  [ -f $PROD_PATH/license/Slovenian.txt ] && [ ! -f $CANDLEHOME/LAP/license/Slovenian.txt ] && cp -f $PROD_PATH/license/Slovenian.txt $CANDLEHOME/LAP/license/ > /dev/null 2>&1
}

function checkLevel { #-------------------------------------------------#IV51017
  # Compare the levels (version and release) of two products.  The new  #IV51017
  # product (the one to be installed) is the basis for the comparison   #IV51017
  # with the current product (the one in CANDLEHOME).                   #IV51017
  #   $1 = the name of the current .ver file.                           #IV51017
  #   $2 (if specified) = the name of the new .ver file.                #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset -i curRel=0 curVer=0 newRel=0 newVer=0 useVRMF=0

  curFile=$curRegPath/$1
  newFile=$newRegPath/$1

  [ -z "$2" ] || newFile=$newRegPath/$2

  # Ensure that both files exist.

  #$NLS-E$ "could not read \"$curFile\""
  [ -r "$curFile" ] || Err_report fail "`nls_replace 'KCI1008E' \\"$curFile\\"`"
  #$NLS-E$ "could not read \"$newFile\""
  [ -r "$newFile" ] || Err_report fail "`nls_replace 'KCI1008E' \\"$newFile\\"`"

  # Get VRMF from .ver file and remove leading whitespace and leading zero
  # Resulting VRMF will be 7 digit e.g. 6200000
  curVRMF=$(grep "^VRMF *=" $curFile | cut -d"=" -f2)
  curVer=${curVRMF##+( )+(0)}

  # If no VRMF, get Ver and Rel instead and convert to VRMF format
  if [ $curVer -eq 0 ]
  then
     curVer=$(grep "^ver *=" $curFile | cut -d"=" -f2)
     #$NLS-E$ "could not determine the version number from \"$curFile\""
     [ $curVer -ne 0 ] || Err_report fail "`nls_replace 'KCI1009E' \\"$curFile\\"`"
     curRel=$(grep "^rel *=" $curFile | cut -d"=" -f2)
     #$NLS-E$ "could not determine the release number from \"$curFile\""
     [ -n "$curRel" ] || Err_report fail "`nls_replace 'KCI1010E' \\"$curFile\\"`"
     # Convert Ver & Rel to VRMF format, this will result in 7 digit number
     # e.g. 5051000 where Ver=505 and Rel=100 i.e. VVVRRR0
     curVer=$curVer${curRel}0
  fi

  # Get VRMF from new .ver file and remove leading whitespace and leading zero
  # Resulting VRMF will be 7 digit e.g. 6200000
  newVRMF=$(grep "^VRMF *=" $newFile | cut -d"=" -f2)
  newVer=${newVRMF##+( )+(0)}

  # If no VRMF, get Ver and Rel instead and convert to VRMF format
  if [ $newVer -eq 0 ]
  then
     newVer=$(grep "^ver *=" $newFile | cut -d"=" -f2)
     #$NLS-E$ "could not determine the version number from \"$newFile\""
     [ $newVer -ne 0 ] || Err_report fail "`nls_replace 'KCI1009E' \\"$newFile\\"`"
     newRel=$(grep "^rel *=" $newFile | cut -d"=" -f2)
     #$NLS-E$ "could not determine the release number from \"$newFile\""
     [ -n "$newRel" ] || Err_report fail "`nls_replace 'KCI1010E' \\"$newFile\\"`"
     # Convert Ver & Rel to VRMF format, this will result in 7 digit number
     # e.g. 5051000 where Ver=505 and Rel=100 i.e. VVVRRR0
     newVer=$newVer${newRel}0
  fi

  # Compare the versions (which are by now all in VRMF format)
  if [ newVer -lt curVer ]
  then
     levelStrg=Older
  elif [ newVer -gt curVer ]
  then
     levelStrg=Newer
  else
     levelStrg=Same
  fi

  return
}

function checkLevel2 { #------------------------------------------------#IV51017
  # Compare the levels (version and release) of any two products. The   #IV51017
  # "ref" product is the basis for comparison with the "cmp" product.   #IV51017
  #    $1 = the complete pathname of the reference .ver file.           #IV51017
  #    $2 = the complete pathname of the .ver file to compare with the  #IV51017
  #         reference.                                                  #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset -i refRel=0 refVer=0 cmpRel=0 cmpVer=0

  refFile=$1
  cmpFile=$2

  # Ensure that both files exist.

  #$NLS-E$ "could not read \"$refFile\""
  [ -r "$refFile" ] || Err_report fail "`nls_replace 'KCI1008E' \\"$refFile\\"`"
  #$NLS-E$ "could not read \"$cmpFile\""
  [ -r "$cmpFile" ] || Err_report fail "`nls_replace 'KCI1008E' \\"$cmpFile\\"`"

  # Get and compare the versions.

  refVer=$(grep "^ver *=" $refFile | cut -d"=" -f2)
  cmpVer=$(grep "^ver *=" $cmpFile | cut -d"=" -f2)
  #$NLS-E$ "could not determine the version number from \"$refFile\""
  [ -n "$refVer" ] || Err_report fail "`nls_replace 'KCI1009E' \\"$refFile\\"`"
  #$NLS-E$ "could not determine the version number from \"$cmpFile\""
  [ -n "$cmpVer" ] || Err_report fail "`nls_replace 'KCI1009E' \\"$cmpFile\\"`"

  if [ cmpVer -lt refVer ]
  then
    levelStrg2=Older
    return
  fi

  if [ cmpVer -gt refVer ]
  then
    levelStrg2=Newer
    return
  fi

  # If the versions are identical, compare the releases.

  refRel=$(grep "^rel *=" $refFile | cut -d"=" -f2)
  cmpRel=$(grep "^rel *=" $cmpFile | cut -d"=" -f2)
  #$NLS-E$ "could not determine the release number from \"$refFile\""
  [ -n "$refRel" ] || Err_report fail "`nls_replace 'KCI1010E' \\"$refFile\\"`"
  #$NLS-E$ "could not determine the release number from \"$cmpFile\""
  [ -n "$cmpRel" ] || Err_report fail "`nls_replace 'KCI1010E' \\"$cmpFile\\"`"

  # Same version, older release.

  if [ cmpRel -lt refRel ]
  then
    levelStrg2=Older
    return
  fi

  # Same version, same release.

  if [ cmpRel -eq refRel ]
  then
    levelStrg2=Same
    return
  fi

  # Same version, newer release.

  levelStrg2=Newer
  return
}

function cleanUp { #----------------------------------------------------#IV51017
  # $1 should equal either the word "trap" or a number.                 #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset -i rc=0
  typeset name=""

  #$NLS-E$ "caught $1, cleanup"
  [ "$1" = "trap" ] && Err_report info "`nls_replace 'KCI1011I' \"$1\"`"
  [[ "$1" = +([0-9]) ]] && rc=$1

  rm -f $CANDLETMP/*.sh $CANDLETMP/*.tbl $refVFile

  # EJF (13046) do not remove jr*.ver files as they are used later when install is re-run
  tmpCurrDir=$(pwd)
  cd $CANDLETMP

# defect 31944, exit install we change to 4
  if [[ "$rc" != 4 ]]
  then
    rm -f $(print $(ls *.ver 2>/dev/null | grep -v '^jr')) > /dev/null 2>&1
  else
    rc=0                # reset to 0
  fi

  cd $tmpCurrDir

  rm -f $thisTok
  rm -f $RegBackup/Pre_CT99_Roma

  # Delete backup LAP, or restore it
  if [ -d "$CANDLEHOME/LAP" ]
  then
    [ -d "$CANDLEHOME/LAP_old" ] && rm -rf "$CANDLEHOME/LAP_old"
  else
    [ -d "$CANDLEHOME/LAP_old" ] && mv "$CANDLEHOME/LAP_old" "$CANDLEHOME/LAP"
  fi

  exit $rc  # HP-UX needs the dollar sign.
}

# Display the release level of the current! installer.

function dspInstLvl { #-------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset curRel="" curVer="" regFile=""

  #$NLS-E$ "specify or set CANDLEHOME, and rerun"
  [ -n "$CANDLEHOME" ] || Err_report fail "`nls_replace 'KCI1012E' CANDLEHOME`"

  regFile=$CANDLEHOME/registry/$ciVFile
  #$NLS-E$ "could not read file \"$regFile\""
  [ -r $regFile ] || Err_report fail "`nls_replace 'KCI1013E' \\"$regFile\\"`"

  # Get the version and release numbers.

  curVer=$(grep "^ver *=" $regFile | cut -d"=" -f2)
  #$NLS-E$ "could not determine the version number from \"$regFile\""
  [ -n "$curVer" ] || Err_report fail "`nls_replace 'KCI1009E' \\"$regFile\\"`"
  curRel=$(grep "^rel *=" $regFile | cut -d"=" -f2)
  #$NLS-E$ "could not determine the release number from \"$regFile\""
  [ -n "$curRel" ] || Err_report fail "`nls_replace 'KCI1010E' \\"$regFile\\"`"

  #$NLS-E$ "\nThe version and release levels of the installer are:"
  echo
  nls_replace 'KCI1014I'
  print "  ver = $curVer"
  print "  rel = $curRel \n"
}

# Determine the latest (newest) fixpack for a product.

function findLatest { #-------------------------------------------------#IV51017
  # $1 = list of fixpack .ver file names (e.g. ciux6110.ver)            #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset allNames name vFile
  typeset -i lateRel=0 lateVer=0 rel=0 ver=0

  # Find the latest (largest) version number.

  for vFile in $@
  do
    ver=$(grep "^ver *=" $newRegPath/$vFile 2> /dev/null | cut -d"=" -f2)
	#$NLS-E$ "could not determine the version number from \"$newRegPath/$vFile\""
    [ -n "$ver" ] || Err_report fail "`nls_replace 'KCI1009E' \\"$newRegPath/$vFile\\"`"
    [ ver -gt lateVer ] && lateVer=ver
  done

  # Find which .ver files specify the latest version.

  for vFile in $@
  do
    name=$(grep -l "^ver *= *$lateVer" $newRegPath/$vFile 2> /dev/null)
    name=${name##*/}  # Remove the path.
    allNames="$allNames $name"
  done

  #$NLS-E$ "could not find version number \"$lateVer\" in any .ver file"
  [ -n "$allNames" ] || Err_report fail "`nls_replace 'KCI1015E' \\"$lateVer\\"`"

  # Find the latest (largest) release number from the latest (i.e. those with largest version number) .ver files.

  for vFile in $allNames
  do
    rel=$(grep "^rel *=" $newRegPath/$vFile 2> /dev/null | cut -d"=" -f2)
	#$NLS-E$ "could not determine the release number from \"$newRegPath/$vFile\""
    [ -n "$rel" ] || Err_report fail "`nls_replace 'KCI1010E' \\"$newRegPath/$vFile\\"`"
    [ rel -gt lateRel ] && lateRel=rel
  done

  # Determine the name of the latest (newest) fixpack.

  for vFile in $allNames
  do
    lateVFile=$(grep -l "^rel *= $lateRel" $newRegPath/$vFile 2> /dev/null)
    lateVFile=${lateVFile##*/}  # Remove the path.
  done

  #$NLS-E$ "could not find the latest fixpack"
  [ -n "$lateVFile" ] || Err_report fail "`nls_replace 'KCI1016E'`"
}

function cpMo { #-------------------------------------------------------#IV51017
  # Copy nls_replace to CANDLEHOME/bin and msg directory to             #IV51017
  # CANDLEHOME/config/nls.                                              #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
    [ ! -d $CANDLEHOME/bin ] && mkdir $CANDLEHOME/bin > /dev/null 2>&1
    [ ! -d $CANDLEHOME/bin ] && Err_report fail "`nls_replace 'KCI1091E' \\"$CANDLEHOME/bin\\"`"

    [ ! -d $CANDLEHOME/config ] && mkdir $CANDLEHOME/config > /dev/null 2>&1
    [ ! -d $CANDLEHOME/config ] && Err_report fail "`nls_replace 'KCI1091E' \\"$CANDLEHOME/config\\"`"

    [ ! -d $CANDLEHOME/config/nls ] && mkdir $CANDLEHOME/config/nls > /dev/null 2>&1
    [ ! -d $CANDLEHOME/config/nls ] && Err_report fail "`nls_replace 'KCI1091E' \\"$CANDLEHOME/config/nls\\"`"

    cp -R $nls_replace_dir/msg $CANDLEHOME/config/nls > /dev/null 2>&1

    [ ! -d $CANDLEHOME/config/nls/msg ] && Err_report warn "`nls_replace 'KCI1018I' \\"msg\\" \\"$CANDLEHOME/config/nls\\"`"

    cp $nls_replace_dir/nls_replace $CANDLEHOME/bin > /dev/null 2>&1
    [ ! -a $CANDLEHOME/bin/nls_replace ] && Err_report warn "`nls_replace 'KCI1018I' \\"nls_replace\\" \\"$CANDLEHOME/bin\\"`"

    chmod 755 $CANDLEHOME/bin/nls_replace > /dev/null 2>&1
	#$NLS-E$ "could not change access permissions on \"$CANDLEHOME/bin/nls_replace\""
    [ $? -eq 0 ] || Err_report warn "`nls_replace 'KCI1019I' \\"$CANDLEHOME/bin/nls_replace\\"`"
}

function unloadTmpFile { #----------------------------------------------#IV51017
  # Unload a single file from an archive's tmp/ directory.              #IV51017
  # $1 = name of file to unload.                                        #IV51017
  # $2 = full pathname of archive.                                      #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset ext tFile

  #$NLS-E$ "cannot read package \"$2\""
  [ -r "$2" ] || Err_report fail "`nls_replace 'KCI1020E' \\"$2\\"`"

  ext=${2##*.}  # Get the extension, either "tar", "ctp" or "zxx".

  if [[ "$ext" = t* ]] # Uncompressed tar file.
  then
    tar -$tarOpt $2 tmp/$1 > /dev/null 2>&1
    # do chown and chmod for Linux.  defect 83100
    if [[ "$chownRoot" = y ]]
    then
      for tvf in $(tar tvf $2 tmp/$1 2> /dev/null | $AWK '{print $6}')
      do
        chown root:root $tvf
        chmod 755 $tvf
      done
    fi
	#$NLS-E$ "could not unload file \"$1\""
    [ -r tmp/$1 ] || Err_report fail "`nls_replace 'KCI1021E' \\"$1\\"`"
  else  # Compressed tar file.
    zcat < $2 | tar -$tarOpt - tmp/$1 > /dev/null 2>&1
	#$NLS-E$ "could not unload file \"$1\""
    [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1021E' \\"$1\\"`"
    # do chown and chmod for Linux.  defect 83100
    if [[ "$chownRoot" = y ]]
    then
      for tvf in $(zcat < $2 | tar tvf - tmp/$1 2> /dev/null | $AWK '{print $6}')
      do
        chown root:root $tvf
        chmod 755 $tvf
      done
    fi
  fi
}

function checkDiskSpace { #---------------------------------------------#IV51017
  # Check for free disk space. We have two inputs:                      #IV12556
  #                                                                     #IV12556
  #    ${1} is the directory to check (CANDLEHOME)                      #IV12556
  #    ${2) is the number of kilobytes required                         #IV12556
  #                                                                     #IV12556
  # We have two outputs in the form of global variables:                #IV12556
  #                                                                     #IV12556
  #    ${kbAvail} is the number of free kilobytes in ${1}.              #IV12556
  #    ${kbTmpAvail} is the number of free kilobytes in /tmp.           #IV12556
  #                                                                     #IV12556
  # Both kbAvail and kbTmpAvail are defined by install.sh as ksh        #IV12656
  # integers (typeset -i), passed as character strings to the Java      #IV12556
  # side of the installer, and subsequently converted to Java integers  #IV12556
  # for arithmetic comparisons.                                         #IV12556
  #                                                                     #IV12556
  # For both ksh and Java, integers have a theoretical upper limit of   #IV12556
  # 2147483647. If the limit is exceeded, bad things happen. For this   #IV12556
  # exercise, the limit represents 2 terabytes: plenty of free disk     #IV12556
  # space for our needs. To avoid conversion problems, we artificially  #IV12556
  # deflate larger values down to the upper limit. The deflation is     #IV12556
  # performed as follows:                                               #IV12556
  #                                                                     #IV12556
  #    a) Obtain the values in character strings using the local "*_c"  #IV12556
  #       variable names and pad them with zero characters.             #IV12556
  #    b) Use string compares to determine whether the values exceed    #IV12556
  #       the theoretical limit.                                        #IV12556
  #    c) Convert values as appropriate to integer form and store them  #IV12556
  #       into their global variable names.                             #IV12556
  #---------------------------------------------------------------------#IV12556
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset thisOS=$(uname -s)

  #---------------------------------------------------------------------#IV12556
  # Define the df command to use. Solaris gives us output in POSIX      #IV12556
  # format. For everyone else we specify -P to get POSIX output.        #IV12556
  #                                                                     #IV12556
  # We also define a common filter for df command output. Regardless of #IV12556
  # the df incantation, we grep for the '%' character to filter out the #IV12556
  # df header and we use the head command to grab the first (and        #IV12556
  # hopefully *only*) line of useful data. Finally, we use awk to grab  #IV12556
  # the 3rd field from the end. The number of fields varies because the #IV12556
  # file system name is sometimes emitted on its own line. However, the #IV12556
  # available free space is always relative to the end of the line.     #IV12556
  #---------------------------------------------------------------------#IV12556
  typeset dfCmd="df -kP"  # All platforms except Solaris                #IV12556
  [ "$thisOS" = "SunOS" ] && dfCmd="df -k"
  typeset dfFilt="| grep '%' | head -1 | $AWK '{av=NF-2; print \$av}'"; #IV12556

  #---------------------------------------------------------------------#IV12556
  # Define the theoretical maximum in a padded string for character     #IV12556
  # comparison.                                                         #IV12556
  #---------------------------------------------------------------------#IV12556
  typeset kbMax="2147483647";                                           #177708.1
  # The printf function does not support the format "%030.30s" for      #177708
  # padding the string with leading zeros on all supported platforms.   #177708
  # Use "%30.30s" to blank pad and then translate blanks to zeros       #177708
  # Assigning a string variable containing only digits to an integer    #177708.1
  # variable can cause octal conversion if the string value only        #177708.1
  # contains digits less than 8 and has one or more leading zeros.      #177708.1
  # NEVER assign a string variable containing only digits that starts   #177708.1
  # with a leading zero to an integer variable.                         #177708.1
  # Test the string variable containing leading zeros for alignment.    #177708.1
  # Assign the string variable without leading zeros for safety.        #177708.1
  typeset kbMax_t=$( printf "%30.30s" $kbMax | tr ' ' '0' );            #177708.1
                                                                        #IV12556
  #---------------------------------------------------------------------#IV12556
  # Determine the amount of free space available in ${1}:CANDLEHOME.    #IV12556
  #---------------------------------------------------------------------#IV12556
  typeset kbAvail_c=$( eval ${dfCmd} ${1} 2> /dev/null ${dfFilt} );     #IV12556
                                                                        #IV12556
  if [ -n "${kbAvail_c}" ] ; then                                       #IV12556
    #$NLS-E$ "calculating available disk space"
    Err_report info "`nls_replace 'KCI1022I'`"
                                                                        #IV12556
    typeset kbAvail_t=$( printf "%30.30s" "${kbAvail_c}" | tr ' ' '0' ); #177708.1
    if [[ "${kbAvail_t}" > "${kbMax_t}" ]]; then                        #177708.1
       kbAvail="${kbMax}";                                              #IV12556
    else                                                                #IV12556
       kbAvail="${kbAvail_c}";                                          #IV12556
    fi;                                                                 #IV12556
  else
    #$NLS-E$ "df could not determine the available disk space in \"$1\""
    Err_report fail "`nls_replace 'KCI1023E' df \\"$1\\"`"
  fi

  #---------------------------------------------------------------------#IV12556
  # Determine the amount of free space available in /tmp. For AIX, we   #IV12556
  # have a special filter to remove mounted /tmp file systems in lieu   #IV12556
  # of the local /tmp file system.                                      #IV12556
  #---------------------------------------------------------------------#IV12556
  typeset tmpFilt="";                                                   #IV12556
  [ "${thisOS}" = "AIX" ] && tmpFilt="| grep -v ':/tmp' ";              #IV12556
                                                                        #IV12556
  typeset kbTmpAvail_c=$( eval ${dfCmd} /tmp 2> /dev/null ${tmpFilt} ${dfFilt} ); #IV12556
                                                                        #IV12556
  if [ -n "${kbTmpAvail_c}" ] ; then                                    #IV12556
    #$NLS-E$ "calculating available disk space"
    Err_report info "`nls_replace 'KCI1022I'`"
                                                                        #IV12556
    typeset kbTmpAvail_t=$( printf "%30.30s" "${kbTmpAvail_c}" | tr ' ' '0' ); #177708.1
    if [[ "${kbTmpAvail_t}" > "${kbMax_t}" ]]; then                     #177708.1
       kbTmpAvail="${kbMax}";                                           #IV12556
    else                                                                #IV12556
       kbTmpAvail="${kbTmpAvail_c}";                                    #IV12556
    fi;                                                                 #IV12556
  else
    #$NLS-E$ "df could not determine the available disk space in /tmp"
    Err_report fail "`nls_replace 'KCI1023E' df /tmp`"
  fi

  #---------------------------------------------------------------------#IV12556
  # Report available free space in ${1}:CANDLEHOME.                     #IV12556
  #---------------------------------------------------------------------#IV12556
  if [ $2 -eq 0 ]  # The value of kbAvail is still needed even if there is nothing to install.
  then
    #$NLS-E$ "\"$kbAvail\" kilobytes available"
    Err_report info "`nls_replace 'KCI1024I' \\"$kbAvail\\"`"
    return
  fi

  if [ kbAvail -gt $2 ]
  then
    #$NLS-E$ "\"$kbAvail\" kilobytes available; only \"$2\" kilobytes required for the package(s)"
    Err_report info "`nls_replace 'KCI1025I' \\"$kbAvail\\" \\"$2\\"`"
	#$NLS-E$ "OK to install"
    Err_report info "`nls_replace 'KCI1026I'`"
  else
    #$NLS-E$ "\"$2\" kilobytes required for the package(s); only \"$kbAvail\" kilobytes available"
    Err_report fail "`nls_replace 'KCI1027E' \\"$2\\" \\"$kbAvail\\"`"
  fi
}

function findFamilies { #-----------------------------------------------#IV51017
  # Determine if there are CCC and/or Roma products available for       #IV51017
  # installation.                                                       #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset pc1 pkg pTbl=proddsc.tbl tFile=tmp/proddsc.tbl

  typeset cccPC=$CANDLETMP/cccPC.$$ romaPC=$CANDLETMP/romaPC.$$ jarPC=$CANDLETMP/jarPC.$$ verPC=$CANDLETMP/verPC.$$
  touch $cccPC $romaPC $jarPC $verPC

  # Get all of the possible product codes from all available product tables.

  for pkg in $(ls $PKGLOC/ci*.tar 2> /dev/null)  # Look in the CI packages for a product description table.
  do
    tar -$tarOpt $pkg $tFile > /dev/null 2>&1
    # do chown and chmod for Linux.  defect 83100
    if [[ "$chownRoot" = y ]]
    then
      for tvf in $(tar tvf $pkg $tFile 2> /dev/null | $AWK '{print $6}')
      do
        chown root:root $tvf
        chmod 755 $tvf
      done
    fi
    [ -r $tFile ] || continue  # Forget it.
    # xb is now a product shared between CCC and Roma (LL - 4/2003)
    # CCC product codes.
    cat $CANDLEHOME/$tFile | grep -v "[Rr]oma" | egrep -v "^(ax|ci|jr|xb|\#|\*)" | cut -d"|" -f1 >> $cccPC 2> /dev/null
    # Roma product codes.
    cat $CANDLEHOME/$tFile | grep "[Rr]oma" | egrep -v "^(ax|ci|jr|xb|\#|\*)" | cut -d"|" -f1 >> $romaPC 2> /dev/null
    break  # One is enough.
  done

  #  Add the codes from the product table in $CANDLEHOME/registry.

  if [ -r $curRegPath/$pTbl ]
  then
    # CCC product codes.
    cat $curRegPath/$pTbl | grep -v "[Rr]oma" | egrep -v "^(ax|ci|jr|xb|\#|\*)"  | cut -d"|" -f1 >> $cccPC 2> /dev/null
    # Roma product codes.
    cat $curRegPath/$pTbl | grep "[Rr]oma" | egrep -v "^(ax|ci|jr|xb|\#|\*)" | cut -d"|" -f1 >> $romaPC 2> /dev/null
  fi

  #$NLS-E$ "could not determine product codes"
  [[ -s "$cccPC" || -s "$romaPC" ]] || Err_report fail "`nls_replace 'KCI1028E'`"

  # Get the product codes from the available jar files.

  ls $PKGLOC/*.j* 2> /dev/null | sed "s?.*/??" | cut -c1-2 > $jarPC
  [ -s $jarPC ] || find $PKGLOC -name "*.j*" 2> /dev/null | sed "s?.*/??" | cut -c1-2 > $jarPC
  #$NLS-E$ "could not find any jar files in \"$PKGLOC\""
  [ -s $jarPC ] || Err_report fail "`nls_replace 'KCI1029E' \\"$PKGLOC\\"`"
  sort -u $jarPC -o $jarPC 2> /dev/null  # Keep only unique product codes.

  # Get the installed product codes from the Registry

  ls $curRegPath/*.ver 2> /dev/null | sed "s?.*/??" | cut -c1-2 > $verPC
  [ -s $verPC ] && sort -u $verPC -o $verPC 2> /dev/null  # Keep only unique product codes.

  # See if there are CCC products available.

  if [[ -r "$cccPC" && -s "$cccPC" ]]
  then
    sort -u $cccPC -o $cccPC 2> /dev/null  # Keep only unique product codes.

    # Search the install media
    for pc1 in $(< $cccPC)
    do
      foundSth=$(grep $pc1 $jarPC 2> /dev/null)
      if [ -n "$foundSth" ]
      then
        hasCCC=true  # hasCCC is declared in Main.
        break
      fi
    done

    # Search the installed products (if any)
    if [ -s $verPC ]
    then
      for pc1 in $(< $cccPC)
      do
        foundSth=$(grep $pc1 $verPC 2> /dev/null)
        if [ -n "$foundSth" ]
        then
          hadCCC=true  # hadCCC is declared in Main.
          break
        fi
      done
    fi
  fi

  # See if there are Roma products available.

  if [[ -r "$romaPC" && -s "$romaPC" ]]
  then
    sort -u $romaPC -o $romaPC 2> /dev/null  # Keep only unique product codes.

    # See if this is a RQ bundling of Roma (Candle Application Services Pac)
    foundSth=$(grep "rq" $jarPC 2> /dev/null)
    [ -n "$foundSth" ] && hasCASP=true  # hasCASP is declared in Main.

    # See if this package has an LDAP (for later eBP warning)
    foundSth=$(grep "xs" $jarPC 2> /dev/null)
    [ -n "$foundSth" ] && hasLDAP=true  # hasLDAP is declared in Main.

    # Search the install media
    for pc1 in $(< $romaPC)
    do
      foundSth=$(grep $pc1 $jarPC 2> /dev/null)
      if [ -n "$foundSth" ]
      then
        hasRoma=true  # hasRoma is declared in Main.
        break
      fi
    done

    # Search the installed products (if any)
    if [ -s $verPC ]
    then
      # See if this is a RQ bundling of Roma (Candle Application Services Pac)
      foundSth=$(grep "rq" $verPC 2> /dev/null)
      [ -n "$foundSth" ] && hadCASP=true  # hadCASP is declared in Main.

      for pc1 in $(< $romaPC)
      do
        foundSth=$(grep $pc1 $verPC 2> /dev/null)
        if [ -n "$foundSth" ]
        then
          hadRoma=true  # hadRoma is declared in Main.
          break
        fi
      done
    fi
  fi

  #if package is not roma by default set CCC.
  #New agents can have no entry in proddsc.tbl. New entry will be added during install process.
  [ -n "$hasRoma" ] || hasCCC=true


  # See if CMS is in the packaging
  foundSth=$(grep "ms" $jarPC 2> /dev/null)
  [ -n "$foundSth" ] && hasCMS=true  # hasCMS is declared in Main.

  rm -f $cccPC $jarPC $romaPC $verPC
}

#
# @14

function unTarPkg { #---------------------------------------------------#IV51017
  # Unload a .tar  or .ctpfile.                                         #IV51017
  #   $1 = full pathname of the file to untar.                          #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset -i i=0 maxTry=2
  typeset ext rc

  #$NLS-E$ "cannot read package \"$1\""
  [ -r "$1" ] || Err_report fail "`nls_replace 'KCI1030E' \\"$1\\"`"

  ext=${1##*.}  # ext (extension) will be either "ctp", "tar", tdd or "zdd" where d = a digit.
  while [ i -lt maxTry ]
  do
    if [[ "$ext" = t* ]]  # Uncompressed tar file.
    then
      tar -$tarOpt $1 > /dev/null 2>&1
      rc=$?
      # do chown and chmod for Linux.  defect 83100
      if [[ "$chownRoot" = y ]]
      then
    	# First sed removes leading "./" from tar archive. This is for the CMVC 203307 defect.
        for tvf in $(tar tvf $1 2> /dev/null | $AWK '{print $6}' | sed -e 's/^\.\/$//' | sed -e 's/\(.*\)\/.*/\1/' | sort | uniq)
        do
          chown -R root:root $tvf
          chmod -R 755 $tvf
        done
      fi
      [ -r $1 ] || rc=-1
    else  # Compressed tar file.
      zcat < $1 | tar -$tarOpt - > /dev/null 2>&1
      rc=$?
      # do chown and chmod for Linux.  defect 83100
      if [[ "$chownRoot" = y ]]
      then
    	# CMVC 203307
        for tvf in $(zcat < $1 | tar tvf - 2> /dev/null | $AWK '{print $6}' | sed -e 's/^\.\/$//' | sed -e 's/\(.*\)\/.*/\1/' | sort | uniq)
        do
          chown -R root:root $tvf
          chmod -R 755 $tvf
        done
      fi
    fi

    if [ "$rc" = "0" ]
    then
      return
    else
      i=i+1
    fi
  done

  # If we get to here, the untar may have failed.

  #$NLS-E$ "unarchive of \"$1\" may have failed"
  Err_report warn "`nls_replace 'KCI1031I' \\"$1\\"`"
}

function execRMDialog { #-----------------------------------------------#IV51017
  # Ask the user what readme file they want to view.                    #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset -i i=1

  # Build a list of descriptions.

  for rmf in $(ls $CANDLEHOME/doc/readme*.txt)
  do
    desc=${rmf##*/}       # Remove the pathname.
    desc=${desc#readme_}  # Remove the boilerplate.
    desc=${desc%.txt}     # Ditto for the extension.
    desc=$(print $desc 2> /dev/null | sed "s/_/ /g")  # Replace the underscores with blanks.
    desc=$(print $desc 2> /dev/null | sed "s,--,/,g")  # Replace two dashes with slash. @12
    rmDescs[i]="$desc"
    i=i+1
  done

  #$NLS-E$ "Exit this menu."
  rmDescs[i]="`nls_replace 'KCI1032I'`"  # Add an escape clause.
  maxVal=$i

  # Generate a "select" menu.

  i=1
  #$NLS-E$ "\nSelect one of the following readme files: \n"
  echo
  nls_replace 'KCI1033I'
  echo
  while [ i -le maxVal ]
  do
    print "$i) ${rmDescs[i]}"
    i=i+1
  done

  # Force the user to make a selection.

  while [ anything ]
  do
    #$NLS-E$ "Please enter a valid number:  \c"
    print "`nls_replace 'KCI1034I'` \c"
    read rmIdx
    [[ $rmIdx = +([0-9]) ]] || continue  # Make sure it's a number.
    [[ "$rmIdx" -gt 0 && "$rmIdx" -le maxVal ]] && break  # It's a valid number.
  done
}

function checkCIElements { #--------------------------------------------#IV51017
  # Version all CI elements with respect to the base package.           #IV51017
  #   $1 = names of all elements without an extension (e.g. cienv1).    #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset ele="" ext=tar

  for ele in $@
  do
    #$NLS-E$ "cannot read element package \"$PKGLOC/$ele.$ext\""
    [ -r "$PKGLOC/$ele.$ext" ] || Err_report fail "`nls_replace 'KCI1035E' \\"$PKGLOC/$ele.$ext\\"`"

    if [ "$hasRegDir" = "no" ]  # If necessary, extract the .ver file.
    then
      unloadTmpFile $ele.ver $PKGLOC/$ele.$ext
    fi

	#$NLS-E$ "cannot read file \"$newRegPath/$ele.ver\""
    [ -r "$newRegPath/$ele.ver" ] || Err_report fail "`nls_replace 'KCI1036E' \\"$newRegPath/$ele.ver\\"`"

    # Compare the level of the element with that of the base package.  Install only if they are the same.

    checkLevel2 $newRegPath/$ciVFile $newRegPath/$ele.ver
	#$NLS-E$ "element package \"$ele\" at different level than base package"
    [ "$levelStrg2" = "Same" ] || Err_report fail "`nls_replace 'KCI1037E' \\"$ele\\"`"
  done
}

# Version all CI fixpack elements with respect to the base fixpack.

function checkCIFixElements { #-----------------------------------------#IV51017
  # Version all CI fixpack elements with respect to the base fixpack.   #IV51017
  # $1 = name of base fixpack without an extension (e.g. ciux6110)      #IV51017
  # $2 = names of all element packages with extensions (e.g.            #IV51017
  #      ciux6110.t01).                                                 #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset baseName=$1 ele="" ext=""

  shift  # Skip the base.
  for ele in $@
  do
    ext=${ele##*.}  # Get the extension.
    ele=${ele%.*}$ext  # Construct the .ver file name (e.g. ciux6110t01).
	#$NLS-E$ "cannot read file \"$newRegPath/$ele.ver\""
    [ -r "$newRegPath/$ele.ver" ] || Err_report fail "`nls_replace 'KCI1036E' \\"$newRegPath/$ele.ver\\"`"

    # Compare the level of the element with that of the base package.  Install only if they are the same.

    checkLevel2 $newRegPath/$baseName.ver $newRegPath/$ele.ver
	#$NLS-E$ "element package \"$ele\" at different level than base package"
    [ "$levelStrg2" = "Same" ] || Err_report fail "`nls_replace 'KCI1037E' \\"$ele\\"`"
  done
}

function checkJRElements { #--------------------------------------------#IV51017
  # Version all JR elements with respect to the base package.           #IV51017
  #   $1 = names of all elements with an extension (e.g. jraix4.z01).   #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset bName="" ele="" ext="" verName=""

  for ele in $@
  do
    #$NLS-E$ "cannot read element package \"$PKGLOC/$ele\""
    [ -r "$PKGLOC/$ele" ] || Err_report fail "`nls_replace 'KCI1038E' \\"$PKGLOC/$ele\\"`"

    # Construct the name of the .ver file.

    ext=${ele##*.}          # ext -> z01
    bName=${ele%.*}         # bName -> jraix4
    verName=$bName$ext.ver  # verName -> jraix4z01.ver

	#$NLS-E$ "cannot read file \"$newRegPath/$verName\""
    [ -r "$newRegPath/$verName" ] || Err_report fail "`nls_replace 'KCI1036E' \\"$newRegPath/$verName\\"`"

    # Compare the level of the element with that of the base package.  Install only if they are the same.

    checkLevel2 $newRegPath/$bName.ver $newRegPath/$verName
	#$NLS-E$ "element package \"$ele\" at different level than base package"
    [ "$levelStrg2" = "Same" ] || Err_report fail "`nls_replace 'KCI1037E' \\"$ele\\"`"
  done
}

function lookForFixPacks { #--------------------------------------------#IV51017
  # See if there are any fixpacks in the product directory.             #IV51017
  # For now (6/22/2005) make it appear there are no fixpacks because    #IV51017
  # fixpack design isn't complete. The test below will think some std   #IV51017
  # packages are fixpacks based on the test for an 8 char name of which #IV51017
  # the last 4 char must be digits. However, new platform codes will    #IV51017
  # cause this to fail, such as axli6243.jar ... the 6243 fits the      #IV51017
  # fixpack test, but is actually the platform code for intel linux     #IV51017
  # (kernel 2.4, 32-bit).                                               #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  hasFixes=false
  return
}

function startGUI { #---------------------------------------------------#IV51017
  # Install products or maintenance from the GUI.                       #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  #$NLS-E$ "Please set DISPLAY environmental variable, and rerun $PROGNAME"
  [ -n "$DISPLAY" ] || Err_report fail "`nls_replace 'KCI1039E' DISPLAY $PROGNAME`"

  cd $CANDLEHOME/bin > /dev/null 2>&1
  #$NLS-E$ "could not cd to \"$CANDLEHOME/bin\" directory"
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI0683E' \\"$CANDLEHOME/bin\\"`"

  export PATH=$savePath:/usr/bin/X11:/usr/openwin/bin

  $JREEXE -classpath $CLASSPATH $JRE_DEBUG_ARGS $SAFE_MODE $DEPLOY_MODE ${SUSE_VERSION} -DGlobalDbgLevel="$TRACE_LEVEL" -DInstallRASConfig="$CANDLEHOME/config/ITMInstallRAS.properties" ITMinstall.JFrame1 -dsa $kbAvail -ca $cArch -ch $CANDLEHOME -cd $PKGLOC -ad "$archDesc" $DEBUG_CON $instFam $tivoli

  [ $? -eq 0 ] || cleanUp 2
}

function startSupport { #-----------------------------------------------#IV51017
  # Install support from command Line.                                  #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset -i rc=0

  # Create the reference .ver file.  This file is used to determine if anything was installed.

  touch $refVFile > /dev/null 2>&1
  #$NLS-E$ "could not create reference version file"
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1040E'`"

  cd $CANDLEHOME/bin > /dev/null 2>&1
  #$NLS-E$ "could not cd to \"$CANDLEHOME/bin\" directory"
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI0683E' \\"$CANDLEHOME/bin\\"`"

  # Silent mode passes parameter filename to Java
  silentParmFile=""
  [ -n "$silentMode" ] && silentParmFile="-p $PARM_FILE"

  $JREEXE -classpath $CLASSPATH $JRE_DEBUG_ARGS $SAFE_MODE $DEPLOY_MODE ${SUSE_VERSION} -DGlobalDbgLevel="$TRACE_LEVEL" -DInstallRASConfig="$CANDLEHOME/config/ITMInstallRAS.properties" ITMinstall.CandleInstall -ch $CANDLEHOME -dl $PKGLOC -sa $kbAvail -ty supp -ca $cArch -ta $kbTmpAvail $DEBUG_CON $instFam $tivoli  $silentParmFile -i $(print $CDNAME | cut -c1-6) -k $AUTHKEY # Run the command.
  rc=$?
}

function startCmdLn { #-------------------------------------------------#IV51017
  # Install products or maintenance from the command line.              #IV51017
  #   $1 will be either "fix" or "prod"                                 #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset -i rc=0

  # Create the reference .ver file.  This file is used to determine if anything was installed.

  touch $refVFile > /dev/null 2>&1
  #$NLS-E$ "could not create reference version file"
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1040E'`"

  cd $CANDLEHOME/bin > /dev/null 2>&1
  #$NLS-E$ "could not cd to \"$CANDLEHOME/bin\" directory"
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI0683E' \\"$CANDLEHOME/bin\\"`"

  # Silent mode passes parameter filename to Java
  silentParmFile=""
  [ -n "$silentMode" ] && silentParmFile="-p $PARM_FILE"

  # Add support for DISABLE_HTTP setting in Deploy mode only                     # IV63269
  # if ../specconfig.txt exists, read it and get the DISABLE_HTTP value          # IV63269
  # append java parm to DEPLOY_MODE if valid DISABLE_HTTP value found            # IV63269
  if [ -n "$DEPLOY_MODE" ]; then                                                 # IV63269
    dhFile="$bindir/../specconfig.txt";                                          # IV63269
    dhFound="";                                                                  # IV63269
    if [ -f "$dhFile" ]; then                                                    # IV63269
    while IFS== read dhVar dhVal dhJunk ; do                                     # IV63269
      if [[ "$dhVar" = DISABLE_HTTP && "$dhVal" = @(YES|NO) ]]; then             # IV63269
        dhFound=$dhVal;                                                          # IV63269
        break;                                                                   # IV63269
      fi;                                                                        # IV63269
    done < $dhFile ;                                                             # IV63269
    fi                                                                           # IV63269
    [ -n "$dhFound" ] && DEPLOY_MODE="$DEPLOY_MODE -Ditm.disableHttp=$dhFound";  # IV63269
  fi                                                                             # IV63269

  $JREEXE -classpath $CLASSPATH $JRE_DEBUG_ARGS $SAFE_MODE $DEPLOY_MODE ${SUSE_VERSION} -DGlobalDbgLevel="$TRACE_LEVEL" -DInstallRASConfig="$CANDLEHOME/config/ITMInstallRAS.properties" ITMinstall.CandleInstall -ch $CANDLEHOME -dl $PKGLOC -sa $kbAvail -ty $1 -ca $cArch -ta $kbTmpAvail $DEBUG_CON $instFam $tivoli  $silentParmFile -i $(print $CDNAME | cut -c1-6) -k $AUTHKEY # Run the command.

  rc=$?

  # Call to cleanup some files we have around
  cleanupSystem

  # From 62x->630, both the trigger file name and usage for IBM License Management   #IV65375
  # Tool were changed. Under the old scheme, ITM*.SYS2 marker files were created for #IV65375
  # a variety of components. Under the new scheme, *.swtag files are created only    #IV65375
  # for chargeable components. As part of the conversion, the old files need to be   #IV65375
  # deleted. We only do this once.                                                   #IV65375
  if [[ ! -a "${CANDLEHOME}/registry/.ilmt_cleanup_done" ]]; then                    #IV65375
     find "${CANDLEHOME}" -name "ITM*.SYS2" -type f -exec rm -f \{\} \; 2> /dev/null #IV65375
     touch "${CANDLEHOME}/registry/.ilmt_cleanup_done";                              #IV65375
  fi;                                                                                #IV65375

  if [ rc -eq 2 ]  # Roma specific processing.
  then
    $CANDLEHOME/bin/cannoxterm.sh romaoa $CANDLEHOME
  else
    [ rc -eq 0 ] || cleanUp 2
  fi

  [ "$instFam" = "-ccc" ] || return
  # Set permissions on certain agents.

  # No SetPerm required for Tandem system
  if [[ "$(uname -s)" != "NONSTOP_KERNEL" ]]
  then
  cmd=$CANDLEHOME/bin/SetPerm
  if [ ! -x $cmd ]
  then
    #$NLS-E$ "command \"$cmd\" not found"
    Err_report warn "`nls_replace 'KCI1041I' \\"$cmd\\"`"

  else
    # run SetPerm only if root user
    if [ "${thisRealUser}" = "root" ]; then                             #IZ94626.1
      saveMask="$(umask)"
      umask 0022        # APAR 89533 - Don't create links directory with 777 permission.
      $cmd -a  # Run the command (-a to suppress output)
      umask $saveMask   # APAR 89533 - Don't affect other things
    else
      # For non-root user, just display a warning
      echo
	  #$NLS-E$ "The root user must run bin/SetPerm from the ITM home to complete the setup"
      Err_report warn "`nls_replace 'KCI1042I' bin/SetPerm`"
    fi
  fi	#end ! -x $cmd
  fi	#end uname -s

  # Only print the CandleConfig message if something was installed.

  for file in $(ls $curRegPath/*.ver)
  do
    if [ "$file" -nt "$refVFile" ]  # Products have been installed.
    then
      if [[ $file != *@(tpa|tms|tpw|tpd|tps).ver ]]
      then
        #$NLS-E$ "\n You may now configure any locally installed IBM Tivoli Monitoring product via"
        #$NLS-E$ " the \"$CANDLEHOME/bin/itmcmd config\" command. \n"
	    echo
	    nls_replace 'KCI1045I' \""$CANDLEHOME/bin/itmcmd config"\"
	    echo
	    break
      fi
    fi
  done
}

function startGUIRP { #-------------------------------------------------#IV51017
  # Create remote packages via GUI.                                     #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  #$NLS-E$ "Please set DISPLAY environmental variable, and rerun $PROGNAME"
  [ -n "$DISPLAY" ] || Err_report fail "`nls_replace 'KCI1046E' DISPLAY $PROGNAME`"

  export PATH=$savePath:/usr/bin/X11:/usr/openwin/bin

#  print $JREEXE -classpath $SAFE_MODE $CLASSPATH ITMinstall.jclone_agents -ch $CANDLEHOME -dsa $kbAvail -ca $cArch -ad "$archDesc" -pl $PKGLOC -td $tarDir -pm $prodMode $DEBUG_CON

  $JREEXE -classpath $JRE_DEBUG_ARGS $SAFE_MODE $CLASSPATH -DGlobalDbgLevel="$TRACE_LEVEL" -DInstallRASConfig="$CANDLEHOME/config/ITMInstallRAS.properties" ITMinstall.jclone_agents -ch $CANDLEHOME -dsa $kbAvail -ca $cArch -ad "$archDesc" -pl $PKGLOC -td $tarDir -pm $prodMode $DEBUG_CON
  [ $? -eq 0 ] || cleanUp 2

  cd $CANDLEHOME/bin
  restart_proc start
  cd - 2> /dev/null > /dev/null

  cleanUp 0
}

function startCmdLnRP { #-----------------------------------------------#IV51017
  # Create remote packages via the command line.                        #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
#  print $JREEXE -classpath $CLASSPATH $SAFE_MODE ITMinstall.CandleClone -ch $CANDLEHOME -dsa $kbAvail -ca $cArch -pl $PKGLOC -td $tarDir -pm $prodMode $DEBUG_CON

  $JREEXE -classpath $CLASSPATH $JRE_DEBUG_ARGS $SAFE_MODE -DGlobalDbgLevel="$TRACE_LEVEL" -DInstallRASConfig="$CANDLEHOME/config/ITMInstallRAS.properties" ITMinstall.CandleClone -ch $CANDLEHOME -dsa $kbAvail -ca $cArch -pl $PKGLOC -td $tarDir -pm $prodMode $DEBUG_CON
  [ $? -eq 0 ] || cleanUp 2

  cd $CANDLEHOME/bin
  restart_proc start
  cd - 2> /dev/null > /dev/null

  cleanUp 0
}

function startCmdLnDepot { #--------------------------------------------#IV51017
  # Install packages to the agent depot via the command line.           #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  PROPS=-Ddebugcon=true
  if [ "$DEBUG_CON" = "" ]
  then
     PROPS=-Ddebugcon=false
  fi
  $JREEXE $PROPS -classpath $CLASSPATH $JRE_DEBUG_ARGS -DGlobalDbgLevel="$TRACE_LEVEL" -DInstallRASConfig="$CANDLEHOME/config/ITMInstallRAS.properties" ITMinstall/AgentBundleDepotInstall -ch $CANDLEHOME -dsa $kbAvail -ca $cArch -pl $PKGLOC $DEBUG_CON
  [ $? -eq 0 ] || cleanUp 2

  cd $CANDLEHOME/bin
  restart_proc start
  cd - 2> /dev/null > /dev/null

  cleanUp 0
}

function startGUIDepot { #----------------------------------------------#IV51017
  # Install packages to the agent depot via the GUI.                    #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  PROPS=-Ddebugcon=true
  if [ "$DEBUG_CON" = "" ]
  then
     PROPS=-Ddebugcon=false
  fi
  $JREEXE $PROPS -classpath $CLASSPATH $JRE_DEBUG_ARGS -DGlobalDbgLevel="$TRACE_LEVEL" -DInstallRASConfig="$CANDLEHOME/config/ITMInstallRAS.properties" ITMinstall/AgentBundleDepotInstall -ch $CANDLEHOME -dsa $kbAvail -ca $cArch -pl $PKGLOC -m gui $DEBUG_CON
  [ $? -eq 0 ] || cleanUp 2

  cd $CANDLEHOME/bin
  restart_proc start
  cd - 2> /dev/null > /dev/null

  cleanUp 0
}

function procTokFiles { #-----------------------------------------------#IV51017
  # Delete old token files, and warn if there is another install        #IV51017
  # running in this <candlehome>.                                       #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset -i maxAge=5  # Remove "remote" token files older than 5 days.
  typeset -i age=0 dot=0 doy=0
  typeset PID name

  foundSth=$(ls ${baseTok}* | grep -v $thisTok 2> /dev/null) # Other installs?
  [ -n "$foundSth" ] || return

  doy=$(date +"%j")  # Day of the year (001 - 366)

  # Remove token files:
  # 1) created on this machine that are no longer connected to any process.
  # 2) created on other machines that are older than maxAge.

  for name in $(ls ${baseTok}* | grep -v $thisTok)
  do
    foundSth=$(cat $name | grep "$thisMach" 2> /dev/null)
    if [ -n "$foundSth" ]  # The token file was created on this machine.
    then
      PID=${name##*_}
      foundSth=$(ps -p $PID | grep $PID 2> /dev/null)
      [ -z "$foundSth" ] && rm -f $name
    else
      foundSth=$(cat $name | grep "|" 2> /dev/null) # Ensure the day of year field exists.
      [ -n "$foundSth" ] || continue

      dot=$(cat $name | cut -d"|" -f2 | cut -d"|" -f1)  # Day the token file was created (001 - 366).

      if [ doy -ge dot ]
      then
        age=doy-dot
      else  # This could happen towards the end of a year, so "normalize" the day of the year.
        doy=366+doy
        age=doy-dot
      fi

      [ age -gt maxAge ] && rm -f $name
    fi
  done

  # Warn the user about possible simultaneous installs.

  foundSth=$(ls ${baseTok}* | grep -v $thisTok 2> /dev/null) # Check for other installs.
  if [ -n "$foundSth" ]
  then
    #$NLS-E$ "\nAn install may already be running in \"$CANDLEHOME\" from"
    #$NLS-E$ "the following machine(s): \n"
    echo
	nls_replace 'KCI1047I' \"$CANDLEHOME\"
	echo
	for name in $(ls ${baseTok}* | grep -v $thisTok)
    do
      print "  \c"; cat $name
    done

    # If not deploy and not silent; then prompt; else force 'NO'       191283
    if [[ -z "$DEPLOY_MODE" && -z "$silentMode" ]]; then
      repeat=1
      while [ $repeat -eq 1 ]; do
        #$NLS-E$ "\nContinue with this installation [ y or n; \"n\" is default ]?  \c"
        echo
        print "`nls_replace 'KCI1048I'` [1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`; \"2\" `nls_replace 'KCI0199I'` ]?  \c"
        read
        [ -z "$REPLY" ] && REPLY=2
        case $REPLY in
          y*|Y*|1) repeat=0
                 ;;
          n*|N*|2) repeat=0
                 rm -f $thisTok
                 cleanUp 0
                 ;;
        esac
      done
    else
      rm -f $thisTok
      cleanUp 2
    fi
  fi
}

function order_proc { #-------------------------------------------------#IV51017
  # Sort lines representing running processes before they are stopped   #IV51017
  # or started.                                                         #IV51017
  # Parameters:                                                         #IV51017
  #   1. Mode, start or stop string                                     #IV51017
  #   2. OriginalLines, list lines in the format:                       #IV51017
  #      code$fieldSep... or code ...                                   #IV51017
  # Output:                                                             #IV51017
  #   orderedProcesses, array that contains all lines from              #IV51017
  #   'OriginalLines' but ordered in desired way                        #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
	Mode=$1
	shift
	OriginalLines=$*
	cqLine=""
 	kfLine=""
	osLine=""
	otherLines=""
	msLine=""
	for proc in $OriginalLines
	do
		[ -n "$fieldSep" ] && Code=$(print "$proc" | cut -d "$fieldSep" -f1) || Code=$(print $proc | cut -c1-2)

		case "$Code" in
		cq) cqLine=$proc ;;
		kf) kfLine=$proc ;;
		ux|lz) osLine="$osLine $proc" ;;
		ms) msLine=$proc ;;
		*) otherLines="$otherLines $proc";;
		esac
	done

	cienvVRMF=$(grep "^VRMF *=" $CANDLEHOME/registry/cienv.ver 2> /dev/null | cut -d"=" -f2)
	cienvVer=${cienvVRMF##+( )+(0)}

	if [[ -z $cienvVer ]]
	then
		cienvVer=$(grep "^ver *=" $CANDLEHOME/registry/cienv.ver 2> /dev/null | cut -d"=" -f2)
		cienvVer=${cienvVer##+( )}

		cienvRel=$(grep "^rel *=" $CANDLEHOME/registry/cienv.ver 2> /dev/null | cut -d"=" -f2)
		cienvRel=${cienvRel##+( )}

		cienvVer=$cienvVer${cienvRel}0
	fi

	archPrefix=${cArch%%+([0-9])}
	tepsVerFile=$CANDLEHOME/registry/cq${archPrefix}*.ver

	tepsVRMF=$(grep "^VRMF *=" $tepsVerFile 2> /dev/null | cut -d"=" -f2)
	tepsVer=${tepsVRMF##+( )+(0)}

	if [[ -z $tepsVer ]]
	then
		tepsVer=$(grep "^ver *=" $tepsVerFile 2> /dev/null | cut -d"=" -f2)
		tepsVer=${tepsVer##+( )}

		tepsRel=$(grep "^rel *=" $tepsVerFile 2> /dev/null | cut -d"=" -f2)
		tepsRel=${tepsRel##+( )}

		tepsVer=$tepsVer${tepsRel}0
	fi

	if [[ $tepsVer -ge "6200000" && $cienvVer -ge "6230000" ]]
	then
		if [ "$Mode" = "stop" ]
		then
			orderedProcesses="$cqLine $osLine $otherLines $msLine"
		else
			orderedProcesses="$msLine $otherLines $osLine $cqLine"
		fi
	else
		if [ "$Mode" = "stop" ]
		then
			orderedProcesses="$cqLine $kfLine $osLine $otherLines $msLine"
		else
			orderedProcesses="$msLine $otherLines $osLine $kfLine  $cqLine"
		fi
	fi
}

function restart_proc { #-----------------------------------------------#IV51017
  # routine to stop or start process.                                   #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
	Code=""
	Instance=""
	Userid=""
	OSAgentUserid=""
	ret=0
	typeset RestartOSAgent="NO"
	if [ -n "$DEPLOY_MODE" ]
	then
		trap "true" 13 15  # Set trap.
		trap "cleanUp trap" 1 2 3  # Set trap.
	else
		trap "cleanUp trap" 1 2 3 15  # Set trap.
	fi

	# Gather agent codes and sort it in the order: TEPS, EHS, Os Agent, other agents, TEMS
	order_proc $1 $fld23
	for proc in $orderedProcesses
	do
		[ -n "$fieldSep" ] && Code=$(print "$proc" | cut -d "$fieldSep" -f1) || Code=$(print $proc | cut -c1-2)
		[ -n "$fieldSep" ] && PID=$(print "$proc" | cut -d "$fieldSep" -f2) || PID=$(print $proc | cut -c3-)
		[ -n "$fieldSep" ] && Instance=$(print "$proc" | cut -d "$fieldSep" -f3) || Instance=$(print $proc | cut -c4-)

		# IZ17921 need to check userid on running processes
		[ -n "$fieldSep" ] && Userid=$(print "$proc" | cut -d "$fieldSep" -f4) || Userid=$(print $proc | cut -c5-)

		# Get the product description from the proddsc.tbl file.  If there are running processes,
		# then this file should exist.

		if [[ -r $pFile ]]
		then
			desc=$(grep "^$Code|" $pFile |head -1 | cut -d"|" -f2)  # desc -> "NetView/AIX Alert Adapter"
			[ -n "$desc" ] || desc=$Code  # Just in case.
		fi

		# Don't restart TEMS, TEPS and TEP if deploy
		if [[ "$Code" = ms ]]
		then
			if [[ -z "$DEPLOY_MODE" ]]
			then
				# IZ17921 add userid if different
				if [[ "${thisRealUser}" = "${Userid}" ]]; then  #IZ94626.1
					./CandleServer $1 $Instance
				else
					if [[ -n "${su_value}" ]]; then           #IZ94626.1
						$su_value - $Userid -c "$CANDLEHOME/bin/CandleServer $1 $Instance"
					else
						#$NLS-E$ "Unable to execute su as non-root userid" #IZ94626.1
						Err_report warn "`nls_replace 'KCI0246I' 'su'`"    #IZ94626.1
					fi
				fi
				CandleServerRet=$?
				#defect 54036
				if [[ $CandleServerRet != 0 ]]
				then
					# when TEMS wasn't stopped/started then we return -1 - this will used only to stop installation in that case.
					ret=1
				fi
			fi
		else
		    if [[ -n "$DEPLOY_MODE" ]]
		    then
		      [[ "$Code" = @(cq|cj) ]] && continue
		    fi

			# check if option -f is supported
			caOpt=""
			if [[ "$1" = "stop" ]]
			then
				currentFile=$CANDLEHOME/registry/cienv.ver
				currentVer=$(grep "^ver *=" $currentFile 2> /dev/null | cut -d"=" -f2)
				if [[ "$currentVer" -ge "620" ]];
				then
					[[ "$Code" = "kf" || -n "$DEPLOY_MODE" ]] && caOpt="-f"
				fi
			fi

			if [[ $Instance != "None" ]]
			then
				if [[ $Code = "um" ]]
				then
                          # IZ17921 add userid if different
                          if [[ "${thisRealUser}" = "${Userid}" ]]; then #IZ94626.1
				    ./CandleAgent $caOpt -p $Instance $1 $Code
                          else
					if [[ -n "${su_value}" ]]; then           #IZ94626.1
						$su_value - $Userid -c "$CANDLEHOME/bin/CandleAgent $caOpt -p $Instance $1 $Code"
					else
						#$NLS-E$ "Unable to execute su as non-root userid" #IZ94626.1
						Err_report warn "`nls_replace 'KCI0246I' 'su'`"    #IZ94626.1
					fi
                          fi
				else
					if [[ $Code = "ud" && $1 = "stop" ]]
					then
						hostNameInRunInfo=$(print "$fld123" | grep '|'$PID'|' | cut -d "$fieldSep" -f1 | head -1)
					fi
					Instance=${Instance#${hostNameInRunInfo}_}
					# IZ17921 add userid if different
					if [[ "${thisRealUser}" = "${Userid}" ]]; then #IZ94626.1
						./CandleAgent $caOpt -o $Instance $1 $Code
					else
						if [[ -n "${su_value}" ]]; then     #IZ94626.1
							$su_value - $Userid -c "$CANDLEHOME/bin/CandleAgent $caOpt -o $Instance $1 $Code"
						else
							#$NLS-E$ "Unable to execute su as non-root userid" #IZ94626.1
							Err_report warn "`nls_replace 'KCI0246I' 'su'`"    #IZ94626.1
						fi
					fi
				fi
			else
				if [[ "$1" = "start" ]]
				then
					if [[ $Code = "lz" ]]
					then
	                  		RestartOSAgent=$Code
						OSAgentUserid=$Userid
					else
						if [[ $Code = "ux" ]]
						then
      	            			RestartOSAgent=$Code
							OSAgentUserid=$Userid
						else
						    startAgent=yes
						    [[ $Code = "sy" ]] && [[ -n "$syNotStart" ]] && startAgent=no
						    [[ $Code = "hd" ]] && [[ -n "$hdNotStart" ]] && startAgent=no
						    if [[ "$startAgent" = "yes" ]]
						    then
							# IZ17921 add userid if different
							if [[ "${thisRealUser}" = "${Userid}" ]]; then #IZ94626.1
								./CandleAgent $1 $Code
							else
								if [[ -n "${su_value}" ]]; then #IZ94626.1
									$su_value - $Userid -c "$CANDLEHOME/bin/CandleAgent $1 $Code"
								else
									#$NLS-E$ "Unable to execute su as non-root userid" #IZ94626.1
									Err_report warn "`nls_replace 'KCI0246I' 'su'`" #IZ94626.1
								fi
							fi
						    fi
						fi
					fi
				else
					# IZ17921 add userid if different
					if [[ "${thisRealUser}" = "${Userid}" ]]; then #IZ94626.1
						./CandleAgent $caOpt $1 $Code
					else
						if [[ -n "${su_value}" ]]; then     #IZ94626.1
							$su_value - $Userid -c "$CANDLEHOME/bin/CandleAgent $caOpt $1 $Code"
						else
							#$NLS-E$ "Unable to execute su as non-root userid" #IZ94626.1
							Err_report warn "`nls_replace 'KCI0246I' 'su'`"    #IZ94626.1
						fi
					fi
				fi
			fi
	      fi
	done

	if [[ $RestartOSAgent != "NO" ]]; then                            #IZ94626.1
         restartcmd="${CANDLEHOME}/bin/CandleAgent ${1} ${RestartOSAgent}"; #IV27631
         if [[ "${thisRealUser}" = "${Userid}" ]]; then                 #IZ94626.1
            ${restartcmd};                                              #IZ94626.1
         else                                                           #IZ94626.1
	      if [[ -n "${su_value}" ]]; then                             #IZ94626.1
	         $su_value - ${Userid} -c "${restartcmd}";                #IZ94626.1
	      else                                                        #IZ94626.1
	         #$NLS-E$ "Unable to execute su as non-root userid"       #IZ94626.1
	         Err_report warn "`nls_replace 'KCI0246I' 'su'`"          #IZ94626.1
	      fi                                                          #IZ94626.1
         fi                                                             #IZ94626.1
	fi                                                                #IZ94626.1
	return $ret
}

function cinfoRunning { #-----------------------------------------------#IV51017
   # Obtain and convert "cinfo -r" output into the following form:      #IZ96646
   #                                                                    #IZ96646
   #     hostname|pc|pid|instance|realuser                              #IZ96646
   #                                                                    #IZ96646
   # On SunOS and Linux, "ps -f" returns the effective user where other #IV21233
   # platforms return the real user. Because we are using this to       #IV96646
   # control whether/how we stop and restart any agent processes, we    #IV96646
   # need the real user. The new cinfo (with IZ96646) returns the real  #IV96646
   # user, but older ones do not. So, this function modifies the output #IV96646
   # from cinfo as needed.                                              #IZ96646
   #--------------------------------------------------------------------#IZ96646
   eval $(itmsh_trace_func "${0}" "${@}");                              #IV51017
                                                                        #IV51017
   typeset x y z;                           # local work variables      #IZ96646
   ${CANDLEHOME}/bin/cinfo -r | grep '\.\.running' | while read x; do   #IZ96646
      set -A y ${x};                        # break words into array    #IZ96646
      # get the real userid on Solaris, Linux                           #IV21233
      case $thisOS in                                                   #IV21233
         SunOS)                                                         #IV21233
            y[3]=$(ps -p ${y[2]} -o ruser | tail +2 | tr -d ' ');       #IV21233
         ;;                                                             #IV21233
         Linux)                                                         #IV21233
            y[3]=$(ps -p ${y[2]} -o ruser | tail -n +2 | tr -d ' ');    #IV21233
         ;;                                                             #IV21233
      esac;                                                             #IV21233
      z=$(( ${#y[@]} - 2 ));                # penultimate word position #IZ96646
      print -- "${y[0]}|${y[1]}|${y[2]}|${y[${z}]}|${y[3]}";            #IZ96646
   done;                                    # (while(read x))           #IZ96646
}                                           # (cinfoRunning())          #IZ96646

function checkCandleManage { #------------------------------------------#IV51017
  # See if there is CandleManage running in the background, and warn    #IV51017
  # the user if so.                                                     #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  UNIX95= ps -eo ppid,args | fgrep "$CANDLEHOME/JRE/" | awk '{print $1}' | fgrep -x "`UNIX95= ps -eo pid,args | fgrep '/CandleManage' | awk '{print $1}'`" > /dev/null 2>&1
  if [[ $? -eq 0 ]]
  then
    # Check for silent mode
    if [ -n "$silentMode" ]
    then
      #$NLS-E$ "The Manage Tivoli Enterprise Monitoring Services is running in the background"
      Err_report warn "`nls_replace 'KCI1178W'`"
    else
      #$NLS-E$ "\nThe Manage Tivoli Enterprise Monitoring Services is running in the background, please close it before continuing.  \c"
      print "\n`nls_replace 'KCI1179I'`  \c"

      repeat=1
      while [ $repeat -eq 1 ]; do
        #$NLS-E$ "\nContinue with this installation [ y or n; \"y\" is default ]?  \c"
        print "\n`nls_replace 'KCI1054I'` [ 1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`; \"1\" `nls_replace 'KCI0199I'` ]?  \c"

        read
        [ -z "$REPLY" ] && REPLY=1
          case $REPLY in

            n*|N*|2)
              cleanUp 0  # Exit.
              repeat=0
            ;;

            y*|Y*|1)
              repeat=0
            ;;
          esac
        done
    fi
  fi
}


function checkMaxOpenFilesLimit { #-------------------------------------#IV51017
  # Check soft limit of a maximum number of open files (only for AIX)   #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
	if [ "$(uname)" = "AIX" ] ; then
		curOpenFiles=$(( $(lsof -p 0 2> /dev/null | wc -l) + $(lsof -p ^0 2> /dev/null | wc -l) ))
		softLimit=$(ulimit -n 2> /dev/null)

		if [[ $softLimit = +([0-9]) && $(( $curOpenFiles+1000 )) -ge $softLimit ]] ; then
			#$NLS-E$ "Current limit of a maximum number of open files is too small. It is recommended to increase this limit by 1000 as installation may fail."
    		Err_report warn "`nls_replace 'KCI1359W'`"

			if [ -z "$PARM_FILE" ] ; then
				repeat=1
				while [ $repeat -eq 1 ]; do
					#$NLS-E$ "\nContinue with this installation [ y or n; \"y\" is default ]?  \c"
					print "\n`nls_replace 'KCI1054I'` [ 1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`; \"1\" `nls_replace 'KCI0199I'` ]?  \c"

					read
					[ -z "$REPLY" ] && REPLY=1
					case $REPLY in
						n*|N*|2)
							cleanUp 0  # Exit.
							repeat=0
						;;

						y*|Y*|1)
							repeat=0
						;;
					esac
				done
			fi
		fi
	fi
}

function checkProcesses { #---------------------------------------------#IV51017
  # See if there is a CMS or if there are any agents that are still     #IV51017
  # running. If so, warn the user.                                      #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset pFile=$CANDLEHOME/registry/proddsc.tbl
  typeset rFile=$CANDLEHOME/config/.ConfigData/RunInfo
  typeset PID desc  msg pc proc

  [ -r $rFile ] || return  # It must be a new install if there is no RunInfo file.

  # Determine whether it's BigK or dinkySQL.

  fieldSep=$(grep "FIELDSEP" $rFile | cut -d "=" -f2 2> /dev/null)

  # See if there are any agents/CMS's running from this <candlehome> on this machine.

  fld123=$(cinfoRunning)  # fld123 -> HOST|ms|35594|TEMS|itmuser
  fld23=$(print "$fld123" | cut -d "$fieldSep" -f2-5)  # fld23 -> ms|35594|TEMS|itmuser
  [ -n "$fld23" ] || return


  # Check to see if the processes are actually running.

  abort="N"

  for proc in $fld23
  do
    pc=$(print "$proc" | cut -d "$fieldSep" -f1)
    PID=$(print "$proc" | cut -d "$fieldSep" -f2)
    userid=$(print "$proc" | cut -d "$fieldSep" -f4)


    if [[ "$thisUser" != "root"  && "$thisUser" != "$userid" ]]
    then
      #$NLS-E$ "The userid associated with the $pc product code and $PID process id does not match the current user"
      Err_report warn "`nls_replace 'KCI0244I' \\"$pc\\" \\"$PID\\"`"
      abort="Y"
      continue
    fi

    if [ -z "$msg" ]
    then
	#$NLS-E$ "\\nThe following processes are currently running: \\n"
	msg="\\n`nls_replace 'KCI1049I'` \\n"
    fi
  done


  # IZ17921 check to see if we should abort

    if [[ $abort = "Y" ]]
    then
	#$NLS-E$ "KCI0245E A running process(s) was detected whose associated userid does not match the current userid.  The install will be aborted"
	Err_report fail "`nls_replace 'KCI0245E'`"
    fi

  if [ -n "$msg" ]
  then
    print "$msg"

    for proc in $fld23
    do
      pc=$(print "$proc" | cut -d "$fieldSep" -f1)

      # Get the product description from the proddsc.tbl file.  If there are running processes,
      # then this file should exist.

      if [ -r $pFile ]
      then
        desc=$(grep "^$pc|" $pFile |head -1 | cut -d"|" -f2)  # desc -> "NetView/AIX Alert Adapter"
        [ -n "$desc" ] || desc=$pc  # Just in case.
      fi

      PID=$(print "$proc" | cut -d "$fieldSep" -f2)

	  #$NLS-E$ "  Product = $desc  PID = $PID"
      print "  `nls_replace 'KCI1050I'` = $desc  PID = $PID"
    done

    # Check for silent mode
    if [ -n "$silentMode" ]
    then
      # See if they ask to ignore the running proceses (looking for something like "INSTALL_IGNORE_RUNNING_PROCESSES=YES")
      [ -f "$PARM_FILE" ] && continueInstall=$(grep -i "INSTALL_IGNORE_RUNNING_PROCESSES" $PARM_FILE 2>/dev/null | grep -i "y" 2>/dev/null)

      # Warn and exit if they didn't ask to ignore running processes
      if [ -z "$continueInstall" ]
      then
	    #$NLS-E$ "Silent install aborting due to running processes detected on this CandleHome."
        Err_report msgs "`nls_replace 'KCI1051I'`"
        cleanUp 1	# Exit - cc=1
      fi

      # Otherwise just continue with the install
	  #$NLS-E$ "Silent install option to ignore running processes detected"
      Err_report warn "`nls_replace 'KCI1052I'`"
      cd $CANDLEHOME/bin
      restart_proc stop
	  restart_proc_rc=$?
	  if [[ $restart_proc_rc != 0 ]]
	  then
	    #$NLS-E$ "Silent install aborting due to stopping TEMS failure. Chcek logs for more details."
        Err_report msgs "`nls_replace 'KCI1232I'`"
		#$NLS-E$ "You can try to manually stop TEMS and again run installation."
		Err_report fail "`nls_replace 'KCI1234I'`"
	  fi
      cd -
      return
    fi

	repeat=1
	while [ $repeat -eq 1 ]; do
		#$NLS-E$ "Existing products found to be running will be restarted during installation."
    	Err_report warn "`nls_replace 'KCI1053I'`"
		#$NLS-E$ "\nContinue with this installation [ y or n; \"y\" is default ]?  \c"
    	print "\n`nls_replace 'KCI1054I'` [ 1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`; \"1\" `nls_replace 'KCI0199I'` ]?  \c"

    	read
    	[ -z "$REPLY" ] && REPLY=1
    	case $REPLY in

      		n*|N*|2) cleanUp 0  # Exit.
      			repeat=0
	    	 ;;

          	y*|Y*|1)
          	    repeat=0
     			cd $CANDLEHOME/bin
     			restart_proc stop
				restart_proc_rc=$?
				if [[ $restart_proc_rc != 0 ]]
				then
					#$NLS-E$ "Install aborting due to stopping TEMS failure. Chcek logs for more details."
					Err_report msgs "`nls_replace 'KCI1233I'`"
					#$NLS-E$ "You can try to manually stop TEMS and again run installation."
					Err_report fail "`nls_replace 'KCI1234I'`"
				fi
     			cd -
             ;;
    	esac
	done
    echo
  fi
}

function BspWarn { #----------------------------------------------------#IV51017
  # Warn user about the dangers of installing over an old Roma          #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  #$NLS-E$ "****************** WARNING ***************************"
  Err_report msgs "`nls_replace 'KCI1055I'`"
  #$NLS-E$ "You are installing eBP over an existing BSP $LDAPVer"
  #$NLS-E$ "installation.   This will migrate your LDAP to the"
  #$NLS-E$ "newer version,  and requires your LDAP server to be"
  #$NLS-E$ "shut down.  Please ensure that there are no remote"
  #$NLS-E$ "BSP applications  using this server.  If this file"
  #$NLS-E$ "system is accessible from other systems,  please"
  #$NLS-E$ "ensure that there are no applications running on"
  #$NLS-E$ "other systems from this CandleHome."

  Err_report msgs "`nls_replace 'KCI1056I' $LDAPVer`"
  Err_report msgs "`nls_replace 'KCI1057I'`"
  Err_report msgs "`nls_replace 'KCI1058I'`"
  Err_report msgs "`nls_replace 'KCI1059I'`"
  Err_report msgs " "
  #$NLS-E$ "Please stop your LDAP server now if it's currently running."
  Err_report msgs "`nls_replace 'KCI1060I' LDAP`"
  #$NLS-E$ "****************** WARNING ***************************"
  Err_report msgs "`nls_replace 'KCI1055I'`"
  sleep 5
}

function Bsp21Rename { #------------------------------------------------#IV51017
  # Routine renames obsolete Roma 2.1 directory names for a BSP 3.1     #IV51017
  # install. This entire routine can be removed when 2.1 is no longer   #IV51017
  # supported.                                                          #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  RegBackup=$1

  # We don't do any renames except for a prior BSP 2.1
  [ -r "$RegBackup/Pre_CT99_Roma" ] || return

  oldName=$(cat $RegBackup/Pre_CT99_Roma|head -1)
  oldPath=${oldName%/ldap}
  oldArch=${oldPath#$CANDLEHOME/roma/}
  typeset thisOS=$(uname -s)
  #
  # Roma directory remap from BSP 2.1->3.1
  # 2.1	-> 3.1
  # aix4 	-> aix42
  # aix43 	-> aix42
  # hp10 	-> hp11
  # hp102 	-> hp11
  # sol25 	-> sol26
  # sol273 	-> sol26

  # New install paths for BSP 3.1
  [ "$thisOS" = "AIX" ] && newPath=$CANDLEHOME/roma/aix42
  [ "$thisOS" = "HP-UX" ] && newPath=$CANDLEHOME/roma/hp11
  [ "$thisOS" = "SunOS" ] && newPath=$CANDLEHOME/roma/sol26

  # If the new path already exists as a file, rename it
  [ -f $newPath ] && mv $newPath ${newPath}__RenamedFile

  # If the old path matches the new one, we're done
  [ "$oldPath" = "newPath" ]  && return

  # Rename any (known) old path to the new one
  [ "$oldArch" = "aix4" ]  && mv $oldPath $newPath
  [ "$oldArch" = "aix43" ] && mv $oldPath $newPath
  [ "$oldArch" = "hp10" ]  && mv $oldPath $newPath
  [ "$oldArch" = "hp102" ] && mv $oldPath $newPath
  [ "$oldArch" = "sol25" ]  && mv $oldPath $newPath
  [ "$oldArch" = "sol273" ] && mv $oldPath $newPath

  # Reset the tag file to the new name
  find $newPath -name ldap -print|head -1 > $RegBackup/Pre_CT99_Roma
}

function shoPrequis { #-------------------------------------------------#IV51017
  # Display the license text in command-line mode.                      #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
#$NLS-E$ "\n    Before installing IBM Tivoli Monitoring agents, you must"
#$NLS-E$ "install at least one IBM Tivoli Enterprise Monitoring Server.  You"
#$NLS-E$ "will need the host name or IP address and port number for the"
#$NLS-E$ "monitoring server to configure any agents.\n"
echo
nls_replace 'KCI1061I'
nls_replace 'KCI1062I'
echo
sleep 5
}

shoLicense()
{
  which nroff  2> /dev/null > /dev/null  # See if nroff is available
  if [ $? = 0 ]                          #  we just do "more" if it's not
  then
    nroff $license | more -s
    print ""
  else
    more -s $license
  fi
  print " "
}

function getVFNames { #-------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  typeset -i i=0
  typeset arch fullName name pc verName

  ls $CANDLEHOME/registry/*.ver > /dev/null 2>&1
  [ $? -eq 0 ] || return  # Nothing to process.

  for name in $(ls $CANDLEHOME/registry/*.ver)
  do
    verName=${name##*/}  # Remove the pathname.
    pc=$(print $verName | cut -c1-2 2> /dev/null)   # Get the product code.
    [[ $pc = @(cz|or|ox|oy|ul|um|ux) ]] || continue   # Only a few products need special permissions.

    arch=$(print $verName | cut -c3- 2> /dev/null)  # Determine the bin arch.
    [[ $arch = @(aix|hp|sol)* ]] || continue  # Skip CMS data pieces.

    vfNames[i]=$verName
    i=i+1
  done

  procNum=${#vfNames[*]}
}

function checkIconv { #-------------------------------------------------#IV51017
  # check if iconv exists in the system                                 #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  iconv_path="$(whence iconv 2>/dev/null)"
  if [ -z "$iconv_path" ]
  then
    # iconv does not exist in the system.
    # This message should not be globalised.
    print "************************* WARNING *************************"
    print "The program iconv is not available and should be installed."
    print "************************* WARNING *************************"
  fi
}

# This cleans up the system before and after we run the entire install
# - mainly used to clean our /tmp
cleanupSystem()
{

  # Our plugin work dir
  rm -rf /tmp/plugin_tmp > /dev/null 2>&1

  #on HP-IA, the temporary directory is /var/tmp
  if [ -d /var/tmp/plugin_tmp ]
  then
      rm -rf /var/tmp/plugin_tmp > /dev/null 2>&1
  fi

}

function checkSYHDupgrade { #-------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  if [ "$1" = "post" ]
  then
  	if [ -z "$syNotInstalled" ]
  	then
      syVerFile=$(ls $curRegPath/sy*.ver 2>/dev/null | grep -v syt.*ver)
      if [ -f "$syVerFile" ]
      then
        # Read ver, rel and VRMF. Truncate space and 0
        newSYver=$(grep "^ver *=" $syVerFile 2>/dev/null | cut -d"=" -f2)
        newSYver=${newSYver##+( )+(0)}
        newSYrel=$(grep "^rel *=" $syVerFile 2>/dev/null | cut -d"=" -f2)
        newSYrel=${newSYrel##+( )+(0)}
        newSYvrmf=$(grep "^VRMF *=" $syVerFile 2>/dev/null | cut -d"=" -f2)
        newSYvrmf=${newSYvrmf##+( )+(0)}

        oldSYverTrim="$(echo $oldSYver | sed 's/^[ 0]+//')"

        # not in deploy or in deploy and oldVer=610
        [[ $DEPLOY_MODE = "" || "$oldSYverTrim" = "610" ]] && [[ "$newSYver" != "$oldSYver" || "$newSYrel" != "$oldSYrel" || "$newSYvrmf" != "$oldSYvrmf" ]] && syNotStart=yes
      fi
    fi

    if [ -z "$hdNotInstalled" ]
    then
      hdVerFile=$(ls $curRegPath/hd*.ver 2>/dev/null | grep -v hdt.*ver)
      if [ -f "$hdVerFile" ]
      then
        # Read ver, rel and VRMF. Truncate space and 0
        newHDver=$(grep "^ver *=" $hdVerFile 2>/dev/null | cut -d"=" -f2)
        newHDver=${newHDver##+( )+(0)}
        newHDrel=$(grep "^rel *=" $hdVerFile 2>/dev/null | cut -d"=" -f2)
        newHDrel=${newHDrel##+( )+(0)}
        newHDvrmf=$(grep "^VRMF *=" $hdVerFile 2>/dev/null | cut -d"=" -f2)
        newHDvrmf=${newHDvrmf##+( )+(0)}

        oldHDverTrim="$(echo $oldHDver | sed 's/^[ 0]+//')"

        # not in deploy or in deploy and oldVer=610
        [[ $DEPLOY_MODE = "" || "$oldHDverTrim" = "610" ]] && [[ "$newHDver" != "$oldHDver" || "$newHDrel" != "$oldHDrel" || "$newHDvrmf" != "$oldHDvrmf" ]] && hdNotStart=yes
      fi
    fi
  fi

  if [ "$1" = "pre" ]
  then
    syVerFile=$(ls $curRegPath/sy*.ver 2>/dev/null | grep -v syt.*ver)
    if [ -f "$syVerFile" ]
    then
      # Read ver, rel and VRMF. Truncate space and 0
      oldSYver=$(grep "^ver *=" $syVerFile 2>/dev/null | cut -d"=" -f2)
      oldSYver=${oldSYver##+( )+(0)}
      oldSYrel=$(grep "^rel *=" $syVerFile 2>/dev/null | cut -d"=" -f2)
      oldSYrel=${oldSYrel##+( )+(0)}
      oldSYvrmf=$(grep "^VRMF *=" $syVerFile 2>/dev/null | cut -d"=" -f2)
      oldSYvrmf=${oldSYvrmf##+( )+(0)}

    else
      syNotInstalled=yes
    fi
    hdVerFile=$(ls $curRegPath/hd*.ver 2>/dev/null | grep -v hdt.*ver)
    if [ -f "$hdVerFile" ]
    then
      # Read ver, rel and VRMF. Truncate space and 0
      oldHDver=$(grep "^ver *=" $hdVerFile 2>/dev/null | cut -d"=" -f2)
      oldHDver=${oldHDver##+( )+(0)}
      oldHDrel=$(grep "^rel *=" $hdVerFile 2>/dev/null | cut -d"=" -f2)
      oldHDrel=${oldHDrel##+( )+(0)}
      oldHDvrmf=$(grep "^VRMF *=" $hdVerFile 2>/dev/null | cut -d"=" -f2)
      oldHDvrmf=${oldHDvrmf##+( )+(0)}
    else
      hdNotInstalled=yes
    fi
  fi
}

function isPlatformSupported { #----------------------------------------#IV51017
  # This function is used to determine if the ITM installer is running  #IV51017
  # on a supported platform. The platform is supported when there is a  #IV51017
  # JRE for the current architecture and a appropriate Gskit package.   #IV51017
  # If the ITM installer is running on supported platform, then  "0" is #IV51017
  # returned.                                                           #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  # @40 check for Application Support Files only install
  if [ "$PLATFORM_SUPPORTED_CHECK" = "NO" ]; then
    # Extract the .ver file for this new media installer
    unloadTmpFile $ciVFile $PKGLOC/$ciTar
    mediaFile=$CANDLEHOME/tmp/$ciVFile
    mediaVer=$(grep "^ver *=" $mediaFile 2> /dev/null | cut -d"=" -f2)
    # Also get the version for the current installer
    currentFile=$CANDLEHOME/registry/$ciVFile
    currentVer=$(grep "^ver *=" $currentFile 2> /dev/null | cut -d"=" -f2)
    if [[ "$currentVer" -ge "$mediaVer" ]]; then
    # At this point, since only Application Support Files are being installed, no platform support checks should be performed.
      return 0
    else
      # IV13614 We are tring to install in a directory that does not have a current ITM environment (version is backlevel from this media)
      # Display an appropriate message so user can correct this, rather than letting it fall through and getting a generic message.
      # $NLS-E$ "KCI1297E The installer level [$1] on the application support media is higher than the installer level [$2] which is currently installed.  Installation terminated"
       Err_report fail "`nls_replace 'KCI1297I' $mediaVer $currentVer`"
    fi

  fi
  arch=$1 #li6242, li6266, aix533 ect.
  tmpDir=$CANDLEHOME/tmp
  jreOS=${arch%%[0-9]*} #aix, li
  cd $tmpDir
  if [ $? -eq 0 ]
  then
    rm -rf tmp > /dev/null 2>&1
  else
    # $NLS-E$ "\n\"$CANDLEHOME\tmp" does not exist"
    Err_report fail "`nls_replace 'KCI1069I' \\"$tmpDir\\"`"
  fi

  # Copy the jr<arch>.ver files from the media to the $CANDLEHOME/tmp/tmp directory if they exist.
  # Otherwise,  assume they were extracted in cienv.tar and
  # copy all of the jr*.ver files from cienv.tar to the $CANDLEHOME/tmp/tmp directory.

  jreOS=${cArch%%[0-9]*} #aix, li, sol, hp, etc
  [ -d tmp ] || mkdir tmp > /dev/null 2>&1

  for file in `ls $PKGLOC/jr$jreOS*.tar $PKGLOC/jr$jreOS*.ctp 2> /dev/null`
  do
    pkgName=$(basename $file)
    pkgName=${pkgName%.*}
    if [ -f $PKGLOC/$pkgName.ver ]
    then
      cp $PKGLOC/$pkgName.ver tmp/$pkgName.ver > /dev/null 2>&1
    else
	  tar -$tarOpt $PKGLOC/cienv.tar tmp/$pkgName.ver > /dev/null 2>&1
	fi
  done
  # Check if there is supported JRE for current arch
  export LANG=C
  jre=$(grep -l "^runArch *= *$arch" tmp/jr$jreOS*.ver 2> /dev/null)
  export LANG=$saveLang
  if [[ -z "$jre" ]] #if no JRE found
  then
    rm -rf tmp > /dev/null 2>&1
    cd $CANDLEHOME
    return 1 #no jre found, platform not supported
  fi
  rm -rf tmp > /dev/null 2>&1
  for file in `ls $PKGLOC/GSKit/gs$jreOS* 2> /dev/null`
  do
    pkgName=$(basename $file)
    pkgName=${pkgName%.*}
	unloadTmpFile $pkgName.ver $file
  done
  # Check if there is GSkit package for current arch
  export LANG=C
  gskit=$(grep -l "^runArch *= *$arch" tmp/gs$jreOS*.ver 2> /dev/null)
  export LANG=$saveLang
  rm -rf tmp > /dev/null 2>&1
  cd $CANDLEHOME
  [[ -z "$gskit" ]] && return 2 #no gskit found, platform not supported
  return 0
}

function checkPlatformBeforeUpgrade { #---------------------------------#IV51017
  # Check if a platform is supported before installing anything.        #IV51017
  # Unpack files to the CANDLEHOME/tmp directory and run validation.    #IV51017
  # It is assumed that $CANDLEHOME is the current directory.            #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
    cd $CANDLETMP
    tar xvf $PKGLOC/cienv.tar tmp/archdsc.tbl >/dev/null 2>&1
    tar xvf $PKGLOC/cienv.tar bin >/dev/null 2>&1

    # Check if the installed CI is greater than the media CI            IV67172
    if [ -r "$curRegPath/$ciVFile" ]                                  # IV67172
    then                                                              # IV67172
      [[ -z "$newRegPath" ]] && newRegPath=$CANDLETMP/tmp ;           # IV67172
      unloadTmpFile $ciVFile $PKGLOC/$ciTar ;  # Extract the .ver file. IV67172
      checkLevel $ciVFile  # Compare the media and installed CI.        IV67172
      if [ "$levelStrg" = "Older" ]                                   # IV67172
      then                                                            # IV67172
         cp "$CANDLEHOME/bin/dynarch.shl" "$CANDLETMP/bin/dynarch.shl" ; # IV67172
         cp "$CANDLEHOME/registry/archdsc.tbl" "$CANDLETMP/tmp/archdsc.tbl" ; # IV67172
      fi                                                              # IV67172
    fi                                                                # IV67172

    . $CANDLETMP/bin/dynarch.shl $CANDLETMP/tmp/archdsc.tbl
    cd $CANDLEHOME
    rm -rf $CANDLETMP/tmp
    rm -rf $CANDLETMP/bin

    if [ -n $cArch ]
    then
        isPlatformSupported $cArch
        [[ $? -ne 0 ]] && Err_report fail "`nls_replace 'KCI1148E'`"
        platformChecked="YES"
    fi
}

function blockOmegamonUpgrade { #---------------------------------------#IV51017
  # block upgrade from releases older than ITM6.1                       #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  export LANG=C
  productsList="cq ms cj sy hd lz ux um"
  for product in $productsList
  do
    for file in `ls $CANDLEHOME/registry/$product*.ver 2>/dev/null | grep $pc[a-z]*[0-9][0-9]*\.ver`
    do
      cVer=$(grep "^ver *=" $file 2>/dev/null | cut -d"=" -f2)
      [[ "$cVer" -lt 610 ]] && Err_report fail "`nls_replace 'KCI1256E'`"
    done
  done
  export LANG=$saveLang
}

function validateSecureGroup { #----------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  # first check if secureMode file exists
  secureFile=$CANDLEHOME/registry/SecureMode
  if [[ -f "$secureFile" ]];
  then
    line=$(cat $secureFile 2>/dev/null)
    mode=$(print $line | cut -f1 -d:)
    group=$(print $line | cut -f3 -d:)
    if [[ -n "$group" && "$group" != "$SECURE_GROUP" ]];
    then
      # $NLS-E$ KCI1347E Group $1 is different than common group $2 previously used to secure IBM Tivoli Monitoring environment.
      Err_report fail "`nls_replace 'KCI1347E' $SECURE_GROUP $group`"
    fi
  fi

  # verify if group name is valid and user is a member of SECURE_GROUP
  testFile=/tmp/secureTest.$$
  touch "$testFile" > /dev/null 2>&1
  if [[ $? = 0 ]];
  then
  	# testFile created without any problem, use chgrp to validate group
    chgrp "$SECURE_GROUP" "$testFile" > /dev/null 2>&1
    rcGroup=$?
    rm -f "$testFile" > /dev/null 2>&1
    [[ "$rcGroup" = 0 ]] && valid=yes
  else
    # use alternative method when testFile can't be created, this check is only for non root user
    [[ "${thisRealUser}" = "root" ]] && [[ -n "$(groups | grep -w $SECURE_GROUP)" ]] && valid=yes #IZ94626.1
  fi

  [[ -z "$valid" ]] && Err_report fail "`nls_replace 'KCI1346E' $SECURE_GROUP $thisUser`" # $NLS-E$ The specified group $1 is not a valid group or user $2 is not a member of this group.
}


function checkMetaprobes { #--------------------------------------------#IV51017
  # inform user about possible issues if metaprobes are installed       #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  # check only for base itm images
  if [[ "$CDNAME" = tmv+([0-9]) ]];
  then
    # determine system architecture
    [[ -f "$CANDLEHOME/bin/dynarch.shl" ]] || return
    . $CANDLEHOME/bin/dynarch.shl $CANDLEHOME/registry/archdsc.tbl 2>/dev/null

    # check if system architecture is 64bit xLinux
    [[ "$cArch" = "lx8266" ]] || return

    # check if tems is available on installation media
    istemscd=$(ls $PKGLOC/ms*.jar 2>/dev/null | grep -v mstms | head -1)
    [[ -z "$istemscd" ]] && return

    # check if installed tems is pre 6.3
    temsverfile=$(ls $CANDLEHOME/registry/ms*.ver 2>/dev/null | grep -v mstms | head -1)
    [[ -z "$temsverfile" ]] && return
    msver=$(grep "^ver *=" $temsverfile 2>/dev/null | cut -d"=" -f2)
    [[ "$msver" -ge 630 ]] && return

    # check if cf metaprobe is installed
    iscf=$(ls $CANDLEHOME/registry/cf*.ver 2>/dev/null | grep -v cftms)
    [[ -n "$iscf" ]] && displayMetaprobeWarning && return

    # check if dp metaprobe is installed and dp ver is less than 511
    isdp=$(ls $CANDLEHOME/registry/dp*.ver 2>/dev/null | grep -v dptms)
    dpver=$(grep "^ver *=" $CANDLEHOME/registry/dptms.ver 2>/dev/null | cut -d"=" -f2)
    [[ -n "$isdp" ]] && [[ "$dpver" -lt 511 ]] && displayMetaprobeWarning && return

    # check if ip metaprobe is installed and ip ver is less than 420
    isip=$(ls $CANDLEHOME/registry/ip*.ver  2>/dev/null | grep -v iptms)
    ipver=$(grep "^ver *=" $CANDLEHOME/registry/iptms.ver 2>/dev/null | cut -d"=" -f2)
    [[ -n "$isip" ]] && [[ "$ipver" -lt 420 ]] && displayMetaprobeWarning && return

    # check if m5 metaprobe is installed and m5 ver is less than 510
    ism5=$(ls $CANDLEHOME/registry/m5*.ver 2>/dev/null | grep -v m5tms)
    m5ver=$(grep "^ver *=" $CANDLEHOME/registry/m5tms.ver 2>/dev/null | cut -d"=" -f2)
    [[ -n "$ism5" ]] && [[ "$m5ver" -lt 510 ]] && displayMetaprobeWarning && return
  fi
}

function displayMetaprobeWarning { #------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  #$NLS-E$ The launched installation image contains a 64-bit Tivoli Enterprise Monitoring Server which is not compatible with the currently installed Meta Probes. Continuing with this installation will disable the Meta Probes until the appropriate application support files for Tivoli Enterprise Monitoring Server are installed. Please refer to the following technote for more information: http://www-01.ibm.com/support/docview.wss?uid=swg#
  msg=$(nls_replace 'KCI1363W' "http://www.ibm.com/support/docview.wss?uid=swg21623643")
  print ""
  print $msg
  print $msg >> $logFile
}

function display63Prereqs { #-------------------------------------------#IV51017
  # display information about prerequisites introduced in version 6.3   #IV51017
  #---------------------------------------------------------------------#IV51017
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV51017
                                                                        #IV51017
  if [[ "$CDNAME" = "tmv630" ]];
  then
    #$NLS-E$ IBM Tivoli Monitoring version 6.3 introduces a number of platform improvements which may require operating system updates. It is important that you ensure this system meets these requirements prior to installing or updating your installation. Please refer to the following technote for more information: http://www.ibm.com/support/docview.wss?uid=swg#
    msg=$(nls_replace 'KCI1362W' "http://www.ibm.com/support/docview.wss?uid=swg21623341")
    print ""
    print $msg
  fi
}

function getCANDLEXXXX { #----------------------------------------------#IV67523
  # Obtain CANDLEHOME and CANDLEDATA settings from the user. We must be #IV67523
  # called for CANDLEHOME first, then CANDLEDATA (some CANDLEDATA       #IV67523
  # things depend on this.)                                             #IV67523
  #                                                                     #IV67523
  # This routine works in both interactive and silent mode. In          #IV67523
  # interactive mode, we prompt for values and confirmations. In silent #IV67523
  # mode, we get the values from the source CANDLEXXXX variables. In    #IV67523
  # either case, we validate the input and try to create the directory. #IV67523
  #                                                                     #IV67523
  # When we're done, we set the CANDLEXXXX variable to the resulting    #IV67523
  # value. By the time we do this, we know the directory exists, is     #IV67523
  # writable, and is fully canonicalized (".", "~", links, etc. are     #IV67523
  # all resolved).                                                      #IV67523
  #                                                                     #IV67523
  # If a failure occurs, we stop install.sh altogether using the        #IV67523
  # Err_report fail mechanism. The caller never regains control.        #IV67523
  #---------------------------------------------------------------------#IV67523
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV67523
                                                                        #IV67523
  typeset ivarname="${1}";                  # CANDLEHOME or CANDLEDATA  #IV67523
  typeset idefault="${2}";                  # /opt/IBM/ITM              #IV67523
                                                                        #IV67523
  typeset varvalue=$(eval 'print "${'${ivarname}'}"');                  #IV67523
                                                                        #IV67523
  typeset oval="";                          # output value (directory)  #IV67523
                                                                        #IV67523
  typeset enterprompt;                      # "Enter name of xxx..."    #IV67523
  typeset msg;                              # message to user           #IV67523
  typeset resp;                             # response from user        #IV67523
  typeset YorN;                             # response from user        #IV67523
  typeset x;                                # temporary variable        #IV67523
                                                                        #IV67523
  typeset CD2CHmark;                        # ${CANDLEDATA}/CANDLEHOME  #IV67523
  typeset CD2CHlink;                        # CD2CHmark resolved        #IV67523
                                                                        #IV67523
  typeset useCH4CD="no";                    # use CANDLEHOME for DATA?  #IV67523
                                                                        #IV67523
  if [[ -z "${silentMode}" ]] && [[ "${ivarname}" = "CANDLEDATA" ]] && [[ "${isSERVERimage}" = "yes" ]]; then #IV67523
    #$NLS-E$ "\nThis installation image contains the monitoring server or portal server." #IV67523
    #$NLS-E$ "\nThe monitoring server and portal server will not start with separate installation and data directories." #IV67523
    #$NLS-E$ "\nIf you want to install the monitoring server or portal server, you must use the installation directory for data." #IV67523
    #$NLS-E$ "\nIf you want to install a different component, you may specify a different location for data." #IV67523
    #$NLS-E$ "\nDo you want to place data in the installation directory or specify a different location?" #IV67523
    msg=$(nls_replace 'KCI1379I')                                       #IV67523
    #$NLS-E$ "\n[1-Use installation directory"                          #IV67523
    msg="${msg}\n[ 1-"$(nls_replace 'KCI1380I');                        #IV67523
    #$NLS-E$ ", 2-Specify another directory"                            #IV67523
    msg="${msg}, 2-"$(nls_replace 'KCI1381I');                          #IV67523
    #$NLS-E$ "; \"1\" is default"                                       #IV67523
    msg="${msg}; \"1\" "$(nls_replace 'KCI0199I');                      #IV67523
    msg="${msg} ] ";                                                    #IV67523
    while true; do                                                      #IV67523
      print -n "${msg}";                    # display the prompt        #IV67523
      read YorN;                            # get the answer            #IV67523
      case "${YorN}" in                                                 #IV67523
        y*|Y*|1|"")                                                     #IV67523
          useCH4CD="yes";                   # use CANDLEHOME            #IV67523
          break;                            # stop YorN prompt loop     #IV67523
          ;;                                # (case(y))                 #IV67523
        n*|N*|2)                            # nope                      #IV67523
          useCH4CD="no";                    # don't use CANDLEHOME      #IV67523
          break;                            # stop YorN prompt loop     #IV67523
          ;;                                # (case(n))                 #IV67523
        *)                                                              #IV67523
          #$NLS-E$ "invalid response, please enter \"1\" or \"2\""      #IV67523
          nls_replace 'KCI1075I' "\"1\"" "\"2\"";                       #IV67523
          ;;                                # (case(*))                 #IV67523
      esac;                                 # (case(YorN))              #IV67523
    done;                                   # (while(true))             #IV67523
  fi;                                       # (if(-n silent...))        #IV67523
                                                                        #IV67523
  #---------------------------------------------------------------------#IV67523
  # Get/check the value from the user. They may have passed it already  #IV67523
  # in an environment variable or we may have to prompt for it.         #IV67523
  #---------------------------------------------------------------------#IV67523
  if [[ "${useCH4CD}" = "yes" ]]; then      # use CANDLEHOME for...     #IV67523
     oval="${CANDLEHOME}";                  # ...CANDLEDATA             #IV67523
  elif [[ -z "${silentMode}" ]]; then       # interactive mode...       #IV67523
    if [[ "${ivarname}" = "CANDLEHOME" ]]; then                         #IV67523
      #$NLS-E$ "Enter the name of the IBM Tivoli Monitoring installation directory."; #IV67523
      enterprompt=$(nls_replace 'KCI1378I');                            #IV67523
    else                                                                #IV67523
      #$NLS-E$ "Enter the name of the IBM Tivoli Monitoring data directory."; #IV67523
      enterprompt=$(nls_replace 'KCI1377I');                            #IV67523
    fi;                                                                 #IV67523
                                                                        #IV67523
    while [[ -z "${oval}" ]]; do            # until we have output val  #IV67523
      if [[ -n "${varvalue}" ]]; then       # if CANDLEXXXX set for us  #IV67523
        resp="${varvalue}";                 # ...take it as our input   #IV67523
        varvalue="";                        # and forget for next time  #IV67523
      else                                  # prompt for it...          #IV67523
        #$NLS-E$ "\nEnter the name of..."                               #IV67523
        print "\n${enterprompt}";                                       #IV67523
        print -n $(nls_replace 'KCI1067I' "${idefault}")" ";            #IV67523
        read resp;                          # get user's answer         #IV67523
      fi;                                   # (else(-n $oval))          #IV67523
      resp="${resp##+([${IFS}])}";          # strip leading spaces      #IV77439
      resp="${resp%%+([${IFS}])}";          # strip trailing spaces     #IV77439
      [[ -z "${resp}" ]] && resp="${idefault}";                         #IV67523
                                                                        #IV67523
      #-----------------------------------------------------------------#IV67523
      # Validate/confirm the response.                                  #IV67523
      # - If CANDLEDATA==CANDLEHOME, ask if we can share.               #IV67523
      # - If it already exists, ask if we can use.                      #IV67523
      # - If it doesn't exist, ask if we can create.                    #IV67523
      #-----------------------------------------------------------------#IV67523
      if [[ "${resp}" != "${resp%%+(#)*}" ]]; then                      #IV67523
        #$NLS-E$ "\nDo not use the hash mark (#) character..."          #IV67523
        nls_replace 'KCI1360W';                                         #IV67523
      elif [[ "${resp}" != "${resp%%+([${IFS}])*}" ]]; then             #IV67523
        #$NLS-E$ "\n\"xxx\"  - blanks are not permitted..."             #IV67523
        nls_replace 'KCI1068I' "\"${resp}\"";                           #IV67523
      else                                                              #IV67523
        if [[ $(ls -d "${resp}" 2> /dev/null) = "" ]]; then             #IV67523
          #$NLS-E$ "\n$1 does not exist"                                #IV67523
          msg=$(nls_replace 'KCI1069I' "\"${resp}\"");                  #IV67523
          #$NLS-E$ "Try to create it... ?"                              #IV67523
          msg="${msg}\n"$(nls_replace 'KCI1070I');                      #IV67523
       else                                                             #IV67523
          x=$(resolveDir "${resp}");                                    #IV67523
          if [[ -z "${x}" ]]; then                                      #IV67523
            #$NLS-E$ "could not determine the full pathname of $1"      #IV67523
            nls_replace 'KCI1081E' "\"${oval}\"";                       #IV67523
            continue;                                                   #IV67523
          elif [[ ! -d "${x}" ]]; then                                  #IV67523
            #$NLS-E$ "$1 already exists but is not a directory."        #IV67523
            nls_replace 'KCI1382E' "\"${resp}\"";                       #IV67523
            continue;                                                   #IV67523
          elif [[ "${x}" != "${resp}" ]]; then                          #IV67523
            #$NLS-E$ "$1 resolves to $2. Using $3 going forward."       #IV67523
            nls_replace 'KCI1383I' "\"${resp}\"" "\"${x}\"" "\"${x}\""; #IV67523
            resp="${x}";                                                #IV67523
          fi;                                                           #IV67523
                                                                        #IV67523
          if [[ "${ivarname}" = "CANDLEDATA" ]]; then                   #IV67523
            CD2CHmark="${resp}/CANDLEHOME";                             #IV67523
            CD2CHlink=$(ls -l "${CD2CHmark}" 2> /dev/null);             #IV67523
            CD2CHlink="${CD2CHlink##* -> }";                            #IV67523
            if [[ -n "${CD2CHlink}" ]] && [[ "${CANDLEHOME}" != "${CD2CHlink}" ]]; then    #IV67523
              #$NLS-E$ "Data directory $1 is already in use by installation directory $2." #IV67523
              nls_replace 'KCI1433E' "\"${resp}\"" "\"${CD2CHlink}\"";                     #IV67523
              continue;                                                 #IV67523
            fi;                                                         #IV67523
          fi;                                                           #IV67523
                                                                        #IV67523
          if [[ "${ivarname}" = "CANDLEDATA" ]] && [[ "${resp}" = "${CANDLEHOME}" ]]; then #IV67523
            #$NLS-E$ "Installation directory and data directory are the same."             #IV67523
            msg=$(nls_replace 'KCI1384I');                              #IV67523
            #$NLS-E$ "OK to share it... ?"                              #IV67523
            msg="${msg}\n"$(nls_replace 'KCI1385I');                    #IV67523
          else                                                          #IV67523
            #$NLS-E$ "\nDirectory $1 already exists."                   #IV67523
            msg=$(nls_replace 'KCI1078I' "\"${resp}\"");                #IV67523
            #$NLS-E$ "OK to use it... ?"                                #IV67523
            msg="${msg}\n"$(nls_replace 'KCI1079I');                    #IV67523
          fi;                                                           #IV67523
        fi;                                                             #IV67523
                                                                        #IV67523
        if [[ "${ivarname}" = "CANDLEDATA" ]]; then                     #IV67523
          if [[ "${resp}" = "${CANDLEHOME}" ]]; then                    #IV67523
            : ;                                                         #IV67523
          elif [[ "${resp}" = "${CANDLEHOME}"* ]]; then                 #IV67523
            #$NLS-E$ "Data directory $1 cannot be a subdirectory of installation directory $2." #IV67523
            nls_replace 'KCI1386E' "\"${resp}\"" "\"${CANDLEHOME}\"";   #IV67523
            continue;                                                   #IV67523
          elif [[ "${CANDLEHOME}" = "${resp}"* ]]; then #IV67523
            #$NLS-E$ "Data directory $1 cannot be a parent of installation directory $2." #IV67523
            nls_replace 'KCI1387E' "\"${resp}\"" "\"${CANDLEHOME}\"";   #IV67523
            continue;                                                   #IV67523
          fi;                                                           #IV67523
        fi;                                                             #IV67523
                                                                        #IV67523
        msg="${msg} [ 1-"$(nls_replace 'KCI0197I');                     #IV67523
        msg="${msg}, 2-"$(nls_replace 'KCI0198I');                      #IV67523
        msg="${msg}; \"1\" "$(nls_replace 'KCI0199I');                  #IV67523
        msg="${msg} ]? ";                                               #IV67523
        while true; do                                                  #IV67523
          print -n "${msg}";                # display the prompt        #IV67523
          read YorN;                        # get the answer            #IV67523
          case "${YorN}" in                                             #IV67523
            y*|Y*|1|"")                                                 #IV67523
              oval="${resp}";               # lock in the response      #IV67523
              break;                        # stop YorN prompt loop     #IV67523
              ;;                            # (case(y))                 #IV67523
            n*|N*|2)                        # nope                      #IV67523
              break;                        # stop YorN prompt loop     #IV67523
              ;;                            # (case(n))                 #IV67523
            *)                                                          #IV67523
              #$NLS-E$ "invalid response, please enter \"y\" or \"n\""  #IV67523
              nls_replace 'KCI1075I' "\"1\"" "\"2\"";                   #IV67523
              ;;                            # (case(*))                 #IV67523
          esac;                             # (case(YorN))              #IV67523
        done;                               # (while(true))             #IV67523
      fi;                                   # (else($resp...))          #IV67523
    done;                                   # (while(-z ${oval}))       #IV67523
  else                                      # silent mode...            #IV67523
    resp="${varvalue##+([${IFS}])}";        # strip leading spaces      #IV67523
    resp="${resp%%+([${IFS}])}";            # strip trailing spaces     #IV67523
    [[ -z "${resp}" ]] && resp="${idefault}";                           #IV67523
                                                                        #IV67523
    #-------------------------------------------------------------------#IV67523
    # If the directory already exists, resolve it and make sure it's    #IV67523
    # not a data already in use by another CANDLEHOME.                  #IV67523
    #-------------------------------------------------------------------#IV67523
    if [[ $(ls -d "${resp}" 2> /dev/null) != "" ]]; then                #IV67523
      x=$(resolveDir "${resp}");                                        #IV67523
      if [[ -z "${x}" ]]; then                                          #IV67523
        #$NLS-E$ "could not determine the full pathname of $1"          #IV67523
        msg=$(nls_replace 'KCI1081E' "\"${oval}\"");                    #IV67523
        Err_report fail "${msg}";                                       #IV67523
      elif [[ "${x}" != "${resp}" ]]; then                              #IV67523
        #$NLS-E$ $1 resolves to $2. Using $3 going forward."            #IV67523
        nls_replace 'KCI1383I' "\"${resp}\"" "\"${x}\"" "\"${x}\"";     #IV67523
        resp="${x}";                                                    #IV67523
      fi;                                   # (elif(x!=resp))           #IV67523
                                                                        #IV67523
      if [[ "${ivarname}" = "CANDLEDATA" ]]; then                       #IV67523
        CD2CHmark="${resp}/CANDLEHOME";                                 #IV67523
        CD2CHlink=$(ls -l "${CD2CHmark}" 2> /dev/null);                 #IV67523
        CD2CHlink="${CD2CHlink##* -> }";                                #IV67523
        if [[ -n "${CD2CHlink}" ]] && [[ "${CANDLEHOME}" != "${CD2CHlink}" ]]; then    #IV67523
          #$NLS-E$ "Data directory $1 is already in use by installation directory $2." #IV67523
          msg=$(nls_replace 'KCI1433E' "\"${resp}\"" "\"${CD2CHlink}\"");              #IV67523
          Err_report fail "${msg}";                                     #IV67523
        fi;                                 # (if(-n CD2CHlink...))     #IV67523
      fi;                                   # (if(ivarname=CANDLEDATA)) #IV67523
    fi;                                     # (if(-d resp))             #IV67523
                                                                        #IV67523
    #-------------------------------------------------------------------#IV67523
    # Since we're in silent mode here, all of these must either a) set  #IV67523
    # 'oval' to a valid directory or b) call 'Err_report fail' so we    #IV67523
    # stop the install process.                                         #IV67523
    #-------------------------------------------------------------------#IV67523
    if [[ -z "${resp}" ]] && [[ "${ivarname}" = "CANDLEDATA" ]]; then   #IV67523
      # Silent mode requires CANDLEHOME. So, if no CANDLEDATA is set,   #IV67523
      # we just reuse CANDLEHOME for data.                              #IV67523
      oval="${CANDLEHOME}";                                             #IV67523
    elif [[ -z "${resp}" ]]; then                                       #IV67523
      # Silent mode requires CANDLEHOME. This is checked earlier so we  #IV67523
      # should never get here for that variable.                        #IV67523
      oval="${idefault}";                                               #IV67523
    elif [[ "${resp}" != "${resp%%+(#)*}" ]]; then                      #IV67523
      #$NLS-E$ "\nDo not use the hash mark (#) character..."            #IV67523
      msg=$(nls_replace 'KCI1360W');                                    #IV67523
      Err_report fail "${msg}";                                         #IV67523
    elif [[ "${resp}" != "${resp%%+([${IFS}])*}" ]]; then               #IV67523
      #$NLS-E$ "\n$1 - blanks are not permitted..."                     #IV67523
      msg=$(nls_replace 'KCI1068I' "\"${resp}\"");                      #IV67523
      Err_report fail "${msg}";                                         #IV67523
    elif [[ -a "${resp}" ]] && [[ ! -d "${resp}" ]]; then               #IV67523
      #$NLS-E$ "$1 already exists but is not a directory."              #IV67523
      msg=$(nls_replace 'KCI1382I' "\"${resp}\"");                      #IV67523
      Err_report fail "${msg}";                                         #IV67523
    elif [[ "${ivarname}" = "CANDLEDATA" ]] && [[ "${resp}" = "${CANDLEHOME}" ]]; then #IV67523
      oval="${resp}";                       # data/code shared in CH    #IV67523
    elif [[ "${ivarname}" = "CANDLEDATA" ]] && [[ "${resp}" = "${CANDLEHOME}"* ]]; then #IV67523
      #$NLS-E$ "Data directory $1 cannot be a subdirectory of installation directory $2." #IV67523
      msg=$(nls_replace 'KCI1386E' "\"${resp}\"" "\"${CANDLEHOME}\"");  #IV67523
      Err_report fail "${msg}";                                         #IV67523
    elif [[ "${ivarname}" = "CANDLEDATA" ]] && [[ "${CANDLEHOME}" = "${resp}"* ]]; then #IV67523
      #$NLS-E$ "Data directory $1 cannot be a parent of installation directory $2." #IV67523
      msg=$(nls_replace 'KCI1387E' "\"${resp}\"" "\"${CANDLEHOME}\"");  #IV67523
      Err_report fail "${msg}";                                         #IV67523
    else                                                                #IV67523
      oval="${resp}";                       # lock in the value         #IV67523
    fi;                                     # (else(resp...))           #IV67523
  fi;                                       # (else(-n silentMode))     #IV67523
                                                                        #IV67523
  #---------------------------------------------------------------------#IV67523
  # If the directory does not exist, try to create it.                  #IV67523
  #---------------------------------------------------------------------#IV67523
  if [[ ! -d "${oval}" ]]; then                                         #IV67523
    #$NLS-E$ "Creating directory $1...";                                #IV67523
    nls_replace 'KCI1388I' "\"${oval}\"";                               #IV67523
    x=$(mkdir -p "${oval}" 2>&1);                                       #IV67523
    rc=${?};                                                            #IV67523
    if [[ ${rc} -ne 0 ]]; then                                          #IV67523
      #$NLS-E$ "An error occurred while creating $1.\n...xxx"           #IV67523
      msg=$(nls_replace 'KCI1423E' "\"${oval}\"" "${rc}");              #IV67523
      msg="${msg}\n...${x}";                                            #IV67523
      Err_report fail "${msg}";                                         #IV67523
    else                                                                #IV67523
      chmod a-s "${oval}" > /dev/null 2>&1                              #IV67523
                                                                        #IV67523
      x=$(resolveDir "${oval}");                                        #IV67523
      if [[ -z "${x}" ]]; then                                          #IV67523
        #$NLS-E$ "could not determine the full pathname of $1"          #IV67523
        msg=$(nls_replace 'KCI1081E' "\"${oval}\"");                    #IV67523
        Err_report fail "${msg}";                                       #IV67523
      elif [[ "${x}" != "${oval}" ]]; then                              #IV67523
        #$NLS-E$ "$1 resolves to $2. Using $3 going forward."           #IV67523
        nls_replace 'KCI1383I' "\"${resp}\"" "\"${x}\"" "\"${x}\"";     #IV67523
        oval="${x}";                                                    #IV67523
      fi;                                                               #IV67523
    fi;                                                                 #IV67523
  elif [[ ! -w "${oval}" ]]; then                                       #IV67523
    #$NLS-E$ "cannot write to $1"                                       #IV67523
    msg=$(nls_replace 'KCI1080E' "\"${oval}\"");                        #IV67523
    Err_report fail "${msg}";                                           #IV67523
  fi;                                                                   #IV67523
                                                                        #IV67523
  #---------------------------------------------------------------------#IV67523
  # Set the resulting CANDLEXXXX variable.                              #IV67523
  #---------------------------------------------------------------------#IV67523
  eval 'export '"${ivarname}"'="'"${oval}"'"';                          #IV67523
                                                                        #IV67523
  return;                                                               #IV67523
}                                           # (getCANDLEXXXX())         #IV67523
                                                                        #IV67523
resolveDir() { #--------------------------------------------------------#IV67523
  # Perform directory canonicalization. In other words, the input       #IV67523
  # directory string is converted to a fully qualified name (no         #IV67523
  # wildcards, "..", ".", and so on. For example, "/tmp/.." resolves    #IV67523
  # to "/". The input directory string must refer to a directory that   #IV67523
  # exists.                                                             #IV67523
  #                                                                     #IV67523
  # AIX and Linux provide the "readlink" command to perform this        #IV67523
  # function. If readlink does not exist, we use "cd -P". This          #IV67523
  # approach changes the directory *and* sets PWD to be the             #IV67523
  # canonicalized directory name. So, we switch to the directory, grab  #IV67523
  # its name, and switch back to the original working directory.        #IV67523
  #                                                                     #IV67523
  # Solaris has some sort of buffering issue when the cd shell command  #IV67523
  # sets $PWD and when the pwd shell command returns the value. In      #IV67523
  # in some cases, one or both of these leave extra garbage on the end  #IV67523
  # of the output value. And of course, this creates problems for us    #IV67523
  # later when we try to use it. We get around the problem by using     #IV67523
  # the /usr/bin/pwd binary, which always returns the correct value     #IV67523
  # without corruption.                                                 #IV67523
  #---------------------------------------------------------------------#IV67523
  typeset result;                           # localize output variable  #IV67523
  result=$(readlink -f ${1} 2> /dev/null);  # try readlink              #IV67523
  if [[ ${?} -ne 0 ]]; then                 # if no readlink or failed  #IV67523
    cd -P ${1} > /dev/null 2>&1;            # use cd to change/resolve  #IV67523
    if [[ ${?} -ne 0 ]]; then               # if cd failed...           #IV67523
      result="";                            # indicate same to caller   #IV67523
    else                                    # get result from cd        #IV67523
      if [[ "${thisOS}" = "SunOS" ]]; then  # $PWD/pwd shell broken...  #IV67523
        result=$(/usr/bin/pwd);             # ...use pwd binary         #IV67523
      else                                  # $PWD is ok...             #IV67523
        result="${PWD}";                    # ...use its value          #IV67523
      fi;                                   # (elif($os="sol"))         #IV67523
      cd ${OLDPWD} > /dev/null 2>&1;        # restore working directory #IV67523
    fi;                                     # (else($?!=0))             #IV67523
  fi;                                       # (if($?!=0))               #IV67523
  print "${result}";                        # pass result to caller     #IV67523
}                                           # (resolveDir())            #IV67523
                                                                        #IV67523
function buildCANDLEDATA { #--------------------------------------------#IV67523
  # Build the CANDLEDATA directory for the first time. The directory    #IV67523
  # itself already exists. We're just setting ITM up to use it.         #IV67523
  #---------------------------------------------------------------------#IV67523
  eval $(itmsh_trace_func "${0}" "${@}");                               #IV67523
                                                                        #IV67523
  typeset cmd="${bindir}/unix/itmCANDLEDATA.sh";                        #IV67523
          cmd="${cmd} -h \"${CANDLEHOME}\"";                            #IV67523
          cmd="${cmd} -l \"${CANDLEDATA}\"";                            #IV67523
          cmd="${cmd} -b \"yes\"";                                      #IV67523
                                                                        #IV67523
  msg=$(eval "${cmd}" 2>&1);                # run itmCANDLEDATA.sh      #IV67523
                                                                        #IV67523
  if [[ ${?} -eq 0 ]]; then                 # if itmCANDLEDATA.sh ok... #IV67523
    print -n "${msg}";                      # ...just reflect messages  #IV67523
  else                                      # but failures...           #IV67523
    Err_report fail "${msg}";               # ...go through Err_report  #IV67523
  fi;                                       # (if($? != 0))             #IV67523
                                                                        #IV67523
  return;                                   # control to caller         #IV67523
}                                           # (buildCANDLEDATA())       #IV67523
                                                                        #IV67523
###################################################################
#                          Main                                   #
###################################################################
export saveLang=$LANG

bindir=`pwd`/$(dirname $0)
[ "$(print $0 | grep '\./')" = "" ] && [ "$(print $0 | grep '^/.*')" = "" ] && [ `pwd` = '/' ] && bindir=`pwd`$(dirname $0)
[ "$(print $0 | grep '^/.*')" != "" ] && bindir=$(dirname $0)

unalias -a;                                 # avert alias interference  #IV51017
if [[ -a "${bindir}/unix/itmsh_trace_main" ]]; then                     #IV51017
   . ${bindir}/unix/itmsh_trace_main;       # embed/start tracing       #IV51017
elif [[ -a "${bindir}/itmsh_trace_main" ]]; then                        #IV51017
   . ${bindir}/itmsh_trace_main;            # embed/start tracing       #IV51017
fi;                                         # (if(-e itmsh_trace_main)) #IV51017
                                                                        #IV51017
if [ -d $bindir/unix ]
then
nls_replace_dir=$bindir/unix/nls
else
nls_replace_dir=$bindir
fi
export PATH="/usr/bin:${nls_replace_dir}:$PATH";                        #IV51017

UsrCmd="$0 $*"
PROGNAME=$(basename $0)
PROGDIR=$(dirname $0)

cd "$PROGDIR"

typeset -i i=1 idx=0 j=0 kbAvail=0 kbTmpAvail=0 kbReq=0 kbytes=0 numEle=0 numItems=0
typeset curJVF="" ext="" newJVF="" osn="" toJVF="" AWK="awk"
typeset hasCCC="" hasFixes=false hasReadmes=false hasRoma="" hasCASP="" hasLDAP="" hasCMS=""
typeset hadCCC="" hadRoma="" hadCASP="" hasLDAP=""
typeset parDir prodMode tarDir  # Used by CandleRemote.
typeset license="license.txt"
typeset fld23 fld123 fieldSep
typeset hostNameInRunInfo
typeset platformChecked="NO"

typeset baseTok thisMach thisRec thisTok thisUser  # Variables used for the token file.

set -A mainMenu
set -A mainMenu2


CDNAME=tmv630
export CDNAME
AUTHKEY=414931f
# @40 This is the trigger for logic check in isPlatformSupported routine:
#   - Set to NO if the install is for media containing Application Support files only
#   - Unset ( or keep original setting as __PLATFORM_SUPPORTED_CHECK__ ) for base and agent media install
PLATFORM_SUPPORTED_CHECK=NO
DEBUG_CON=""
SAFE_MODE=""
silentMode=""
PARM_FILE=""
CLI_OPTION="installProduct"
DEPLOY_MODE=""
PROD_PATH=$(pwd);                                                       #IV67523
SUSE_VERSION=""
SECURE_GROUP=""
ciTar=cienv.tar  # NOTE:  this file is not compressed.
ciVFile=cienv.ver
instCI=no instCIFix=no instJRE=no
instPkgs="ci*.tar jr*.tar jr*.ctp jr*.z* *.jar *.j0*"
oldJRE=jre1.1.6
reqDirs="JRE bin config doc download logs misc registry tables tmp xfer keyfiles"
tables="archdsc.tbl imdprof.tbl imdver.tbl proddsc.tbl"

typeset thisOS=$(uname -s)

#-----------------------------------------------------------------------#IZ94626.1
# Use the id command to get our real and effective user names.          #IZ94626.1
#-----------------------------------------------------------------------#IZ94626.1
if [[ "${thisOS}" = "SunOS" ]]; then                                    #IZ94626.1
   id_cmd="/usr/xpg4/bin/id";                                           #IZ94626.1
else                                                                    #IZ94626.1
   id_cmd="id";                                                         #IZ94626.1
fi;                                                                     #IZ94626.1
thisRealUser=$(${id_cmd} -unr);             # get real user name        #IZ94626.1
thisEffectiveUser=$(${id_cmd} -un);         # get effective user name   #IZ94626.1
thisUser="${thisEffectiveUser}";            # generally, we want euid   #IZ94626.1
                                                                        #IZ94626.1
#-----------------------------------------------------------------------#IZ94626.1
# If our real userid is root, then we can use the su command. Note      #IZ94626.1
# that it doesn't matter if our effective userid is root. The AIX and   #IZ94626.1
# HP-UX versions of su determine authorization based on real userid,    #IZ94626.1
# and will fail if our real userid is not root.                         #IZ94626.1
#-----------------------------------------------------------------------#IZ94626.1
if [[ "${thisRealUser}" = "root" ]]; then                               #IZ94626.1
   su_value=$(locateCMD su);                                            #IZ94626
else                                                                    #IZ94626.1
   su_value="";                                                         #IZ94626.1
fi;                                                                     #IZ94626.1
                                                                        #IZ96646.1
#-----------------------------------------------------------------------#IZ96646.1
# On Solaris, if the real and effective user are not the same, we set   #IZ96646.1
# umask 000 so plugin processing doesn't run into permission issues     #IZ96646.1
# with the system temporary directory.                                  #IZ96646.1
#-----------------------------------------------------------------------#IZ96646.1
if [[ "${thisOS}" = "SunOS" ]]; then                                    #IZ94626.1
   if [[ "${thisRealUser}" != "${thisEffectiveUser}" ]]; then           #IZ94626.1
      umask 000;                                                        #IZ96646.1
   fi;                                                                  #IZ94626.1
fi;                                                                     #IZ94626.1

# need to use nawk if it's available
type nawk >/dev/null 2>&1
[[ $? -eq 0 ]] && AWK=nawk

# check if iconv exists in the system
checkIconv

while getopts ":h:l:j:d:o:p:k:acqvgs" opt; do                           #IV67523
  case $opt in
    a) DEPLOY_MODE="-Ditm.deploy=true"
       ;;
    h) CANDLEHOME=$(eval print $OPTARG)                                 #IV67523
       ;;
    l) CANDLEDATA=$(eval print $OPTARG)                                 #IV67523
       ;;
    j) JAVALOC=$OPTARG
       JAVALOC=$(eval print $JAVALOC)  # Process ~ if it was entered.
       ;;
    d) PROD_PATH=$OPTARG
       PROD_PATH=$(eval print $PROD_PATH)
	   ;;
    c) DEBUG_CON="-debugcon"
       ;;
    o) CLI_OPTION="installProduct"
       [ -n "$(print $OPTARG | grep -i "product" | grep -i "depot")" ] && CLI_OPTION="installProductToDepot"
       ;;
    p) PARM_FILE=$OPTARG
       export PARM_FILE
       silentMode="-t 5"
       ;;
    g) GSKIT_DOWNGRADE="yes"
       export GSKIT_DOWNGRADE
       ;;
    q) silentMode="-t 5"
       VERBOSE="yes" # by default, verbose in silent mode for deploy
       export VERBOSE
       ;;
    k) SECURE_GROUP=$OPTARG
       export SECURE_GROUP
       ;;
    v) VERBOSE="yes"
       export VERBOSE
       ;;
    s) SKIP_STARTUP_UPDATE="yes"
       ;;
    ?) Use_report
       ;;
    :) Use_report
       ;;
  esac
done

#-----------------------------------------------------------------------#IV51017
# Resolve/validate PROD_PATH.                                           #IV51017
#-----------------------------------------------------------------------#IV51017
[[ $PROD_PATH = @(.|./*) ]] && PROD_PATH=$(pwd)/$PROD_PATH              #IV67523
#$NLS-E$ "\"$PROD_PATH\" is not a directory"                                     #IV67523
[ -d $PROD_PATH ] || Err_report fail "`nls_replace 'KCI1084E' \\"$PROD_PATH\\"`" #IV67523
#$NLS-E$ "cannot read \"$PROD_PATH\""                                            #IV67523
[ -r $PROD_PATH ] || Err_report fail "`nls_replace 'KCI1085E' \\"$PROD_PATH\\"`" #IV67523
nls_replace_dir=$PROD_PATH/unix/nls                                     #IV67523
export TEXTDOMAINDIR=$nls_replace_dir/msg                               #IV67523
export PATH=$nls_replace_dir:$PATH                                      #IV67523
                                                                        #IV67523
#-----------------------------------------------------------------------#IV51017
# If tracing wasn't started from where install.sh was launched, try     #IV51017
# from the install image and from CANDLEHOME/bin.                       #IV51017
#-----------------------------------------------------------------------#IV51017
if [[ -z "${ITMSH_TRACE_EMBEDDED}" ]]; then                             #IV51017
   if [[ -a "${PROD_PATH}/unix/itmsh_trace_main" ]]; then               #IV51017
      . "${PROD_PATH}/unix/itmsh_trace_main";    # embed/start tracing  #IV51017
   elif [[ -a "${CANDLEHOME}/bin/itmsh_trace_main" ]]; then             #IV51017
      . "${CANDLEHOME}/bin/itmsh_trace_main";    # embed/start tracing  #IV51017
   fi;                                      # (if(-e...))               #IV51017
fi;                                         # (if(-e itmsh_trace_main)) #IV51017
                                                                        #IV51017
# Set ITMLOCALINS variable so that Prerequisite Scanner can differentiate
# between remote deploy and local installation.
if [ -z "$DEPLOY_MODE" ]; then
	export ITMLOCALINS=true
fi

checkMaxOpenFilesLimit

#check if correct version of psksh is installed:
typeset -i number20=0 number19=0
number20=020
number19=019
[ number19 -gt number20 ] && print -u2 "`nls_replace 'KCI1318I'`"

#try to check if scirpt was run with wildcard path and more than 1 path was resolved (defect 71768, 72307)
if [ $# -gt 0 ] && [ -n $1 ] && [ `echo $1 | cut -c1` != "-"  ] && [ 'install.sh' = `basename $1` ]
then
   #$NLS-E$ "  KCI1176E You have probably run install.sh using wildcards in path and this path was resolved to more than one string."
   print -u2 "`nls_replace 'KCI1176W' -c`"
fi

shift OPTIND-1
[ $# -eq 0 ] || Use_report

if [ -n "$DEPLOY_MODE" ]
then
	trap "true" 13 15  # Set trap.
	trap "cleanUp trap" 1 2 3  # Set trap.
else
	trap "cleanUp trap" 1 2 3 15  # Set trap.
fi

# For AIX installations called by Remote Deploy, the PATH is based on the OS
# agent which launchs install.sh.  This PATH does not include /usr/sbin
# therefore it must be added to run slibclean, rmitab and mkitab.
# [[ "$(uname)" = "AIX" ]] && [[ -n "$DEPLOY_MODE" ]] && PATH=$PATH:/usr/sbin
# Adding it always as user may not have /usr/sbin in the PATH
[[ "$(uname)" = "AIX" ]] && PATH=$PATH:/usr/sbin

# In silent mode, make sure all information is known up front
if [ -n "$silentMode" ]
then
  #$NLS-E$ "ITM home directory must be specified in silent mode (-h)"
  [ -z "$CANDLEHOME" ] && Err_report fail "`nls_replace 'KCI1063E' -h`"
  #$NLS-E$ "Input parameter file must be specified in silent mode (-p)"
  [ -z "$PARM_FILE" ] && Err_report fail "`nls_replace 'KCI1064E' -p`"
  #$NLS-E$ "Valid input parameter absolute file path must be specified in silent mode (-p)"
  [ -r /$PARM_FILE ] || Err_report fail "`nls_replace 'KCI1065E' -p`"
fi

welcome="INSTALL"
[[ "$AUTHKEY" = "__AUTHKEY__" ]] && welcome="UPGRADE"
print $welcome
[ -f ./non_ibm_license ] && grep "IBM TIVOLI MONITORING (\"ITM\")" ./non_ibm_license

# check write permissions for /tmp directory
[ -w "/tmp" ] || Err_report fail "`nls_replace 'KCI1250E'`"

# check for selinux
if [[ "$thisOS" = "Linux" ]];
then
  eval selinuxenabled >/dev/null 2>&1
  #$NLS-E$ "KCI1373W: SELinux (Secure Enhanced Linux) is set to \"enforced\" or \"permissive\" mode. \"Enforced\" mode is not supported by IBM Tivoli Monitoring. \"Permissive\" mode may require additional system configuration steps. Perform the following steps to set the mode to \"disabled\" or refer to IBM Tivoli Monitoring documentation for more details:\n1. Set SELinux \"disabled\" mode.\n2. Reboot the system.\n3. Execute the \"prelink -a\" command before restarting the ITM installation."
  [[ $? -eq 0 ]] && nls_replace 'KCI1373W'
fi

# Call to cleanup some files we have around
cleanupSystem

#-----------------------------------------------------------------------#IV67523
# Get/validate CANDLEHOME.                                              #IV67523
#-----------------------------------------------------------------------#IV67523
if [[ -z "${CANDLEHOME}" ]]; then                                       #IV67523
   CHprompt="/opt/IBM/ITM";                                             #IV67523
else                                                                    #IV67523
   CHprompt="${CANDLEHOME}";                                            #IV67523
fi;                                                                     #IV67523
getCANDLEXXXX "CANDLEHOME" "${CHprompt}";                               #IV67523
                                                                        #IV67523
#-----------------------------------------------------------------------#IV67523
# Determine whether this is a server image (with TEMS or TEPS). If so,  #IV67523
# it means some extra messaging and prompting with respect to how we    #IV67523
# manage CANDLEDATA.                                                    #IV67523
#-----------------------------------------------------------------------#IV67523
ls "${PROD_PATH}/unix" 2> /dev/null | egrep "^(ms|cq)" > /dev/null;     #IV67523
if [[ ${?} -eq 0 ]]; then                                               #IV67523
   export isSERVERimage="yes";                                          #IV67523
else                                                                    #IV67523
   export isSERVERimage="no";                                           #IV67523
fi;                                                                     #IV67523
                                                                        #IV67523
#-----------------------------------------------------------------------#IV67523
# Get/validate CANDLEDATA:                                              #IV67523
# - We don't ask for CANDLEDATA for existing CANDLEHOMEs that are       #IV67523
#   already in use. The user must run itmCANDLEDATA.sh to convert       #IV67523
#   existing CANDLEHOMEs. If the system has already been converted, we  #IV67523
#   ensure the CANDLEDATA environment variable or -l command option     #IV67523
#   match.                                                              #IV67523
# - We do ask for CANDLEDATA for new CANDLEHOMEs. If they specify a     #IV67523
#   separate directory for CANDLEDATA, we set that up using the         #IV67523
#   itmCANDLEDATA.sh utility.                                           #IV67523
#-----------------------------------------------------------------------#IV67523
CHregistry="${CANDLEHOME}/registry";                                    #IV67523
if [[ -a "${CHregistry}/cienv.ver" ]]; then      # CANDLEHOME in use?   #IV67523
  CH2CDmark="${CHregistry}/CANDLEDATA";     # yes, check existing link  #IV67523
  CH2CDlink=$(ls -l "${CH2CDmark}" 2> /dev/null);                       #IV67523
  if [[ -n "${CH2CDlink}" ]]; then          # does link entry exist?    #IV67523
    x="${CH2CDlink##* -> }";                                            #IV67523
    if [[ "${CH2CDlink}" = "${x}" ]] || [[ ! -d "${x}" ]]; then         #IV67523
      #$NLS-E$ "Link $1 is broken."                                     #IV67523
      msg=$(nls_replace 'KCI1391E' "\"${CH2CDmark}\"");                 #IV67523
      Err_report fail "${msg}";                                         #IV67523
    else                                                                #IV67523
      CH2CDlink="${x}";                     # link is ok, use it        #IV67523
    fi;                                     # (else(CH2CDlink=x))       #IV67523
                                                                        #IV67523
    if [[ -z "${CANDLEDATA}" ]]; then       # if nothing from user      #IV67523
      CANDLEDATA="${CH2CDlink}";            # use existing link as-is   #IV67523
    elif [[ "${CANDLEDATA}" != "${CH2CDlink}" ]]; then                  #IV67523
      #$NLS-E$ "The specified data directory $1 differs from existing link $2." #IV67523
      msg=$(nls_replace 'KCI1392E' "\"${CANDLEDATA}\"" "\"${CH2CDlink}\"");     #IV67523
      Err_report fail "${msg}";                                         #IV67523
    else                                                                #IV67523
      CANDLEDATA="${CH2CDlink}";            # use existing link as-is   #IV67523
    fi;                                     # (if(CH2CDlink...))        #IV67523
  elif [[ -z "${CANDLEDATA}" ]]; then       # if nothing from user      #IV67523
    CANDLEDATA="${CANDLEHOME}";             # just use CANDLEHOME       #IV67523
  elif [[ "${CANDLEDATA}" != "${CANDLEHOME}" ]]; then                   #IV67523
    #$NLS-E$ "Installation directory $1 has not been configured to use data directory $2." #IV67523
    #$NLS-E$ "\nUse itmCANDLEDATA.sh to convert an existing installation."                 #IV67523
    msg=$(nls_replace 'KCI1393E' "\"${CANDLEHOME}\"" "\"${CANDLEDATA}\"");                 #IV67523
    Err_report fail "${msg}";                                           #IV67523
  fi;                                       # (else(-n...))             #IV67523
else                                        # not first time...         #IV67523
  getCANDLEXXXX "CANDLEDATA" "${CANDLEHOME}";    # yes, ask for CDATA   #IV67523
  if [[ "${CANDLEDATA}" != "${CANDLEHOME}" ]]; then                     #IV67523
    buildCANDLEDATA;                        # establish the links       #IV67523
  fi;                                       # (if($CANDLEDATA!=$CA...)) #IV67523
fi;                                         # (else(-a cienv.ver))      #IV67523
                                                                        #IV67523
#-----------------------------------------------------------------------#IV67523
# Warn not to install TEMS or TEPS as appropriate.                      #IV67523
#-----------------------------------------------------------------------#IV67523
if [[ "${isSERVERimage}" = "yes" ]] &&                                  #IV67523
   [[ "${CANDLEHOME}" != "${CANDLEDATA}" ]]; then                       #IV67523
  #$NLS-E$ "\nWARNING: This installation image contains the monitoring server or portal server." #IV67523
  #$NLS-E$ "\nWARNING: Your installation directory is $1."                                       #IV67523
  #$NLS-E$ "\nWARNING: Your data directory is $2."                                               #IV67523
  #$NLS-E$ "\nWARNING: Do not install the monitoring server or portal server!"                   #IV67523
  #$NLS-E$ "\nWARNING: The monitoring server and portal server will not start with separate installation and data directories."; #IV67523
  #$NLS-E$ "\n"                                                                                  #IV67523
  nls_replace 'KCI1390I' "\"${CANDLEHOME}\"" "\"${CANDLEDATA}\"";       #IV67523
fi;                                         # (if(isSERVERimage...))    #IV67523
                                                                        #IV67523
export cArch
export newRegPath

# verify encryption key for silent mode installation
if [[ -n "$silentMode" ]];
then
  if [[ -f "$PARM_FILE" ]];
  then
    silentKey=$(grep "^INSTALL_ENCRYPTION_KEY" $PARM_FILE | head -1 )
    silentKey=${silentKey#*=}
    if [[ -n "$silentKey" ]];
    then
      # disallow certain characters
      echo $silentKey | grep "[\$=|]" > /dev/null 2>&1
      [[ $? = 0 ]] && invalidKey=true

      # length must be exactly 32
      len=$(echo $silentKey|wc -c)
      len=$(($len-1))
      [[ "$len" = "32" ]] || invalidKey=true

      #$NLS-E$ "INSTALL_ENCRYPTION_KEY must be exactly 32 characters long and can not contain the following characters \"\$=|\""
      [[ -n "$invalidKey" ]] && Err_report error "`nls_replace 'KCI0751E' INSTALL_ENCRYPTION_KEY` \"\$=|\"" 10
    fi
  fi
fi

# check if secure group passed to install.sh is valid
[[ -n "$SECURE_GROUP" ]] && validateSecureGroup

#If pdks is not installed in correct version, log message to log file (for silent installation)
[ -d $CANDLEHOME/logs ] || mkdir -p $CANDLEHOME/logs > /dev/null 2>&1   #IV67523
logFile=$CANDLEHOME/logs/candle_installation.log
[ number19 -gt number20 ] && print  "`nls_replace 'KCI1318I'`" >>  $logFile 2>&1

#This is in case we overinstall a JRE.
chmod -R 755 $CANDLEHOME/JRE > /dev/null 2>&1

# Set the package location.  The packages could be in a local directory (if downloaded from the Internet) or
# in the "unix" subdirectory of a CD file system.

PKGLOC=""

for pkg in $instPkgs
do
  if [ -f $PROD_PATH/$pkg ]  # Packages found in a local directory.
  then
    PKGLOC=$PROD_PATH
    break
  fi
done

# If no packages were found in a local directory, then they must be on a CD where they will be stored
# in the "unix" subdirectory.
if [ -z "$PKGLOC" ]
then
  tmp=$PROD_PATH/unix
  #$NLS-E$ "\"$tmp\" is not a directory"
  [ -d $tmp ] || Err_report fail "`nls_replace 'KCI1086E' \\"$tmp\\"`"
  #$NLS-E$ "cannot read \"$tmp\""
  [ -r $tmp ] || Err_report fail "`nls_replace 'KCI1087E' \\"$tmp\\"`"
  for pkgs in $instPkgs
  do
    for pkg in $(ls $tmp/$pkgs 2>/dev/null)
    do
      if [ -f $pkg ]  # Packages found on the CD.
      then
        PKGLOC=$tmp
        break
      fi
    done
  done
fi

#$NLS-E$ "could not find any product packages under directory \"$PROD_PATH\""
[ -n "$PKGLOC" ] || Err_report fail "`nls_replace 'KCI1088E' \\"$PROD_PATH\\"`"

# @10 - removed the following restriction.
#[ "$CANDLEHOME" != "$PKGLOC" ] || Err_report fail "product path should not be set to CANDLEHOME - \"$CANDLEHOME\""

# START - determine the variables needed by CandleRemote.

# Determine the location of the base installation script(s), *install.sh.  This will be either the $PKGLOC
# directory or the parent directory of $PKGLOC.

prodMode=""
tarDir=""

if [[ -f $PKGLOC/install.sh || -f $PKGLOC/cinstall.sh || -f $PKGLOC/ninstall.sh ]]
then
  tarDir=$PKGLOC  # Execute the tar command at the location of *install.sh.
  prodMode="eDelivery"  # The quotes are just for readability.
else
  parDir=$(dirname $PKGLOC)
  if [[ -f $parDir/install.sh || -f $parDir/cinstall.sh || -f $parDir/ninstall.sh ]]
  then
    tarDir=$parDir
    prodMode="newCD"  # Just a guess for now.  See following code.
  fi
fi

#$NLS-E$ "could not locate the base installation script"
[ -n "$prodMode" ] || Err_report fail "`nls_replace 'KCI1089E'`"

# Determine where the .ver files are located.  For "eDelivery," they are always inside the .jar files.

if [ "$prodMode" = "newCD" ]
then
  [ -d $PKGLOC/registry ] && prodMode="oldCD"
fi

# END - determine the variables needed by CandleRemote.

# Backup the registry so we can refer to previous version numbers during the
# install for possible data migration purposes.
# Non-compliant installs like Roma(CT98) will just get a token file in the
# backup area.
RegPath=$CANDLEHOME/registry
RegBackup=$CANDLEHOME/old_registry
typeset LDAPVer=""
[ -d $RegBackup ] || mkdir $RegBackup

# Check for a pre-CT99 roma.  /roma will exist and ideally /registry
# will not.  However, someone may have aborted an install, so we look
# in the registry for some common Roma ver files, and if there are none
# then we have a pre-CT99 roma for sure, and we create a token file.

# If we do find the common Roma ver files, we clean out the backup registry
# in preparation for copying the ver files.  This also deletes any token file.
[ "$thisOS" = "AIX" ] && os="aix"

if [ "$thisOS" = "HP-UX" ]
then
os=hp
[ "$(uname -m)" = "ia64" ] && os=hpi
fi

[ "$thisOS" = "SunOS" ] && os="sol"
[ "$thisOS" = "Linux" ] && os="l"

RomaPath=$(find $CANDLEHOME/* -name ldap -print | grep "roma/$os" | head -1) #IZ75211
if [ -n "$RomaPath" ]
then
  xx=$(ls $RegPath/bx${os}*.ver 2>/dev/null)
  xx=${xx}$(ls $RegPath/xm${os}*.ver 2>/dev/null)
  xx=${xx}$(ls $RegPath/xc${os}*.ver 2>/dev/null)
  xx=${xx}$(ls $RegPath/xs${os}*.ver 2>/dev/null)
  if [ -z "$xx" ]
  then
    print $RomaPath > $RegBackup/Pre_CT99_Roma
    chmod 755 $RegBackup/Pre_CT99_Roma > /dev/null 2>&1
    LDAPVer="2.10"
  else
    # clean out the backup registry
    rm -f $RegBackup/* > /dev/null 2>&1

    # Determine the previous version of LDAP installed (if any)
    ver=$(grep "ver =" $RegPath/xs${os}*.ver 2>/dev/null|head -1|cut -d" " -f3)
    [ -n "$ver" ] && LDAPVer=$(print $ver|cut -c 1).$(print $ver|cut -c 2-3)
  fi
fi

[ -d $RegPath ] && cp $RegPath/*.ver $RegBackup > /dev/null 2>&1

# Create the CANDLEHOME directories.

cd $CANDLEHOME > /dev/null 2>&1
#$NLS-E$ "could not cd to \"$CANDLEHOME\" directory"
[ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1090E' \\"$CANDLEHOME\\"`"

for dir in $reqDirs
do
  [ -d $dir ] || mkdir $dir > /dev/null 2>&1
  #$NLS-E$ "could not create directory \"$CANDLEHOME/$dir\""
  [ -d $dir ] || Err_report fail "`nls_replace 'KCI1091E' \\"$CANDLEHOME/$dir\\"`"
  [ -w $dir ] || chmod 755 $dir > /dev/null 2>&1
  #$NLS-E$ "could not add write access to directory \"$CANDLEHOME/$dir\""
  [ -w $dir ] || Err_report fail "`nls_replace 'KCI1092E' \\"$CANDLEHOME/$dir\\"`"
done

# The name of the docs directory has been changed to doc for consistency with NT.

if [ -d $CANDLEHOME/docs ]
then
# Skip the error handling since an empty docs dir will result in an error.
  mv $CANDLEHOME/docs/* $CANDLEHOME/doc/ > /dev/null 2>&1
  rmdir $CANDLEHOME/docs > /dev/null 2>&1
fi

export CANDLETMP=$CANDLEHOME/tmp
curRegPath=$CANDLEHOME/registry
refVFile=$CANDLEHOME/registry/citnt.ver

# LL - start token file stuff.

baseTok=$CANDLETMP/candle_install_

thisOS=$(uname -a | cut -d" " -f1)  # thisOS -> AIX
#

# tar file option for unix and root use oxf. defect 12436
# set flag to do chown and chmod for Linux.  defect 83100
tarOpt=xhf                                                              #IV67523
chownRoot=n
if [[ "$thisUser" = "root" ]]
then
  if [[ "$thisOS" != "Linux" ]]
  then
    tarOpt=oxhf                                                         #IV67523
  else
    chownRoot=y
  fi
fi

if [[ "$thisOS" = NONSTOP_KERNEL ]]
then
  tarOpt=oxhf                                                           #IV67523
fi

# 17037 if we are root, we can cleanup unused modules and libs
[[ "$thisUser" = "root" &&  "$thisOS" = "AIX" ]] && slibclean 2>/dev/null

# Create the token file.

thisMach=$(uname -a | cut -d" " -f2)  # thisMach -> vulcan
thisTok=$baseTok${thisMach}_$$
touch $thisTok > /dev/null 2>&1
#$NLS-E$ "could not create token file"
[ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1093E'`"

# Build the description record, and add it to the token file.  This record will look like:
# "AIX vulcan billy 17440 Friday Jun 08 14:21 2001 |159|".  The number between pipes (|)
# is the day of the year the file was created.

thisRec="$thisOS $thisMach $thisUser $$ $(date +"%A %B %d %H:%M %Y |%j|")"
print "$thisRec" > $thisTok

# Delete old token files, and warn if there is another install running in this <candlehome>.

procTokFiles

# LL - end token file stuff.


#block upgrade from releases older than 6.1
blockOmegamonUpgrade

#  Skip this step during remote deploys to allow for the case in which prereqs are already installed and are not part of the deploy package
if [[ -z "$DEPLOY_MODE" ]]
then
  #Defect 78393
  #Determine if the machine's architecture is deprecated in new version before unpacking installer
  dynShl=$CANDLEHOME/bin/dynarch.shl
  #check if scirpt exist and it is executeble (This means that ITM is installed on this machine/location)
  if [ -x $dynShl ] && [ -f $CANDLEHOME/registry/archdsc.tbl ]
  then

  # To determine the architecture, use payloads from the installation media and
  # not resources already installed in CANDLEHOME.
	oldPROGNAME=$PROGNAME
    checkPlatformBeforeUpgrade
    PROGNAME=$oldPROGNAME
  fi
fi

su_value=$(locateCMD su);                                               #IZ94626

checkCandleManage # Look for running CandleManage.

checkProcesses  # LL - look for running agents/CMS.

# 17037 if we are root, we can cleanup unused modules and libs
[[ "$thisUser" = "root" &&  "$thisOS" = "AIX" ]] && slibclean 2>/dev/null

hasCCC=""
hasRoma=""
hasCMS=""
#shoPrequis
#$NLS-E$ "searching for product families; please wait"
Err_report info "`nls_replace 'KCI1094I'`"

checkDiskSpace $CANDLEHOME 1000

findFamilies  # Determine what product families are available for installation.

# RQ (Candle Application Services Pac) is not allowed to install over Roma
# but is allowed to install over RQ (which has Roma components)
if [ -n "$hasCASP" ]
then
  #$NLS-E$ "this product does not install over eBP"
  [[ -z "$hadCASP" && -n "$hadRoma" ]] && Err_report fail "`nls_replace 'KCI1095E' eBP`"
fi

# If packages are available for more than one product family, ask the user to select a family.

instFam=""
#$NLS-E$ "Exit install."
exInst="`nls_replace 'KCI1096I'`"

if [[ -n "$hasCCC" && -n "$hasRoma" ]]
then
  #$NLS-E$ "Install Candle Commmand Center products."
  cccP="`nls_replace 'KCI1097I'`"
  #$NLS-E$ "Install Roma products."
  romaP="`nls_replace 'KCI1098I'`"

  #$NLS-E$ "\nSelect one of the following:  \n"
  echo
  nls_replace 'KCI1099I'
  echo
  #$NLS-E$ "Please enter a valid number:  "
  PS3="`nls_replace 'KCI1100I'`"
  select num in "$cccP" "$romaP" "$exInst"
  do
    case $num in
      $cccP)
        instFam="-ccc"
        break ;;
      $romaP)
        instFam="-roma"
        break ;;
      $exInst)
        cleanUp 0 ;;
    esac
  done
  echo
fi

# Determine what family to install if packages exist for only one family.

if [ -z "$instFam" ]
then
  if [ -n "$hasCCC" ]
  then
    instFam="-ccc"
  elif [ -n "$hasRoma" ]
  then
    instFam="-roma"
  fi
fi

# IBM/Tivoli or Candle presentation (use Candle whenever a CMS is in the packaging)
tivoli="-tivoli"
# EJF (#6333) remove this line to allow tivoli to remain set
#[ -n "$hasCMS" ] && tivoli=""

# Ensure that product packages exist.  The check against $PKGLOC is needed since findFamilies() excludes
# ax, ci and jr packages.

#$NLS-E$ "could not find any product packages under directory \"$PROD_PATH\""
[[ -n "$instFam" && -n "$PKGLOC" ]] || Err_report fail "`nls_replace 'KCI1101E' \\"$PROD_PATH\\"`"

# See if any fixpacks exist in $PKGLOC.

hasFixes=false
lookForFixPacks  # If fixpacks are available, hasFixes=true.
#print ... hasFixes = $hasFixes

# See if there are any readme's available for viewing.

hasReadmes=false
ls $CANDLEHOME/doc/readme*.txt > /dev/null 2>&1
[ $? -eq 0 ] && hasReadmes=true

# For eBP over-installs, warn them about shutting down running LDAPs
[[ "$instFam" = "-roma" && -n "$LDAPVer" && -n "$hasLDAP" ]] && BspWarn

# All menu items.

#$NLS-E$ "Install products or maintenance via GUI."
gui="`nls_replace 'KCI1102I'`"
#$NLS-E$ "Install products to the local host."
cmdLn="`nls_replace 'KCI1103I'`"
#$NLS-E$ "Install maintenance via command line."
cmdLnM="`nls_replace 'KCI1104I'`"
#$NLS-E$ "Create remote packages via GUI."
guiRP="`nls_replace 'KCI1105I'`"
#$NLS-E$ "Create remote packages via command line."
cmdLnRP="`nls_replace 'KCI1106I'`"
#$NLS-E$ "View readme files."
vRM="`nls_replace 'KCI1107I'`"
#$NLS-E$ "Install products to depot for remote deployment (requires TEMS)."
cmdLnDepot="`nls_replace 'KCI1108I'`"
#$NLS-E$ "Install products to depot via GUI."
guiDepot="`nls_replace 'KCI1109I'`"
#$NLS-E$ "Install product support."
supportCLI="`nls_replace 'KCI1201I'`"

# Build an accurate list of menu items.

if [ "$hasFixes" = "false" ]
then
  #$NLS-E$ "Install products via GUI."
  gui="`nls_replace 'KCI1110I'`"
fi

# EJF (#12006) remove GUI install option
#mainMenu[0]="$gui"
mainMenu[0]="$cmdLn"
mainMenu2[0]="Install products to the local host."
[ "$hasFixes" = "true" ] && mainMenu[${#mainMenu[*]}]="$cmdLnM"
[ "$hasFixes" = "true" ] && mainMenu2[${#mainMenu2[*]}]="Install maintenance via command line."
# Uncomment the next 2 lines to activate GUI remote package creation, feature 14459
#mainMenu[${#mainMenu[*]}]="$guiRP"
#mainMenu2[${#mainMenu2[*]}]="Create remote packages via GUI."
# Uncomment the next 2 lines to activate command line remote package creation, feature 14459
#mainMenu[${#mainMenu[*]}]="$cmdLnRP"
#mainMenu2[${#mainMenu2[*]}]="Create remote packages via command line."

# Uncomment the next 2 lines to activate GUI depot option
# mainMenu[${#mainMenu[*]}]="$guiDepot"
# mainMenu2[${#mainMenu2[*]}]="Install products to depot via GUI."

#
# d15022 abstract      DEV: Remove depot install choice from Exchange Unix installer
# d35743   (modifies this code to add hlv410 and vlv410)
#
# depot install is not valid for the following cds from the
# install.sh shell script :
#
# m3v410     IBM Tivoli OMEGAMON Data Files for z/OS
# q5v610     ITM for Microsoft Cluster Servers
# 3zv610     ITM for Windows Active Directory (ADO)
# vwv525     ITM for NetView on z/OS 5.2.5
# hlv410     OMEGAMON z/OS Management Console
# vlv410     OMEGAMON XE for z/VM and Linux
#
#
# use the 'tacmd addBundles' CLI to add components from these
# cds to a depot
if [[ "$CDNAME" != @(m3v4*|q5v*|3zv*|vwv*|hlv*|vlv*) ]]; then
    if [[ "$(uname -s)" != "NONSTOP_KERNEL" ]]
    then
    	mainMenu[${#mainMenu[*]}]="$cmdLnDepot"
		mainMenu2[${#mainMenu2[*]}]="Install products to depot for remote deployment (requires TEMS)."
    fi
fi
[ "$hasReadmes" = "true" ] && mainMenu[${#mainMenu[*]}]="$vRM"
[ "$hasReadmes" = "true" ] && mainMenu2[${#mainMenu2[*]}]="View readme files."
mainMenu[${#mainMenu[*]}]="$supportCLI"
mainMenu2[${#mainMenu2[*]}]="Install product support"
mainMenu[${#mainMenu[*]}]="$exInst"
mainMenu2[${#mainMenu2[*]}]="Exit install."
numItems=${#mainMenu[*]}  # The number of elements in the array.

# warn user about os prereqs introduced in version 6.3
display63Prereqs

# warn user about possible issue with metaprobes after tems upgrade
checkMetaprobes

# Generate a "select" menu, and ask the user to select an install mode.  It's too early in the script
# to actually invoke the Java installation routines.  Howevever, displaying this menu later in the script
# would make the install awkward for the user.

while [ anything ]
do
  guiRep=no cmdLnRep=no cmdLnMRep=no guiRPRep=no cmdLnRPRep=no vRMRep=no cmdLnDepotRep=no guiDepotRep=no supportCLIRep=no

  # Force the user to make a selection, unless we're in silent mode.
  if [ -n "$silentMode" ]
  then
    [ "$CLI_OPTION" = "installProduct" ] && cmdLnRep=yes
    [ "$CLI_OPTION" = "installProductToDepot" ] && cmdLnDepotRep=yes
	#$NLS-E$ "installing products via silent mode"
    Err_report info "`nls_replace 'KCI1111I'`"
  else
    i=1 j=0
	#$NLS-E$ "\nSelect one of the following:  \n"
    echo
	nls_replace 'KCI1112I'
	echo
    while [ j -lt numItems ]
    do
      print "$i) ${mainMenu[j]}"
      i=i+1
      j=j+1
    done

    while [ anything ]
    do
	  #$NLS-E$ "\nPlease enter a valid number:  \c"
      print "\n`nls_replace 'KCI1113I'`  \c"
      read
      [[ $REPLY = +([0-9]) ]] || continue  # Make sure it's a number.
      [[ "$REPLY" -gt 0 && "$REPLY" -le numItems ]] && break  # It's a valid number.
    done
    idx=$REPLY

    # Determine what the user wants to do.

# EJF (#12006) remove GUI install option
#    [ idx -eq 1 ] && guiRep=yes  # The first item is always GUI mode.
    [ idx -eq numItems ] && cleanUp 4  # The last item is always exit.

# EJF (#12006) remove GUI install option (changed "idx -gt 1" to "idx -gt 0")
    if [ idx -gt 0 -a idx -lt numItems ]  # Skip the first and last items since they're known.
    then
      idx=idx-1  # Set the real array index.
      sel=${mainMenu2[$idx]}
#      print ... sel = $sel
      foundSth=$(print "$sel" | grep "product" | grep -v "depot" 2> /dev/null)  # Install products via command line.
      [ -n "$foundSth" ] && cmdLnRep=yes
      foundSth=$(print "$sel" | grep "maintenance" 2> /dev/null) # Install maintenance via command line.
      [ -n "$foundSth" ] && cmdLnMRep=yes
      foundSth=$(print "$sel" | grep "remote" 2> /dev/null | grep "GUI" 2> /dev/null) # Create remote packages via GUI.
      [ -n "$foundSth" ] && guiRPRep=yes
      foundSth=$(print "$sel" | grep "remote" 2> /dev/null | grep "command" 2> /dev/null) # Create remote packages via command line.
      [ -n "$foundSth" ] && cmdLnRPRep=yes
      foundSth=$(print "$sel" | grep "readme" 2> /dev/null) # View readme files.
      [ -n "$foundSth" ] && vRMRep=yes
      foundSth=$(print "$sel" | grep "depot" | grep "GUI" 2> /dev/null) # Install products via GUI to agent depot
      [ -n "$foundSth" ] && guiDepotRep=yes
      foundSth=$(print "$sel" | grep "depot" 2> /dev/null) # Install products via command line to agent depot
      [ -n "$foundSth" ] && cmdLnDepotRep=yes
      foundSth=$(print "$sel" | grep "support" 2> /dev/null) # Install product support
      [ -n "$foundSth" ] && supportCLIRep=yes
    fi
  fi

  if [ "$vRMRep" = "yes" ]
  then
    typeset rmIdx=""
    typeset -i maxVal=0
    set -A rmDescs

    execRMDialog  # Execute the readme dialog.

    # Convert the description selected in the menu into a file name and then display the readme.

    if [[ -n "$rmIdx" && "$rmIdx" -lt maxVal ]]  # It must be less than maxVal since maxVal is "exit."
    then
      desc="${rmDescs[$rmIdx]}"  # Get the description.
      desc=$(print $desc 2> /dev/null | sed "s/ /_/g")  # Add the underscores.
      desc="readme_$desc.txt"  # Complete the file name.
      if [ -f $CANDLEHOME/doc/$desc ]
      then
        more $CANDLEHOME/doc/$desc
      else
	    #$NLS-E$ "could not find readme \"$CANDLEHOME/doc/$desc\""
        Err_report warn "`nls_replace 'KCI1114I' \\"$CANDLEHOME/doc/$desc\\"`"
      fi
    fi
  else
    break
  fi
done
echo

# EJF - per defect 9191, if adding to depot ensure that a TEMS is installed
if [[ "$cmdLnDepotRep" = "yes" || "$guiDepotRep" = "yes" ]] && [[ ! -f $CANDLEHOME/config/.ConfigData/kmsenv ]]
then
   #$NLS-E$ "No Tivoli Enterprise Monitoring Server (TEMS) is installed under $CANDLEHOME"
   Err_report fail "`nls_replace 'KCI1115E' $CANDLEHOME`"
fi

# Determine where the .ver files are located for new packages.  If there is a "registry" subdirectory in
# $PKGLOC, then look for the files in the $PKGLOC/registry directory.  .ver files in this directory can
# be read directly.  If there is no "registry" directory under $PKGLOC, the .ver files are contained
# within the product packages, and these files must be extracted from the packages and placed in $CANDLETMP.
# We cannot extract to $PKGLOC since this directory may be on a read-only CD.

hasRegDir=no

if [ -d $PKGLOC/registry ]
then
  newRegPath=$PKGLOC/registry
  hasRegDir=yes
else
  newRegPath=$CANDLETMP
  hasRegDir=no
fi

# See if there is a installer (i.e. $ciTar) package in $PKGLOC.  If so, then we need to version and possibly
# update the current installer (if we're installing into an existing CANDLEHOME).

instCI=no
if [ -f $PKGLOC/$ciTar ]
then
  #$NLS-E$ Initializing ...
  nls_replace 'KCI1116I'
  #$NLS-E$ "processing base CI package"
  Err_report info "`nls_replace 'KCI1117I'`"

  #$NLS-E$ "cannot read package \"$PKGLOC/$ciTar\""
  [ -r $PKGLOC/$ciTar ] || Err_report fail "`nls_replace 'KCI1118E' \\"$PKGLOC/$ciTar\\"`"

  if [ "$hasRegDir" = "no" ]
  then
    unloadTmpFile $ciVFile $PKGLOC/$ciTar  # Extract the .ver file.
  fi

  #$NLS-E$ "cannot read file \"$newRegPath/$ciVFile\""
  [ -r "$newRegPath/$ciVFile" ] || Err_report fail "`nls_replace 'KCI1119E' \\"$newRegPath/$ciVFile\\"`"

  # See if our own CI classes are installed.

  if [ -r "$curRegPath/$ciVFile" ]  # Our classes are installed.
  then
    checkLevel $ciVFile  # Check if the "to be installed" CI classes are newer than the ones in CANDLEHOME.
    [ "$levelStrg" = "Same" ] && instCI=Same  # levelStrg is set by function checkLevel().
    [ "$levelStrg" = "Newer" ] && instCI=yes  # levelStrg is set by function checkLevel().
  else  # Our classes are not installed.
    instCI=yes
  fi
fi

# Check if we are trying to install to depot and version of installer on CD is newer then the version in CandleHome,
# in that case we exit since we do not want to CI upgraded in CH when no components are actually installed just added to depot
if [[ "$cmdLnDepotRep" = "yes" || "$guiDepotRep" = "yes" ]] && [[ "$instCI" = "yes" ]]
then
  echo
  #$NLS-E$ Media currently being used is unable to install products to depot, use "tacmd" command to populate the remote deploy depot.
  # Refer to documentation for more details.
  print "`nls_replace 'KCI1177E'`"
  cleanUp 2
fi

# Check prereqs for JRE. Skip the check for the remote deploy or when user
# doesn't want it in silent mode.
silentPrereqCheck=""
if [ -n "$silentMode" ]; then
	if [ -f "$PARM_FILE" ]; then
		silentPrereqCheck=`grep "^PREREQ_CHECK=" $PARM_FILE | cut -d "=" -f 2`
	fi

	# Assign default value in case silent config file doesn't contain
	# PREREQ_CHECK variable else validate the string.
	if [ -z "$silentPrereqCheck" ]; then
		silentPrereqCheck="YES"
	else
		case $silentPrereqCheck in
			[Yy][Ee][Ss]|[Tt][Rr][Uu][Ee])
				silentPrereqCheck="YES"
				break
				;;
			[No][Oo]|[Ff][Aa][Ll][Ss][Ee])
				silentPrereqCheck="NO"
				break
				;;
		esac
	fi
fi

if [ -z "$DEPLOY_MODE" ]; then
	# Run PRS if not in silent mode or user wants it to be run in silent mode.
	# This allows to turn off PRS in silent mode.
	if [ -z "$silentMode" -o "$silentPrereqCheck" = "YES" ]; then
		prereqchecker=${PKGLOC}/prereqchecker/prereq_checker.sh

		# Dots are required in order to overcome CMVC keyword expansion...
		uniqSessionIDStamp=`date +'%Y.%m.%d.-%H.%M.%S.' | tr -d "."`
		prsdir=${CANDLEHOME}/logs/prereqscan/${uniqSessionIDStamp}
		jreprsoutputdir=${prsdir}/KJR

		[ -d $jreprsoutputdir ] || mkdir -p $jreprsoutputdir > /dev/null 2>&1
		#$NLS-E$ "could not create directory \"$jreprsoutputdir\""
		[ -d $jreprsoutputdir ] || Err_report fail "`nls_replace 'KCI1091E' \\"$jreprsoutputdir\\"`"
		#$NLS-E$ "KCI0460E cannot write to directory \"$jreprsoutputdir\""
		[ -w $jreprsoutputdir ] || Err_report fail "`nls_replace 'KCI0460E' \\"$jreprsoutputdir\\"`"

		$prereqchecker "KJR" PATH=$CANDLEHOME outputdir="${jreprsoutputdir}" \
		    > /dev/null 2>&1

		# If prerequisite scanning for JRE has passed then we don't do anything - this
		# is invisible for the user. However when it fails we want to inform the user
		# and ask if he/she wants to continue installation.
		if [ $? -ne 0 ]; then
			grep FAIL ${jreprsoutputdir}/result.txt > /dev/null 2>&1
			if [ $? -eq 0 ]; then
				#$NLS-E$ "Prerequisite scanning for JRE failed:"
				print "`nls_replace 'KCI1367E'`"
				# If in silent mode then exit as there's high possibility that ITM
				# will fail to install or not work properly
				if [ -n "$silentMode" ]; then
					#$NLS_E$ FAIL	Tivoli Enterprise-suppliedd JRE	<jrsprsoutputdir>/result.txt
					print "\t`nls_replace 'KCI1368E'`\t\tTivoli Enterprise-supplied JRE\t${jreprsoutputdir}/result.txt"
					cleanUp 2
				fi
				#$NLS_E$ FAIL	Tivoli Enterprise-suppliedd JRE	<jrsprsoutputdir>/result.txt
				print "\t`nls_replace 'KCI1368E'`\t\tTivoli Enterprise-supplied JRE\t${jreprsoutputdir}/result.txt"
				#$NLS-E$ Property						Result	Found								Expected
				print "\t\t`nls_replace 'KCI1369I'`\t\t\t\t`nls_replace 'KCI1370I'`\t\t`nls_replace 'KCI1371I'`\t\t\t\t\t`nls_replace 'KCI1372I'`"
				print "\t\t============================================================================================================"
				grep FAIL ${jreprsoutputdir}/result.txt 2>/dev/null | grep -v Overall | awk '{print "\t\t", $0}'

				while true; do
					#$NLS-E$ "\nContinue with this installation [ y or n; \"n\" is default ]?  \c"
					echo
					print "`nls_replace 'KCI1048I'` [1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`; \"2\" `nls_replace 'KCI0199I'` ]?  \c"
		        	read Response
		        	case "$Response" in
					1|[Yy]|[Yy][Ee]|[Yy][Ee][Ss])
						break
						;;
		        	2|[Nn]|[Nn][Oo]|"")
		        		#$NLS-E$ User response, cannot continue installation.
		        		Err_report fail "`nls_replace 'KCI1230I'`"
		        		;;
		        	*)
		        		#$NLS-E$ "invalid response, please enter \"y\" or \"n\""
						Err_report warn "`nls_replace 'KCI1075I' \\"1\\" \\"2\\"`"
						;;
		        	esac
		        done
			fi
		fi
	fi
fi # End of "[ -z DEPLOY_MODE ]" if

# Install the installer.

#Defect 81121- moved this setting to the top of Main to make sure we have saved the LANG
#export saveLang=$LANG

savePath=$PATH

########################## defect 77379
if [ "$instCI" = "Same" ]
then
  aFile=tmp/archdsc.tbl
  tar -$tarOpt $PKGLOC/cienv.tar $aFile > /dev/null 2>&1
fi

if [[ "$instCI" = "no" || "$instCI" = "Same" ]]
then
  # Copy the jr<arch>.ver files from the media to the tmp directory if they exist.
  # Otherwise,  assume they were extracted from in cienv.tar.

  jreOS=${cArch%%[0-9]*} #aix, li, sol, hp, etc
  [ -d tmp ] || mkdir tmp > /dev/null 2>&1
  for file in `ls $PKGLOC/jr${jreOS}*.tar $PKGLOC/jr${jreOS}*.ctp 2> /dev/null`
  do
    pkgName=$(basename $file)
    pkgName=${pkgName%.*}
    if [ -f $PKGLOC/$pkgName.ver ]
    then
      cp $PKGLOC/$pkgName.ver tmp/$pkgName.ver > /dev/null 2>&1
    else
      tar -$tarOpt $PKGLOC/cienv.tar tmp/$pkgName.ver > /dev/null 2>&1
    fi
  done
fi
########################## defect 77379

kbytes=0
if [ "$instCI" = "yes" ]
then
  # See if there is enough room to install.

  export LANG=C
  kbytes=$(grep "^size[A-Z,a-z]* *=" $newRegPath/$ciVFile | cut -d"=" -f2 2> /dev/null)
  export LANG=$saveLang
  #$NLS-E$ "could not determine the size of the base CI package"
  [ kbytes -gt 0 ] || Err_report fail "`nls_replace 'KCI1120E'`"

  if [[ "$(uname -s)" = "NONSTOP_KERNEL" ]]	# Tandem df doesn't work as we expect, hard code now.
  then
      kbAvail=10000000
  else
      checkDiskSpace $CANDLEHOME $kbytes  # install.sh exits if space is not available.
  fi

  # Process any elements.

  eleNames="" i=1 numEle=0

  export LANG=C
  numEle=$(grep "^numElements *=" $newRegPath/$ciVFile 2> /dev/null | cut -d"=" -f2)
  export LANG=$saveLang
  if [[ $? -eq 0 && numEle -gt 1 ]]  # The installer is contained in multiple packages.
  then
    #$NLS-E$ "processing element CI packages"
    Err_report info "`nls_replace 'KCI1121I'`"
    while [ i -lt numEle ]
    do
      eleNames="$eleNames cienv$i"  # For CI, element names begin with 1.
      i=i+1
    done
    checkCIElements $eleNames
  fi

  #$NLS-E$ "unloading CI package(s)"
  Err_report info "`nls_replace 'KCI1122I'`"
  for pkg in $ciTar $eleNames
  do
    bName=${pkg%.*}  # Remove any extension.
    unTarPkg $PKGLOC/$bName.tar
  done

  # Copy the .ver file for the base package, and add write access.

  cp $newRegPath/$ciVFile $curRegPath/ > /dev/null 2>&1
  #$NLS-E$ "could not copy version file \"$ciVFile\""
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1123E' \\"$ciVFile\\"`"
  chmod 755 $curRegPath/$ciVFile > /dev/null 2>&1
  #$NLS-E$ "could not change access permissions on \"$ciVFile\""
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1124E' \\"$ciVFile\\"`"

  #46553 cpInstProg removed
  #cpInstProg $ciTar  # Copy install.sh to CANDLEHOME/bin.

  # Copying the mo directory and nls_replace file
  cpMo

  #65392 backup existing installer translation directory
  rm -f $CANDLEHOME/tmp/nls_msg.tar > /dev/null 2>&1
  tar chf $CANDLEHOME/tmp/nls_msg.tar -C $CANDLEHOME config/nls/msg bin/nls_replace > /dev/null #IV67523
  #$NLS-E$ "could not create translation backup file"
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1238E'`"
  # Copy the jr<arch>.ver files from the media to the tmp directory if they exist.
  # Otherwise,  assume they were extracted from in cienv.tar.

  [ -d tmp ] || mkdir tmp > /dev/null 2>&1
  jreOS=${cArch%%[0-9]*} #aix, li, sol, hp, etc
  for file in `ls $PKGLOC/jr${jreOS}*.tar $PKGLOC/jr${jreOS}*.ctp 2> /dev/null`
  do
    pkgName=$(basename $file)
    pkgName=${pkgName%.*}
    if [ -f $PKGLOC/$pkgName.ver ]
    then
      cp $PKGLOC/$pkgName.ver tmp/$pkgName.ver > /dev/null 2>&1
    fi
  done

fi

# Determine if there is an installer (ci) fixpack that needs to be applied.

typeset allVFiles=""

ciFixPacks=$(ls $PKGLOC/ciux*.t* 2> /dev/null)
if [ -n "$ciFixPacks" ]
then
  #$NLS-E$ "processing CI fixpacks"
  Err_report info "`nls_replace 'KCI1125I'`"

  for fp in $ciFixPacks
  do
    #$NLS-E$ "cannot read package \"$fp\""
    [ -r $fp ] || Err_report fail "`nls_replace 'KCI1126E' \\"$fp\\"`"

    # Determine the name of the .ver file.  If the fixpack name is ciuxdddd.t00 where d is a digit,
    # the .ver file will be named ciuxdddd.ver.  If the fixpack name is ciuxdddd.t01, ciuxdddd.t02,
    # etc, the .ver file will be named ciuxddddt01.ver, ciuxddddt02.ver, etc.

    vFile=${fp##*/}   # Remove the pathname.
    ext=${vFile##*.}  # Get the extension; ext -> t00, t01, t02, ...
    if [ "$ext" = "t00" ]
    then
      vFile=${vFile%.*}.ver      # vFile -> ciux6110.ver
    else
      vFile=${vFile%.*}$ext.ver  # vFile -> ciux6110t01.ver
    fi

    if [ "$hasRegDir" = "no" ]  # Unload the ciux*.ver files if there is no registry directory.
    then
      unloadTmpFile $vFile $fp  # Extract the .ver file.
    fi
	#$NLS-E$ "cannot read file \"$newRegPath/$vFile\""
    [ -r "$newRegPath/$vFile" ] || Err_report fail "`nls_replace 'KCI1127E' \\"$newRegPath/$vFile\\"`"

    [ "$ext" = "t00" ] && allVFiles="$allVFiles $vFile"  # Consider only the base package.
  done

  # Find the name of the .ver file for the latest ci fixpack.

  if [ -n "$allVFiles" ]
  then
    findLatest $allVFiles  # findLatest() will place the name of the .ver file in variable lateVFile.
    ciLateVFile=$lateVFile  # Save the name for future processing.
    checkLevel $ciVFile $ciLateVFile  # See if the fixpack needs to be installed.
    [ "$levelStrg" = "Newer" ] && instCIFix=yes
  fi
fi

# Install a ci fixpack.

kbytes=0
if [ "$instCIFix" = "yes" ]
then
  # See if there is enough room.

  export LANG=C
  kbytes=$(grep "^size[A-Z]* *=" $newRegPath/$ciLateVFile | cut -d"=" -f2)
  export LANG=$saveLang
  #$NLS-E$ "could not determine the size of the CI fixpack"
  [ kbytes -gt 0 ] || Err_report fail "`nls_replace 'KCI1128E'`"

  if [[ "$(uname -s)" = "NONSTOP_KERNEL" ]]	# Tandem df doesn't work as we expect, hard code now.
  then
      kbAvail=10000000
  else
      checkDiskSpace $CANDLEHOME $kbytes  # install.sh exits if space is not available.
  fi


  # Process any elements.

  eleNames="" i=1 numEle=0
  typeset baseName=${ciLateVFile%.*}

  export LANG=C
  numEle=$(grep "^numElements *=" $newRegPath/$ciLateVFile 2> /dev/null | cut -d"=" -f2)
  export LANG=$saveLang
  if [[ $? -eq 0 && numEle -gt 1 ]]  # The fixpack is contained in multiple packages.
  then
    #$NLS-E$ "processing element CI fixpacks"
    Err_report info "`nls_replace 'KCI1129I'`"
    while [ i -lt numEle ]
    do
      [ i -le 9 ] && ext=t0$i || ext=t$i
      eleNames="$eleNames $baseName.$ext"
      i=i+1
    done
    checkCIFixElements $baseName $eleNames
  fi

  #$NLS-E$ "installing CI fixpack \"$baseName\""
  Err_report info "`nls_replace 'KCI1130I' \\"$baseName\\"`"
  for pkg in $baseName.t00 $eleNames
  do
    unTarPkg $PKGLOC/$pkg
  done

  # Preserve existing fixID records.  Adding the values to an array will remove the newline
  # character returned from the grep operation.

  export LANG=C
  set -A fixIDs $(grep "fixID" $curRegPath/$ciVFile | grep -v $baseName | cut -d"=" -f2 2> /dev/null)
  export LANG=$saveLang

  # Copy the base .ver file (while changing its name), and add write access.

  cp $newRegPath/$ciLateVFile $curRegPath/$ciVFile > /dev/null 2>&1
  #$NLS-E$ "could not copy version file \"$ciLateVFile\""
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1131E' \\"$ciLateVFile\\"`"
  chmod 755 $curRegPath/$ciVFile > /dev/null 2>&1
  #$NLS-E$ "could not change access permissions on \"$ciVFile\""
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1132E' \\"$ciVFile\\"`"

  # Add the fixpack names which have been installed to the end of the .ver file.

  print "" >> $curRegPath/$ciVFile 2> /dev/null
  for pkg in ${fixIDs[*]} $baseName
  do
    print "fixID = $pkg" >> $curRegPath/$ciVFile 2> /dev/null
  done

  #46553 cpInstProg removed
  #cpInstProg $baseName.t00  # Copy install.sh to CANDLEHOME/bin.
fi

# Copy the tables only if they don't already exist in CANDLEHOME/registry.  Remember, the old and new tables
# will be merged by CandleInstall.
for txt in $tables
do
  if [ ! -r $curRegPath/$txt ]
  then
    if [ -r $newRegPath/$txt ]
    then
      cp $newRegPath/$txt $curRegPath/ > /dev/null 2>&1
      chmod 755 $curRegPath/$txt > /dev/null 2>&1
    fi
  fi

  #$NLS-E$ "cannot read file \"$curRegPath/$txt\""
  [ -r $curRegPath/$txt ] || Err_report fail "`nls_replace 'KCI1163E' \\"$curRegPath/$txt\\"`"
done

# Copy the kcirunas.cfg file to CANDLEHOME/config if it does not already exist there.

RUNASFILE=kcirunas.cfg
RUNASIMG=""
RUNASCFG="$CANDLEHOME/config/$RUNASFILE"
[[ -z "$RUNASIMG" && -f "$PROGDIR/$RUNASFILE" ]] && RUNASIMG="$PROGDIR/$RUNASFILE"
[[ -z "$RUNASIMG" && -f "$PKGLOC/../$RUNASFILE" ]] && RUNASIMG="$PKGLOC/../$RUNASFILE"
[[ -z "$RUNASIMG" && -f "$PKGLOC/$RUNASFILE" ]] && RUNASIMG="$PKGLOC/$RUNASFILE"
#$NLS-E$ "cannot locate file \"$RUNASFILE\" - please try the install again"
[[ -z "$RUNASIMG" ]] && Err_report fail "`nls_replace 'KCI11072E' \\"$RUNASFILE\\"`"

if [[ ! -f "$RUNASCFG" ]]
then
  cp "$RUNASIMG" "$RUNASCFG"
  #$NLS-E$ "could not copy file \"$RUNASIMG\" to \\"$RUNASCFG\\""
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI11074E' \\"$RUNASIMG\\" \\"$RUNASCFG\\"`"
fi

# Locate the architecture description table. This table is needed to run dynarch.shl.

archTbl=archdsc.tbl
if [ -r $newRegPath/$archTbl ]  # Use the "new" table if there is one.
then
  archTbl=$newRegPath/$archTbl
else
  archTbl=$curRegPath/$archTbl
fi
#$NLS-E$ "cannot read file \"$archTbl\" - please try the install again"
[ -r $archTbl ] || Err_report fail "`nls_replace 'KCI1133E' \\"$archTbl\\"`"

# Display the license agreement after we've unpacked it
# Command-line modes need to display the license text first

# Which license agreement to display? (default is license.txt)
[ -n "$hasRoma" ]             && license="terms.agr"
[ "$instFam" = "-romaMQSEC" ] && license="mqterms.agr"
[ "$instFam" = "-pqedit" ] && license="terms.agr_MsgEdit"
export LANG=C
licName=$(print $license | tr 'a-z' 'A-Z')
export LANG=$saveLang
# Get user agreement on licensing using LAP tool if its on CD, or Candle license if not
lapDisplayMode="none"

if [ -d "$PROD_PATH/unix/LAP" ]
then
  if [[ "$cmdLnRep" = "yes" || "$cmdLnMRep" = "yes" || "$cmdLnRPRep" = "yes" || "$cmdLnDepotRep" = "yes" ]]
  then
    lapDisplayMode="-text_only"
  else
    lapDisplayMode="-win_style 0"
  fi
else
  # If license file isn't there (maybe deleted), check /misc for a spare copy
  [ -r $license ]  || cp -p $CANDLEHOME/misc/.$license $license 2> /dev/null
  # also check the top level of the cd for the license file
  [ -r $license ]  || cp -p $PROD_PATH/$license $license 2> /dev/null

  # Get license agreement unless we're in silent mode
  if [ -z "$silentMode" ]
  then
    if [[ "$cmdLnRep" = "yes" || "$cmdLnMRep" = "yes" || "$cmdLnRPRep" = "yes" || "$cmdLnDepotRep" = "yes" ]]
    then
      while true		# license.txt gets option to postpone view @03
      do
        print " "
		#$NLS-E$ "Do you agree to the terms of the license in file $licName on the CD?"
        print "`nls_replace 'KCI1134I' $licName`"
		#$NLS-E$ "Enter Yes, No, or R to read the agreement now."
        print "1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`, `nls_replace 'KCI1135I' 3`"
		#$NLS-E$ "(choosing \"No\" exits the installation process) (Y/N/R): \c"
        print "`nls_replace 'KCI1136I' \\"2\\"` (1/2/3): \c"
        read Response
        case "$Response" in
          n*|N*|2) exit 0
                 ;;
          r*|R*|3) shoLicense
                 ;;
          y*|Y*|1) break
                 ;;
        esac
      done
    fi
  fi
fi

# Determine the machine's architecture.

dynShl=$CANDLEHOME/bin/dynarch.shl
#$NLS-E$ "cannot execute \"$dynShl\""
[ -x $dynShl ] || Err_report fail "`nls_replace 'KCI1137E' \\"$dynShl\\"`"
. $dynShl $archTbl  # dynarch.shl will determine a value for cArch.
#$NLS-E$ "could not determine the machine's architecture"
[ -n "$cArch" ] || Err_report fail "`nls_replace 'KCI1138E'`"
#$NLS-E$ "computed architecture = \"$cArch\""
Err_report info "`nls_replace 'KCI1139I' \\"$cArch\\"`"

#$NLS-E$ "HP 10 is not a supported operating system"
[[ "$cArch" = "hp10" ]] && Err_report fail "`nls_replace 'KCI1140E'`"
#$NLS-E$ "HP 10.20 is not a supported operating system"
[[ "$cArch" = "hp102" ]] && Err_report fail "`nls_replace 'KCI1141E'`"

#
# f14583 abstract      Install support for TEPS on Intel SuSE9
#
# TEPS needs to differentiate between Red and SUSE linux
export LANG=C
if [[ -f /etc/SuSE-release ]]
then
   VERSION=`cat /etc/SuSE-release | grep 'VERSION' | grep -v '^#' | sed  -e 's#[^0-9]##g'`
   SUSE_VERSION="-Dsuse_version=${VERSION}"
fi
export LANG=$saveLang

# Get the names of all JRE packages (products and fixpacks) in the product directory.

typeset jrPkgs=""

# Get a list of the JRE packages we have in the media
export LANG=C
jrPkgs=$(ls $PKGLOC/jr*.tar  $PKGLOC/jr*.ctp $PKGLOC/jr*.z* 2> /dev/null)
export LANG=$saveLang

#always unload JRE packages .ver files to tmp directory
#unless the JRE .ver file is also external to the JRE package
if [ -n "$jrPkgs" ]
then
  #$NLS-E$ "processing JRE packages"
  Err_report info "`nls_replace 'KCI1142I'`"
  for jre in $jrPkgs
  do
  	#
  	# if the jre does not have read permission and
  	# this is not deploy mode exit
  	if [[ ! -r $jre && -z "$DEPLOY_MODE" ]]
  	then
	    #$NLS-E$ "cannot read package \"$jre\""
  		Err_report fail "`nls_replace 'KCI1143E' \\"$jre\\"`"
  	fi

    bName=${jre##*/}  # Remove the pathname.

    # Construct the name of the .ver file.

    ext=${bName##*.}  # ext -> tar, ctp, z00, z01, z02, ...
    if [[ "$ext" = "tar" || "$ext" = "ctp" || "$ext" = "z00" ]]
    then
      vFile=${bName%.*}.ver      # vFile -> jraix4.ver
    else
      vFile=${bName%.*}$ext.ver  # vFile -> jraix4z01.ver
    fi

    # copy jre .ver from installation media if available
    if [[ -f $PKGLOC/$vFile ]]
    then
      cp $PKGLOC/$vFile $CANDLEHOME/tmp/ > /dev/null 2>&1
    else
      unloadTmpFile $vFile $jre  # Extract the .ver file.
    fi

	#$NLS-E$ "cannot read file \"$newRegPath/$vFile\""
    [ -r "$newRegPath/$vFile" ] || Err_report fail "`nls_replace 'KCI1144E' \\"$newRegPath/$vFile\\"`"

  done
fi

# See if there is a JRE installed.

osn=${cArch%%[0-9]*}  # Get the name (e.g. aix) of the operating system.
#$NLS-E$ "About to check for installed JRE"
Err_report info "`nls_replace 'KCI1145I'`"
curJVF=$(grep -l "^runArch *= *$cArch$" $curRegPath/jr$osn*.ver 2> /dev/null)
if [ -z "$curJVF" ]  # Look for a "runArch = open" record.
then
  export LANG=C
  set -A ta $(grep -l "^runArch *= *open" $curRegPath/jr$osn*.ver 2> /dev/null)
  export LANG=$saveLang
  numEle=${#ta[*]}
  # There may be more than one .ver file with an "open" record, so use the latest .ver file.
  [ numEle -gt 0 ] && curJVF=${ta[numEle-1]}

	# 17901.1 - additional check for a 32-bit JRE
  if [ -z "$curJVF" ]  # Look for the 32-bit version if this is 64-bit
  then
    # get architecture bit (3 = 32 or 6 = 64)
    typeset -i len=0
    len=$(print $cArch|wc -c)
    len=$(($len-1))
    bitNum=$(print $cArch|cut -c$len)
    [ -z "$bitNum" ] && bitNum=3

    # if we're 64-bit
    if [ "$bitNum" = 6 ]
    then
	  #$NLS-E$ "64bit JRE not found looking for installed 32bit JRE"
      Err_report info "`nls_replace 'KCI1146I'`"
      newRunArch=$(print $cArch | sed 's/6$/3/g') # replace ending 6 with 3
      export LANG=C
	  set -A ta $(grep -l "^runArch *= *$newRunArch$" $curRegPath/jr$osn+([0-9]).ver 2> /dev/null)
      export LANG=$saveLang
	  numEle=${#ta[*]}
      # There may be more than one .ver file, so use the latest .ver file.
      [ numEle -gt 0 ] && curJVF=${ta[numEle-1]}
	  #$NLS-E$ "current JRE is \"$curJVF\""
      Err_report info "`nls_replace 'KCI1147I' \\"$curJVF\\"`"
    fi
  fi
fi
[ -n "$curJVF" ] && curJVF=${curJVF##*/}  # Remove the pathname.

# See if there is a new JRE to install.

export LANG=C
newJVF=$(grep -l "^runArch *= *$cArch$" $newRegPath/jr$osn+([0-9]).ver 2> /dev/null)  # Exclude *zxx .ver files.
export LANG=$saveLang
if [ -z "$newJVF" ]  # Look for a "runArch = open" record.
then
  export LANG=C
  set -A ta $(grep -l "^runArch *= *open" $newRegPath/jr$osn+([0-9]).ver 2> /dev/null)
  export LANG=$saveLang
  numEle=${#ta[*]}
  # There may be more than one .ver file with an "open" record, so use the latest .ver file.
  [ numEle -gt 0 ] && newJVF=${ta[numEle-1]}

  # 13526.1 - additional check for a 32-bit JRE
  if [ -z "$newJVF" ]  # Look for the 32-bit version if this is 64-bit
  then
    # get architecture bit (3 = 32 or 6 = 64)
    typeset -i len=0
    len=$(print $cArch|wc -c)
    len=$(($len-1))
    bitNum=$(print $cArch|cut -c$len)
    [ -z "$bitNum" ] && bitNum=3

    # if we're 64-bit
    if [ "$bitNum" = 6 ]
    then
      newRunArch=$(print $cArch | sed 's/6$/3/g') # replace ending 6 with 3
      export LANG=C
	  set -A ta $(grep -l "^runArch *= *$newRunArch$" $newRegPath/jr$osn+([0-9]).ver 2> /dev/null)
      export LANG=$saveLang
	  numEle=${#ta[*]}
      # There may be more than one .ver file, so use the latest .ver file.
      [ numEle -gt 0 ] && newJVF=${ta[numEle-1]}
    fi
  fi
fi

[ -n "$newJVF" ] && newJVF=${newJVF##*/}  # Remove the pathname.

# Determine if a JRE needs to be installed.

instJRE=no
#d94520- If we are installing from an application support only dvd, display a more useful error message
#   for this scenario. This would happen if the user is trying to install an application support image that
#   does not contain Java into a directory that does not contain an existing ITM environment. This scenario
#   is not supported.
if [[ "$PLATFORM_SUPPORTED_CHECK" = "NO" ]]
then

   # $NLS-E$ "KCI1292I Installations from application support media must be installed into a directory containing an existing IBM Tivoli Monitoring environment."
   [[ -z "$curJVF" && -z "$newJVF" ]] && Err_report fail "`nls_replace 'KCI1292I'`"
else

   #$NLS-E$ "This is not a supported platform or the installation media does not contain any components supported on this platform."
   [[ -z "$curJVF" && -z "$newJVF" ]] && Err_report fail "`nls_replace 'KCI1148E'`"
fi

#  Skip this step during remote deploys to allow for the case in which prereqs are already installed and are not part of the deploy package
if [[ -z "$DEPLOY_MODE" ]]
then
  #double check if this is supported platform, by checking jr & gs .ver files.
  if [[ $platformChecked = "NO" ]]; then
    isPlatformSupported $cArch;
    [[ $? -ne 0 ]] && Err_report fail "`nls_replace 'KCI1148E'`"
  fi
fi

[[ -n "$curJVF" && -z "$newJVF" ]] && { instJRE=no; toJVF=$curJVF; }

[[ -z "$curJVF" && -n "$newJVF" ]] && { instJRE=yes; toJVF=$newJVF; }

if [[ -n "$curJVF" && -n "$newJVF" ]]
then
  # See if the JRE in the product directory is newer than the one in CANDLEHOME.
  checkLevel $curJVF $newJVF
  if [ "$levelStrg" = "Newer" ]
  then
    instJRE=yes
    toJVF=$newJVF
  else
    instJRE=no
    toJVF=$curJVF
  fi
fi

# Install the JRE.

kbytes=0
if [ "$instJRE" = "yes" ]
then
  # See if there is enough room to install.

  export LANG=C
  kbytes=$(grep "^size[A-Z,a-z]* *=" $newRegPath/$newJVF 2> /dev/null | cut -d"=" -f2)
  export LANG=$saveLang
  #$NLS-E$ "could not determine the size of the base JRE package"
  [ kbytes -gt 0 ] || Err_report fail "`nls_replace 'KCI1149E'`"

  if [[ "$(uname -s)" = "NONSTOP_KERNEL" ]]	# Tandem df doesn't work as we expect, hard code now.
  then
      kbAvail=10000000
  else
      checkDiskSpace $CANDLEHOME $kbytes  # install.sh exits if space is not available.
  fi


  # Determine the name of the base package.

  bName=${newJVF%.*}  # Remove the .ver extension.
  bPkg=$(ls $PKGLOC/$bName.z00 2> /dev/null)  # Look first for the latest .z package
  [ -n "$bPkg" ] || bPkg=$(ls $PKGLOC/$bName.tar $PKGLOC/$bName.ctp 2> /dev/null)
  if [ -n "$bPkg" ]
  then
    bPkg=${bPkg##*/}  # Remove the pathname
  else
    # 13526.1 - for remote deploy, there may not be a native 64-bit JRE

    # get archtecture bit (3 = 32 or 6 = 64)
    typeset -i len=0
    len=$(print $cArch|wc -c)
    len=$(($len-1))
    bitNum=$(print $cArch|cut -c$len)
    [ -z "$bitNum" ] && bitNum=3

    # if we're 64-bit and can't find a 64-bit JRE, look for 32-bit
    if [ "$bitNum" = 6 ]
    then
      newRunArch=$(print $cArch | sed 's/6$/3/g') # replace ending 6 with 3
      export LANG=C
      set -A ta $(grep -l "^runArch *= *$newRunArch$" $newRegPath/jr$osn+([0-9]).ver 2> /dev/null)
      export LANG=$saveLang
	  numEle=${#ta[*]}
      # There may be more than one .ver file, so use the latest .ver file.
      [ numEle -gt 0 ] && newJVF=${ta[numEle-1]}
      [ -n "$newJVF" ] && newJVF=${newJVF##*/}  # Remove the pathname.
    fi
    bName=${newJVF%.*}  # Remove the .ver extension.
    bPkg=$(ls $PKGLOC/$bName.z00 2> /dev/null)  # Look first for the latest .z package
    [ -n "$bPkg" ] || bPkg=$(ls $PKGLOC/$bName.tar $PKGLOC/$bName.ctp 2> /dev/null)

    if [ -n "$bPkg" ]
    then
      bPkg=${bPkg##*/}  # Remove the pathname
    else
	  #$NLS-E$ " could not find a base JRE package in \"$PKGLOC\""
      Err_report fail "`nls_replace 'KCI1150E' \\"$PKGLOC\\"`"
    fi
  fi

  eleNames="" i=1 numEle=0
  if [ "$bPkg" = "$bName.z00" ]  # There may be elements, so process them.
  then
    numEle=$(grep "^numElements *=" $newRegPath/$newJVF 2> /dev/null | cut -d"=" -f2)
    if [[ $? -eq 0 && numEle -gt 1 ]]  # The JRE is contained in multiple packages.
    then
	  #$NLS-E$ "versioning element JRE packages"
      Err_report info "`nls_replace 'KCI1151I'`"
      while [ i -lt numEle ]
      do
        [ i -lt 10 ] && eleNames="$eleNames $bName.z0$i"
        [ i -ge 10 ] && eleNames="$eleNames $bName.z$i"
        i=i+1
      done
      checkJRElements $eleNames
    fi
  fi

  # EJF (#12868) If a JRE is going to be overwritten remove it first
  # (41680) we should remove base JRE only. There might be preReq JRE, we shouldn't remove it.
  # (53460) we should remove the JRE only, which will be overwritten.
  jrName=$(print $curJVF | cut -d . -f1 | cut -c3-)
  if [[ -n "$jrName" && -d $CANDLEHOME/JRE/$jrName ]]
  then
    #$NLS-E$ "removing old JRE"
    Err_report info "`nls_replace 'KCI1152I'`"
    if [ "$curJVF" = "$newJVF" ]
    then
    	rm -rf $CANDLEHOME/JRE/$jrName > /dev/null 2>&1
    	if [[ $? -eq 0 ]]
    	then
	  	#$NLS-E$ "old JRE has been removed"
      	Err_report info "`nls_replace 'KCI1153I'`"
      	if [[ -n "$curRegPath" && -n "$curJVF" && -f "$curRegPath/$curJVF" ]]
      	then
			#--------------------------------------------------------------#IZ95119
			# In lieu of removing the ver file, delete its content so we   #IZ95119
			# maintain its ownership and permissions when the new version  #IZ95119
			# takes its place later on below.                              #IZ95119
			#--------------------------------------------------------------#IZ95119
			print -n "" > $curRegPath/$curJVF 2>&1                         #IZ95119
			#$NLS-E$ "could not remove old JRE version file \"$curJVF\""
        	[ $? -eq 0 ] || Err_report warn "`nls_replace 'KCI1154I' \\"$curJVF\\"`"
      	fi
    	else
	  	#$NLS-E$ "unable to remove all or part of old JRE"
      	Err_report warn "`nls_replace 'KCI1155I'`"
    	fi
    fi
  fi

  #$NLS-E$ "unloading JRE package(s)"
  Err_report info "`nls_replace 'KCI1156I'`"
  for pkg in $bPkg $eleNames
  do
    unTarPkg $PKGLOC/$pkg
    if [ "$(uname -s)" = "NONSTOP_KERNEL" ]
    then
	while true
	do
  	    repeat=true
  	    if [ -z "$JAVALOC" ]
	    then
		        #$NLS-E$ "\nEnter the java installed location.(Default is /usr/tandem/java)"
				echo
    	        nls_replace 'KCI1157I' /usr/tandem/java
				export LANG=C
    	        read resp
				resp=$(eval print $resp)
    	        export LANG=$saveLang
				[ -z "$resp" ] && resp=/usr/tandem/java
	    else
		resp="$JAVALOC"
	    fi

	    if [[ ! -d "$resp" ]]
	    then
		#$NLS-E$ "\n$resp is not a correct directory."
		echo
		nls_replace 'KCI1158I' $resp
		[ -n "$silentMode" ] && exit	# if this is silent mode exit now
		JAVALOC=""
		continue
	    else
		if [ ! -x $resp/jre/bin/java ]
		then
		    #$NLS-E$ "\nThe $resp/jre/bin/java is not an executable file."
			echo
		    nls_replace 'KCI1159I' $resp/jre/bin/java
		    [ -n "$silentMode" ] && exit		# silent mode exit here.
		    JAVALOC=""
		    continue
		fi
	    fi

	    JreVersion=1.4.2
	    JreVersion15=1.5
	    jreCmd="$resp/jre/bin/java -version"
	    [ -f /tmp/jrever.txt ] && rm -f /tmp/jrever.txt
	    $jreCmd 2>/tmp/jrever.txt
	    jrever=$(grep -e "$JreVersion" -e "$JreVersion15" /tmp/jrever.txt)
	    [ -f /tmp/jrever.txt ] && rm -f /tmp/jrever.txt
	    if [[ -z $jrever ]]
	    then
		$resp/jre/bin/java -version
		#$NLS-E$ "\nThis is not a correct jre version. You have to install jre 1.4.2 or 1.5."
		print
		nls_replace 'KCI1175E'
		exit
	    fi
	    JAVALOC=$resp
	    break
	done
        ln -sf $JAVALOC/jre/bin $CANDLEHOME/JRE/$cArch/bin
        ln -sf $JAVALOC/jre/lib  $CANDLEHOME/JRE/$cArch/lib
    fi
  done

  #45988.1
  chmod -R 755 $CANDLEHOME/JRE > /dev/null 2>&1

  # Copy the .ver file for the base package, and add write access.

  cp $newRegPath/$newJVF $curRegPath/ > /dev/null 2>&1
  #$NLS-E$ "could not copy version file \"$newJVF\""
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1161E' \\"$newJVF\\"`"
  chmod 755 $curRegPath/$newJVF > /dev/null 2>&1
  #$NLS-E$ "could not change access permissions on \"$newJVF\""
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1162E' \\"$newJVF\\"`"
fi

  if [[ "$(uname -s)" = "NONSTOP_KERNEL" ]]	# Tandem df doesn't work as we expect, hard code now.
  then
      kbAvail=10000000
  else
      checkDiskSpace $CANDLEHOME $kbytes  # install.sh exits if space is not available.
  fi


# The JRE is now installed under $CANDLEHOME/JRE.

[ -d "$CANDLEHOME/$oldJRE" ] && mv $CANDLEHOME/$oldJRE $CANDLEHOME/$oldJRE.old > /dev/null 2>&1

# Determine which JRE to use (The JRE name is stored in env var JREEXE.), and set the CLASSPATH.
# Function setJavaEnv is defined in dynarch.shl.

setJavaEnv $cArch

# For Roma 2.1 overinstalls, we may have to rename some directories
[ "$instFam" = "-roma" ] && Bsp21Rename $RegBackup

# Display the LAP license agreement, if it was packaged
[ "$lapDisplayMode" != "none" ] && run_LAPTool $lapDisplayMode

# Recalculate available space and insure we have enough space for temporary files
# and logging to start the next phase of the installation process.

kbytes=10000
if [[ "$(uname -s)" = "NONSTOP_KERNEL" ]]	# Tandem df doesn't work as we expect, hard code now.
then
    kbAvail=10000000
else
    checkDiskSpace $CANDLEHOME $kbytes  # install.sh exits if space is not available.
fi

checkSYHDupgrade pre

# Start the installer.
if [[ "$supportCLIRep" = "yes" ]]
then
cmdLnRep="no"
cmdLnDepotRep="no"
startSupport
fi
[ "$guiRep" = "yes" ] && startGUI
[ "$cmdLnRep" = "yes" ] && startCmdLn prod
[ "$cmdLnMRep" = "yes" ] && startCmdLn fix
[ "$guiRPRep" = "yes" ] && startGUIRP
[ "$cmdLnRPRep" = "yes" ] && startCmdLnRP
[ "$cmdLnDepotRep" = "yes" ] && startCmdLnDepot
[ "$guiDepotRep" = "yes" ] && startGUIDepot

# Restore the library path.  It was saved in dynarch.shl.  The restoration must be done after CandleInstall
# has completed.

if [ -n "$saveLibPath" ]
then
  [[ "$1" = aix+([0-9]) ]] && export LIBPATH="$saveLibPath"
  [[ "$1" = hp+([0-9]) ]] && export SHLIB_PATH="$saveLibPath"
  [[ "$1" = hpi+([0-9]) ]] && export SHLIB_PATH="$saveLibPath"
  [[ "$1" = sol+([0-9]) ]] && export LD_LIBRARY_PATH="$saveLibPath"
  [[ "$1" = tsf+([0-9]) ]] && export LD_LIBRARY_PATH="$saveLibPath"
fi

export LANG="$saveLang"
export PATH="$savePath"

# change owner and group fot Linux if ID is root defect 12436
if [[ "$thisOS" = "Linux" && "$thisUser" = "root" ]]
then
    chown -R root $CANDLEHOME/bin $CANDLEHOME/config $CANDLEHOME/HTML $CANDLEHOME/images $CANDLEHOME/samples $CANDLEHOME/misc $CANDLEHOME/JRE  $CANDLEHOME/classes  $CANDLEHOME/tmp
    chgrp -R root $CANDLEHOME/bin $CANDLEHOME/config $CANDLEHOME/HTML $CANDLEHOME/images $CANDLEHOME/samples $CANDLEHOME/misc $CANDLEHOME/JRE  $CANDLEHOME/classes  $CANDLEHOME/tmp
    [ -d $CANDLEHOME/tables ] && chmod -R 777 $CANDLEHOME/tables
fi

if [[ -z "$SECURE_GROUP" ]];
then
  # Set write permissions for some directories
  chmod -R 777 $CANDLEHOME/bin 2> /dev/null
  chmod -R 777 $CANDLEHOME/config 2> /dev/null
  chmod -R 777 $CANDLEHOME/logs 2> /dev/null
  chmod -R 777 $CANDLEHOME/tmp 2> /dev/null

  # These products require more write permissions than others

  # defect 18183, the permssions set in SetPerm are being overwritten here
  chmod -R 777 $CANDLEHOME/*/um/work 2> /dev/null
  chmod -R 777 $CANDLEHOME/*/um/tables 2> /dev/null
  chmod -R 777 $CANDLEHOME/*/um/scripts 2> /dev/null
  chmod -R 777 $CANDLEHOME/*/um/metafiles 2> /dev/null
  chmod -R 777 $CANDLEHOME/*/qi 2> /dev/null

  # EJF (#12046) ensure that the images directory has read access for everyone
  chmod -R 755 $CANDLEHOME/images 2> /dev/null
fi

# set AUTORUN_SECURE variable to skip secureMain execution in UpdateAutoRun.sh
export AUTORUN_SECURE=no

# begin feature 204833 (RFE 33478)
# When -s switch for install.sh is used during pristine install we skip updating
# startup scripts. In this case value 0 is stored in registry/AutoStart file.
# All subsequent installations/upgrades will also skip startup script update.
# In order to restore startup script update functionality it's enough to remove
# registry/AutoStart file.
if [ -n "$SKIP_STARTUP_UPDATE" ]; then
    if [ -f "$CANDLEHOME/registry/AutoStart" ]; then
        #$NLS-E$ "KCI1366W Option -s (skip startup script update) ignored - this option is available during a pristine installation only."
        Err_report warn "`nls_replace 'KCI1366W'`"
    else
        echo 0 > "$CANDLEHOME/registry/AutoStart" 2> /dev/null
    fi
fi
# end feature 204833 (RFE 33478)

# defect 44550, run UpdateAutoRun.sh only for root user
# $CANDLEHOME/bin/UpdateAutoRun.sh $CANDLEHOME
if [ "${thisRealUser}" = "root" ]; then                                 #IZ94626.1
    $CANDLEHOME/bin/UpdateAutoRun.sh -h $CANDLEHOME
else
    # For non-root user, just display a warning
    #$NLS-E$ "The root user must run bin/UpdateAutoRun.sh from the ITM home to complete the setup"
    Err_report warn "`nls_replace 'KCI1042I' bin/UpdateAutoRun.sh`"

    # For non-root on aix system, display warning
    #$NLS-E$ "The root user must run 'slibclean' utility and restart all components afterwards"
    [[ $os = "ai" ]] && [[ -z "$localGSKitEnabled" ]] && [[ -f "$CANDLEHOME/config/gsKit.config" ]] && Err_report warn "`nls_replace 'KCI1255I'`"
fi

# secure ITM installation
$CANDLEHOME/bin/itmsecure.sh install

checkSYHDupgrade post
cd $CANDLEHOME/bin
restart_proc start

if [[ -n "$syNotStart" || -n "$hdNotStart" ]];
then
  oldSYver=$(print $oldSYver | sed 's/[ ]*$//')
  oldHDver=$(print $oldHDver | sed 's/[ ]*$//')
  if [[ -n "$syNotStart" && ! $oldSYver = "610" ]] || [[ -n "$hdNotStart" && ! $oldHDver = "610" ]];
  then
    nls_replace 'KCI1317I'
    else
    nls_replace 'KCI1251I'
  fi
fi

cd - 2> /dev/null > /dev/null

cleanUp 0
